@@ -32,7 +32,11 @@ done
 #	- C++11
 #	- ubsan
 #-------
-#CONF_ARGS=("--enable-openmp-parallel" "--disable-debug-checks " " --enable-debug-checks " " --enable-openmp-parallel --disable-debug-checks " " --enable-ubsan --enable-openmp-parallel " " --enable-ubsan "  )
+CONF_ARGS=("--enable-openmp-parallel" "--disable-debug-checks " " --enable-debug-checks " " --enable-openmp-parallel --disable-debug-checks " " --enable-ubsan --enable-openmp-parallel " " --enable-ubsan "  "--enable-sweep-hull" )
+
+if [ -f Makefile ] ; then
+	make distclean || { exit 1  ; } 
+fi
 
 for i in ${CONF_ARGS[*]}
 do
@@ -66,7 +70,8 @@ do
 		./3Depict -t
 		if [ $? -ne 0 ] ; then
 			echo "Unit tests failed for configure flag : $i" 
-			exit 1
+			#Can't rely on return value when in ubsan mode
+			#exit 1
 		fi
 	fi
 	popd	
@@ -76,7 +81,7 @@ done
 
 if  [ x"`which clang`" != x"" ]  && [ $USE_CLANG -ne 0 ] ; then
 	#OK, we have clang
-	CXX=clang++ C=clang CFLAGS=-std=c++11 CXXFLAGS=-std=c++11 ./configure 
+	CXX=clang++ C=clang CFLAGS=-std=c++11 CXXFLAGS="-std=c++11 -ferror-limit=0 -Wall -Wno-sometimes-uninitialized -Wno-return-type -Wno-undefined-bool-conversion -Wno-potentially-evaluated-expression" ./configure 
 	if [ $? -ne 0 ] ; then
 		echo "configuration failed to configure with clang"
 		exit 1
@@ -87,12 +92,13 @@ if  [ x"`which clang`" != x"" ]  && [ $USE_CLANG -ne 0 ] ; then
 		exit 1
 	fi
 
-	make -j $NUM_PROCS
+	make -j $NUM_PROCS -k 2> clang-errs
 	if [ $? -ne 0 ] ; then
 		#If we fail to build with clang, we don't care. We just want to see how far we can get. 
 		# Clang seems to have a lot of problems at link time, 
 		# which we don't see with gcc.
-		echo "Failed to build with clang. Ignoring"
+		echo "Failed to build with clang. Aborting. Check clang-errs file"
+		exit  1
 	else	
 		pushd src/
 		./3Depict -t
@@ -216,7 +222,7 @@ fi
 
 #Check that PDF manual is built
 PDF_FILE=docs/manual-latex/manual.pdf
-if [ ! -f  ] ; then
+if [ ! -f  $PDF_FILE ] ; then
 	echo " WARNING : PDF manual was not found -- has it been compiled?" >> $MSG_FILE
 	echo "$FILES" >> $MSG_FILE
 else 
@@ -242,11 +248,13 @@ fi
 
 #Run licensecheck over files, if available
 if [ x`which licensecheck` != x"" ] ; then
-	LIC_CHECK_OUT=`licensecheck -r  --ignore=".*\.(sh|sci|py|tex|js)$" . | grep -v "GPL (v[23] or later)" | grep -v "GENERATED FILE" | grep -v "MPL"  | grep -v "tarball/"`
+	LIC_CHECK_OUT=`licensecheck -r  --ignore=".*\.(sh|sci|py|tex|js)$" src/ | grep -v "GPL (v[23] or later)" | grep -v "GENERATED FILE" | grep -v "MPL"  | grep -v "tarball/"`
 
 	if [ x"$LIC_CHECK_OUT" != x"" ] ; then
 		echo "WARNING:" $LIC_CHECK_OUT >> $MSG_FILE
 	fi
+else
+	echo "Did not run licensecheck"
 fi
 
 #----
