@@ -11,6 +11,50 @@
 #HOST_VAL=x86_64-w64-mingw32 #For mingw64 (windows 64 bit)
 #HOST_VAL=i686-w64-mingw32 #For mingw32 (Windows 32 bit)
 
+#Setup clone of hg repository, if required
+if [ ! -d code ] || [ ! -d code/3Depict ]  ; then
+	read -p "Code dir is missing -- shall we clone it?? (y/n):" answer
+	
+	case $answer in
+		y | Y ) 
+			if [ x`which hg` == x"" ] ; then
+				echo "No mercurial.. Can't clone, aborting."
+				exit 1
+			fi
+
+			DEFCLONEURL="http://hg.code.sf.net/p/threedepict/code"
+
+			read -p "Path to clone (include protocol if url...), blank for default" clonedir
+		
+			if [ x"$clonedir " == x"" ] ; then 
+				CLONEURL=$DEFCLONEURL
+			else
+				CLONEURL=$clonedir	
+			fi
+
+			mkdir code  || { echo "failed to create code dir" ; exit 1 ; }
+			pushd code
+			
+			#Clone code
+			hg clone $CLONEURL  3Depict || { echo "Failed to clone" ; exit 1 ; }
+			popd
+
+			#obtain patches, and then put it into place
+			cp -R code/3Depict/packaging/mingw-debian-cross/patches . || { echo "Failed to obtain patches from mingw packging dir" ; exit 1 ; }
+
+
+
+			;; 
+		n | N )
+			echo "You said no."
+			;;
+		*) 
+			echo "Aborting - please answer Y, or N, or create the \"code\" directory, with "
+			exit 1
+			;;
+	esac #ends the case list
+fi
+
 if [ ! -f host_val ] ; then
 	echo "Please select 32 or 64 bit by typing \"32\" or \"64\" (32/64)"
 	read HOST_VAL
@@ -88,16 +132,19 @@ PATCHES_FTGL_POSTCONF="ftgl-override-configure-2"
 PATCHES_ICONV="iconv-fix-alias2.patch"
 
 #1) gettext-tools fails in various places, but we don't actually need it, so turn it off
-#2) gettext fails to correctly determine windows function call prefix.
-#   should be fixed for gettext > 0.18.1.1 ?
-#   https://lists.gnu.org/archive/html/bug-gettext/2012-12/msg00071.html
-PATCHES_GETTEXT="gettext-fix-configure-versions"    #gettext-win32-prefix
+PATCHES_GETTEXT="gettext-fix-configure-versions gettext-disable-tools"   
 
-PATCHES_GLEW="glew-makefile.base"
+PATCHES_GLEW="glew-makefile.base glew-disable-dllentrypoint"
 
-PATCHES_MATHGL="mathgl-openmp-linker-flag mathgl-disable-things"
-PATCHES_QHULL="qhull-ptr.patch"
-PATCH_LIST="$PATCHES_WXWIDGETS_POST $PATCHES_GSL $PATCHES_ZLIB $PATCHES_LIBPNG $PATCHES_GETTEXT $PATCHES_FTGL $PATCHES_GLEW $PATCHES_MATHGL $PATCHES_FTGL_POSTCONF $PATCHES_ICONV $PATCHES_QHULL"
+PATCHES_MATHGL="mathgl-disable-things"
+
+PATCHES_QHULL="qhull-ptr.patch" # qhull-2009-configure_ac.patch"
+
+#Vigra's cmakelists assumes you can execute compiled binaries on the host
+# this obviously won't work for cross compilation
+PATCHES_LIBVIGRA="vigra-cpp-version.patch"
+
+PATCH_LIST="$PATCHES_WXWIDGETS_POST $PATCHES_GSL $PATCHES_ZLIB $PATCHES_LIBPNG $PATCHES_GETTEXT $PATCHES_FTGL $PATCHES_GLEW $PATCHES_MATHGL $PATCHES_FTGL_POSTCONF $PATCHES_ICONV $PATCHES_QHULL $PATCHES_LIBVIGRA"
 
 
 BUILD_STATUS_FILE="$BASE/build-status"
@@ -159,20 +206,27 @@ function grabDeps()
 {
 	pushd deps 2>/dev/null
 
-	DEB_PACKAGES="qhull expat freetype ftgl gettext gsl libpng libxml2 mathgl tiff zlib glew libvigraimpex"
+	DEB_PACKAGES="qhull expat freetype ftgl gettext gsl libxml2 mathgl tiff zlib glew libvigraimpex libpng-dev"
 	if [ x$DIST_NAME == x"Ubuntu" ] || [ x$DIST_NAME == x"LinuxMint" ]  ; then 
        		LIBJPEGNAME="libjpeg6b"
 	else
 		#Libjpeg seems to be forked/renamed very frequently in debian
 		# Likely a new libjpeg will need to be picked each time this script is run
-		LIBJPEGNAME="libjpeg9"
+		LIBJPEGNAME="libjpeg-turbo"
 	fi
 	DEB_PACKAGES="$DEB_PACKAGES $LIBJPEGNAME"
 
 	GET_PACKAGES=""
 	for i in $DEB_PACKAGES
 	do
-		FNAME=`ls packages/${i}_*.orig.* 2> /dev/null`
+
+		#libpng-dev is a virtual package?
+		if [ x"$i" != x"libpng-dev" ] ; then
+			FNAME=`ls packages/${i}_*.orig.* 2> /dev/null`
+		else
+			FNAME=`ls packages/libpng*_*.orig.* 2> /dev/null`
+		fi
+
 		#If filename is empty, we will need to retreive it from
 		# interwebs
 		if [ x"$FNAME" == x"" ] ; then
@@ -211,6 +265,49 @@ function grabDeps()
 		mv *.orig.* *.debian.* *.dsc *.diff.* packages 
 	fi
 
+	#---
+	#Download dsc package not in archive (pull from debian mentors)
+	FILES_LIBVD="libvd_1.1.0+svn7-1.dsc libvd_1.1.0+svn7-1.debian.tar.xz libvd_1.1.0+svn7.orig.tar.gz"
+	URL_LIBVD="https://mentors.debian.net/debian/pool/main/libv/libvd/"
+	SHA_LIBVD="70ad1c77d807f0eea259ae4b88a0eb1284401467 afac97ecfc678965a241f7132d14ebb8c09d0be1 d66f48b3b2d00d3543cb1d3de572ef9c9acd4875"
+
+
+	#Check SHA1sum hash to ensure integrity
+	#----
+	rm sha1_libvd.tmp
+	for i in $FILES_LIBVD
+	do
+		if [ ! -f $i ] ; then
+			if [ ! -f packages/$i  ] ; then
+				wget ${URL_LIBVD}${i} || { echo "Unable to download libvd DSC"; exit 1; }
+			else
+				mv packages/$i .
+			fi
+		fi
+
+		sha1sum $i | awk '{ print $1 }' >>sha1_libvd.tmp
+	done
+
+	
+	SHA_DOWNLOADS=`cat sha1_libvd.tmp | tr '\n' ' '  | sed 's/ *$//'`
+	if [ x"${SHA_DOWNLOADS}" != x"${SHA_LIBVD}" ] ; then
+		echo "SHA1 sums were : |"${SHA_DOWNLOADS}"|"
+		echo " Should be: |"${SHA_LIBVD}"|"
+		exit 1
+	fi
+	rm sha1_libvd.tmp
+	#----
+	
+	#extract
+	if [ ! -d libvd-1.1.0+svn7 ] ; then	
+		dpkg-source --no-check -x libvd*dsc || { echo "LIBVD Package extraction failed"; exit 1; }
+	fi
+
+	#Move into package folder
+	mv libvd*{dsc,gz,xz} packages/
+	#---
+
+
 	#Check that we have untarred all the required packages
 	# (eg we downloaded one, and wiped it at some stage)
 	# if not, pull the package out, and re-build it
@@ -269,6 +366,28 @@ function grabDeps()
 		sudo apt-get install nsis || { echo "Failed installation"; exit 1; }
 	fi
 
+	#Download wxwidgets as needed
+	#----
+
+	WXVER=wxWidgets-3.1.0
+	if [ ! -f packages/${WXVER}.tar.bz2 ] ; then
+		wget "https://github.com/wxWidgets/wxWidgets/releases/download/v3.1.0/${WXVER}.tar.bz2"
+		mv ${WXVER}.tar.bz2 packages/
+	fi
+
+	#check SHA256
+	if [ x`sha256sum packages/${WXVER}.tar.bz2 | awk '{print $1}'` != x'e082460fb6bf14b7dd6e8ac142598d1d3d0b08a7b5ba402fdbf8711da7e66da8' ] ; then
+		echo "SHA256 sum mismatch for wxwidgets";
+		exit 1
+	fi
+	
+	#Extract
+	if [ ! -d ${WXVER} ] ; then
+		tar -jxf packages/${WXVER}.tar.bz2 || { echo "failed extracting wxwidgets "; exit 1; }
+	fi	
+	#----
+
+
 	popd 2> /dev/null
 
 }
@@ -385,7 +504,10 @@ function build_glew()
 	cp patches/glew-makefile.base patches/glew-makefile
 	sed -i "s@HOST_VAL@$HOST_VAL@" patches/glew-makefile
 	sed -i "s@BASEDIR@$BASE@" patches/glew-makefile
-	
+	#FIXME: I patched out the bin/ subdir, around line 155
+	echo "There is a patch missing to remove the tool build"
+	exit 1
+
 	pushd deps >/dev/null
 	pushd glew-* >/dev/null
 
@@ -402,17 +524,28 @@ function build_glew()
 	make clean
 	rm -f configure.log
 	
-	LD=$CC make -j $NUM_PROCS || { echo "glew build failed"; exit 1; } 
+	LD=$CC CC=$CC CXX=$CXX LDFLAGS=-lstdc++ make -j $NUM_PROCS || { echo "glew build failed"; exit 1; } 
 
 	make install DESTDIR="$BASE"|| { echo "glew install failed"; exit 1; } 
 
 	popd >/dev/null
 	popd >/dev/null
 
+
+	#Check that glew built as a dynamic library. this should cause the init funcction
+	# to be preceded by the windows dynamic link prefix, __imp__
+	IMP_PREFIX=`find ./ -name \*dll | xargs x86_64-w64-mingw32-objdump -x | grep Init | grep __imp`
+	if [ x"$IMP_PREFIX" == x"" ] ; then
+		echo " Glew build failed, due to missing __imp__ prefix (dynamic link)"
+		exit 1
+	fi
+
 	#remove static library, as this can be incorrectly caught by linker
 	# leading to confusing messages
 	rm ${BASE}/lib/libglew*.a
 
+	#FIXME: The DLL and the include files are not being installed properly
+
 	echo "glew" >> $BUILD_STATUS_FILE
 }
 
@@ -424,7 +557,7 @@ function build_libpng()
 		return;
 	fi
 	pushd deps >/dev/null
-	pushd libpng-* >/dev/null
+	pushd libpng[0-9.]*-* >/dev/null
 	
 	if [ $? -ne 0 ] ; then
 		echo "libpng dir missing, or duplicated?"
@@ -433,7 +566,7 @@ function build_libpng()
 
 	make clean
 
-	./configure --host=$HOST_VAL --enable-shared --disable-static || { echo "Libpng configure failed"; exit 1; } 
+	./configure --host=$HOST_VAL --prefix=/ --enable-shared --disable-static || { echo "Libpng configure failed"; exit 1; } 
 
 	#Hack to strip linker version script
 	# eg as discussed : 
@@ -642,7 +775,7 @@ function build_qhull2012()
 	make SO="dll" -j $NUM_PROCS 
 	find ./ -name \*dll -exec cp {} ${BASE}/bin/	
 	make SO="dll" -j $NUM_PROCS || { echo "qhull build failed"; exit 1; } 
-	make install DESTDIR="$BASE"|| { echo "qhull install failed"; exit 1; } 
+	make SO="dll" install DESTDIR="$BASE"|| { echo "qhull install failed"; exit 1; } 
 
 	popd >/dev/null
 	popd >/dev/null
@@ -658,7 +791,7 @@ function build_qhull2012()
 
 function build_qhull2015()
 {
-	NAME="libqhull"
+	NAME="libqhull_2015"
 	ISBUILT_ARG=${NAME}
 	isBuilt
 	if [ $ISBUILT -eq 1 ] ; then
@@ -688,7 +821,7 @@ function build_qhull2015()
 	#  IMHO, once I manually alter a cmakefile, the cache should fucking well work out that it is out-of-date, as I keep getting nasty suprises.
 	# https://cmake.org/Bug/view.php?id=14820 .
 	rm -f CMakeCache.txt
-	cmake -DCMAKE_INSTALL_PREFIX="$BASE" -DCMAKE_TOOLCHAIN_FILE=../../patches/cmake-toolchain$BITS_VAL 
+	cmake -DCMAKE_INSTALL_PREFIX="$BASE" -DCMAKE_TOOLCHAIN_FILE=../../patches/cmake-toolchain$BITS_VAL -DWITH_BOOST=0 -DWITH_FFTW=0 -DWITH_LEMON=0 
 
 
 	#TODO: Better test (using c)
@@ -721,7 +854,7 @@ function build_qhull2015()
 
 function build_qhull2009()
 {
-	NAME="libqhull"
+	NAME="libqhull_2009"
 	ISBUILT_ARG=${NAME}
 	isBuilt
 	if [ $ISBUILT -eq 1 ] ; then
@@ -736,16 +869,35 @@ function build_qhull2009()
 		exit 1
 	fi
 
+
+	#The as-distributed version of libtool no longer builds dlls with my gcc, it just fails silently. Rebuild
+	libtoolize --install
+	autoreconf
+	automake --add-missing
+
+	#FIXME: Need to change the ptrT type on 64 bit, in mem.h
+	echo "Need to change ptrT, mem.h..."
+	exit 1
+
+	
+	make distclean
+
 	./configure --host=$HOST_VAL --enable-shared --disable-static --prefix=/ || { echo "$NAME configure failed"; exit 1; } 
 
 	sed -i "s/ gcc$/${HOST_VAL}-gcc/" Makefile
 	sed -i "s/ g++$/${HOST_VAL}-g++/" Makefile
 
+	
 	make SO="dll" -j $NUM_PROCS 
 	find ./ -name \*dll -exec cp {} ${BASE}/bin/	
 	make SO="dll" -j $NUM_PROCS || { echo "qhull build failed"; exit 1; } 
 	make install DESTDIR="$BASE"|| { echo "qhull install failed"; exit 1; } 
 
+	if [ x`find ./ -name \*.dll` == x"" ] ; then
+		echo "Qhull DLL missing, after build. That's not right... (last time due to -no-undefined not given to linker flags, which libtool then decides means should do a static build. Updating libtool makes this more sane, and errors out)"
+		exit 1;
+	fi
+
 	popd >/dev/null
 	popd >/dev/null
 
@@ -756,10 +908,56 @@ function build_qhull2009()
 	echo ${NAME} >> $BUILD_STATUS_FILE
 }
 
+function build_libvd()
+{
+	echo "FIXME: You need to disable the soname generation in the libvd makefile, otherwise the dll wont load properly"
+	exit 1
+
+	NAME="libvd"
+	ISBUILT_ARG=${NAME}
+	isBuilt
+	if [ $ISBUILT -eq 1 ] ; then
+		return;
+	fi
+	pushd deps >/dev/null
+	pushd libvd*[0-9]* >/dev/null
+	
+	if [ $? -ne 0 ] ; then
+		echo "${NAME} dir missing, or duplicated?"
+		exit 1
+	fi
+
+	pushd src/
+	#Make sure we do not pull any VS code in
+	rm -rf msvc
+
+
+	make clean
+
+	#Change the hard-coded compiler as needed
+	if [ x`grep ${HOST_VAL} Makefile` == x"" ] ; then
+
+		sed -i s/gcc/${HOST_VAL}-gcc/ Makefile || { echo " compilation replacement failed" ; exit 1 ; }
+		sed -i s/g++/${HOST_VAL}-g++/ Makefile || { echo " compilation replacement failed" ; exit 1 ; }
+	fi
+
+	#FIXME: Getting errors about undefined reference to pglUseProgram (and friends). These are the shader
+	# definitions, which are missing under windows
+	#  This is because it picks up the msvc folder. We now remove this. 
+	# however I have not yet made a patch for this
+	LDFLAGS="-L/home/pcuser/mingw64/lib/ -lopengl32 -lglu32 -lglew32 -lstdc++" make || { echo "libvd compilation failed"; exit 1; }
+
+	popd
+
+	FIX_LA_FILE_ARG=libvd
+	fix_la_file
+	echo ${NAME} >> $BUILD_STATUS_FILE
+}
+
 #FIXME: This does not work. Qhull uses a strange combination of cmake
 # and hand makefiles, so propagating correct cross-compiling 
 # parameters is quite tricky
-function build_qhull2015()
+function build_qhull2015b()
 {
 	NAME="libqhull"
 	ISBUILT_ARG=${NAME}
@@ -884,18 +1082,30 @@ function build_wx()
 		echo "wxwidgets dir missing, or duplicated?"
 		exit 1
 	fi
-	WX_VER=`grep "WX_RELEASE =" Makefile`
+	WX_VER=`egrep "WX_RELEASE\s*=" Makefile.in | awk -F= '{print $2}' | sed 's/\s*//'`
 
-	if [ x"${WX_VER}" == x"3.0" ] ; then
-		echo "WX needs to be at least 3.1, but is 3.0..."
-		exit 1
-	fi
+	case ${WX_VER} in
+		"3.0" )  
+			echo "WX needs to be at least 3.1, but is 3.0..."
+			echo "this is due to a propgrid bug"
+			exit 1
+			;;
+		[0-9].[0-9])
+			;;
+		"" )
+			echo "blah! - wx version not set!"
+			exit 1
+			;;
+	esac
 
 	make clean
 
-	WX_DISABLE="--disable-compat26 --disable-compat28 --disable-ole --disable-dataobj --disable-ipc --disable-apple_ieee --disable-zipstream --disable-protocol_ftp --disable-mshtmlhelp --disable-aui --disable-mdi --disable-postscript --disable-datepick --disable-splash --disable-wizarddlg --disable-joystick --disable-loggui --disable-debug --disable-logwin --disable-logdlg --disable-tarstream --disable-fs_archive --disable-fs_inet --disable-fs_zip --disable-snglinst --disable-sound --disable-variant --without-regex"
+	#It might be possible to make this more aggressive. WX includes a lot of stuff we don't have any use for.
+	# Linux distributions disale their web stuff (eg disable-webkit), as there are apparently serious security issues with wx's implementation
+	# as a bonus, the more we disable, the less we have to compile!
+	WX_DISABLE="--disable-compat26 --disable-compat28 --disable-ipc --disable-apple_ieee --disable-zipstream --disable-protocol_ftp --disable-mshtmlhelp --disable-aui --disable-mdi --disable-postscript --disable-datepick --disable-splash --disable-wizarddlg --disable-joystick --disable-loggui --disable-debug --disable-logwin --disable-logdlg --disable-tarstream --disable-fs_archive --disable-fs_zip --disable-snglinst --disable-sound --without-regex --disable-stc --disable-bannerwindow --disable-splash --disable-mediactr --disable-webkit"
 
-	./configure --host=$HOST_VAL --enable-shared --disable-static --with-opengl --enable-unicode --without-regex --prefix=/ || { echo "wxwidgets configure failed"; exit 1; } 
+	./configure --host=$HOST_VAL --enable-shared --disable-static --with-opengl --enable-unicode $WX_DISABLE --enable-wxurl --prefix=/ || { echo "wxwidgets configure failed"; exit 1; } 
 
        #TODO: Where is this coming from ???
 	for i in `find ./ -name Makefile | grep -v samples | grep -v wxPython`
@@ -913,7 +1123,7 @@ function build_wx()
 
 	pushd ./bin/
 
-	if [ -l wx-config ] ; then
+	if [ -L wx-config ] ; then
 		unlink wx-config
 	fi
 
@@ -926,7 +1136,7 @@ function build_wx()
 	fi
 
 	if [ x"$WX_CONFIG_FILE" == x"" ] ; then
-		WX_CONFIG_FILE=`find ${BASE}/lib/wx/config/ -type f  -executable -name x\*mingw32-msw-unicode-\*`
+		WX_CONFIG_FILE=`find ${BASE}/lib/wx/config/ -type f  -executable -name \*mingw32-msw-unicode-\*`
 	fi	
 
 	if [ x"$WX_CONFIG_FILE" == x"" ] ; then
@@ -941,18 +1151,25 @@ function build_wx()
 	PATCH_LEVEL=1
 	sed -i "s@REPLACE_BASENAME@${BASE}@" wx-config || { echo "Failed to update wx-config with build root,. Aborting";  exit 1; }
 	popd
+	#FIXME : We still need to fix some problems with wx-config, wx-config --rescomp returns empty
 
 	pushd ./lib/
 	ln -s wx-${WX_VER}/wx/ wx
 	popd
 
 	pushd ./include/wx-${WX_VER}/wx/msw/
-	if [ x$BITS_VAL == 64 ] ; then
+	if [ x"$BITS_VAL" == x"64" ] ; then
+		echo "Replacing manifest"
 		cp amd64.manifest wx.manifest
 	fi
 	popd	
 
 
+
+	pushd ./include
+	ln -s wx-${WX_VER}/wx/ wx
+	popd
+
 	echo ${NAME} >> $BUILD_STATUS_FILE
 
 }
@@ -975,13 +1192,28 @@ function build_freetype()
 
 	tar -xjf freetype-[0-9]*bz2  || { echo "freetype decompress failed" ; exit 1; } 
 
-	pushd freetype-[0-9]*
+	#For some reason, the debian package puts freetype in its own self-subdir
+	pushd freetype-[0-9].[0-9].[0-9]
 	make clean
-	./configure --host=$HOST_VAL --enable-shared --disable-static --without-png --with-harfbuzz=no --prefix=/ || { echo "freetype configure failed"; exit 1; } 
+	#Create out-of-tree build zone
+	rm -rf build/
+	mkdir build
+	pushd build
+
+	LIBS=-lpng cmake -DWITH_HarfBuzz=OFF -DWITH_PNG=ON -DWITH_ZLIB=ON -DBUILD_SHARED_LIBS:BOOL=true -DCMAKE_INSTALL_PREFIX="$BASE" -DCMAKE_TOOLCHAIN_FILE=${BASE}/patches/cmake-toolchain$BITS_VAL -DPNG_PNG_INCLUDE_DIR=${BASE}/include/  ../
+
+	if [ $? -ne 0 ] ; then
+		echo "Cmake failed"
+		exit 1
+	fi
+
+	#Old configure-based build system
+#	./configure --host=$HOST_VAL --enable-shared --disable-static --without-png --with-harfbuzz=no --prefix=/ || { echo "freetype configure failed"; exit 1; } 
 
 	make -j $NUM_PROCS || { echo "freetype build failed"; exit 1; } 
 	
-	make install DESTDIR="$BASE"|| { echo "freetype install failed"; exit 1; } 
+	make install || { echo "freetype install failed"; exit 1; } 
+	popd 
 
 	popd >/dev/null
 	
@@ -997,7 +1229,10 @@ function build_freetype()
 	pushd include >/dev/null
 	ln -s freetype2/freetype/
 	popd >/dev/null
-	
+
+	#Freetype does not install .dll properly, do by hand
+	find ./ -name libfreetype*dll -exec cp {} ${BASE}/lib/ \; || { echo "Unable to find freetype dll. Aborting" ; exit 1 ;}
+
 	echo ${NAME} >> $BUILD_STATUS_FILE
 }
 
@@ -1061,7 +1296,10 @@ function build_gettext()
 	applyPatches
 	automake
 
-	./configure --host=$HOST_VAL --disable-threads --enable-shared --disable-static --prefix=/ || { echo "$NAME configure failed"; exit 1; } 
+	#The CFlags/cxxflags thing is due to a really old gettext bug
+	# (who is maintaining that these days? the fix is known for years!) 
+	# http://savannah.gnu.org/bugs/?36443
+	CFLAGS="$CFLAGS -O2" CXXFLAGS="$CXXFLAGS -O2" ./configure --host=$HOST_VAL --disable-threads --enable-shared --disable-static --prefix=/ || { echo "$NAME configure failed"; exit 1; } 
 
 	make -j $NUM_PROCS || { echo "$NAME build failed"; exit 1; } 
 	
@@ -1082,7 +1320,9 @@ function build_gettext()
 	fi
 	popd >/dev/null
 	popd >/dev/null
-	
+
+	#FIXME: libintl.h does notget installed properly
+
 	FIX_LA_FILE_ARG=libintl
 	fix_la_file
 	FIX_LA_FILE_ARG=libcharset
@@ -1111,13 +1351,13 @@ function build_mathgl()
 	APPLY_PATCH_ARG=$PATCHES_MATHGL
 	applyPatches
 
-	if [ -d $BASEDIR/include/mgl2 ] ; then
-		echo "there are mgl2 headers already installed. Abort abort!"\
+	if [ -d ${BASE}/include/mgl2 ] ; then
+		echo "there are mgl2 headers already installed. Abort abort!"
 		exit 1
 	fi
 
 	rm -f CMakeCache.txt
-	LIBS=-lpng cmake -Denable-gsl="yes" -Denable-mpi="no"  -DCMAKE_INSTALL_PREFIX="$BASE" -DCMAKE_TOOLCHAIN_FILE=../../patches/cmake-toolchain$BITS_VAL -DPNG_PNG_INCLUDE_DIR=${BASEDIR}/include/
+	LIBS=-lpng cmake -Denable-gsl="yes" -Denable-jpeg="no" -Denable-mpi="no" -Denable-ltdl="no" -Denable-python="no"  -DCMAKE_INSTALL_PREFIX="$BASE" -DCMAKE_TOOLCHAIN_FILE=../../patches/cmake-toolchain$BITS_VAL -DPNG_PNG_INCLUDE_DIR=${BASEDIR}/include/
 
 	make -j $NUM_PROCS
 
@@ -1158,7 +1398,7 @@ function build_libvigra()
 	APPLY_PATCH_ARG=$PATCHES_LIBVIGRA
 	applyPatches
 
-	cmake -DCMAKE_INSTALL_PREFIX="$BASE" -DCMAKE_TOOLCHAIN_FILE=../../patches/cmake-toolchain$BITS_VAL -DPNG_PNG_INCLUDE_DIR=${BASEDIR}/include/
+	cmake -DCMAKE_INSTALL_PREFIX="$BASE" -DCMAKE_TOOLCHAIN_FILE=../../patches/cmake-toolchain$BITS_VAL -DPNG_PNG_INCLUDE_DIR=${BASE}/include/
 
 	make -j $NUM_PROCS
 	
@@ -1211,8 +1451,9 @@ function build_ftgl()
 	sed -i 's/return glBegin(GL_POINTS)/return 0;/' configure
 
 
+	FTFLAG=-I${BASE}/include/freetype2/
 
-	./configure --host=$HOST_VAL --enable-shared --disable-static --prefix=/ || { echo "ftgl configure failed"; exit 1; } 
+	LIBS="-lfreetype -lz" CXXFLAGS=$FTFLAG CFLAG=$FTFLAG ./configure --host=$HOST_VAL --with-ft-prefix=${BASE}---enable-shared --disable-static --prefix=/ || { echo "ftgl configure failed"; exit 1; } 
 
 
 	#MAkefile refers to ECHO variable for reporting completion, which does not exist
@@ -1333,7 +1574,7 @@ function build_3Depict()
 	autoreconf
 	automake --add-missing
 
-	CONF_FLAG="--host=$HOST_VAL --with-libqhull-link=-lqhull"
+	CONF_FLAG="--host=$HOST_VAL --with-libqhull-link=-lqhull_r --enable-sweep-hull --enable-libvd"
 	if [ $IS_RELEASE -ne 0 ] ; then
 		CONF_FLAG="$CONF_FLAG --disable-debug-checks --enable-openmp-parallel"
 	fi
@@ -1393,6 +1634,8 @@ function build_3Depict()
 	#HACK - find all -I// and -L// and replace them with something sane
 	find ./ -name Makefile -exec sed -i "s@-I//@-I${BASE}/@" {} \;
 	find ./ -name Makefile -exec sed -i "s@-L//@-L${BASE}/@" {} \;
+	#HACK : remove -lm, which picks up host's lib, rather than mingw build
+	find ./ -name Makefile -exec sed -i "s@-lgslcblas -lm@-lgslcblas@" {} \;
 
 	#Actually perform build	
 	make -j$NUM_PROCS
@@ -1402,25 +1645,19 @@ function build_3Depict()
 	fi
 
 	#if the locales are missing, try to rebuild them
-	if [ x`find locales/ -name \*.mo` = x""  ] ; then
-		
-		if [ x`which msgmerge` == x"" ] ; then
-			echo "Translations do not appear to be built for 3depict. Need to install translation builder, gettext."
-			sudo apt-get install gettext
-		fi
-		pushd translations
-		./makeTranslations || { echo "Translation build failed."; exit 1; }
-
-		mkdir -p ../locales/
-		for i in *.mo
-		do
-			TARGDIR=../locales/`echo $i | sed ' s/\.mo//' | sed 's/3Depict_//'`  
-			mkdir -p $TARGDIR
-			cp $i $TARGDIR/
-		done
-		popd
+	if [ x`which msgmerge` == x"" ] ; then
+		echo "Translations do not appear to be built for 3depict. Need to install translation builder, gettext."
+		sudo apt-get install gettext
 	fi
 
+	pushd translations
+
+	#change the install dir
+	sed -i "s@/usr/share/@${BASE}/code/3Depict/@" makeTranslations 
+
+	./makeTranslations install || { echo "Translation build failed."; exit 1; }
+	popd
+
 	popd > /dev/null
 }
 
@@ -1447,6 +1684,19 @@ function make_package()
 		fi
 	fi
 
+	mkdir locale
+
+	#Copy locales (eg wxstd.mo)
+	SUPPORTED_LOCALS="de es fr"
+	for i in $SUPPORTED_LOCALS
+	do
+
+		cp -Rp ${BASE}/share/locale/$i locale/$i || { echo "copying locale support files failed" : exit 1; }
+	done
+
+	#remove trash locale, which we don't need to ship
+	find ./locale/ -name bug-gnu-gettext@gnu.org.mo -exec rm {} \;
+
 
 	NSI_FILE=./windows-installer.nsi
 
@@ -1580,13 +1830,55 @@ function make_package()
 		TARGET_FILE=3Depict-${VERSION}-${HOST_EXT}-debug.exe
 	fi
 	
-	mv 3Depict-setup.exe  $TARGET_FILE
+	mv Setup.exe  $TARGET_FILE
 	echo "File written to : `pwd`/$TARGET_FILE"
 	echo "-------------------"
 	
 	popd > /dev/null
 }
 
+#Run wine host - assumes you have wine already installed
+function run_wine()
+{
+	if [ $BITS_VAL == 64 ] ; then
+		export WINEPREFIX=~/.wine64/
+		export WINEARCH=win64
+	else
+		echo "error in script, bits val not right"
+		return
+	fi
+	if [ x"`which wine`" == x"" ] ; then 
+		return
+	fi
+
+	pushd ${BASE}code/3Depict/src/
+
+	if [ $IS_RELEASE -ne 1 ] ; then
+		#Run the unit tests
+		wine 3Depict.exe -t
+	fi
+
+	if [ x"${TARGET_FILE}" == x"" ] ;then
+
+		NSI_FILE=windows-installer.nsi
+		VERSION=0.0.21
+		if [ $IS_RELEASE -ne 0 ] ; then
+			echo "Release mode enabled:"
+			TARGET_FILE=3Depict-${VERSION}-${HOST_EXT}.exe
+		else
+			echo "Release mode disabled:"
+			TARGET_FILE=3Depict-${VERSION}-${HOST_EXT}-debug.exe
+		fi
+	fi
+
+	#Remove any old code
+	rm -rf "${WINEPREFIX}/drive_c/Program\ Files/3Depict/"
+	#unpack installer using wine 
+	wine ${BASE}/code/3Depict/${TARGET_FILE}
+	#run program
+	wine "${WINEPREFIX}/drive_c/Program Files/3Depict/3Depict.exe"
+}
+
 #Check that we have a suitable host system
 checkHost
 
@@ -1646,24 +1938,30 @@ DESTDIR=${BASE}
 
 build_zlib
 build_libtiff
-build_libpng
+build_libpng 
 build_libjpeg
 build_libxml2
 build_gsl
-build_qhull2012
+#build_qhull2009 
 #build_qhull2015
 build_expat
-build_freetype
+build_freetype  
 build_libiconv
-build_gettext 
+build_gettext  
 build_ftgl 
-build_glew
+build_glew  
 build_libvigra
 
-build_mathgl 
+build_mathgl
 build_wx
+build_libvd 
+
 
 build_3Depict
 
 make_package
 
+#Normally this should be disabled
+if [ 0 -eq 1 ] ; then
+	run_wine
+fi
