@@ -1,6 +1,6 @@
 /* 
- * APTClasses.h - Generic APT components code
- * Copyright (C) 2015  D Haley
+ * APTFileIO.cpp - Generic APT components code
+ * Copyright (C) 2018, D Haley
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
 #include "../../common/stringFuncs.h"
 #include "../../common/basics.h"
 #include "../../common/translation.h"
-
+#include "../../common/endianTest.h"
 
 
 #include <cstring>
@@ -109,11 +109,28 @@ const char *LAWATAP_ATO_ERR_STRINGS[] = { "",
 				};
 //---------
 
+//TAPSIM binary errors and associated error strings
+enum
+{
+	TAPSIM_ERR_FILE_FORMAT_FAIL,
+	TAPSIM_ERR_FILE_OPEN_FAIL,
+	TAPSIM_ERR_ABORTED
+};
+	
+
+const char *TAPSIM_ERR_STRINGS[] = {"",
+			NTRANS("Error interpreting file contents"),
+			NTRANS("Error opening file"),
+			NTRANS("Aborted opening file"),
+			};
+
+//This is a performance tunable option. Some systems seek slowly
+#define USE_SEEKING_LIMITLOAD
+#if defined(USE_SEEKING_LIMITLOAD)
 unsigned int LimitLoadPosFile(unsigned int inputnumcols, unsigned int outputnumcols, const unsigned int index[], vector<IonHit> &posIons,const char *posFile, size_t limitCount,
 	       	unsigned int &progress, ATOMIC_BOOL &wantAbort,bool strongSampling)
 {
 
-
 	//Function is only defined for 4 columns here.
 	ASSERT(outputnumcols == 4);
 	//buffersize must be a power of two and at least outputnumcols*sizeof(float)
@@ -274,6 +291,165 @@ unsigned int LimitLoadPosFile(unsigned int inputnumcols, unsigned int outputnumc
 	delete[] buffer2;
 	return 0;
 }
+#else
+unsigned int LimitLoadPosFile(unsigned int inputnumcols, unsigned int outputnumcols, const unsigned int index[], vector<IonHit> &posIons,const char *posFile, size_t limitCount,
+	       	unsigned int &progress, ATOMIC_BOOL &wantAbort,bool strongSampling)
+{
+	//Function is only defined for 4 columns here.
+	ASSERT(outputnumcols == 4);
+	//buffersize must be a power of two and at least outputnumcols*sizeof(float)
+	const unsigned int BUFFERSIZE=inputnumcols * sizeof(float);
+	const unsigned int BUFFERSIZE2=outputnumcols * sizeof(float);
+
+
+	//Obtain size of file
+	size_t fileSize;
+	if(!getFilesize(posFile,fileSize))
+		return POS_OPEN_FAIL;
+	
+	if(!fileSize)
+		return POS_EMPTY_FAIL;
+	
+	//calculate the number of points stored in the POS file
+	size_t pointCount=0;
+	size_t maxIons;
+	size_t maxCols = inputnumcols * sizeof(float);
+	//regular case
+	if(fileSize % maxCols)
+		return POS_SIZE_MODULUS_ERR;	
+	maxIons =fileSize/maxCols;
+	limitCount=std::min(limitCount,maxIons);
+
+	//If we are going to load the whole file, don't use a sampling method to do it.
+	if(limitCount == maxIons)
+		return GenericLoadFloatFile(inputnumcols, outputnumcols, index, posIons,posFile,progress, wantAbort);
+	 
+	//open pos file
+	std::ifstream CFile(posFile,std::ios::binary);
+	if(!CFile)
+		return POS_OPEN_FAIL;
+
+	//Allocate buffers
+	//--
+	char *buffer,*buffer2;
+	try
+	{
+		buffer=new char[BUFFERSIZE];
+	}
+	catch(std::bad_alloc &m)
+	{
+		return POS_ALLOC_FAIL;
+	}
+		
+	try
+	{
+		buffer2=new char[BUFFERSIZE2];
+	}
+	catch(std::bad_alloc &m)
+	{
+		delete[] buffer;
+		return POS_ALLOC_FAIL;
+	}
+	//--
+
+
+	//Use a sampling method to load the pos file
+	std::vector<size_t> ionsToLoad;
+	try
+	{
+		posIons.resize(limitCount);
+
+		RandNumGen rng;
+		rng.initTimer();
+		unsigned int dummy;
+		randomDigitSelection(ionsToLoad,maxIons,rng,
+				limitCount,dummy,strongSampling);
+	}
+	catch(std::bad_alloc)
+	{
+		delete[] buffer;
+		delete[] buffer2;
+		return POS_ALLOC_FAIL;
+	}
+
+	//sort again
+	//NOTE: I tried to use a functor here to get progress
+	// It was not stable with parallel sort.
+	// Can we use several partial_sorts to get progress?
+	std::sort(ionsToLoad.begin(),ionsToLoad.end());
+	unsigned int curProg = PROGRESS_REDUCE;	
+
+	size_t curPos=0, curIon=0;
+	while( (CFile.tellg() < fileSize) && curIon < ionsToLoad.size())
+	{
+		//Read in a single input entry
+		
+		if(!CFile.good())
+		{
+			delete[] buffer;
+			delete[] buffer2;
+			return POS_READ_FAIL;
+		}
+		CFile.read(buffer,BUFFERSIZE);
+
+		if(curPos != ionsToLoad[curIon])
+		{
+			curPos++;
+			continue;
+		}
+		curPos++;
+		 
+		
+		for (size_t i = 0; i < outputnumcols; i++) // iterate through floats
+			memcpy(&(buffer2[i * sizeof(float)]), &(buffer[index[i] * sizeof(float)]), sizeof(float));
+		
+		posIons[curIon].setHit((float*)buffer2);
+		//Data bytes stored in pos files are big
+		//endian. flip as required
+		#ifdef __LITTLE_ENDIAN__
+			posIons[curIon].switchEndian();	
+		#endif
+		
+		if(posIons[curIon].hasNaN())
+		{
+			delete[] buffer;
+			delete[] buffer2;
+			return POS_NAN_LOAD_ERROR;	
+		}
+	
+		if(posIons[curIon].hasInf())
+		{
+			delete[] buffer;
+			delete[] buffer2;
+			return POS_INF_LOAD_ERROR;	
+		}
+		
+		pointCount++;
+		if(!curProg--)
+		{
+
+			progress= (unsigned int)((float)(CFile.tellg())/((float)fileSize)*100.0f);
+			curProg=PROGRESS_REDUCE;
+			if(wantAbort)
+			{
+				delete[] buffer;
+				delete[] buffer2;
+				posIons.clear();
+				return POS_ABORT_FAIL;
+				
+			}
+		}
+
+		curIon++;
+	}
+
+
+	delete[] buffer;
+	delete[] buffer2;
+	return 0;
+}
+#endif
+
 
 unsigned int GenericLoadFloatFile(unsigned int inputnumcols, unsigned int outputnumcols, 
 		const unsigned int index[], vector<IonHit> &posIons,const char *posFile, 
@@ -692,8 +868,6 @@ unsigned int limitLoadTextFile(unsigned int maxCols,
 
 unsigned int LoadATOFile(const char *fileName, vector<IonHit> &ions, unsigned int &progress, ATOMIC_BOOL &wantAbort,unsigned int forceEndian)
 {
-
-
 	//open pos file
 	std::ifstream CFile(fileName,std::ios::binary);
 
@@ -712,7 +886,7 @@ unsigned int LoadATOFile(const char *fileName, vector<IonHit> &ions, unsigned in
 	//   say that the XYZ is added later.
 	// - Header is 2 32 binary
 	// - File is serialised as little-endian
-	// - Various incompatible versions exist. Unclear how to distinguish
+	// - Various incompatible versions exist. Unclear how to distinguish. 
 
 	// Larson et al say that 
 	// - there are 14 fields
@@ -726,6 +900,8 @@ unsigned int LoadATOFile(const char *fileName, vector<IonHit> &ions, unsigned in
 	// we assume that the endian-ness must be auto-detected somehow, as no sources
 	// agree on file endian-ness. If we cannot detect it, we assume little endian
 
+	//Known versions of file : 2, 5
+
 	//Header (8 bytes), record 14 entries, 4 bytes each
 
 	const size_t LAWATAP_ATO_HEADER_SIZE=8;
@@ -751,9 +927,10 @@ unsigned int LoadATOFile(const char *fileName, vector<IonHit> &ions, unsigned in
 	//Assume that we can have a new version that doesn't affect the readout
 	// assume that earlier versions are compatible. This means, for a random byte
 	// in a random length (modulo) file, 
-	// we have a 1-4/255 chance of rejection from this test, and a 1/56 chance of
+	// we have a 1-5/255 chance of rejection from this test, and a 1/56 chance of
 	// rejection from filesize, giving a ~0.02% chance of incorrect acceptance.
-	if(!versionByte || versionByte > 4)
+	// Known versions : 2, 3, 5
+	if(!versionByte || versionByte > 5)
 		return LAWATAP_ATO_VERSIONCHECK_ERR;
 
 
@@ -782,7 +959,7 @@ unsigned int LoadATOFile(const char *fileName, vector<IonHit> &ions, unsigned in
 		ASSERT(forceEndian < 3);
 #ifdef __LITTLE_ENDIAN__
 		endianFlip=(forceEndian == 2);
-#elif __BIG_ENDIAN
+#elif defined(__BIG_ENDIAN__)
 		endianFlip=(forceEndian == 1);
 #endif
 	}
@@ -922,6 +1099,11 @@ unsigned int LoadATOFile(const char *fileName, vector<IonHit> &ions, unsigned in
 
 			ions[curPos] = IonHit(buffer);
 			curPos++;
+
+			//Update progress every ~1k ions.
+			// this is deliberately a power of two to speed
+			// the modulus operator (not benchmarked)
+			progress = (unsigned int)(float)CFile.tellg()/(float)fileSize*100.0f;
 			
 		}
 	}
@@ -933,6 +1115,8 @@ unsigned int LoadATOFile(const char *fileName, vector<IonHit> &ions, unsigned in
 			CFile.read((char*)buffer,LAWATAP_ATO_RECORD_SIZE);
 			ions[curPos] = IonHit(buffer);
 			curPos++;
+			
+			progress = (unsigned int)(float)CFile.tellg()/(float)fileSize*100.0f;
 		}
 	}
 
@@ -942,6 +1126,151 @@ unsigned int LoadATOFile(const char *fileName, vector<IonHit> &ions, unsigned in
 	return 0;
 }
 
+//Load a TAPSIM Binfile, which is the input geometry format for TAPSIM
+unsigned int loadTapsimBinFile(const char *fileName, unsigned int &progress, ATOMIC_BOOL &wantAbort, vector<IonHit> &posIons)
+{
+	ifstream f(fileName,std::ios::binary); 
+
+	if(!f)
+		return TAPSIM_ERR_FILE_OPEN_FAIL;
+
+	f.seekg(0,std::ios::end);
+	size_t fileSize;
+	fileSize = f.tellg();
+	f.seekg(0,std::ios::beg);
+
+	//TAPSIM's binary geometry input format is not totally clearly documented
+	// but an example is provided. So best efforts are us.
+
+	std::string str;
+	getline(f,str);
+
+	vector<string> s;
+	splitStrsRef(str.c_str(),' ',s);
+
+	const unsigned int HEADER_ENTRIES=4;
+	if(s.size() != HEADER_ENTRIES)
+		return TAPSIM_ERR_FILE_FORMAT_FAIL;
+	
+	bool binaryMode;
+	
+
+	if(s[0] == "BINARY")
+	{
+		//Payload is in binary format
+		binaryMode=true;
+	}
+	else if(s[0] == "ASCII")
+	{
+		//payload is in ascii format
+		binaryMode=false;
+	}
+	else
+		return TAPSIM_ERR_FILE_FORMAT_FAIL;
+
+	//maximum size of a TAPSIM record, in bytes
+	unsigned int MAX_RECORD_SIZE=18;
+	char *buffer=new char[MAX_RECORD_SIZE];
+	if(binaryMode)
+	{
+		size_t numEntries;
+		if(stream_cast(numEntries,s[1]))
+		{
+			delete[] buffer;
+			return TAPSIM_ERR_FILE_FORMAT_FAIL;
+		}
+		
+		//ID specifies atom type
+		const bool idsPresent=true;
+		//There is an inconsistency between the example files given and the documentation
+		// The documentation says the ids are optional. The examples have the option disabled, 
+		// but provide the IDs anyway - this changes the payload size.
+		if(!(s[2] == "1" || s[2] == "0" ) )
+		{
+			//UNknown value, abort
+			delete[] buffer;
+			return TAPSIM_ERR_FILE_FORMAT_FAIL;
+		}
+		
+
+		size_t numbersPresent;
+		if(s[3] == "1" )
+			numbersPresent=true;
+		else if (s[3] == "0") 
+			numbersPresent=false;
+		else
+		{
+			delete[] buffer;
+			return TAPSIM_ERR_FILE_FORMAT_FAIL;
+		}
+
+		//Contrary to documentation, ids appear to be "short", not "unsigned int"	
+		size_t recordSize=( (numbersPresent ? 1:0 )*sizeof(unsigned int) + 
+			3*sizeof(float) + (idsPresent ?1:0)*sizeof(short));
+
+		//Check that payload size matches expected size
+		if(fileSize -f.tellg() !=recordSize*numEntries)
+		{
+			delete[] buffer;
+			return TAPSIM_ERR_FILE_FORMAT_FAIL;
+		}
+
+		IonHit h;
+		posIons.resize(numEntries);
+		for(size_t ui=0;ui<numEntries; ui++)
+		{
+			f.read(buffer,recordSize);
+
+			//Transfer position data
+#ifdef __LITTLE_ENDIAN__
+			h.setPos(Point3D((float*)(buffer)));
+
+			//assign the ID as the mass to charge
+			h.setMassToCharge(*( (short*)(buffer+12) ));
+#elif defined(__BIG_ENDIAN__)
+			{
+			Point3D pt((float*)(buffer));
+			pt.switchEndian();
+			h.setPos(pt);
+
+			uint16_t *s;
+			s =( (uint16_t*)(buffer+12) );
+			uint16_tSwapBytes(s);
+
+			
+
+			}
+			
+#endif
+		
+			posIons[ui]=h;
+
+			progress = (float)ui/ (float)numEntries*100.0f;
+
+			if(wantAbort)
+			{
+				delete[] buffer;
+				return TAPSIM_ERR_ABORTED;
+			}
+		}
+
+#pragma omp parallel for
+		for(size_t ui=0;ui<numEntries;ui++)
+		{
+			//rescale back to nm, rather than in m
+			posIons[ui].setPos(posIons[ui].getPos()*1e9);
+		}
+	}
+	else
+	{
+		//TODO: IMPLEMENT ME
+		ASSERT(false);
+	}
+
+
+	delete[] buffer;
+	return 0;
+}
 
 
 #ifdef DEBUG
@@ -980,7 +1309,7 @@ bool writeATO(const std::string &filename,bool flip, unsigned int nPoints)
 
 	if(!flip)
 	{
-		//Fkip the endinanness
+		//Flip the endinanness
 		for(size_t ui=0;ui<4;ui++)
 			floatSwapBytes(buffer+ui);
 	}
@@ -989,7 +1318,7 @@ bool writeATO(const std::string &filename,bool flip, unsigned int nPoints)
 	outF.write((char*)&intData,4);
 
 	intData=3;
-	//Write out verion num  as "3"
+	//Write out version num  as "3"
 	outF.write((char*)&intData,4);
 
 	for(size_t ui=0;ui<nPoints;ui++)
@@ -1028,7 +1357,7 @@ bool testATOFormat()
 	TEST((ions[0].getPos().sqrDist(Point3D(1,1,0)) < sqrtf(std::numeric_limits<float>::epsilon())),"Checking read/write OK");
 	//Load using auto-detection of endinanness
 
-	//Load, forcing assuming cont4ents are little endianness as requried
+	//Load, forcing assuming contents are little endianness as required
 	TEST(!LoadATOFile(filename.c_str(),ions,dummyProgress,wantAbort,1),"ATO load test (forced endianness)");
 	TEST(ions.size() == 100,"ion size check");
 	TEST((ions[0].getPos().sqrDist(Point3D(1,1,0)) < sqrtf(std::numeric_limits<float>::epsilon())),"checking read/write OK");
