@@ -1,6 +1,6 @@
 /*
  * APTRanges.cpp - Atom probe rangefile class 
- * Copyright (C) 2015  D Haley
+ * Copyright (C) 2018, D Haley
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -117,7 +117,7 @@ bool RangeFile::decomposeIonNames(const std::string &name,
 
 	//true - was last, or now am on ion name
 	//false - am still on multiplier
-	int nameMode=true;
+	bool nameMode=true;
 	for(size_t ui=1;ui<name.size();ui++)
 	{
 		if(!isascii(name[ui]))
@@ -1069,16 +1069,19 @@ unsigned int RangeFile::detectFileType(const char *rangeFile)
 		if(!f)
 			return RANGE_FORMAT_END_OF_ENUM;
 
+		vector<string> strs;
 		std::string tmpStr;
 		size_t nCount;
+		bool brokeOut=false;
 
 		//retrieve the line
-		getline(f,tmpStr);
+		if(!getline(f,tmpStr))
+			goto skipoutRNGChecks;	
+
 		//strip exterior whitespace
 		tmpStr=stripWhite(tmpStr);
 
 		//break it apart
-		vector<string> strs;
 		splitStrsRef(tmpStr.c_str()," ",strs);
 
 		//Drop the whitepace
@@ -1115,8 +1118,11 @@ unsigned int RangeFile::detectFileType(const char *rangeFile)
 
 		while(nCount--)
 		{
-			getline(f,tmpStr);
-
+			if(!getline(f,tmpStr))
+			{
+				tmpStr.clear();
+				break;
+			}
 			//shouldn't hit eof
 			if(f.eof() || (!f.good()) )
 			{
@@ -1135,9 +1141,15 @@ unsigned int RangeFile::detectFileType(const char *rangeFile)
 
 		//Now, spin forwards until we either hit EOF or our double-dash marker
 		while( ! (f.eof() || f.bad()) )
+		// note windows may not set fail bits properly, so we monitor 
+		// the break on getline
 		{
+			tmpStr.clear();
 			if(!getline(f,tmpStr))
+			{
+				brokeOut=true;
 				break;
+			}
 
 			if(tmpStr.size() > 2 &&
 				tmpStr[0] == '-' && tmpStr[1] == '-')
@@ -1149,12 +1161,13 @@ unsigned int RangeFile::detectFileType(const char *rangeFile)
 			}
 		}
 
-		if(!f.good())
+		if(!f.good() || brokeOut)
 		{
 			//we did not see a double-dash, must be a vanilla ORNL file
 			typeStatus[RANGE_FORMAT_DBL_ORNL]=STATUS_IS_NOT;
 		}
 
+		//FIXME: Don't use a goto?
 skipoutRNGChecks:
 		;
 	}
@@ -1184,7 +1197,8 @@ skipoutRNGChecks:
 
 			//Get line, stripped of whitespace
 			std::string tmpStr;
-			getline(f,tmpStr);
+			if(!getline(f,tmpStr))
+				break;
 			tmpStr=stripWhite(tmpStr);
 
 			//See if we have this header
@@ -1303,7 +1317,11 @@ unsigned int RangeFile::readRNGHeader(FILE *fpRange, vector<pair<string,string>
 			//Gobble chars until we hit the newline
 			do
 			{
-				fgetc(fpRange);
+				if(fgetc(fpRange) == EOF)
+				{
+					delete[] inBuffer;
+					return RANGE_ERR_FORMAT_COLOUR;
+				}
 				peekVal=fpeek(fpRange);
 			}
 			while(peekVal != (int)'\n'&&
@@ -1373,7 +1391,7 @@ unsigned int RangeFile::readRNGFreqTable(FILE *fpRange, char *inBuffer,const uns
 	//remove whitespace from each entry
 	for(size_t ui=0;ui<colHeaders.size();ui++)
 	{
-		stripChars(colHeaders[ui],"\f\n\r\t ");
+		colHeaders[ui] = stripChars(colHeaders[ui],"\f\n\r\t ");
 	}
 	
 	stripZeroEntries(colHeaders);
@@ -2212,6 +2230,7 @@ unsigned int RangeFile::openRRNG(FILE *fpRange)
 
 	delete[] inBuffer;
 
+
 	//If we didn't find anything useful, then thats a formatting error
 	if(!haveSeenIonBlock || !numRanges  || !numBasicIons)
 		return RANGE_ERR_FORMAT;
@@ -2388,12 +2407,12 @@ bool RangeFile::range(vector<IonHit> &ions, string ionShortName)
 	return true;
 }
 
-void RangeFile::range(vector<IonHit> &ions)
+void RangeFile::range(vector<IonHit> &ions) const
 {
 	vector<IonHit> rangedVec;
 
 	unsigned int numIons=ions.size();
-	rangedVec.reserve(numIons);
+	rangedVec.reserve(numIons/2);
 
 	for(unsigned int ui=0; ui<numIons; ui++)
 	{
@@ -2407,6 +2426,19 @@ void RangeFile::range(vector<IonHit> &ions)
 	ions.swap(rangedVec);
 }
 
+void RangeFile::range(const vector<IonHit> &inIons, vector<IonHit> &ions) const
+{
+	unsigned int numIons=inIons.size();
+	ions.clear();
+	ions.reserve(numIons/2);
+
+	for(unsigned int ui=0; ui<numIons; ui++)
+	{
+		if(isRanged(inIons[ui]))
+			ions.push_back(inIons[ui]);
+	}
+}
+
 void RangeFile::rangeByRangeID(vector<IonHit> &ions, unsigned int rangeID)
 {
 	vector<IonHit> rangedVec;
@@ -2829,12 +2861,12 @@ unsigned int RangeFile::addRange(float start, float end, unsigned int parentIonI
 		ASSERT(isSelfConsistent());
 	}
 #endif
-	return ranges.size();
+	return ranges.size() -1;
 }
 
 unsigned int RangeFile::addIon(const std::string &shortN, const std::string &longN, const RGBf &newCol)
 {
-	for(unsigned int ui=9; ui<ionNames.size() ;ui++)
+	for(unsigned int ui=0; ui<ionNames.size() ;ui++)
 	{
 		if(ionNames[ui].first == shortN || ionNames[ui].second == longN)
 			return -1;
@@ -2910,7 +2942,7 @@ bool RangeFile::decompose(RangeFile &rng) const
 
 		for(size_t uj=0; uj<fragments.size();uj++)
 		{
-			if(rng.getIonID(fragments[uj].first) == -1)
+			if(rng.getIonID(fragments[uj].first) == (unsigned int)-1)
 			{
 				//make a new random colour
 				RGBf col;
@@ -2936,7 +2968,7 @@ bool RangeFile::isSelfDecomposable() const
 			return false;
 		//ion cannot be decomposed into smaller fragments. 
 		//This is not a decomposable rangefile
-		if(getIonID(ionNames[ui].first) == -1)
+		if(getIonID(ionNames[ui].first) == (unsigned int)-1)
 			return false;
 	}	
 
