@@ -1,6 +1,6 @@
 /*
  * APTRanges.h - Atom probe rangefile class 
- * Copyright (C) 2015  D Haley
+ * Copyright (C) 2018, D Haley
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -195,8 +195,12 @@ class RangeFile
 		bool isRanged(float mass) const;
 		//! Returns true if an ion is ranged
 		bool isRanged(const IonHit &) const;
-		//!Clips out ions that are not inside the range
-		void range(std::vector<IonHit> &ionHits);
+		//!Clips out ions that are not inside any range
+		// this destroys the input ionHits
+		void range(std::vector<IonHit> &ionHits) const;
+		//!Clips out ions that are not inside any range, keeping 
+		void range(const std::vector<IonHit> &ionsIn,
+					std::vector<IonHit> &ionHits) const;
 		//!Clips out ions that dont match the specified ion name
 		/*! Returns false if the ion name given doesn't match
 		 *  any in the rangefile (case sensitive) 
