@@ -1,6 +1,6 @@
 /*
  * ionhit.h - Ion event data class
- * Copyright (C) 2015  D Haley
+ * Copyright (C) 2018, D Haley
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -64,9 +64,8 @@ class IonHit
 		//returns true if any of the 4 data pts are +-inf
 		bool hasInf();
 
-#ifdef __LITTLE_ENDIAN__		
+		//!Flip the endian of the point contained in this class
 		void switchEndian();
-#endif
 		//this does the endian switch for you
 		//but you must supply a valid array.
 		void makePosData(float *floatArr) const;
