@@ -1,6 +1,6 @@
 /*
  *	configFile.cpp  - User configuration loading/saving
- *	Copyright (C) 2015, D Haley 
+ *	Copyright (C) 2018, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -491,7 +491,9 @@ unsigned int ConfigFile::read()
 		if(!XMLHelpFwdToElem(nodePtr,"wantorthocam"))
 		{
 			std::string str;
-			XMLGetAttrib(nodePtr,str,"value");
+			//Check for the value attribute
+			if(!XMLGetAttrib(nodePtr,str,"value"))
+				return CONFIG_ERR_BADFILE;
 
 			//Check if the user wants ortho camera, if no idea, revert to not 
 			if(!boolStrDec(str,wantStartupOrthoCam))
