@@ -1,6 +1,6 @@
 /*
  *	filter.cpp - modular data filter implementation 
- *	Copyright (C) 2015, D Haley 
+ *	Copyright (C) 2018, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -19,12 +19,12 @@
 #include "filter.h"
 #include "plot.h"
 
-#include "common/stringFuncs.h"
 #include "common/translation.h"
 
 #include "wx/wxcomponents.h"
 
 #include "common/voxels.h"
+#include "common/colourmap.h"
 #include "backend/APT/vtk.h"
 
 
@@ -86,6 +86,12 @@ size_t numElements(const vector<const FilterStreamData *> &v, unsigned int mask)
 	return nE;
 }
 
+void Filter::convertFileStringToAbsolute(const string &baseDir, std::string &s) const
+{
+	//Relative files should be marked with a ./
+	if(s.size() > 2 && s.substr(0,2) == "./") 
+		s=baseDir + s.substr(2);
+}
 
 template<>
 bool Filter::applyPropertyNow(bool &prop, const std::string &val, bool &needUp)
@@ -193,6 +199,124 @@ std::string Filter::getBaseErrString(unsigned int errCode)
 	return string("");
 }
 
+void Filter::appendConsoleMessage(const std::string &s)
+{
+	//Create a new console message
+	consoleOutput.push_back(ConsoleMessage(s,this,false));
+}
+
+void Filter::markConsoleCached()
+{
+	for(unsigned int ui=0;ui<consoleOutput.size();ui++)
+		consoleOutput[ui].cached=true;
+}
+
+FilterProperty::FilterProperty()
+{
+}
+
+FilterProperty::FilterProperty(size_t keyV, unsigned int typeV, const std::string &nameV, 
+					const std::string &dataV, const std::string &helpTextV) 
+	 : helpText(helpTextV), type(typeV), key(keyV), data(dataV),name(nameV)
+{
+	
+#ifdef DEBUG
+	checkSelfConsistent();
+#endif
+
+}
+
+FilterProperty::FilterProperty(size_t keyV, const std::string &nameV, 
+		const std::string &dataV, const std::string &helpTextV)
+	 : helpText(helpTextV), type(PROPERTY_TYPE_STRING), key(keyV), data(dataV),name(nameV)
+{
+
+}
+FilterProperty::FilterProperty(size_t keyV, const std::string &nameV, 
+					unsigned int dataV, const std::string &helpTextV) 
+	 : helpText(helpTextV), type(PROPERTY_TYPE_INTEGER), key(keyV), name(nameV)
+{
+	bool res;
+	res=stream_cast(data,dataV);
+	ASSERT(!res);
+#ifdef DEBUG
+	checkSelfConsistent();
+#endif
+
+}
+
+FilterProperty::FilterProperty(size_t keyV, const std::string &nameV, 
+					float dataV, const std::string &helpTextV) 
+	 : helpText(helpTextV), type(PROPERTY_TYPE_REAL), key(keyV), name(nameV)
+{
+	bool res;
+	res=stream_cast(data,dataV);
+	ASSERT(!res);
+#ifdef DEBUG
+	checkSelfConsistent();
+#endif
+
+}
+
+FilterProperty::FilterProperty(size_t keyV, const std::string &nameV, 
+					bool dataV, const std::string &helpTextV) 
+	 : helpText(helpTextV), type(PROPERTY_TYPE_BOOL), key(keyV), name(nameV)
+{
+	data=boolStrEnc(dataV);
+#ifdef DEBUG
+	checkSelfConsistent();
+#endif
+
+}
+
+FilterProperty::FilterProperty(size_t keyV, const std::string &nameV, 
+					const Point3D &dataV, const std::string &helpTextV) 
+	 : helpText(helpTextV), type(PROPERTY_TYPE_POINT3D), key(keyV), name(nameV)
+{
+	stream_cast(data,dataV);
+#ifdef DEBUG
+	checkSelfConsistent();
+#endif
+
+}
+
+FilterProperty::FilterProperty(size_t keyV, const std::string &nameV, 
+					const ColourRGBA &dataV, const std::string &helpTextV) 
+	 : helpText(helpTextV), type(PROPERTY_TYPE_COLOUR), key(keyV), name(nameV)
+{
+	data=dataV.rgbString();
+
+#ifdef DEBUG
+	checkSelfConsistent();
+#endif
+
+}
+
+FilterProperty::FilterProperty(size_t keyV, const std::string &nameV, 
+					const ColourRGBAf &dataV, const std::string &helpTextV) 
+	 : helpText(helpTextV), type(PROPERTY_TYPE_COLOUR), key(keyV), name(nameV)
+{
+	data=dataV.toColourRGBA().rgbString();
+
+#ifdef DEBUG
+	checkSelfConsistent();
+#endif
+
+}
+
+FilterProperty::FilterProperty(size_t keyV, const std::string &nameV, 
+		std::vector<std::pair<unsigned int, string> > &choices, 
+		unsigned int selected, const std::string &helpTextV) 
+	 : helpText(helpTextV), type(PROPERTY_TYPE_CHOICE), key(keyV), name(nameV)
+{
+
+	data=choiceString(choices,selected);
+#ifdef DEBUG
+	checkSelfConsistent();
+#endif
+
+}
+
 #ifdef DEBUG
 bool FilterProperty::checkSelfConsistent() const
 {	
@@ -274,6 +398,17 @@ void FilterPropGroup::addProperty(const FilterProperty &prop, size_t group)
 		groupNames.resize(groupCount,string(""));
 	}	
 
+
+#ifdef DEBUG
+	for(unsigned int ui=0; ui<properties.size();ui++)
+	{
+		//Check for key uniqueness
+		ASSERT(prop.key != properties[ui].key);
+		//Note, you can't check for help text uniqueness, or key name uniqueness
+		// as these can repeat (eg for range selection).
+	}
+#endif
+
 	keyGroupings.push_back(make_pair(prop.key,group));
 	properties.push_back(prop);
 }
@@ -413,34 +548,40 @@ PlotStreamData::PlotStreamData() :  r(1.0f),g(0.0f),b(0.0f),a(1.0f),
 	streamType=STREAM_TYPE_PLOT;
 	errDat.mode=PLOT_ERROR_NONE;
 
-	hardMinX=hardMinY=-std::numeric_limits<float>::max();
-	hardMaxX=hardMaxY=std::numeric_limits<float>::max();
+	for(unsigned int ui=0;ui<2;ui++)
+	{
+		for(unsigned int uj=0;uj<2;uj++)
+			plotBoundsActive[ui][uj]=false;
+	}
+
 }
 
-void PlotStreamData::autoSetHardBounds()
+
+void PlotStreamData::setPlotBound(unsigned int axis, unsigned int offset, float v)
 {
-	if(xyData.size())
-	{
-		hardMinX=std::numeric_limits<float>::max();
-		hardMinY=std::numeric_limits<float>::max();
-		hardMaxX=-std::numeric_limits<float>::max();
-		hardMaxY=-std::numeric_limits<float>::max();
+	ASSERT(axis < 2);
+	ASSERT(offset < 2);
+
+	plotBoundsActive[axis][offset] = true;
+	plotBounds[axis][offset] = v;
 
-		for(size_t ui=0;ui<xyData.size();ui++)
-		{
-			hardMinX=std::min(hardMinX,xyData[ui].first);
-			hardMinY=std::min(hardMinY,xyData[ui].second);
-			hardMaxX=std::max(hardMinX,xyData[ui].first);
-			hardMaxY=std::max(hardMaxY,xyData[ui].second);
-		}
-	}
-	else
-	{
-		hardMinX=hardMinY=-1;
-		hardMaxX=hardMaxY=1;
-	}
 }
 
+float PlotStreamData::getPlotBound(unsigned int axis, unsigned int offset) const
+{
+	ASSERT(plotBoundsActive[axis][offset]);
+
+	return plotBounds[axis][offset];
+}
+
+bool PlotStreamData::getPlotBoundActive(unsigned int axis, unsigned int offset) const
+{
+	ASSERT(axis < 2);
+	ASSERT(offset < 2);
+	return plotBoundsActive[axis][offset];
+}
+
+
 bool PlotStreamData::save(const char *filename) const
 {
 
@@ -492,16 +633,6 @@ void PlotStreamData::checkSelfConsistent() const
 	//region's should have a colour and ID vector of same size
 	ASSERT(regionID.size() ==regionR.size());
 
-	//log plots should have hardmin >=0
-	ASSERT(!(logarithmic && hardMinY < 0) );
-
-	//hardMin should be <=hardMax
-	//--
-	ASSERT(hardMinX<=hardMaxX);
-
-	ASSERT(hardMinY<=hardMaxY);
-	//--
-
 	//Needs to have a title
 	ASSERT(dataLabel.size());
 
@@ -517,6 +648,20 @@ void PlotStreamData::checkSelfConsistent() const
 
 	//Must set the "index" for this plot 
 	ASSERT(index != (unsigned int)-1);
+
+
+	for(unsigned int ui=0;ui<2;ui++)
+	{
+		if(plotBoundsActive[ui][0] && 
+			plotBoundsActive[ui][1])
+		{
+			//Plot bounds must not be inverted
+			// - lower bound must be less than upper bound
+			ASSERT(plotBounds[ui][0] < plotBounds[ui][1]);
+		}
+	}
+					
+
 }
 #endif
 
@@ -527,6 +672,7 @@ Plot2DStreamData::Plot2DStreamData()
 	b=a=1.0f;
 
 	scatterIntensityLog=false;
+	wantImageLog=false;
 }
 
 size_t Plot2DStreamData::getNumBasicObjects() const
@@ -553,7 +699,7 @@ void Plot2DStreamData::checkSelfConsistent() const
 		ASSERT(scatterIntensity.empty());
 
 
-	ASSERT(plotType < PLOT_TYPE_ENUM_END);
+	ASSERT(plotStyle < PLOT_TYPE_ENUM_END);
 }
 void RangeStreamData::checkSelfConsistent() const
 {
@@ -672,6 +818,10 @@ void IonStreamData::estimateIonParameters(const std::vector<const FilterStreamDa
 	map<vector<float>, unsigned int> ionColourMap;
 	std::string lastStr;
 
+	//Create a  default grey colour, if we can't do better
+	r=g=b=0.5;
+	a=1.0;
+
 	//Sum up the relative frequencies
 	for(unsigned int ui=0; ui<inData.size(); ui++)
 	{
@@ -801,10 +951,13 @@ size_t IonStreamData::getNumBasicObjects() const
 
 
 VoxelStreamData::VoxelStreamData() : representationType(VOXEL_REPRESENT_POINTCLOUD),
+	colourMapMode(COLOURMAP_MODE_BUILTIN),colourMap(COLOURMAP_VIRIDIS),
 	r(1.0f),g(0.0f),b(0.0f),a(0.3f), splatSize(2.0f),isoLevel(0.5f)
 {
 	streamType=STREAM_TYPE_VOXEL;
 	data = new Voxels<float>;
+	colourMin=0;
+	colourMax=1;
 }
 
 VoxelStreamData::VoxelStreamData(const Filter *f) : FilterStreamData(f), representationType(VOXEL_REPRESENT_POINTCLOUD),
@@ -970,7 +1123,7 @@ unsigned int Filter::collateIons(const vector<const FilterStreamData *> &dataIn,
 				for(size_t ui=0;ui<dataSize; ui++)
 					outVector[offset+ui]=d->data[ui];
 
-				if(Filter::wantAbort)
+				if(*Filter::wantAbort)
 					return FILTER_ERR_ABORT;
 				offset+=d->data.size();
 
@@ -982,6 +1135,44 @@ unsigned int Filter::collateIons(const vector<const FilterStreamData *> &dataIn,
 
 	return 0;
 }
+
+unsigned int Filter::collateIons(const vector<const FilterStreamData *> &dataIn,
+				std::vector<Point3D> &p, ProgressData &prog, size_t totalDataSize)
+{
+	//Build monolithic point set
+	//---
+	p.resize(totalDataSize);
+
+	size_t dataSize=0;
+
+	prog.filterProgress=0;
+	if(*Filter::wantAbort)
+		return FILTER_ERR_ABORT;
+
+	for(unsigned int ui=0;ui<dataIn.size() ;ui++)
+	{
+		switch(dataIn[ui]->getStreamType())
+		{
+			case STREAM_TYPE_IONS: 
+			{
+				const IonStreamData *d;
+				d=((const IonStreamData *)dataIn[ui]);
+
+				if(extendDataVector(p,d->data,	prog.filterProgress,
+						dataSize))
+					return FILTER_ERR_ABORT;
+
+				dataSize+=d->data.size();
+			}
+			break;	
+			default:
+				break;
+		}
+	}
+	//---
+
+	return 0;
+}
 				
 
 void Filter::updateOutputInfo(const std::vector<const FilterStreamData *> &dataOut)
@@ -1105,8 +1296,9 @@ bool Filter::boolToggleTests()
 			
 			FilterProperty p2;
 			p2 = propGroup.getPropValue(p.key);
+			string strMsg = string("displayed bool property can't be toggled") + p.name;
 			//Check the property values
-			TEST(p2.data == p.data,"displayed bool property can't be toggled");
+			TEST(p2.data == p.data,strMsg.c_str());
 			
 			//Toggle value back to original status
 			if(p2.data== "0")
