@@ -1,6 +1,6 @@
 /*
  *	filter.h - Data filter header file. 
- *	Copyright (C) 2015, D Haley 
+ *	Copyright (C) 2018, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -55,7 +55,7 @@ class Voxels;
 
 const unsigned int NUM_CALLBACK=50000;
 
-const unsigned int IONDATA_SIZE=4;
+const unsigned int IONDATA_SIZE=16;
 
 //!Filter types  -- must match array FILTER_NAMES
 enum
@@ -114,9 +114,6 @@ enum
 	BINDING_RECT_CORNER_MOVE
 };
 
-extern const char *STREAM_NAMES[];
-
-
 //Representations
 enum 
 {
@@ -124,9 +121,17 @@ enum
 	VOXEL_REPRESENT_POINTCLOUD,
 	VOXEL_REPRESENT_ISOSURF,
 	VOXEL_REPRESENT_AXIAL_SLICE,
+	VOXEL_REPRESENT_SLICE,
+#ifdef ENABLE_LIBVD
+	VOXEL_REPRESENT_VOLUME_RENDER,
+#endif
 	VOXEL_REPRESENT_END
 };
 
+extern const char *STREAM_NAMES[];
+
+
+
 //Error codes for each of the filters. 
 //These can be passed to the getErrString() function for
 //a human readable error message
@@ -140,6 +145,7 @@ enum
 };
 
 
+
 //!Generic filter error codes
 enum
 {
@@ -149,10 +155,38 @@ enum
 //---
 //
 
+inline 
+void assignIonData(Point3D &p, const IonHit &h)
+{
+	p = h.getPosRef();
+}
+	 
+inline
+void assignIonData(IonHit &p, const IonHit &h)
+{
+	p = h;
+}
+
 //!Return the number of elements in a vector of filter data - i.e. the sum of the number of objects within each stream. Only masked streams (STREAM_TYPE_*) will be counted
 size_t numElements(const std::vector<const FilterStreamData *> &vm, unsigned int mask=STREAMTYPE_MASK_ALL);
 
 
+//!Message that can be emitted by a filter for output to a textual console
+class ConsoleMessage
+{
+	public:
+		ConsoleMessage() {cached=0;}
+		ConsoleMessage(const std::string &s, const Filter *p,bool isCached) { parent=p;message=s;cached=isCached;}
+		//Message content and filter that generated it
+		const Filter *parent;
+		//!Textual message to display
+		std::string message;
+		//!Whether this message was generated in the latest refresh,
+		// or if it came from cached output
+		bool cached;
+
+		void operator=(const ConsoleMessage &oth);
+};
 
 //!Abstract base class for data types that can propagate through filter system
 class FilterStreamData
@@ -181,6 +215,7 @@ class FilterStreamData
 		//Cross-checks fields to determine if (best guess)
 		///data structure has a sane combination of values
 		virtual void checkSelfConsistent() const {}
+		virtual const string getName() const =0; 
 #endif
 
 };
@@ -190,7 +225,7 @@ class FilterProperty
 	public:
 	//!Human readable short help (tooltip) for each of the keys
 	std::string helpText;
-	//!Data type for this element
+	//!Data type for this element (PROPERTY_TYPE_{BOOL,FLOAT,etc} (constants.h))
 	unsigned int type;
 	//!Unique key value for this element
 	size_t key;
@@ -202,7 +237,45 @@ class FilterProperty
 	//!name of property
 	std::string name;
 
+	FilterProperty();
+
+	//Initialise property with explicit type, using string
+ 	FilterProperty(size_t key, unsigned int type, const std::string &name, 
+ 					const std::string &data, const std::string &helpText);
+ 	//Initialise string property
+ 	FilterProperty(size_t key, const std::string &name, 
+			const std::string &data, const std::string &helpText);
+
+	//Initialise bool property 
+	FilterProperty(size_t key, const std::string &name, 
+			bool data, const std::string &helpText);
+	
+	//Initialise float property 
+	FilterProperty(size_t key, const std::string &name, 
+			float data, const std::string &helpText);
+ 
+	//Initialise uint property 
+	FilterProperty(size_t key, const std::string &name, 
+			unsigned int data, const std::string &helpText);
+
+	//Initialise Point3D property 
+	FilterProperty(size_t key, const std::string &name, 
+			const Point3D &data, const std::string &helpText);
+
+	//Initialise choice property 
+	FilterProperty(size_t key, const std::string &name, 
+			std::vector<std::pair<unsigned int, string> > &data, unsigned int selected,
+				const std::string &helpText);
+	//Intialise colour property	
+	FilterProperty(size_t key, const std::string &name, 
+			const ColourRGBA &data,const std::string &helpText);
+	//Intialise colour property (float version)	
+	FilterProperty(size_t key, const std::string &name, 
+			const ColourRGBAf &data,const std::string &helpText);
+
+#ifdef DEBUG
 	bool checkSelfConsistent() const;
+#endif
 };
 
 class FilterPropGroup
@@ -275,7 +348,7 @@ public:
 
 	size_t getNumBasicObjects() const;
 
-	//Ion colour + transparancy in [0,1] colour space. 
+	//Ion colour + transparency in [0,1] colour space. 
 	float r,g,b,a;
 
 	//Ion Size in 2D opengl units
@@ -294,6 +367,10 @@ public:
 	//!Use heuristics to guess best display parameters for this ionstream. May attempt to leave them alone 
 	void estimateIonParameters(const std::vector<const FilterStreamData *> &inputData);
 	void estimateIonParameters(const IonStreamData *inputFilter);
+
+#ifdef DEBUG
+		const string getName() const { return "Ion Stream";}; 
+#endif
 };
 
 //!Point with m-t-c value data
@@ -307,17 +384,37 @@ public:
 	void clear();
 	
 	unsigned int representationType;
+	//Do we want to use a built-in colour map, or a custom eon
+	unsigned int colourMapMode;
+	//Which built in colour map to use
+	unsigned int colourMap;
+	//Custom colour map to use (only if requested by colourMapMode)
+	std::vector<std::pair<float,ColourRGBAf> > customColourMap;
+	//set to the min/max bounds for the colour map
+	float colourMin,colourMax;
+	//RGBA (eg for isosurface) 
 	float r,g,b,a;
+	//Size of "splats" to use when using the point based reporesentation
 	float splatSize;
+	//Isosurface value to apply
 	float isoLevel;
 	//!Apply filter to input data stream	
 	Voxels<float> *data;
 		
+#ifdef DEBUG
+	const string getName() const { return "Voxel Stream";}; 
+#endif
 };
 
 //!Plotting data
 class PlotStreamData : public FilterStreamData
 {
+	private :
+		//
+		bool plotBoundsActive[2][2];
+		//!Special boundary positions
+		float plotBounds[2][2];
+
 	public:
 		PlotStreamData();
 		PlotStreamData(const Filter *f);
@@ -329,8 +426,6 @@ class PlotStreamData : public FilterStreamData
 		//Get data size
 		size_t getNumBasicObjects() const { return xyData.size();};
 
-		//Use the contained XY data to set hard plot bounds
-		void autoSetHardBounds();
 		//plot colour
 		float r,g,b,a;
 		//plot trace mode - enum PLOT_TRACE
@@ -351,6 +446,8 @@ class PlotStreamData : public FilterStreamData
 
 		//!XY data pairs for plotting curve
 		std::vector<std::pair<float,float> > xyData;
+
+
 		//!Rectangular marked regions
 		std::vector<std::pair<float,float> > regions;
 		std::vector<std::string> regionTitle;
@@ -367,19 +464,26 @@ class PlotStreamData : public FilterStreamData
 		unsigned int index;
 		//!Error bar mode
 		PLOT_ERROR errDat;
-		
-		//!Hard bounds that cannot be exceeded when drawing plot
-		float hardMinX,hardMaxX,hardMinY,hardMaxY;
 
+		//Set sections of the plot that must be visible
+		// offset=0/1 -> lower/upper, axis=0/1 -> x/y axis, respectively
+		void setPlotBound(unsigned int axis, unsigned int offset, float v);
+
+		
+		float getPlotBound(unsigned int axis, unsigned int offset) const;
+		bool getPlotBoundActive(unsigned int axis, unsigned int offset) const;
+		
 #ifdef DEBUG
 		//Cross-checks fields to determine if (best guess)
 		///data structure has a sane combination of values
 		virtual void checkSelfConsistent() const; 
+		const string getName() const { return "Plot Stream";}; 
 #endif
 
 };
 
 //!2D Plotting data
+//TODO: Should probably more carefully structure scatter and density plot content
 class Plot2DStreamData : public FilterStreamData
 {
 	public:
@@ -395,11 +499,11 @@ class Plot2DStreamData : public FilterStreamData
 		//Label for X, Y axes
 		std::string xLabel,yLabel;
 
-		unsigned int plotType;
+		unsigned int plotStyle;
 
 		//!Structured XY data pairs for plotting curve
 		Array2D<float> xyData;
-		//Only rqeuired for xy plots
+		//Only required for xy plots
 		float xMin,xMax,yMin,yMax;
 	
 
@@ -410,14 +514,19 @@ class Plot2DStreamData : public FilterStreamData
 		//optional intensity data for scatter plots
 		std::vector<float> scatterIntensity;
 	
-		//Do we want to plot the scatter intensity in lgo mode?
+		//Do we want to plot the scatter intensity in log mode?
 		bool scatterIntensityLog;	
+		//Do we want image intensity as log?
+		bool wantImageLog;
+		//which colour map do we want to use if using 2D function mode?
+		unsigned int colourMap;
 
 		//!Parent filter index
 		unsigned int index;
 
 #ifdef DEBUG
 		void checkSelfConsistent() const;
+		const string getName() const { return "2DPlot Stream";}; 
 #endif
 };
 
@@ -429,7 +538,7 @@ class DrawStreamData: public FilterStreamData
 		std::vector<DrawableObj *> drawables;
 		//!constructor
 		DrawStreamData(){ streamType=STREAM_TYPE_DRAW;};
-		DrawStreamData(const Filter *f){ streamType=STREAM_TYPE_DRAW;};
+		DrawStreamData(const Filter *f){ streamType=STREAM_TYPE_DRAW; parent=f;};
 		//!Destructor
 		~DrawStreamData();
 		//!Returns 0, as this does not store basic object types -- i.e. is not for data storage per se.
@@ -441,6 +550,7 @@ class DrawStreamData: public FilterStreamData
 		//Cross-checks fields to determine if (best guess)
 		///data structure has a sane combination of values
 		void checkSelfConsistent() const; 
+		const string getName() const { return "Draw Stream";}; 
 #endif
 };
 
@@ -472,6 +582,7 @@ class RangeStreamData :  public FilterStreamData
 
 #ifdef DEBUG
 		void checkSelfConsistent() const ; 
+		const string getName() const { return "Range Stream" ;}; 
 #endif
 };
 
@@ -479,8 +590,9 @@ class RangeStreamData :  public FilterStreamData
 class Filter
 {
 	protected:
-
+		//Should we cache, and is the cache content up-to-date
 		bool cache, cacheOK;
+		//Should we use slower, but better randomisation algorithms
 		static bool strongRandom;
 
 
@@ -491,8 +603,9 @@ class Filter
 	
 
 		//!Temporary console output. Should be only nonzero size if messages are present
-		//after refresh, until cache is cleared
-		std::vector<std::string> consoleOutput;
+		//after refresh, until cache is cleared. First entry is message, second is if 
+		// the message is from cached data
+		std::vector<ConsoleMessage> consoleOutput;
 		//!User settable labelling string (human readable ID, etc etc)
 		std::string userString;
 		//Filter output cache
@@ -502,9 +615,15 @@ class Filter
 
 
 
+		//Convert an internal file string (/ separated) representing a path, to absolute as required. Relative paths should be indicated with a "./"
+		void convertFileStringToAbsolute(const string &basePath, std::string &s) const;
+
 		//Collate ions from filterstream data into an ionhit vector
 		static unsigned int collateIons(const std::vector<const FilterStreamData *> &dataIn,
 				std::vector<IonHit> &outVector, ProgressData &prog, size_t totalDataSize=(size_t)-1);
+		
+		static unsigned int collateIons(const std::vector<const FilterStreamData *> &dataIn,
+				std::vector<Point3D> &outVector, ProgressData &prog, size_t totalDataSize=(size_t)-1);
 
 		//!Propagate the given input data to an output vector
 		static void propagateStreams(const std::vector<const FilterStreamData *> &dataIn,
@@ -519,7 +638,10 @@ class Filter
 		// returns true if change applied OK.
 		template<class T>	
 		bool applyPropertyNow(T &oldProp,const std::string &newVal, bool &needUp);
-	
+
+		//!Add a new console message
+		void appendConsoleMessage(const std::string &s);
+
 
 		//place a stream object into the filter cache, if required
 		// does not place object into filter output - you need to do that yourself
@@ -531,6 +653,241 @@ class Filter
 		//!Get the per-filter error codes
 		virtual std::string getSpecificErrString(unsigned int errCode) const=0;
 
+		//!Extend a point data vector using some ion data. 
+		// Offset is the position to start inserting in the destination array.
+		// Will fail if user abort is detected 
+		template<class T>
+		static unsigned int extendDataVector(std::vector<T> &dest, const std::vector<IonHit> &vIonData,
+						unsigned int &progress, size_t offset)
+		{
+			unsigned int curProg=NUM_CALLBACK;
+			unsigned int n =offset;
+#ifdef _OPENMP
+			//Parallel version
+			bool spin=false;
+			#pragma omp parallel for shared(spin)
+			for(size_t ui=0;ui<vIonData.size();ui++)
+			{
+				if(spin)
+					continue;
+				assignIonData(dest[offset+ ui],vIonData[ui]);
+				
+				//update progress every CALLBACK entries
+				if(!curProg--)
+				{
+					#pragma omp critical
+					{
+					n+=NUM_CALLBACK;
+					progress= (unsigned int)(((float)n/(float)dest.size())*100.0f);
+					if(!omp_get_thread_num())
+					{
+						if(*Filter::wantAbort)
+							spin=true;
+					}
+					}
+				}
+
+			}
+
+			if(spin)
+				return 1;
+#else
+
+			for(size_t ui=0;ui<vIonData.size();ui++)
+			{
+				assignIonData(dest[offset+ ui],vIonData[ui]);
+				
+				//update progress every CALLBACK ions
+				if(!curProg--)
+				{
+					n+=NUM_CALLBACK;
+					progress= (unsigned int)(((float)n/(float)dest.size())*100.0f);
+					if(*(Filter::wantAbort))
+						return 1;
+				}
+
+			}
+#endif
+
+
+			return 0;
+		}
+
+		//Build two sets of points from a single datastream and the given rangefile
+		//Returns 0 on no error, otherwise nonzero
+		template<class T>
+		static size_t buildSplitPoints(const vector<const FilterStreamData *> &dataIn,
+						ProgressData &progress, size_t totalDataSize,
+						const RangeFile *rngF, const vector<bool> &pSourceEnabled, const vector<bool> &pTargetEnabled,
+						vector<T> &pSource, vector<T> &pTarget
+						)
+		{
+			ASSERT(rngF);
+			ASSERT(pSourceEnabled.size() == pTargetEnabled.size());
+			ASSERT(pSourceEnabled.size() == rngF->getNumIons());
+			size_t sizeNeeded[2];
+			sizeNeeded[0]=sizeNeeded[1]=0;
+
+			//Presize arrays
+			for(unsigned int ui=0; ui<dataIn.size() ; ui++)
+			{
+				switch(dataIn[ui]->getStreamType())
+				{
+					case STREAM_TYPE_IONS:
+					{
+						unsigned int ionID;
+
+						const IonStreamData *d;
+						d=((const IonStreamData *)dataIn[ui]);
+						ionID=getIonstreamIonID(d,rngF);
+
+						if(ionID == (unsigned int)-1)
+						{
+
+							//we have ungrouped ions, so work out size individually
+							for(unsigned int uj=0;uj<d->data.size();uj++)
+							{
+								ionID = rngF->getIonID(d->data[uj].getMassToCharge());
+
+								if(ionID == (unsigned int)-1)
+									continue;
+
+								if(pSourceEnabled[ionID])
+									sizeNeeded[0]++;
+								if(pTargetEnabled[ionID])
+									sizeNeeded[1]++;
+							}
+							
+							break;
+						}
+
+						if(pSourceEnabled[ionID])
+							sizeNeeded[0]+=d->data.size();
+
+						if(pTargetEnabled[ionID])
+							sizeNeeded[1]+=d->data.size();
+
+						break;
+					}
+					default:
+						break;
+				}
+			}
+
+			pSource.resize(sizeNeeded[0]);
+			pTarget.resize(sizeNeeded[1]);
+
+			//Fill arrays
+			size_t curPos[2];
+			curPos[0]=curPos[1]=0;
+
+			for(unsigned int ui=0; ui<dataIn.size() ; ui++)
+			{
+				switch(dataIn[ui]->getStreamType())
+				{
+					case STREAM_TYPE_IONS:
+					{
+						unsigned int ionID;
+						const IonStreamData *d;
+						d=((const IonStreamData *)dataIn[ui]);
+						ionID=getIonstreamIonID(d,rngF);
+
+						if(ionID==(unsigned int)(-1))
+						{
+							//we have ungrouped ions, so work out size individually
+							for(unsigned int uj=0;uj<d->data.size();uj++)
+							{
+								ionID = rngF->getIonID(d->data[uj].getMassToCharge());
+
+								if(ionID == (unsigned int)-1)
+									continue;
+
+								if(pSourceEnabled[ionID])
+								{
+									assignIonData(pSource[curPos[0]],d->data[uj]);
+									curPos[0]++;
+								}
+
+								if(pTargetEnabled[ionID])
+								{
+									assignIonData(pTarget[curPos[1]],d->data[uj]);
+									curPos[1]++;
+								}
+							}
+							
+							break;
+						}
+
+						unsigned int dummyProgress=0;
+						if(pSourceEnabled[ionID])
+						{
+							if(extendDataVector(pSource,d->data,
+									     dummyProgress,curPos[0]))
+								return FILTER_ERR_ABORT;
+
+							curPos[0]+=d->data.size();
+						}
+
+						if(pTargetEnabled[ionID])
+						{
+							if(extendDataVector(pTarget,d->data,
+									     dummyProgress,curPos[1]))
+								return FILTER_ERR_ABORT;
+
+							curPos[1]+=d->data.size();
+						}
+
+						break;
+					}
+					default:
+						break;
+				}
+			}
+
+
+			return 0;
+		}
+	
+		//TODO: Replace with a non brute-force option
+		//try to find the ID of a given ionstream by matching to a specific rangefile	
+		static unsigned int getIonstreamIonID(const IonStreamData *d, const RangeFile *r)
+		{
+			if(d->data.empty())
+				return (unsigned int)-1;
+
+			unsigned int tentativeRange;
+
+			tentativeRange=r->getIonID(d->data[0].getMassToCharge());
+
+
+			//TODO: Currently, we have no choice but to brute force it.
+			//In the future, it might be worth storing some data inside the IonStreamData itself
+			//and to use that first, rather than try to brute force the result
+#ifdef _OPENMP
+			bool spin=false;
+			#pragma omp parallel for shared(spin)
+			for(size_t ui=1;ui<d->data.size();ui++)
+			{
+				if(spin)
+					continue;
+				if(r->getIonID(d->data[ui].getMassToCharge()) !=tentativeRange)
+					spin=true;
+			}
+
+			//Not a range
+			if(spin)
+				return (unsigned int)-1;
+
+#else
+			for(size_t ui=1;ui<d->data.size();ui++)
+			{
+				if(r->getIonID(d->data[ui].getMassToCharge()) !=tentativeRange)
+					return (unsigned int)-1;
+			}
+#endif
+
+			return tentativeRange;	
+		}
 	
 	public:	
 		Filter() ;
@@ -661,9 +1018,13 @@ class Filter
 		//!Get the number of outputs for the specified type during the filter's last refresh
 		unsigned int getNumOutput(unsigned int streamType) const;
 
-		//!Get the filter messages from the console. To erase strings, either call erase, or erase cahche
-		void getConsoleStrings(std::vector<std::string > &v) const { v=consoleOutput;};
+		//!Get the filter messages from the console. 
+		void getConsoleStrings(std::vector<ConsoleMessage> &v) const { v=consoleOutput;};
+		//!Erase any console strings
 		void clearConsole() { consoleOutput.clear();};
+	
+		//!Mark any console messages as having originated from a cache
+		void markConsoleCached();
 
 		//!Should filters use strong randomisation (where applicable) or not?
 		static void setStrongRandom(bool strongRand) {strongRandom=strongRand;}; 
@@ -676,7 +1037,10 @@ class Filter
 
 		//Can we be a useful filter, even if given no input specified by the Use mask?
 		virtual bool isUsefulAsAppend() const { return false;}
-	
+
+		//Obtain all of the streams in the input vector that
+		// have type stream type that matches the typename, T.
+		// call as a template function ...::getStreamsOfType<STREAM_TYPE>(...)
 		template<typename T>	
 		static void getStreamsOfType(const std::vector<const FilterStreamData *> &vec, std::vector<const T *> &dataOut);
 
