@@ -1,6 +1,6 @@
 /* 
  * K3DTree-mk2.cpp : 3D Point KD tree - precise implementation 
- * Copyright (C) 2015  D. Haley
+ * Copyright (C) 2018, D Haley
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,8 +18,6 @@
 
 #include "K3DTree-mk2.h"
 
-#include "backend/APT/ionhit.h"
-
 #include <stack>
 #include <queue>
 
@@ -341,7 +339,7 @@ void K3DTreeMk2::ptsInSphere(const Point3D &origin, float radius,
 			{
 				if(leftCube.containedInSphere(origin,sqrRadius))
 				{
-					ASSERT(indexedPoints[nodeIdx].first.sqrDist(origin) < radius*radius);
+					ASSERT(indexedPoints[nodeIdx].first.sqrDist(origin) <=sqrRadius);
 					idxQueue.push(nodes[nodeIdx].childLeft);
 				}
 				else
@@ -359,7 +357,7 @@ void K3DTreeMk2::ptsInSphere(const Point3D &origin, float radius,
 				{
 					//If the right-hand cube is contained within (origin,radius) sphere, then so are all its chilren
 
-					ASSERT(indexedPoints[nodeIdx].first.sqrDist(origin) < sqrRadius);
+					ASSERT(indexedPoints[nodeIdx].first.sqrDist(origin) <=sqrRadius);
 					idxQueue.push(nodes[nodeIdx].childRight);
 				}
 				else
@@ -379,7 +377,7 @@ void K3DTreeMk2::ptsInSphere(const Point3D &origin, float radius,
 	{
 		size_t curIdx;
 		curIdx=idxQueue.front();
-		ASSERT(indexedPoints[curIdx].first.sqrDist(origin) < sqrRadius);
+		ASSERT(indexedPoints[curIdx].first.sqrDist(origin) <=sqrRadius);
 		if(nodes[curIdx].childLeft != (size_t)-1)
 			idxQueue.push(nodes[curIdx].childLeft);
 		
@@ -645,6 +643,50 @@ size_t K3DTreeMk2::findNearestUntagged(const Point3D &searchPt,
 
 }
 
+//TODO: There is almost certainly a better implementation where
+// we consider wider and wider block sizes
+void K3DTreeMk2::findKNearestUntagged(const Point3D &queryPt,
+			const BoundCube &b, unsigned int nnMax, 
+			std::vector<size_t> &pts, bool tag) 
+{
+	pts.clear();
+	if(tag)
+	{
+		vector<size_t> taggedPts;
+		//Keep looking for points
+		while(pts.size() < nnMax)
+		{
+			size_t nearPt;
+			nearPt=findNearestUntagged(queryPt,b);
+			if(nearPt == (size_t)-1)
+				break;
+
+			pts.push_back(nearPt);
+		}
+	}
+	else
+	{
+		std::set<size_t> skipPts;
+		while(skipPts.size() < nnMax)
+		{
+			size_t nearPt;
+			nearPt=findNearestWithSkip(queryPt,b,skipPts);
+			if(nearPt == (size_t)-1)
+				break;
+
+			skipPts.insert(nearPt);
+		}
+
+		pts.reserve(skipPts.size());
+		for(std::set<size_t>::iterator it=skipPts.begin(); it!=skipPts.end(); ++it)
+		{
+			pts.push_back(*it);
+		}
+
+	}
+}
+		
+
 size_t K3DTreeMk2::findNearestWithSkip(const Point3D &searchPt,
 				const BoundCube &domainCube, const std::set<size_t> &skipPts, size_t pseudoRoot) const
 {
