@@ -1,6 +1,6 @@
 /*
- *	binomial.cpp - Binomia distribution randomness testing
- *	Copyright (C) 2015, D Haley 
+ *	binomial.cpp - Binomial distribution randomness testing
+ *	Copyright (C) 2018, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -74,23 +74,13 @@ int countBinnedIons(const std::vector<IonHit> &ions, const RangeFile *rng,
 	}
 
 	//Filter ions by ranging
-	for(size_t ui=0;ui<ions.size();ui++)
-	{
-		unsigned int ionID;
-		ionID = rng->getIonID(ions[ui].getMassToCharge());
-		
-		//Skip unranged ions
-		if(ionID == (unsigned int)-1)
-			continue;
-
-		filteredIons.push_back(ions[ui]);
-	}
+	rng->range(ions,filteredIons);
 	
 	//Obtain the bounding box for the filtered ions
 	BoundCube totalBound;
 	IonHit::getBoundCube(filteredIons,totalBound);
 
-	// Sort the ions according to their Z value
+	// Sort the ions according to their extrusion-direction value
 	//--
 	unsigned int extrusionAxis=segmentOptions.extrusionDirection;
 	IonAxisCompare axisCmp(extrusionAxis);
@@ -100,10 +90,10 @@ int countBinnedIons(const std::vector<IonHit> &ions, const RangeFile *rng,
 
 	//--
 
+	//out-of-extrusion direction axes
 	unsigned int direction[2];
 	float binLen[2];
 	unsigned int nBins[2];
-
 	direction[0]=(extrusionAxis+1)%3;
 	direction[1]=(extrusionAxis+2)%3;
 
@@ -130,7 +120,7 @@ int countBinnedIons(const std::vector<IonHit> &ions, const RangeFile *rng,
 				float s;
 				s= totalBound.getSize(direction[ui]);
 				nBins[ui] = s/targetL+1;
-				binLen[ui] = s/nBins[ui]+1;
+				binLen[ui] = s/nBins[ui];
 			}
 
 			break;
@@ -185,23 +175,35 @@ int countBinnedIons(const std::vector<IonHit> &ions, const RangeFile *rng,
 		xPos =ionOffset[direction[0]]/binLen[0];
 		yPos = ionOffset[direction[1]]/binLen[1];
 
+		//numerical issue where ions appear on the side of a
+		// bin - snap them into this bin 
+		if(xPos == nBins[0])
+			xPos--;
+		if(yPos == nBins[1])
+			yPos--;
 		
 		//Find the bin that this new ion is in,
 		// and its range value
-		unsigned int binIdx,selectionId,range;
+		unsigned int binIdx,selectionId,ionId;
 		binIdx = rowMajorOffset(xPos,yPos,nBins[1]);
 		
-		//get range
-		range=rng->getIonID(filteredIons[ui].getMassToCharge());
-		ASSERT(range!=(unsigned int)-1);
+		//get ion ID
+		ionId=rng->getIonID(filteredIons[ui].getMassToCharge());
+		ASSERT(ionId!=(unsigned int)-1);
 
-		//convert range ID to selection ID
+		//convert ion ID to selection ID
 		//TODO: It might be faster not use the map, but rather
 		// to use a fixed (but oversized) array to do the idx remapping.
 		// the mem cost is pretty negligible for any sensible use case
-		ASSERT(selectionMapping.find(range) != selectionMapping.end());
-		selectionId= selectionMapping.find(range)->second;
+		if(selectionMapping.find(ionId) == selectionMapping.end())
+		{
+			//Ion was not in inintial selection, may have
+			// been disabled by upstream range
+			continue;
+		}
+		selectionId= selectionMapping.find(ionId)->second;
 
+		ASSERT(binIdx < nGrids);
 		//Increment the nIons for the given bin
 		gridEntries[binIdx].nIons[selectionId]++;
 		gridEntries[binIdx].totalIons++;
@@ -222,7 +224,7 @@ int countBinnedIons(const std::vector<IonHit> &ions, const RangeFile *rng,
 			// TODO: Should we initialise zStart to this ion,
 			//   or should we snap it to the next ion we encounter?
 			gridEntries[binIdx].startPt[extrusionAxis] =filteredIons[ui].getPos()[extrusionAxis];
-			gridEntries[binIdx].endPt[extrusionAxis]=filteredIons[ui].getPos()[extrusionAxis];
+			gridEntries[binIdx].endPt[extrusionAxis]=filteredIons[ui].getPos()[extrusionAxis]-lowBound[extrusionAxis];
 
 			//Set the box x-y (where extrusion=z)  coordinates.
 			setGridABCoords(binIdx,direction,nBins,
@@ -330,11 +332,11 @@ void genBinomialHistogram(const vector<GRID_ENTRY> &completedGridEntries,
 
 }
 
-
-void computeBinomialStats(const vector<GRID_ENTRY> &gridEntries, const BINOMIAL_HIST &binHist,
+void computeBinomialStats(const vector<GRID_ENTRY> &gridEntries, BINOMIAL_HIST &binHist,
 				unsigned int nSelected, BINOMIAL_STATS &stats)
 {
 	stats.nBlocks=gridEntries.size();	
+	//FIXME: Hack
 	stats.nIons=gridEntries[0].totalIons;
 
 	//Compute mean
@@ -359,6 +361,11 @@ void computeBinomialStats(const vector<GRID_ENTRY> &gridEntries, const BINOMIAL_
 	nChiCounted.resize(nSelected,0);
 	stats.chiSquare.resize(nSelected,0);
 
+	//make space to store the theoretical frequencies for each 
+	// bin subtype
+	binHist.theoreticFrequencies.resize(nSelected);
+
+	//Loop over each ion type, and generate the statistics for it
 	CompareMagFloating<double> cmpMag;
 	for(size_t ui=0;ui<nSelected; ui++)
 	{
@@ -366,8 +373,6 @@ void computeBinomialStats(const vector<GRID_ENTRY> &gridEntries, const BINOMIAL_
 		// of the current species.
 		double p;
 		p = stats.mean[ui]/(double)stats.nIons;
-		unsigned int nTotal;
-		nTotal= gridEntries[ui].totalIons;
 
 		//There is a numerical stability concern here, as we will add small
 		// and large numbers repeatedly. This can cause "drift" when summing. To limit it,
@@ -389,15 +394,17 @@ void computeBinomialStats(const vector<GRID_ENTRY> &gridEntries, const BINOMIAL_
 			unsigned int nTimesObs;
 			nTimesObs=it->second;
 
-			//Don't count bins with a low count, as per Moody et al, we set this to 5.
-			// This can skew the chi-square statistic
-			if(nTimesObs <BINOMIAL_LOWCOUNT_SKEW_THRESHOLD)
-				continue;
 
 
 			//Number of times we should have observed blocks with
 			// current count (it->first) of ion
-			binThrObs=gsl_ran_binomial_pdf (it->first,p,nTotal)*stats.nBlocks;
+			binThrObs=gsl_ran_binomial_pdf (it->first,p,stats.nIons)*stats.nBlocks;
+			binHist.theoreticFrequencies[ui][it->first] = binThrObs;
+
+			//Don't count bins with a low count, as per Moody et al, we set this to 5.
+			// This can skew the chi-square statistic
+			if(nTimesObs <BINOMIAL_LOWCOUNT_SKEW_THRESHOLD)
+				continue;
 
 			if(!binThrObs)
 				continue;
@@ -413,8 +420,37 @@ void computeBinomialStats(const vector<GRID_ENTRY> &gridEntries, const BINOMIAL_
 		stats.chiSquare[ui]=std::accumulate(sortedNumbers.begin(),
 					sortedNumbers.end(),0.0);
 	}
+
+	//Normalise the theoretical frequency
+	binHist.theoreticNormalisedFrequencies.resize(nSelected);
+	for(unsigned int ui=0;ui<nSelected;ui++)
+	{
+		{
+
+		map<unsigned int,double> &mSrc=binHist.theoreticFrequencies[ui];
+		map<unsigned int,double> &mDest=binHist.theoreticNormalisedFrequencies[ui];
+		double sum;
+		sum=0;
+		for(map<unsigned int, double>::const_iterator it=mSrc.begin();
+				it!=mSrc.end();++it)
+			sum+=it->second;
+
+		for(map<unsigned int, double>::const_iterator it=mSrc.begin();
+				it!=mSrc.end();++it)
+		{
+			mDest[it->first] = it->second/sum;
+		}
+
+
+
+		}
+	}
+
 	//--
 
+
+
+
 	//Compute the normalised comparison coefficient, "mu"
 	// Moody et al, Microscopy Research and Techniques. 2008
 	//--- 
@@ -487,8 +523,6 @@ void setGridABCoords(unsigned int binIdx, unsigned int *direction,unsigned int *
 
 #ifdef DEBUG
 #include "common/assertion.h"
-#include "backend/APT/APTRanges.h"
-#include "common/mathfuncs.h"
 
 #include <sys/time.h>
 
@@ -503,7 +537,7 @@ bool testBinomial()
 	TEST(testBinomialGSLChi(),"Binomial GSL");
 	TEST(testBinomialBinning(),"Binomial Binning");
 	TEST(testBinomialRandomnessTruePositive(),"Binomial random correctly detected");
-	TEST(testBinomialRandomnessTrueNegative(),"Binomial non-random correclty deteced");
+	TEST(testBinomialRandomnessTrueNegative(),"Binomial non-random correctly detected");
 	return true;
 }
 
@@ -522,8 +556,8 @@ void generateTestGridEntries(vector<GRID_ENTRY> &gridEntries, unsigned int &nSel
 	gridEntries.resize(NSAMPLE);
 
 	//Fake some binomially sampled data.
-	// The first species will be binomially distributed. The second
-	// is fixed to allow the first to fill quota
+	// The first species will be binomially distributed, then scaled
+	// The second species is fixed to allow the first to fill quota
 	for(size_t ui=0;ui<gridEntries.size();ui++)
 	{
 		// Should give a p-value of < 0.05
@@ -628,6 +662,8 @@ bool testBinomialRandomnessTruePositive()
 	}
 
 
+	//use a sorted summation to prevent small numbers from 
+	//being swamped during fp-addition
 	CompareMagFloating<double> cmpMag;
 	std::sort(sortedNumbers.begin(),sortedNumbers.end(),cmpMag);
 	chiSq=std::accumulate(sortedNumbers.begin(),sortedNumbers.end(),0.0f);
@@ -636,14 +672,14 @@ bool testBinomialRandomnessTruePositive()
 	if(sortedNumbers.size() <=2)
 	{
 		//Skip tests if no frequency is above the skew threshold
-		WARN(false,"Unlikely (but possible) situation occured - all binomial ions were insufficiently frequent. skipping Chi-square");
+		WARN(false,"Unlikely (but possible) situation occurred - all binomial ions were insufficiently frequent. skipping Chi-square");
 	}
 	else
 	{
 		double pValue=1.0-gsl_cdf_chisq_P(chiSq,binHist.mapIonFrequencies[0].size()-1);
 
 
-		//Its random, but because we might run this test a lot, set a very low statistical threhsold
+		//Its random, but because we might run this test a lot, set a very low statistical threshold
 		TEST(pValue > 0.00001,"Confirmation of randomness by pvalue");
 		
 
@@ -696,13 +732,13 @@ bool testBinomialBinning()
 	col.red=1.0f;
 	col.green=col.blue=0.0f;
 	rng.addIon("A","A",col);
+	rng.addRange(0.5,1.5,rng.getIonID("A"));
 	
 	col.red=0.0f;
 	col.blue=1.0f;
 
 	rng.addIon("B","B",col);
-	rng.addRange(0.5,1.5,rng.getIonID("B"));
-	rng.addRange(1.5,2.5,rng.getIonID("B"));
+	rng.addRange(1.51,2.5,rng.getIonID("B"));
 
 	vector<IonHit> ions;
 	ions.resize(100);
@@ -739,8 +775,10 @@ bool testBinomialBinning()
 	TEST(!countBinnedIons(ions,&rng,selectedIons,segOpt,g),
 				"binomial binning (auto brick mode)");
 
+	//Check that we counted at least 30% of the ions
+	TEST(g.size() > (float)ions.size()/segOpt.nIons*0.3,"Full bricks only");
 	//Check that the number of grids is less than the number of ions
-	TEST(g.size() < ions.size()/segOpt.nIons,"Full bricks only");
+	TEST(g.size() <=(float)ions.size()/segOpt.nIons,"Full bricks only");
 
 	size_t total=0;
 	for(size_t ui=0;ui<g.size(); ui++)
@@ -748,6 +786,8 @@ bool testBinomialBinning()
 		//Check grid extrusion is positive
 		TEST(g[ui].startPt[2] < g[ui].endPt[2], "grid extrusion direction");
 		TEST(g[ui].totalIons == segOpt.nIons, "grid ion reported count");
+
+		//Count each type of ion
 		size_t kIons;
 		kIons=0;
 		for(size_t uj=0;uj<g[ui].nIons.size();uj++)
