@@ -1,6 +1,6 @@
 /*
  *	binomial.h - Binomia distribution randomness testing
- *	Copyright (C) 2015, D Haley 
+ *	Copyright (C) 2018, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -97,16 +97,14 @@ int countBinnedIons(const std::vector<IonHit> &ions, const RangeFile *rng,
 			std::vector<GRID_ENTRY> &completedGridEntries);
 
 //Generate a vector of ion frequencies in histogram of segment counts, 
+// nSelected is the number of ions selected for binomial analysis,
+// as given to countBinnedIons
 void genBinomialHistogram(const std::vector<GRID_ENTRY> &completedGridEntries,
 				unsigned int nSelected, BINOMIAL_HIST &binHist);
 
-//convert grid frequencies to compositions
 
-void binomialConvert(const std::vector<std::map<unsigned int,unsigned int> > &ionFrequencies,float binWidth, 
-				std::vector<std::vector<float> > &ionConcentrations); 
-
-
-void computeBinomialStats(const std::vector<GRID_ENTRY> &gridEntries,const BINOMIAL_HIST &binHist,
+//Compute the statistics in BINOMIAL_STATS , and binHist.theoreticFrequencies.
+void computeBinomialStats(const std::vector<GRID_ENTRY> &gridEntries,BINOMIAL_HIST &binHist,
 		unsigned int nSelected, BINOMIAL_STATS &binStats);
 
 #ifdef DEBUG
