@@ -1,6 +1,6 @@
 /*
  *	annotation.cpp - 3D annotations filter for 3depict
- *	Copyright (C) 2015, D Haley 
+ *	Copyright (C) 2018, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -24,9 +24,6 @@ using std::vector;
 using std::pair;
 using std::make_pair;
 
-//grab size when doing convex hull calculations
-const unsigned int HULL_GRAB_SIZE=4096;
-
 enum
 {
 	KEY_POSITION=1,
@@ -50,7 +47,8 @@ enum
 	KEY_LINEAR_TICKSPACING,
 	KEY_ANGLE_POS_ZERO,
 	KEY_ANGLE_POS_ONE,
-	KEY_ANGLE_POS_TWO
+	KEY_ANGLE_POS_TWO,
+	KEY_RULER_MODE
 };
 
 enum
@@ -67,6 +65,18 @@ enum
 	BINDING_LINEAR_SPHERERADIUS
 };
 
+enum
+{
+	RULER_MODE_TICKS, // Ticks equally spaced along axis
+	RULER_MODE_POINT_TO_POINT,
+	RULER_MODE_ENUM_END,
+};
+
+const char *rulerModeStrings[] = 
+{
+	NTRANS("Ticks"),
+	NTRANS("Distance")
+};
 
 const char *annotationModeStrings[] =
 {
@@ -108,6 +118,7 @@ AnnotateFilter::AnnotateFilter() : annotationMode(ANNOTATION_TEXT),
 	linearMeasureSpacing=10.0f;
 	lineSize=1.0f;
 	angleFormatPreDecimal=angleFormatPostDecimal=0;
+	rulerMode=RULER_MODE_TICKS;
 
 	cacheOK=false;
 	cache=true; //By default, we should cache, but decision is made higher up
@@ -146,6 +157,7 @@ Filter *AnnotateFilter::cloneUncached() const
 	p->linearMeasureSpacing=linearMeasureSpacing;
 	p->linearMeasureTicks=linearMeasureTicks;
 	p->lineSize=lineSize;
+	p->rulerMode=rulerMode;
 
 	//We are copying whether to cache or not,
 	//not the cache itself
@@ -167,12 +179,22 @@ unsigned int AnnotateFilter::refresh(const std::vector<const FilterStreamData *>
 
 	//If we are not enabled, do not draw anything into the output
 	if(!active)
+	{
+		progress.filterProgress=100;
 		return 0;
+	}
 
+	progress.step=1;
+	progress.maxStep=1;
+	progress.stepName=TRANS("Draw");
+	
 	DrawStreamData *d; 
 	d = new DrawStreamData;
 	d->parent=this;
 
+	//FIXME: refactor each drawing routine into a new function,
+	// rather than one colossal refresh function
+
 	//Draw text output as needed
 	if( annotationMode == ANNOTATION_TEXT ||
 		annotationMode== ANNOTATION_TEXT_WITH_ARROW)
@@ -187,7 +209,7 @@ unsigned int AnnotateFilter::refresh(const std::vector<const FilterStreamData *>
 		dt->setTextDir(acrossVec);
 		dt->setSize((unsigned int)textSize);
 
-		dt->setAlignment(DRAWTEXT_ALIGN_CENTRE);
+		dt->setHorizAlignment(DRAWTEXT_ALIGN_CENTREX);
 		
 		dt->canSelect=true;
 		SelectionDevice *s = new SelectionDevice(this);
@@ -323,20 +345,18 @@ unsigned int AnnotateFilter::refresh(const std::vector<const FilterStreamData *>
 				//and the rest for the actual integer
 				size_t num;
 				num = angleFormatPreDecimal+angleFormatPostDecimal+1;
-				char *buf = new char[num+1];
+				char *buf = new char[num+2];
 
 				std::string tmp,formatStr;
 				formatStr="%";
 				if(angleFormatPreDecimal)
 				{
-
-					stream_cast(tmp,angleFormatPreDecimal + angleFormatPostDecimal+2);
+					stream_cast(tmp,num+1);
 					formatStr+=std::string("0") + tmp;
 				}
 
 				if(angleFormatPostDecimal)
 				{
-
 					stream_cast(tmp,angleFormatPostDecimal);
 					formatStr+=".";
 					formatStr+=tmp;
@@ -356,7 +376,7 @@ unsigned int AnnotateFilter::refresh(const std::vector<const FilterStreamData *>
 			dt = new DrawGLText(getDefaultFontFile().c_str(),FTGL_POLYGON);
 
 			dt->setString(angleString);
-			dt->setAlignment(DRAWTEXT_ALIGN_CENTRE);
+			dt->setHorizAlignment(DRAWTEXT_ALIGN_CENTREX);
 			//Place the text using
 			//a factor of the text size in
 			//the direction of the average
@@ -388,6 +408,7 @@ unsigned int AnnotateFilter::refresh(const std::vector<const FilterStreamData *>
 		DrawVector *dv;
 		dv = new DrawVector;
 
+		//Draw ruler line
 		dv->setOrigin(position);
 		dv->setColour(rgba.r(),rgba.g(),rgba.b(),rgba.a());
 		dv->setVector(target-position);
@@ -395,116 +416,157 @@ unsigned int AnnotateFilter::refresh(const std::vector<const FilterStreamData *>
 
 		d->drawables.push_back(dv);
 
-		//Compute the tick spacings
-		vector<float> tickSpacings;
-		if(linearFixedTicks)
+		//Ruler has multiple appearance modes,
+		// draw the correct one
+		switch(rulerMode)
 		{
-			tickSpacingsFromFixedNum(0,sqrtf(target.sqrDist(position)),
-					linearMeasureTicks,tickSpacings);
-		}
-		else
-		{
-			tickSpacingsFromInterspace(0,sqrtf(target.sqrDist(position)),
-						linearMeasureSpacing,tickSpacings);
-		}
-
-		if(tickSpacings.size())
-		{
-
-			Point3D measureNormal;
-			measureNormal = target-position;
-			measureNormal.normalise();
-
-			//Construct the drawable text object
-			for(unsigned int ui=0;ui<tickSpacings.size();ui++)
+			case RULER_MODE_TICKS:
 			{
+				//Compute the tick spacings
+				vector<float> tickSpacings;
+				if(linearFixedTicks)
+				{
+					tickSpacingsFromFixedNum(0,sqrtf(target.sqrDist(position)),
+							linearMeasureTicks,tickSpacings);
+				}
+				else
+				{
+					tickSpacingsFromInterspace(0,sqrtf(target.sqrDist(position)),
+								linearMeasureSpacing,tickSpacings);
+				}
+
+				if(tickSpacings.size())
+				{
+
+					Point3D measureNormal;
+					measureNormal = target-position;
+					measureNormal.normalise();
+
+					//Construct the drawable text object
+					for(unsigned int ui=0;ui<tickSpacings.size();ui++)
+					{
+						DrawGLText *dT; 
+						//Create the tick that will be added to the drawables
+						dT = new DrawGLText(getDefaultFontFile().c_str(),FTGL_POLYGON);
+							
+						dT->setColour(rgba.r(),rgba.g()	,rgba.b(),rgba.a());
+						dT->setOrigin(measureNormal*tickSpacings[ui] + position);
+						dT->setUp(upVec);	
+						dT->setTextDir(acrossVec);
+						dT->setSize((unsigned int)fontSizeLinearMeasure);
+
+						string s;
+						stream_cast(s,tickSpacings[ui]);
+
+						dT->setString(s);
+
+						d->drawables.push_back(dT);
+					}
+
+
+
+
+				}
+				break;
+			}
+			case RULER_MODE_POINT_TO_POINT:
+			{
+				//obtain the length of the two ends.
+				float length;
+				length=sqrtf(target.sqrDist(position));
+
 				DrawGLText *dT; 
 				//Create the tick that will be added to the drawables
 				dT = new DrawGLText(getDefaultFontFile().c_str(),FTGL_POLYGON);
 					
 				dT->setColour(rgba.r(),rgba.g()	,rgba.b(),rgba.a());
-				dT->setOrigin(measureNormal*tickSpacings[ui] + position);
+				dT->setOrigin((target+position)*0.5);
 				dT->setUp(upVec);	
 				dT->setTextDir(acrossVec);
 				dT->setSize((unsigned int)fontSizeLinearMeasure);
 
 				string s;
-				stream_cast(s,tickSpacings[ui]);
-
+				stream_cast(s,length);
 				dT->setString(s);
 
 				d->drawables.push_back(dT);
+				break;
+
 			}
-
-
-			//Now draw the end markers
-
-			//Start marker
-			DrawSphere *dS;
-			dS = new DrawSphere;
-			dS->setRadius(sphereMarkerSize);
-			dS->setOrigin(position);
-			dS->setColour(rgba.r(),rgba.g(),rgba.b(),rgba.a());
-
-			dS->canSelect=true;
-			dS->wantsLight=true;
-			
-			SelectionDevice *s= new SelectionDevice(this);
-			SelectionBinding bind[4];
-			//Create binding for sphere translation.
-			//Note that each binding is a bit different, as it
-			//affects each sphere separately.
-			bind[0].setBinding(SELECT_BUTTON_LEFT,0,DRAW_SPHERE_BIND_ORIGIN,
-						BINDING_LINEAR_ORIGIN,position,dS);
-			bind[0].setInteractionMode(BIND_MODE_POINT3D_TRANSLATE);
-			s->addBinding(bind[0]);
-
-			//Create binding for sphere scaling, each binding is the same 
-			bind[1].setBinding(SELECT_BUTTON_LEFT,FLAG_SHIFT,DRAW_SPHERE_BIND_RADIUS,
-						BINDING_LINEAR_SPHERERADIUS,dS->getRadius(),dS);
-			bind[1].setInteractionMode(BIND_MODE_FLOAT_TRANSLATE);
-			bind[1].setFloatLimits(0,std::numeric_limits<float>::max());
-
-			s->addBinding(bind[1]);
-
-			devices.push_back(s);
-			d->drawables.push_back(dS);
-		
-		
-			//Now do the second sphere (end marker)
-			s= new SelectionDevice(this);
-			dS = new DrawSphere;
-			
-			dS->setRadius(sphereMarkerSize);
-			dS->setOrigin(target);
-			dS->setColour(rgba.r(),rgba.g(),rgba.b(),rgba.a());
-
-			dS->canSelect=true;
-			dS->wantsLight=true;
-
-			bind[2].setBinding(SELECT_BUTTON_LEFT,0,DRAW_SPHERE_BIND_ORIGIN,
-						BINDING_LINEAR_TARGET,target,dS);
-			bind[2].setInteractionMode(BIND_MODE_POINT3D_TRANSLATE);
-			s->addBinding(bind[2]);
-
-			//Create binding for sphere scaling, each binding is the same 
-			bind[3].setBinding(SELECT_BUTTON_LEFT,FLAG_SHIFT,DRAW_SPHERE_BIND_RADIUS,
-						BINDING_LINEAR_SPHERERADIUS,dS->getRadius(),dS);
-			bind[3].setInteractionMode(BIND_MODE_FLOAT_TRANSLATE);
-			bind[3].setFloatLimits(0,std::numeric_limits<float>::max());
-			s->addBinding(bind[3]);
-		
-			
-			devices.push_back(s);
-			d->drawables.push_back(dS);
-
-
 		}
+
+
+		//Now draw the end markers
+		
+		//Start marker
+		//==
+		DrawSphere *dS;
+		dS = new DrawSphere;
+		dS->setRadius(sphereMarkerSize);
+		dS->setOrigin(position);
+		dS->setColour(rgba.r(),rgba.g(),rgba.b(),rgba.a());
+
+		dS->canSelect=true;
+		dS->wantsLight=true;
+		
+		SelectionDevice *s= new SelectionDevice(this);
+		SelectionBinding bind[4];
+		//Create binding for sphere translation.
+		//Note that each binding is a bit different, as it
+		//affects each sphere separately.
+		bind[0].setBinding(SELECT_BUTTON_LEFT,0,DRAW_SPHERE_BIND_ORIGIN,
+					BINDING_LINEAR_ORIGIN,position,dS);
+		bind[0].setInteractionMode(BIND_MODE_POINT3D_TRANSLATE);
+		s->addBinding(bind[0]);
+
+		//Create binding for sphere scaling, each binding is the same 
+		bind[1].setBinding(SELECT_BUTTON_LEFT,FLAG_SHIFT,DRAW_SPHERE_BIND_RADIUS,
+					BINDING_LINEAR_SPHERERADIUS,dS->getRadius(),dS);
+		bind[1].setInteractionMode(BIND_MODE_FLOAT_TRANSLATE);
+		bind[1].setFloatLimits(0,std::numeric_limits<float>::max());
+
+		s->addBinding(bind[1]);
+
+		devices.push_back(s);
+		d->drawables.push_back(dS);
+		//==
+	
+
+
+		//Now do the second sphere (end marker)
+		//==
+		s= new SelectionDevice(this);
+		dS = new DrawSphere;
+		
+		dS->setRadius(sphereMarkerSize);
+		dS->setOrigin(target);
+		dS->setColour(rgba.r(),rgba.g(),rgba.b(),rgba.a());
+
+		dS->canSelect=true;
+		dS->wantsLight=true;
+
+		bind[2].setBinding(SELECT_BUTTON_LEFT,0,DRAW_SPHERE_BIND_ORIGIN,
+					BINDING_LINEAR_TARGET,target,dS);
+		bind[2].setInteractionMode(BIND_MODE_POINT3D_TRANSLATE);
+		s->addBinding(bind[2]);
+
+		//Create binding for sphere scaling, each binding is the same 
+		bind[3].setBinding(SELECT_BUTTON_LEFT,FLAG_SHIFT,DRAW_SPHERE_BIND_RADIUS,
+					BINDING_LINEAR_SPHERERADIUS,dS->getRadius(),dS);
+		bind[3].setInteractionMode(BIND_MODE_FLOAT_TRANSLATE);
+		bind[3].setFloatLimits(0,std::numeric_limits<float>::max());
+		s->addBinding(bind[3]);
+	
+		
+		devices.push_back(s);
+		d->drawables.push_back(dS);
+		//==
 	}
 	
 	d->cached=0;
 	getOut.push_back(d);
 
+	progress.filterProgress=100;
 	return 0;
 }
 
@@ -516,262 +578,145 @@ size_t AnnotateFilter::numBytesForCache(size_t nObjects) const
 void AnnotateFilter::getProperties(FilterPropGroup &propertyList) const
 {
 	string tmpStr;
-	FilterProperty p;
 	size_t curGroup=0;
 
-	tmpStr=boolStrEnc(active);	
-
-	p.name=TRANS("Enable");
-	p.data=tmpStr;
-	p.key=KEY_ENABLE;
-	p.helpText=TRANS("Enable/disable annotation");
-	p.type=PROPERTY_TYPE_BOOL;
-	propertyList.addProperty(p,curGroup);
+	FilterProperty pEnable(KEY_ENABLE, TRANS("Enable"),
+		 active,TRANS("Enable/disable annotation"));
+					
+	propertyList.addProperty(pEnable,curGroup);
 
 	if(!active)
+	{
+		propertyList.setGroupTitle(curGroup,TRANS("Status"));
 		return;
+	}
 
 
 
 	vector<pair<unsigned int,string> > choices;
-	
 	for(unsigned int ui=0;ui<ANNOTATION_MODE_END; ui++)
 	{
-		choices.push_back(make_pair((unsigned int)ui,
+		choices.push_back(make_pair(ui,
 					TRANS(annotationModeStrings[ui])));
 	}
+	FilterProperty pMode(KEY_MODE, TRANS("Mode"),
+			choices,annotationMode,TRANS("Type or style of annotation"));
+	propertyList.addProperty(pMode,curGroup);
 
-	tmpStr=choiceString(choices,annotationMode);
-	p.name=TRANS("Mode");
-	p.data=tmpStr;
-	p.key=KEY_MODE;
-	p.helpText=TRANS("Type or style of annotation");
-	p.type=PROPERTY_TYPE_CHOICE;
-	propertyList.addProperty(p,curGroup);
 	propertyList.setGroupTitle(curGroup,TRANS("Mode"));
 	curGroup++;
 
+	FilterProperty p;
 	switch(annotationMode)
 	{
 		case ANNOTATION_TEXT:
 		{
 			//Note to translators, this is short for "annotation text",
 			// or similar
-			p.name=TRANS("Annotation");
-			p.data=annotateText;
-			p.type=PROPERTY_TYPE_STRING;
-			p.key=KEY_ANNOTATE_TEXT;
-			p.helpText=TRANS("Text of annotation");
-			propertyList.addProperty(p,curGroup);
-		
-			stream_cast(tmpStr,position);
-			p.name=TRANS("Origin");
-			p.data=tmpStr;
-			p.type=PROPERTY_TYPE_POINT3D;
-			p.key=KEY_POSITION;
-			p.helpText=TRANS("Position of annotation");
-			propertyList.addProperty(p,curGroup);
+			FilterProperty pMode(KEY_ANNOTATE_TEXT,PROPERTY_TYPE_STRING,
+				TRANS("Annotation"), annotateText, TRANS("Text of annotation"));
+			propertyList.addProperty(pMode,curGroup);
+	
+			FilterProperty pOrigin(KEY_POSITION, TRANS("Origin"),position,
+					TRANS("Position of annotation"));
+			propertyList.addProperty(pOrigin,curGroup);
 			
-			stream_cast(tmpStr,upVec);
-			p.name=TRANS("Up dir");
-			p.data=tmpStr;
-			p.type=PROPERTY_TYPE_STRING;
-			p.key=KEY_UPVEC;
-			p.helpText=TRANS("Vector for up direction of annotation text");
-			propertyList.addProperty(p,curGroup);
+			FilterProperty pUp(KEY_UPVEC, TRANS("Up dir"),upVec,
+					TRANS("Vector for up direction of annotation text"));
+			propertyList.addProperty(pUp,curGroup);
 		
-			stream_cast(tmpStr,acrossVec);
-			p.name=TRANS("Across dir");
-			p.data=tmpStr;
-			p.type=PROPERTY_TYPE_STRING;
-			p.key=KEY_ACROSSVEC;
-			p.helpText=TRANS("Reading direction for annotation");
-			propertyList.addProperty(p,curGroup);
-
-
-			stream_cast(tmpStr,textSize);
-			p.name=TRANS("Text size");
-			p.data=tmpStr;
-			p.type=PROPERTY_TYPE_REAL;
-			p.key=KEY_TEXTSIZE;
-			p.helpText=TRANS("Relative size of annotation text");
-			propertyList.addProperty(p,curGroup);
-
-			break;
-		}
-		case ANNOTATION_ARROW:
-		{
-			stream_cast(tmpStr,position);
-			p.name=TRANS("Start");
-			p.data=tmpStr;
-			p.type=PROPERTY_TYPE_POINT3D;
-			p.key=KEY_POSITION;
-			p.helpText=TRANS("3D position for tail of arrow");
-			propertyList.addProperty(p,curGroup);
-		
-			stream_cast(tmpStr,target);
-			p.name=TRANS("End");
-			p.data=tmpStr;
-			p.type=PROPERTY_TYPE_POINT3D;
-			p.key=KEY_TARGET;
-			p.helpText=TRANS("3D Position to which arrow points");
-			propertyList.addProperty(p,curGroup);
-			
-			propertyList.setGroupTitle(curGroup,TRANS("Positioning"));
-
-			curGroup++;
-
-			stream_cast(tmpStr,annotateSize);
-			p.name=TRANS("Tip radius");
-			p.data=tmpStr;
-			p.type=PROPERTY_TYPE_REAL;
-			p.key=KEY_ARROW_SIZE;
-			p.helpText=TRANS("Size of the arrow head");
-			propertyList.addProperty(p,curGroup);
-
-			stream_cast(tmpStr,lineSize);
-			p.name=TRANS("Line size");
-			p.data=tmpStr;
-			p.type=PROPERTY_TYPE_REAL;
-			p.key=KEY_LINESIZE;
-			p.helpText=TRANS("Thickness of line used to draw arrow stem");
-			propertyList.addProperty(p,curGroup);
+			FilterProperty pAcross(KEY_ACROSSVEC, TRANS("Across dir"),acrossVec,
+						TRANS("Reading direction for annotation"));
+			propertyList.addProperty(pAcross,curGroup);
 			
+			FilterProperty pTextSize(KEY_TEXTSIZE, TRANS("Text size"),textSize,
+						TRANS("Relative size of annotation text"));
+			propertyList.addProperty(pTextSize,curGroup);
 
 			break;
 		}
 		case ANNOTATION_TEXT_WITH_ARROW:
 		{
-			stream_cast(tmpStr,position);
-			p.name=TRANS("Start");
-			p.data=tmpStr;
-			p.type=PROPERTY_TYPE_POINT3D;
-			p.key=KEY_POSITION;
-			p.helpText=TRANS("3D position for tail of arrow");
-			propertyList.addProperty(p,curGroup);
+			FilterProperty pMode(KEY_ANNOTATE_TEXT,PROPERTY_TYPE_STRING,
+				TRANS("Annotation"), annotateText, TRANS("Text of annotation"));
+			propertyList.addProperty(pMode,curGroup);
+			//Fall through
+		}
+		case ANNOTATION_ARROW:
+		{
+			FilterProperty pOrigin(KEY_POSITION, TRANS("Start"),position,
+					TRANS("3D position for tail of arrow"));
+			propertyList.addProperty(pOrigin,curGroup);
 			
 
-			stream_cast(tmpStr,target);
-			p.name=TRANS("End");
-			p.data=tmpStr;
-			p.type=PROPERTY_TYPE_POINT3D;
-			p.key=KEY_TARGET;
-			p.helpText=TRANS("3D Position to which arrow points");
-			propertyList.addProperty(p,curGroup);
+			FilterProperty pEnd(KEY_TARGET, TRANS("End"),position,
+					TRANS("3D position to which arrow points"));
+			propertyList.addProperty(pEnd,curGroup);
 			
-			//Note to translators, this is short for "annotation text",
-			// or similar
-			p.name=TRANS("Annotation");
-			p.data=annotateText;
-			p.type=PROPERTY_TYPE_STRING;
-			p.key=KEY_ANNOTATE_TEXT;
-			p.helpText=TRANS("Text of annotation");
-			propertyList.addProperty(p,curGroup);
+
 			
 			propertyList.setGroupTitle(curGroup,TRANS("Options"));
 			curGroup++;
 
-			stream_cast(tmpStr,textSize);
-			p.name=TRANS("Text size");
-			p.data=tmpStr;
-			p.type=PROPERTY_TYPE_REAL;
-			p.key=KEY_TEXTSIZE;
-			p.helpText=TRANS("Relative size of annotation text");
-			propertyList.addProperty(p,curGroup);
-			
-			stream_cast(tmpStr,upVec);
-			p.name=TRANS("Up dir");
-			p.data=tmpStr;
-			p.type=PROPERTY_TYPE_STRING;
-			p.key=KEY_UPVEC;
-			propertyList.addProperty(p,curGroup);
+			FilterProperty pTextSize(KEY_TEXTSIZE, TRANS("Text size"),textSize,
+						TRANS("Relative size of annotation text"));
+			propertyList.addProperty(pTextSize,curGroup);
+
+			FilterProperty pUp(KEY_UPVEC, TRANS("Up dir"),upVec,
+					TRANS("Vector for up direction of annotation text"));
 		
-			stream_cast(tmpStr,acrossVec);
-			p.name=TRANS("Across dir");
-			p.data=tmpStr;
-			p.type=PROPERTY_TYPE_STRING;
-			p.key=KEY_ACROSSVEC;
-			propertyList.addProperty(p,curGroup);
+			FilterProperty pAcross(KEY_ACROSSVEC, TRANS("Across dir"),acrossVec,
+						TRANS("Reading direction for annotation"));
+			propertyList.addProperty(pAcross,curGroup);
+
+			FilterProperty pArrow(KEY_ARROW_SIZE, TRANS("Arrow radius"),annotateSize,
+					TRANS("Size of the arrow head"));
+			propertyList.addProperty(pArrow,curGroup);
+
 
-			stream_cast(tmpStr,annotateSize);
-			p.name=TRANS("Tip radius");
-			p.data=tmpStr;
-			p.type=PROPERTY_TYPE_REAL;
-			p.key=KEY_ARROW_SIZE;
-			propertyList.addProperty(p,curGroup);
 			break;
 		}
 		case ANNOTATION_ANGLE_MEASURE:
 		{
-			stream_cast(tmpStr,anglePos[0]);
-			p.name=TRANS("Position A");
-			p.data=tmpStr;
-			p.type=PROPERTY_TYPE_POINT3D;
-			p.key=KEY_ANGLE_POS_ZERO;
-			p.helpText=TRANS("Location of first non-central vertex");
-			propertyList.addProperty(p,curGroup);
-
-			stream_cast(tmpStr,anglePos[1]);
-			p.name=TRANS("Origin ");
-			p.data=tmpStr;
-			p.type=PROPERTY_TYPE_POINT3D;
-			p.key=KEY_ANGLE_POS_ONE;
-			p.helpText=TRANS("Location of central vertex");
-			propertyList.addProperty(p,curGroup);
 			
-			stream_cast(tmpStr,anglePos[2]);
-			p.name=TRANS("Position B");
-			p.data=tmpStr;
-			p.type=PROPERTY_TYPE_POINT3D;
-			p.key=KEY_ANGLE_POS_TWO;
-			p.helpText=TRANS("Location of second non-central vertex");
-			propertyList.addProperty(p,curGroup);
+			FilterProperty pFirstPos(KEY_ANGLE_POS_ZERO, TRANS("Position A"),anglePos[0],
+					TRANS("Location of first non-central vertex"));
+			propertyList.addProperty(pFirstPos,curGroup);
+			
+			FilterProperty pOrigin(KEY_ANGLE_POS_ONE, TRANS("Origin"),anglePos[1],
+						TRANS("Location of central vertex"));
+			propertyList.addProperty(pOrigin,curGroup);
+
+			FilterProperty pSecondPos(KEY_ANGLE_POS_TWO, TRANS("Position B"),anglePos[1],
+						TRANS("Location of second non-central vertex"));
+			propertyList.addProperty(pSecondPos,curGroup);
 			
 			propertyList.setGroupTitle(curGroup,TRANS("Positioning"));
 			curGroup++;
 			
-			stream_cast(tmpStr,acrossVec);
-			p.name=TRANS("Across dir");
-			p.data=tmpStr;
-			p.type=PROPERTY_TYPE_STRING;
-			p.key=KEY_ACROSSVEC;
-			p.helpText=TRANS("Reading direction for angle text");
-			propertyList.addProperty(p,curGroup);
+			FilterProperty pUp(KEY_UPVEC, TRANS("Up dir"),upVec,
+					TRANS("Vector for up direction of annotation text"));
+			propertyList.addProperty(pUp,curGroup);
+		
+			FilterProperty pAcross(KEY_ACROSSVEC, TRANS("Across dir"),acrossVec,
+						TRANS("Reading direction for annotation"));
+			propertyList.addProperty(pAcross,curGroup);
 
-			stream_cast(tmpStr,upVec);
-			p.name=TRANS("Up dir");
-			p.helpText=TRANS("Vector for up direction of angle text");
-			p.data=tmpStr;
-			p.type=PROPERTY_TYPE_STRING;
-			p.key=KEY_UPVEC;
-			propertyList.addProperty(p,curGroup);
-
-
-			p.key=KEY_REFLEXIVE;
-			p.name=TRANS("Reflexive");
-			p.data=boolStrEnc(reflexAngle);
-			p.type=PROPERTY_TYPE_BOOL;
-			p.helpText=TRANS("Measure interor (enabled) or exterior angle (disabled)");
-			propertyList.addProperty(p,curGroup);
+			FilterProperty pReflexive(KEY_REFLEXIVE, TRANS("Reflexive"),reflexAngle,
+						TRANS("Measure interor (enabled) or exterior angle (disabled)"));
+			propertyList.addProperty(pReflexive,curGroup);
+		
+			FilterProperty pAngleTextVis(KEY_ANGLE_TEXT_VISIBLE, TRANS("Show Angle"),showAngleText,
+					TRANS("Display angle text (when enabled)"));
+			propertyList.addProperty(pAngleTextVis,curGroup);
 		
 			
-
-			p.name=TRANS("Show Angle");
-			p.data=boolStrEnc(showAngleText);
-			p.type=PROPERTY_TYPE_BOOL;
-			p.key=KEY_ANGLE_TEXT_VISIBLE;
-			p.helpText=TRANS("Display angle text (when enabled)");
-			propertyList.addProperty(p,curGroup);
 		
 			if(showAngleText)
 			{
-				stream_cast(tmpStr,textSize);
-				p.name=TRANS("Text size");
-				p.data=tmpStr;
-				p.type=PROPERTY_TYPE_REAL;
-				p.key=KEY_TEXTSIZE;
-				p.helpText=TRANS("Size of angle text");
-				propertyList.addProperty(p,curGroup);
+				FilterProperty pTextSize(KEY_TEXTSIZE, TRANS("Text size"),textSize,
+							TRANS("Relative size of annotation text"));
+				propertyList.addProperty(pTextSize,curGroup);
 
 
 				std::string tmp2;
@@ -788,122 +733,82 @@ void AnnotateFilter::getProperties(FilterPropGroup &propertyList) const
 					tmpStr+=std::string(".") + tmp2;
 				}
 
-				p.name=TRANS("Digit format");
-				p.data=tmpStr;
-				p.type=PROPERTY_TYPE_STRING;
-				p.key=KEY_ANGLE_FORMAT_STRING;
-				p.helpText=TRANS("Format of angle text; # for numeral position, \'.\' for separator, eg ##.## gives 12.34");
-				propertyList.addProperty(p,curGroup);
+				FilterProperty pDigitFormat(KEY_ANGLE_FORMAT_STRING, TRANS("Digit Format"),tmpStr,TRANS("Format of angle text; # for numeral position, \'.\' for separator, eg ##.## gives 12.34"));
+				propertyList.addProperty(pDigitFormat,curGroup);
 
 			}
 			
-			stream_cast(tmpStr,sphereMarkerSize);
-			p.name=TRANS("Sphere size");
-			p.data=tmpStr;
-			p.type=PROPERTY_TYPE_REAL;
-			p.key=KEY_SPHERE_ANGLE_SIZE;
-			p.helpText=TRANS("Marker sphere size for manipulating tool");
-			propertyList.addProperty(p,curGroup);
 
-		
+			FilterProperty pSphereSize(KEY_SPHERE_ANGLE_SIZE, TRANS("Sphere size"),sphereMarkerSize,
+				TRANS("Marker sphere size for manipulating tool"));
+			propertyList.addProperty(pSphereSize,curGroup);
 
 			break;
 		}
 		case ANNOTATION_LINEAR_MEASURE:
 		{
-			stream_cast(tmpStr,position);
-			p.name=TRANS("Start");
-			p.data=tmpStr;
-			p.type=PROPERTY_TYPE_POINT3D;
-			p.key=KEY_POSITION;
-			p.helpText=TRANS("Ruler beginning 3D location");
-			propertyList.addProperty(p,curGroup);
-		
-			stream_cast(tmpStr,target);
-			p.name=TRANS("End");
-			p.data=tmpStr;
-			p.type=PROPERTY_TYPE_POINT3D;
-			p.key=KEY_TARGET;
-			p.helpText=TRANS("Ruler finish 3D location");
-			propertyList.addProperty(p,curGroup);
-
-			stream_cast(tmpStr,upVec);
-			p.name=TRANS("Up dir");
-			p.data=tmpStr;
-			p.type=PROPERTY_TYPE_STRING;
-			p.key=KEY_UPVEC;
-			p.helpText=TRANS("Vector for up direction of annotation text");
-			propertyList.addProperty(p,curGroup);
-		
-			stream_cast(tmpStr,acrossVec);
-			p.name=TRANS("Across dir");
-			p.data=tmpStr;
-			p.type=PROPERTY_TYPE_STRING;
-			p.key=KEY_ACROSSVEC;
-			p.helpText=TRANS("Reading direction for annotation");
-			propertyList.addProperty(p,curGroup);
+			vector<pair<unsigned int,string> > choices;
 			
-			stream_cast(tmpStr,fontSizeLinearMeasure);
-			p.key=KEY_LINEAR_FONTSIZE;
-			p.name=TRANS("Font Size");
-			p.data=tmpStr;
-			p.type=PROPERTY_TYPE_INTEGER;
-			p.helpText=TRANS("Relative size of annotation text");
-			propertyList.addProperty(p,curGroup);
-			
-		
-			tmpStr=boolStrEnc(linearFixedTicks);
-			p.key=KEY_LINEAR_FIXED_TICKS;
-			p.name=TRANS("Fixed ticks");
-			p.data=tmpStr;
-			p.type=PROPERTY_TYPE_BOOL;
-			p.helpText=TRANS("Use fixed (enabled) number of text markers, or one every fixed distance (disabled)");
-			propertyList.addProperty(p,curGroup);
+			for(unsigned int ui=0;ui<RULER_MODE_ENUM_END; ui++)
+				choices.push_back(make_pair(ui, TRANS(rulerModeStrings[ui])));
 
+			FilterProperty pRulerMode(KEY_RULER_MODE, TRANS("Display Style"),
+				choices,rulerMode, TRANS("Set the appearance of the ruler"));
+			propertyList.addProperty(pRulerMode,curGroup);
+
+			FilterProperty pPosition(KEY_POSITION, TRANS("Start"),position,
+				 TRANS("Ruler beginning location, in 3D"));
+			propertyList.addProperty(pPosition,curGroup);
+		
+			FilterProperty pTarget(KEY_TARGET, TRANS("End"),target,
+				 TRANS("Ruler finish location, in 3D"));
+			propertyList.addProperty(pTarget,curGroup);
+			
+			FilterProperty pUp(KEY_UPVEC, TRANS("Up direction"),upVec,
+				TRANS("Vector for up direction of annotation text"));
+			propertyList.addProperty(pUp,curGroup);
+			
+			FilterProperty pAcross(KEY_ACROSSVEC, TRANS("Across direction"),acrossVec,
+				TRANS("Reading direction for annotation"));
+			propertyList.addProperty(pAcross,curGroup);
+
+			FilterProperty pFontSize(KEY_LINEAR_FONTSIZE, TRANS("Font Size"),
+				fontSizeLinearMeasure, TRANS("Relative size of annotation text"));
+			propertyList.addProperty(pFontSize,curGroup);
+			
+			FilterProperty pFixedTicks(KEY_LINEAR_FIXED_TICKS, TRANS("Fixed Ticks"),
+				linearFixedTicks, TRANS("Use fixed (enabled) number of text markers, or one every fixed distance (disabled)"));
+			propertyList.addProperty(pFixedTicks,curGroup);
+			
 			if(linearFixedTicks)
 			{
-				stream_cast(tmpStr,linearMeasureTicks);
-				p.key=KEY_LINEAR_NUMTICKS;
-				p.name=TRANS("Num Ticks");
-				p.data=tmpStr;
-				p.type=PROPERTY_TYPE_INTEGER;
-				p.helpText=TRANS("Number of tick marks along ruler");
-				propertyList.addProperty(p,curGroup);
+				FilterProperty pNumTicks(KEY_LINEAR_NUMTICKS, TRANS("Num Ticks"),
+					linearMeasureTicks, TRANS("Number of tick marks along ruler"));
+				propertyList.addProperty(pNumTicks,curGroup);
 			}
 			else
 			{
-				stream_cast(tmpStr,linearMeasureSpacing);
-				p.key=KEY_LINEAR_TICKSPACING;
-				p.name=TRANS("Tick Spacing");
-				p.data=tmpStr;
-				p.type=PROPERTY_TYPE_REAL;
-				p.helpText=TRANS("Distance between tick marks along ruler");
-				propertyList.addProperty(p,curGroup);
+				FilterProperty pTickSpacing(KEY_LINEAR_TICKSPACING, string(TRANS("Tick Spacing")), 
+					(float)linearMeasureSpacing,string(TRANS("Distance between tick marks along ruler")));
+				propertyList.addProperty(pTickSpacing,curGroup);
 			}
+		
+			string tmpStr;
+			stream_cast(tmpStr,sphereMarkerSize);	
+			FilterProperty pSphereSize(KEY_SPHERE_ANGLE_SIZE, string(TRANS("Sphere Size")), 
+						(float)sphereMarkerSize, string(TRANS("Marker sphere size for manipulating tool")));
+			propertyList.addProperty(pSphereSize,curGroup);
 			
-			
-			stream_cast(tmpStr,sphereMarkerSize);
-			p.name=TRANS("Sphere size");
-			p.data=tmpStr;
-			p.type=PROPERTY_TYPE_REAL;
-			p.key=KEY_SPHERE_ANGLE_SIZE;
-			p.helpText=TRANS("Marker sphere size for manipulating tool");
-			propertyList.addProperty(p,curGroup);
-
 			break;
 		}
 		default:
 			ASSERT(false);
 	}
 
-
-	p.key=KEY_COLOUR;
-	p.name=TRANS("Colour");
-	p.data=rgba.toColourRGBA().rgbString();
-	p.type=PROPERTY_TYPE_COLOUR;
-	p.helpText=TRANS("Colour for ruler and ticks");
-	propertyList.addProperty(p,curGroup);
-	propertyList.setGroupTitle(curGroup,TRANS("Appearance"));
+	FilterProperty pColour(KEY_COLOUR, TRANS("Colour"), 
+				rgba, TRANS("Colour for ruler and ticks"));
+	propertyList.addProperty(pColour,curGroup);
+	propertyList.setGroupTitle(curGroup,TRANS("Display"));
 }
 
 bool AnnotateFilter::setProperty(  unsigned int key,
@@ -1171,6 +1076,24 @@ bool AnnotateFilter::setProperty(  unsigned int key,
 			needUpdate=true;
 			break;
 		}
+		case KEY_RULER_MODE:
+		{
+			unsigned int newMode;
+			for(newMode=0;newMode<RULER_MODE_ENUM_END;newMode++)
+			{
+				if(stripped == TRANS(rulerModeStrings[newMode]))
+					break;
+			}
+
+			if(newMode == ANNOTATION_MODE_END)
+				return false;
+			if(newMode!=rulerMode)
+			{
+				rulerMode=newMode;
+				needUpdate=true;
+			}
+			break;
+		}
 		
 		default:
 			ASSERT(false);
@@ -1220,6 +1143,9 @@ bool AnnotateFilter::writeState(std::ostream &f,unsigned int format, unsigned in
 			
 			f << tabs(depth+1) << "<angleformat predecimal=\""<< angleFormatPreDecimal 
 					<< "\" postdecimal=\"" << angleFormatPostDecimal<< "\" />" << endl;
+			f << tabs(depth+1) << "<ruler mode=\""<<rulerMode  << "\" spacing=\""<<
+					linearMeasureSpacing <<"\" tickcount=\"" << linearMeasureTicks << 
+					"\" fontsize=\"" << fontSizeLinearMeasure <<  "\"/>"  << endl;
 
 			f << tabs(depth) << "</" <<trueName()<< ">" << endl;
 			break;
@@ -1369,6 +1295,38 @@ bool AnnotateFilter::readState(xmlNodePtr &nodePtr, const std::string &stateFile
 	if(!XMLGetAttrib(nodePtr,angleFormatPostDecimal,"predecimal"))
 		return false;
 
+	//This feature did not exist prior to 2741:2eb7770d3018
+	// (<0.0.21), and thus may not be present
+	xmlNodePtr tmpNode;
+	tmpNode=nodePtr;
+
+	if(!XMLHelpFwdToElem(nodePtr,"ruler"))
+	{	
+		if(!XMLGetAttrib(nodePtr,rulerMode,"mode"))
+			return false;
+		if(rulerMode >=RULER_MODE_ENUM_END)
+			return false;
+
+		if(!XMLGetAttrib(nodePtr,linearMeasureSpacing,"spacing"))
+			return false;
+		if(linearMeasureSpacing<=0 )
+			return false;
+
+		if(!XMLGetAttrib(nodePtr,linearMeasureTicks,"tickcount"))
+			return false;
+		if(linearMeasureTicks <=0)
+			return false;
+
+		if(!XMLGetAttrib(nodePtr,fontSizeLinearMeasure,"fontsize"))
+			return false;
+		if(fontSizeLinearMeasure <=0)
+			return false;
+	}
+	else
+	{
+		//if no instructions, use tick mode
+		rulerMode = RULER_MODE_TICKS;
+	}
 
 	return true;
 }
