@@ -1,6 +1,6 @@
 /*
  *	boundingBox.cpp - Place a bounding box around point datasets
- *	Copyright (C) 2015, D Haley 
+ *	Copyright (C) 2018, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -103,6 +103,7 @@ Filter *BoundingBoxFilter::cloneUncached() const
 	p->threeDText=threeDText;	
 
 	p->lineWidth=lineWidth;
+	p->lineColour=lineColour;
 	p->fontSize=fontSize;
 
 	//We are copying whether to cache or not,
@@ -200,6 +201,7 @@ void BoundingBoxFilter::drawTicks(const BoundCube &bTotal, DrawStreamData *d) co
 			dV->setVector(tickVector);
 			dV->setColour(lineColour.r(),lineColour.g(),
 					lineColour.b(), lineColour.a());
+			dV->setLineSize(lineWidth);
 
 			d->drawables.push_back(dV);
 	
@@ -230,7 +232,8 @@ void BoundingBoxFilter::drawTicks(const BoundCube &bTotal, DrawStreamData *d) co
 				dT->setOrigin(tickPosition + tickVector*2);	
 				dT->setUp(Point3D(0,0,1));	
 				dT->setTextDir(textVector);
-				dT->setAlignment(DRAWTEXT_ALIGN_RIGHT);
+				dT->setHorizAlignment(DRAWTEXT_ALIGN_RIGHT);
+				dT->setVertAlignment(DRAWTEXT_ALIGN_CENTREY);
 
 				d->drawables.push_back(dT);
 			}
@@ -253,10 +256,11 @@ void BoundingBoxFilter::drawTicks(const BoundCube &bTotal, DrawStreamData *d) co
 		dT->setColour(lineColour.r(),lineColour.g(),
 			lineColour.b(),lineColour.a());
 		dT->setSize(fontSize);
-		dT->setOrigin(tickOrigin+ Point3D(-1,-1,-1));
-		dT->setAlignment(DRAWTEXT_ALIGN_RIGHT);
+		dT->setOrigin(tickOrigin+ Point3D(-2,-2,-2));
+		dT->setHorizAlignment(DRAWTEXT_ALIGN_RIGHT);
+		dT->setVertAlignment(DRAWTEXT_ALIGN_CENTREY);
 		dT->setUp(Point3D(0,0,1));	
-		dT->setTextDir(Point3D(-1,-1,0));
+		dT->setTextDir(Point3D(1,1,0));
 		d->drawables.push_back(dT);
 	}
 
@@ -371,7 +375,7 @@ void BoundingBoxFilter::drawDimension(const BoundCube &bTotal, DrawStreamData *d
 		dT->setColour(lineColour.r(),lineColour.g(),
 				lineColour.b(),lineColour.a());
 		dT->setOrigin(centrePt[ui]);	
-		dT->setAlignment(DRAWTEXT_ALIGN_CENTRE);
+		dT->setHorizAlignment(DRAWTEXT_ALIGN_CENTREX);
 		
 		switch(ui)
 		{
@@ -417,110 +421,109 @@ unsigned int BoundingBoxFilter::refresh(const std::vector<const FilterStreamData
 	size_t n=0;
 	Point3D p[4];
 	unsigned int ptCount=0;
-	for(unsigned int ui=0;ui<dataIn.size();ui++)
+	if(totalSize)
 	{
-		switch(dataIn[ui]->getStreamType())
+		for(unsigned int ui=0;ui<dataIn.size();ui++)
 		{
-			case STREAM_TYPE_IONS:
+			switch(dataIn[ui]->getStreamType())
 			{
-				bThis=bTotal;
-				//Grab the first four points to define a volume.
-				//then expand that volume using the boundcube functions.
-				const IonStreamData *d =(const IonStreamData *) dataIn[ui];
-				size_t dataPos=0;
-				unsigned int curProg=NUM_CALLBACK;
-				while(ptCount < 4 && dataPos < d->data.size())
+				case STREAM_TYPE_IONS:
 				{
-					for(unsigned int ui=0; ui<d->data.size();ui++)
+					bThis=bTotal;
+					//Grab the first four points to define a volume.
+					//then expand that volume using the boundcube functions.
+					const IonStreamData *d =(const IonStreamData *) dataIn[ui];
+					size_t dataPos=0;
+					unsigned int curProg=NUM_CALLBACK;
+					while(ptCount < 4 && dataPos < d->data.size())
 					{
-						p[ptCount]=d->data[ui].getPosRef();
-						ptCount++;
-						dataPos=ui;
-						if(ptCount >=4) 
-							break;
+						for(unsigned int ui=0; ui<d->data.size();ui++)
+						{
+							p[ptCount]=d->data[ui].getPosRef();
+							ptCount++;
+							dataPos=ui;
+							if(ptCount >=4) 
+								break;
+						}
 					}
-				}
 
-				//Ptcount will be 4 if we have >=4 points in dataset
-				if(ptCount < 4)
-					break;
-				bThis.setBounds(p,4);
-				//Expand the bounding volume
+					//Ptcount will be 4 if we have >=4 points in dataset
+					if(ptCount < 4)
+						break;
+					bThis.setBounds(p,4);
+					//Expand the bounding volume
 #ifdef _OPENMP
-				//Parallel version
-				unsigned int nT =omp_get_max_threads(); 
+					//Parallel version
+					unsigned int nT =omp_get_max_threads(); 
 
-				BoundCube *newBounds= new BoundCube[nT];
-				for(unsigned int ui=0;ui<nT;ui++)
-					newBounds[ui]=bThis;
+					BoundCube *newBounds= new BoundCube[nT];
+					for(unsigned int ui=0;ui<nT;ui++)
+						newBounds[ui]=bThis;
 
-				bool spin=false;
-				#pragma omp parallel for shared(spin)
-				for(unsigned int ui=dataPos;ui<d->data.size();ui++)
-				{
-					unsigned int thisT=omp_get_thread_num();
-					//OpenMP does not allow exiting. Use spin instead
+					bool spin=false;
+					#pragma omp parallel for shared(spin)
+					for(unsigned int ui=dataPos;ui<d->data.size();ui++)
+					{
+						unsigned int thisT=omp_get_thread_num();
+						//OpenMP does not allow exiting. Use spin instead
+						if(spin)
+							continue;
+
+						if(!curProg--)
+						{
+							#pragma omp critical
+							{
+							n+=NUM_CALLBACK;
+							progress.filterProgress= (unsigned int)((float)(n)/((float)totalSize)*100.0f);
+							}
+
+
+							if(thisT == 0)
+							{
+								if(*Filter::wantAbort)
+									spin=true;
+							}
+						}
+
+						newBounds[thisT].expand(d->data[ui].getPosRef());
+					}
 					if(spin)
-						continue;
-
-					if(!curProg--)
-					{
-						#pragma omp critical
-						{
-						n+=NUM_CALLBACK;
-						progress.filterProgress= (unsigned int)((float)(n)/((float)totalSize)*100.0f);
-						}
-
-
-						if(thisT == 0)
-						{
-							if(*Filter::wantAbort)
-								spin=true;
-						}
+					{			
+						delete[] newBounds;
+						return BOUNDINGBOX_ABORT_ERR;
 					}
 
-					newBounds[thisT].expand(d->data[ui].getPosRef());
-				}
-				if(spin)
-				{			
-					delete d;
+					for(unsigned int ui=0;ui<nT;ui++)
+						bThis.expand(newBounds[ui]);
+
 					delete[] newBounds;
-					return BOUNDINGBOX_ABORT_ERR;
-				}
-
-				for(unsigned int ui=0;ui<nT;ui++)
-					bThis.expand(newBounds[ui]);
-
-				delete[] newBounds;
 #else
-				//Single thread version
-				for(unsigned int ui=dataPos;ui<d->data.size();ui++)
-				{
-					bThis.expand(d->data[ui].getPosRef());
-					if(!curProg--)
+					//Single thread version
+					for(unsigned int ui=dataPos;ui<d->data.size();ui++)
 					{
-						n+=NUM_CALLBACK;
-						progress.filterProgress= (unsigned int)((float)(n)/((float)totalSize)*100.0f);
-						if(*Filter::wantAbort)
+						bThis.expand(d->data[ui].getPosRef());
+						if(!curProg--)
 						{
-							delete d;
-							return BOUNDINGBOX_ABORT_ERR;
+							n+=NUM_CALLBACK;
+							progress.filterProgress= (unsigned int)((float)(n)/((float)totalSize)*100.0f);
+							if(*Filter::wantAbort)
+								return BOUNDINGBOX_ABORT_ERR;
 						}
 					}
-				}
 #endif
-				bTotal.expand(bThis);
-				progress.filterProgress=100;
-				break;
+					bTotal.expand(bThis);
+					break;
+				}
+				default:
+					break;
 			}
-			default:
-				break;
-		}
 
-		//Copy the input data to the output	
-		getOut.push_back(dataIn[ui]);	
+			//Copy the input data to the output	
+			getOut.push_back(dataIn[ui]);	
+		}
 	}
 
+	progress.filterProgress=100;
 	//Append the bounding box if it is valid
 	if(bTotal.isValid())
 	{
@@ -559,34 +562,26 @@ unsigned int BoundingBoxFilter::refresh(const std::vector<const FilterStreamData
 
 void BoundingBoxFilter::getProperties(FilterPropGroup &propertyList) const
 {
-	FilterProperty p;
 	size_t curGroup=0;
 
-	string tmpStr;
-	stream_cast(tmpStr,isVisible);
-	p.name=TRANS("Visible");
-	p.data= tmpStr;
-	p.key=KEY_VISIBLE;
-	p.type=PROPERTY_TYPE_BOOL;
-	p.helpText=TRANS("If true, show box, otherwise hide box");
-	propertyList.addProperty(p,curGroup);
+	FilterProperty pVisible(KEY_VISIBLE, TRANS("Visible"),
+		 isVisible,TRANS("If true, show box, otherwise hide box"));
+	propertyList.addProperty(pVisible,curGroup);
 
 	if(isVisible)
 	{
 		vector<pair<unsigned int,string> > choices;
 		for(size_t ui=0;ui<BOUND_STYLE_ENUM_END; ui++)
 		{
+			std::string tmpStr;
 			tmpStr=TRANS(BOUND_STYLE[ui]);
 			choices.push_back(make_pair(ui,tmpStr));
 		}
+	
+		FilterProperty pStyle(KEY_STYLE, TRANS("Style"),
+			choices,boundStyle,TRANS("Box display mode"));
+		propertyList.addProperty(pStyle,curGroup);
 
-		tmpStr= choiceString(choices,boundStyle);
-		p.name=TRANS("Style");
-		p.data=tmpStr;
-		p.type=PROPERTY_TYPE_CHOICE;
-		p.helpText=TRANS("Box display mode");
-		p.key=KEY_STYLE;
-		propertyList.addProperty(p,curGroup);
 		propertyList.setGroupTitle(curGroup,TRANS("Display mode"));
 		curGroup++;
 
@@ -595,94 +590,44 @@ void BoundingBoxFilter::getProperties(FilterPropGroup &propertyList) const
 		{
 
 			//Properties are X Y and Z counts on ticks
-			stream_cast(tmpStr,fixedNumTicks);
-			p.name=TRANS("Fixed Tick Num");
-			p.data=tmpStr;
-			p.key=KEY_FIXEDOUT;
-			p.type=PROPERTY_TYPE_BOOL;
-			p.helpText=TRANS("If true, evenly use specified number of ticks. Otherwise, use distance to determine tick count");
-			propertyList.addProperty(p,curGroup);
+			FilterProperty pFixedTicks(KEY_FIXEDOUT, TRANS("Fixed Tick Num"),fixedNumTicks,
+				 TRANS("If true, evenly use specified number of ticks. Otherwise, use distance to determine tick count"));
+			propertyList.addProperty(pFixedTicks,curGroup);
 
 			if(fixedNumTicks)
 			{
-				//Properties are X Y and Z counts on ticks
-				stream_cast(tmpStr,numTicks[0]);
-				p.key=KEY_COUNT_X;
-				p.name=TRANS("Num X");
-				p.data=tmpStr;
-				p.type=PROPERTY_TYPE_INTEGER;
-				p.helpText=TRANS("Tick count in X direction");
-				propertyList.addProperty(p,curGroup);
+				FilterProperty pNumTicksX(KEY_COUNT_X, TRANS("Num X"),numTicks[0], TRANS("Tick count in X direction"));
+				propertyList.addProperty(pNumTicksX,curGroup);
 				
-				stream_cast(tmpStr,numTicks[1]);
-				p.key=KEY_COUNT_Y;
-				p.name=TRANS("Num Y");
-				p.data=tmpStr;
-				p.type=PROPERTY_TYPE_INTEGER;
-				p.helpText=TRANS("Tick count in Y direction");
-				propertyList.addProperty(p,curGroup);
+				FilterProperty pNumTicksY(KEY_COUNT_Y, TRANS("Num Y"),numTicks[1], TRANS("Tick count in Y direction"));
+				propertyList.addProperty(pNumTicksY,curGroup);
 				
-				stream_cast(tmpStr,numTicks[2]);
-				p.key=KEY_COUNT_Z;
-				p.name=TRANS("Num Z");
-				p.data=tmpStr;
-				p.type=PROPERTY_TYPE_INTEGER;
-				p.helpText=TRANS("Tick count in Z direction");
-				propertyList.addProperty(p,curGroup);
+				FilterProperty pNumTicksZ(KEY_COUNT_Z, TRANS("Num Z"),numTicks[2], TRANS("Tick count in Z direction"));
+				propertyList.addProperty(pNumTicksZ,curGroup);
 			}
 			else
 			{
-				stream_cast(tmpStr,tickSpacing[0]);
-				p.name=TRANS("Spacing X");
-				p.data= tmpStr;
-				p.key=KEY_SPACING_X;
-				p.type=PROPERTY_TYPE_REAL;
-				p.helpText=TRANS("Distance between ticks on X axis");
-				propertyList.addProperty(p,curGroup);
+				FilterProperty pSpacingX(KEY_SPACING_X, TRANS("Spacing X"),tickSpacing[0],TRANS("Distance between ticks on X axis"));
+				propertyList.addProperty(pSpacingX,curGroup);
+				
+				FilterProperty pSpacingY(KEY_SPACING_Y, TRANS("Spacing Y"),tickSpacing[1],TRANS("Distance between ticks on Y axis"));
+				propertyList.addProperty(pSpacingY,curGroup);
 
-				stream_cast(tmpStr,tickSpacing[1]);
-				p.name=TRANS("Spacing Y");
-				p.data= tmpStr;
-				p.key=KEY_SPACING_Y;
-				p.type=PROPERTY_TYPE_REAL;
-				p.helpText=TRANS("Distance between ticks on Y axis");
-				propertyList.addProperty(p,curGroup);
-
-				stream_cast(tmpStr,tickSpacing[2]);
-				p.name=TRANS("Spacing Z");
-				p.data= tmpStr;
-				p.key=KEY_SPACING_Z;
-				p.type=PROPERTY_TYPE_REAL;
-				p.helpText=TRANS("Distance between ticks on Z axis");
-				propertyList.addProperty(p,curGroup);
+				FilterProperty pSpacingZ(KEY_SPACING_Z, TRANS("Spacing Z"),tickSpacing[2],TRANS("Distance between ticks on Z axis"));
+				propertyList.addProperty(pSpacingZ,curGroup);
 			}
 		}
 
 		if(boundStyle!=BOUND_STYLE_BOX_ONLY)
 		{	
-			tmpStr=boolStrEnc(enableTicks[0]);	
-			p.name=TRANS("Ticks X");
-			p.data= tmpStr;
-			p.key=KEY_SHOW_TICKS_X;
-			p.type=PROPERTY_TYPE_BOOL;
-			p.helpText=TRANS("Display tick marks on X axis");
-			propertyList.addProperty(p,curGroup);
+			FilterProperty pTicksX(KEY_SHOW_TICKS_X, TRANS("Ticks X"),enableTicks[0],TRANS("Display tick marks on X axis"));
+			propertyList.addProperty(pTicksX,curGroup);
 
-			tmpStr=boolStrEnc(enableTicks[1]);	
-			p.name=TRANS("Ticks Y");
-			p.data= tmpStr;
-			p.key=KEY_SHOW_TICKS_Y;
-			p.type=PROPERTY_TYPE_BOOL;
-			p.helpText=TRANS("Display tick marks on Y axis");
-			propertyList.addProperty(p,curGroup);
+			FilterProperty pTicksY(KEY_SHOW_TICKS_Y, TRANS("Ticks Y"),enableTicks[1],TRANS("Display tick marks on Y axis"));
+			propertyList.addProperty(pTicksY,curGroup);
 			
-			tmpStr=boolStrEnc(enableTicks[2]);	
-			p.name=TRANS("Ticks Z");
-			p.data= tmpStr;
-			p.key=KEY_SHOW_TICKS_Z;
-			p.type=PROPERTY_TYPE_BOOL;
-			p.helpText=TRANS("Display tick marks on Z axis");
-			propertyList.addProperty(p,curGroup);
+			FilterProperty pTicksZ(KEY_SHOW_TICKS_Z, TRANS("Ticks Z"),enableTicks[2],TRANS("Display tick marks on Z axis"));
+			propertyList.addProperty(pTicksZ,curGroup);
 
 			propertyList.setGroupTitle(curGroup,TRANS("Tick marks"));
 
@@ -690,42 +635,25 @@ void BoundingBoxFilter::getProperties(FilterPropGroup &propertyList) const
 
 
 		//Colour
-
-		p.name=TRANS("Box Colour");
-		p.data= lineColour.toColourRGBA().rgbString();
-		p.key=KEY_LINECOLOUR;
-		p.type=PROPERTY_TYPE_COLOUR;
-		p.helpText=TRANS("Colour of the bounding box");
-		propertyList.addProperty(p,curGroup);
+		FilterProperty pColour(KEY_LINECOLOUR, TRANS("Box Colour"),
+				lineColour,TRANS("Colour of the bounding box"));
+		propertyList.addProperty(pColour,curGroup);
 
 		//Line thickness
-		stream_cast(tmpStr,lineWidth);
-		p.name=TRANS("Line thickness");
-		p.data= tmpStr;
-		p.key=KEY_LINEWIDTH;
-		p.type=PROPERTY_TYPE_REAL;
-		p.helpText=TRANS("Thickness of the lines used to draw the box");
-		propertyList.addProperty(p,curGroup);
-
+		FilterProperty pLinewidth(KEY_LINEWIDTH, TRANS("Line thickness"),
+			lineWidth,TRANS("Thickness of the lines used to draw the box"));
+		propertyList.addProperty(pLinewidth,curGroup);
+		
 		//Font size	
 		if(boundStyle != BOUND_STYLE_BOX_ONLY)
 		{
-			stream_cast(tmpStr,fontSize);
-			p.key=KEY_FONTSIZE;
-			p.name=TRANS("Font Size");
-			p.data= tmpStr;
-			p.type=PROPERTY_TYPE_INTEGER;
-			p.helpText=TRANS("Relative size for text");
-			propertyList.addProperty(p,curGroup);
+			FilterProperty pFontSize(KEY_FONTSIZE, TRANS("Font Size"),
+				fontSize,TRANS("Relative size for text"));
+			propertyList.addProperty(pFontSize,curGroup);
 
-			
-			p.key=KEY_ABSCOORDS;
-			p.name=TRANS("Abs. Coords");
-			p.data = boolStrEnc(absoluteCoords);
-			p.type=PROPERTY_TYPE_BOOL;
-			p.helpText=TRANS("Show labels using aboslute coo-ordinates");
-	
-			propertyList.addProperty(p,curGroup);
+			FilterProperty pAbsCoords(KEY_ABSCOORDS, TRANS("Abs. Coords"),
+				absoluteCoords,TRANS("Show labels using absolute co-ordinates"));
+			propertyList.addProperty(pAbsCoords,curGroup);
 		}
 	}
 	propertyList.setGroupTitle(curGroup,TRANS("Appearance"));
@@ -800,8 +728,9 @@ bool BoundingBoxFilter::setProperty(  unsigned int key,
 				return false;
 
 
-			if(lineColour.toColourRGBA() != newLineColour) 
-				needUpdate=true;
+			if(lineColour.toColourRGBA() == newLineColour) 
+				return false;
+			
 			lineColour=newLineColour.toRGBAf();
 
 			needUpdate=true;
