@@ -1,6 +1,6 @@
 /*
  *	clusterAnalysis.cpp - Perform clustering data analysis on valued point clouds
- *	Copyright (C) 2015, D Haley 
+ *	Copyright (C) 2018, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -86,12 +86,6 @@ const char CHEM_DIST_DATALABEL[] =NTRANS("Chemistry Distribution");
 
 using std::vector;
 
-//Optimisation tuning value;
-// number of points to expect in a KD query sphere before the bulk query pays off
-//  in terms of algorithm speed
-const float SPHERE_PRESEARCH_CUTOFF = 75;
-
-
 //In link clustering, when we preform size cropping, do we awant to count bulk ions in our analysis?
 const bool WANT_COUNT_BULK_FORCROP=false;
 
@@ -500,7 +494,7 @@ unsigned int ClusterAnalysisFilter::refresh(const std::vector<const FilterStream
 	
 	if(!haveRangeParent)
 	{
-		consoleOutput.push_back(string(TRANS("No range data. Can't cluster.")));
+		appendConsoleMessage(string(TRANS("No range data. Can't cluster.")));
 		return 0;
 	}
 
@@ -510,7 +504,7 @@ unsigned int ClusterAnalysisFilter::refresh(const std::vector<const FilterStream
 	//Check that the user has enabled something as core	
 	if(!haveACore)
 	{
-		consoleOutput.push_back(
+		appendConsoleMessage(
 			string(TRANS("No ranges selected for cluster \"core\". Cannot continue with clustering.")));
 		return NOCORE_ERR;
 	}
@@ -519,7 +513,7 @@ unsigned int ClusterAnalysisFilter::refresh(const std::vector<const FilterStream
 	//Check that the user has enabled something as matrix/bulk. 
 	if(!haveABulk && enableBulkLink)
 	{
-		consoleOutput.push_back(
+		appendConsoleMessage(
 			string(TRANS("No ranges selected for cluster \"bulk\". Cannot continue with clustering.")));
 		return NOBULK_ERR;
 	}
@@ -685,7 +679,7 @@ ASSERT(!(haveBulk && !haveCore));
 		Plot2DStreamData *p = new Plot2DStreamData;
 		p->parent=this;
 
-		p->plotType=PLOT_2D_SCATTER;
+		p->plotStyle=PLOT_2D_SCATTER;
 		p->dataLabel=TRANS("Morphology Plot");
 		p->xLabel=TRANS("\\lambda_1:\\lambda_2 ratio");
 		p->yLabel=TRANS("\\lambda_2:\\lambda_3 ratio");
@@ -733,7 +727,7 @@ ASSERT(!(haveBulk && !haveCore));
 		}
 		else
 		{
-			consoleOutput.push_back(TRANS("No clusters had sufficient dimensionality to compute singular values"));
+			appendConsoleMessage(TRANS("No clusters had sufficient dimensionality to compute singular values"));
 			delete p;
 		}
 	
@@ -794,7 +788,7 @@ ASSERT(!(haveBulk && !haveCore));
 	sDebugConsole=TRANS("Found :");
 	sDebugConsole+=stmp;
 	sDebugConsole+= TRANS(" clusters");
-	consoleOutput.push_back(sDebugConsole);
+	appendConsoleMessage(sDebugConsole);
 
 	size_t totalSize=0;
 
@@ -885,9 +879,9 @@ ASSERT(!(haveBulk && !haveCore));
 
 
 			if(haveBulk)
-				consoleOutput.push_back(TRANS("Compositions (fractional, core+bulk)"));
+				appendConsoleMessage(TRANS("Compositions (fractional, core+bulk)"));
 			else if(haveCore)
-				consoleOutput.push_back(TRANS("Compositions (fractional, core only)"));
+				appendConsoleMessage(TRANS("Compositions (fractional, core only)"));
 
 			std::string compString,tmp;
 			for(unsigned int ui=0;ui<compTable.size();ui++)
@@ -896,7 +890,7 @@ ASSERT(!(haveBulk && !haveCore));
 				compString+="\t\t";
 				stream_cast(tmp,compTable[ui].second);
 				compString+=tmp;
-				consoleOutput.push_back(compString);
+				appendConsoleMessage(compString);
 			}
 
 		}
@@ -905,7 +899,7 @@ ASSERT(!(haveBulk && !haveCore));
 			vector<pair<string,size_t> > freqTable;
 			makeFrequencyTable(i,r->rangeFile,freqTable);
 
-			consoleOutput.push_back(TRANS("Frequencies (core+bulk)"));
+			appendConsoleMessage(TRANS("Frequencies (core+bulk)"));
 
 			std::string freqString,tmp;
 			for(unsigned int ui=0;ui<freqTable.size();ui++)
@@ -914,7 +908,7 @@ ASSERT(!(haveBulk && !haveCore));
 				freqString+="\t\t";
 				stream_cast(tmp,freqTable[ui].second);
 				freqString+=tmp;
-				consoleOutput.push_back(freqString);
+				appendConsoleMessage(freqString);
 			}
 		}
 
@@ -939,88 +933,55 @@ void ClusterAnalysisFilter::getProperties(FilterPropGroup &propertyList) const
 		tmpStr=TRANS("Core Link + Erode");
 		choices.push_back(make_pair((unsigned int)CLUSTER_LINK_ERODE,tmpStr));
 		
-		tmpStr= choiceString(choices,algorithm);
-		p.name=TRANS("Algorithm");
-		p.data=tmpStr;
-		choices.clear();
-		p.type=PROPERTY_TYPE_CHOICE;
-		p.helpText=TRANS("Cluster algorithm mode");
-		p.key=KEY_CLUSTERANALYSIS_ALGORITHM;
-		propertyList.addProperty(p,curGroup);
+		FilterProperty pAlgorithm(KEY_CLUSTERANALYSIS_ALGORITHM, TRANS("Algorithm"),
+			choices,algorithm,TRANS("Cluster algorithm mode"));
+		propertyList.addProperty(pAlgorithm,curGroup);
 		
 		propertyList.setGroupTitle(curGroup,TRANS("Algorithm"));
 		curGroup++;
 	}
 
 	if(algorithm == CLUSTER_LINK_ERODE)
-	
 	{
-		p.name=TRANS("Core Classify");
-		p.data=boolStrEnc(enableCoreClassify);
-		p.type=PROPERTY_TYPE_BOOL;
-		p.helpText=TRANS("Enable core-classifcation pre-step in clustering (Stephenson et al, 2007)");
-		p.key=KEY_CORECLASSIFY_ENABLE;
-		propertyList.addProperty(p,curGroup);
+	
+		FilterProperty pCoreClassify(KEY_CORECLASSIFY_ENABLE, TRANS("Core Classify"),
+			enableCoreClassify,TRANS("Enable core-classifcation pre-step in clustering (Stephenson et al, 2007)"));
+		propertyList.addProperty(pCoreClassify,curGroup);
+
 		if(enableCoreClassify)
 		{
-			stream_cast(tmpStr,coreDist);
-			p.name=TRANS("Core Classify Dist");
-			p.data=tmpStr;
-			p.type=PROPERTY_TYPE_REAL;
-			p.helpText=TRANS("Restrict only atoms by distance to be cluster sources");
-			p.key=KEY_CORECLASSIFYDIST;
-			propertyList.addProperty(p,curGroup);
-		
-			stream_cast(tmpStr,coreKNN);
-			p.name=TRANS("Classify Knn Max");
-			p.data=tmpStr;
-			p.type=PROPERTY_TYPE_INTEGER;
-			p.helpText=TRANS("Require that the kth NN (this number) is within the classify distance, to be a cluster source");
-			p.key=KEY_CORECLASSIFYKNN;
-			propertyList.addProperty(p,curGroup);
+			FilterProperty pCoreClassifyDist(KEY_CORECLASSIFYDIST, TRANS("Core Classify Dist"),
+				coreDist,TRANS("Restrict only atoms joind by this distance to be cluster sources"));
+			propertyList.addProperty(pCoreClassifyDist,curGroup);
+			
+			FilterProperty pCoreKNN(KEY_CORECLASSIFYKNN, TRANS("Classify Knn Max"),
+				coreKNN,TRANS("Require that the kth NN (this number) is within the classify distance, to be a cluster source"));
+			propertyList.addProperty(pCoreKNN,curGroup);
 		}
 		
-		stream_cast(tmpStr,linkDist);
-		p.name=TRANS("Core Link Dist");
-		p.data=tmpStr;
-		p.type=PROPERTY_TYPE_REAL;
-		p.helpText=TRANS("Distance between clusters to allow linking");
-		p.key=KEY_LINKDIST;
-		propertyList.addProperty(p,curGroup);
-
-
-		p.name=TRANS("Bulk Link");
-		p.data=boolStrEnc(enableBulkLink);
-		p.type=PROPERTY_TYPE_BOOL;
-		p.helpText=TRANS("Enable  linking of non-cluster species - eg for composition analysis ");
-		p.key=KEY_BULKLINK_ENABLE;
-		propertyList.addProperty(p,curGroup);
+		FilterProperty pLinkDist(KEY_LINKDIST, TRANS("Core Link Dist"),
+			linkDist,TRANS("Distance between clusters to allow linking"));
+		propertyList.addProperty(pLinkDist,curGroup);
 		
+		FilterProperty pEnableBulk(KEY_BULKLINK_ENABLE, TRANS("Bulk Link"),
+			enableBulkLink,TRANS("Enable linking of non-cluster species - eg for composition analysis"));
+		propertyList.addProperty(pEnableBulk,curGroup);
+
 		if(enableBulkLink)
 		{
-			stream_cast(tmpStr,bulkLink);
-			p.name=TRANS("Bulk Link (Envelope) Dist");
-			p.data=tmpStr;
-			p.type=PROPERTY_TYPE_REAL;
-			p.helpText=TRANS("Distance from core points that form cluster that is used to grab surrounding bulk points");
-			p.key=KEY_BULKLINK;
-			propertyList.addProperty(p,curGroup);
-		
-			p.name=TRANS("Erosion");
-			p.data=boolStrEnc(enableErosion);
-			p.type=PROPERTY_TYPE_BOOL;
-			p.helpText=TRANS("Enable  linking of non-cluster species - eg for composition analysis ");
-			p.key=KEY_ERODE_ENABLE;
-			propertyList.addProperty(p,curGroup);
+			FilterProperty pBulkLink(KEY_BULKLINK, TRANS("Bulk Link (Envelope) dist"),
+				bulkLink,TRANS("Distance from core points, that form a cluster, used to grab surrounding bulk points"));
+			propertyList.addProperty(pBulkLink,curGroup);
+			
+			FilterProperty pErosion(KEY_ERODE_ENABLE, TRANS("Erosion"),
+				enableErosion,TRANS("Enable removal of edge bulk atoms, to remove \"cloud\" of atoms beyond the nominal cluster limits (hole-closing halo)"));
+			propertyList.addProperty(pErosion,curGroup);
+			
 			if(enableErosion)
 			{
-				stream_cast(tmpStr,dErosion);
-				p.name=TRANS("Erode Dist");
-				p.data=tmpStr;
-				p.type=PROPERTY_TYPE_REAL;
-				p.helpText=TRANS("Distance from unclustered material in which bulk points are eroded from cluster");
-				p.key=KEY_ERODEDIST;
-				propertyList.addProperty(p,curGroup);
+				FilterProperty pErosionDist(KEY_ERODEDIST, TRANS("Erode Dist"),
+					dErosion,TRANS("Distance from unclustered material in which bulk points are eroded from cluster"));
+				propertyList.addProperty(pErosionDist,curGroup);
 			}
 		}
 	}
@@ -1029,88 +990,52 @@ void ClusterAnalysisFilter::getProperties(FilterPropGroup &propertyList) const
 
 	curGroup++;
 
-	tmpStr=boolStrEnc(wantCropSize);
-	p.name=TRANS("Size Cropping");
-	p.data=tmpStr;
-	p.type=PROPERTY_TYPE_BOOL;
-	p.helpText=TRANS("Remove clusters based upon size distribution");
-	p.key=KEY_CROP_SIZE;
-	propertyList.addProperty(p,curGroup);
-
+	FilterProperty pErosionDist(KEY_CROP_SIZE, TRANS("Size Cropping"),
+		wantCropSize,TRANS("Remove clusters based upon size distribution"));
+	propertyList.addProperty(pErosionDist,curGroup);
+	
 	if(wantCropSize)
 	{
-		stream_cast(tmpStr,nMin);
-		p.name=TRANS("Min Size");
-		p.data=tmpStr;
-		p.type=PROPERTY_TYPE_INTEGER;
-		p.helpText=TRANS("Remove clusters below this size");
-		p.key=KEY_CROP_NMIN;
-		propertyList.addProperty(p,curGroup);
+		FilterProperty pMinClusterSize(KEY_CROP_NMIN, TRANS("Min Size"),
+			(unsigned int)nMin,TRANS("Remove clusters below this size"));
+		propertyList.addProperty(pMinClusterSize,curGroup);
 		
-		stream_cast(tmpStr,nMax);
-		p.name=TRANS("Max Size");
-		p.data=tmpStr;
-		p.type=PROPERTY_TYPE_INTEGER;
-		p.helpText=TRANS("Remove clusters above this size");
-		p.key=KEY_CROP_NMAX;
-		propertyList.addProperty(p,curGroup);
+		FilterProperty pMaxClusterSize(KEY_CROP_NMAX, TRANS("Max Size"),
+			(unsigned int) nMax,TRANS("Remove clusters above this size"));
+		propertyList.addProperty(pMaxClusterSize,curGroup);
 	}	
 	
-	tmpStr=boolStrEnc(wantClusterSizeDist);
-	p.name=TRANS("Size Distribution");
-	p.data=tmpStr;
-	p.type=PROPERTY_TYPE_BOOL;
-	p.helpText=TRANS("Show number of clusters as a function of cluster size");
-	p.key=KEY_WANT_CLUSTERSIZEDIST;
-		propertyList.addProperty(p,curGroup);
+	FilterProperty pSizeDistribution(KEY_WANT_CLUSTERSIZEDIST, TRANS("Size Distribution"),
+		wantClusterSizeDist,TRANS("Show number of clusters as a function of cluster size"));
+	propertyList.addProperty(pSizeDistribution,curGroup);
+	
 	if(wantClusterSizeDist)
 	{	
-		tmpStr=boolStrEnc(logClusterSize);
-		p.name=TRANS("Log Scale");
-		p.data=tmpStr;
-		p.type=PROPERTY_TYPE_BOOL;
-		p.helpText=TRANS("Use logarithmic scale for size distribution");
-		p.key=KEY_WANT_LOGSIZEDIST;
-		propertyList.addProperty(p,curGroup);
+		FilterProperty pSizeDistLog(KEY_WANT_LOGSIZEDIST, TRANS("Dist. Log Scale"),
+			logClusterSize,TRANS("Use logarithmic scale for size distribution"));
+		propertyList.addProperty(pSizeDistLog,curGroup);
 	}
 
 
-	tmpStr=boolStrEnc(wantClusterMorphology);
-	p.name=TRANS("Morphology Dist.");
-	p.data=tmpStr;
-	p.type=PROPERTY_TYPE_BOOL;
-	p.helpText=TRANS("Create a plot showing cluster aspect ratio");
-	p.key=KEY_WANT_CLUSTERMORPHOLOGY;
-	propertyList.addProperty(p,curGroup);
-
-
-	tmpStr=boolStrEnc(wantClusterID);
-	p.name=TRANS("Cluster Id");
-	p.data=tmpStr;
-	p.type=PROPERTY_TYPE_BOOL;
-	p.helpText=TRANS("Assign cluster output a unique per-cluster value (id).");
-	p.key=KEY_WANT_CLUSTERID;
-	propertyList.addProperty(p,curGroup);
+	FilterProperty pWantClusterMorph(KEY_WANT_CLUSTERMORPHOLOGY, TRANS("Morphology Dist."),
+		wantClusterMorphology,TRANS("Create a plot showing cluster aspect ratio"));
+	propertyList.addProperty(pWantClusterMorph,curGroup);
+	
+	FilterProperty pWantClusterId(KEY_WANT_CLUSTERID, TRANS("Cluster Id"),
+		wantClusterID,TRANS("Assign cluster output a unique per-cluster value (id)."));
+	propertyList.addProperty(pWantClusterId,curGroup);
 
 	if(!wantClusterID)
 	{
-		tmpStr=boolStrEnc(wantClusterComposition);
-		p.name=TRANS("Chemistry Dist.");
-		p.data=tmpStr;
-		p.type=PROPERTY_TYPE_BOOL;
-		p.helpText=TRANS("Create a plot showing chemistry for each cluster size");
-		p.key=KEY_WANT_COMPOSITIONDIST;
-		propertyList.addProperty(p,curGroup);
+		FilterProperty pWantComposition(KEY_WANT_COMPOSITIONDIST, TRANS("Chemistry Dist."),
+			wantClusterComposition,TRANS("Create a plot showing chemistry for each cluster size"));
+		propertyList.addProperty(pWantComposition,curGroup);
 		
 		if(wantClusterComposition)
 		{	
-			tmpStr=boolStrEnc(normaliseComposition);
-			p.name=TRANS("Normalise");
-			p.data=tmpStr;
-			p.type=PROPERTY_TYPE_BOOL;
-			p.helpText=TRANS("Convert cluster counts to composition");
-			p.key=KEY_NORMALISE_COMPOSITION;
-			propertyList.addProperty(p,curGroup);
+			FilterProperty pNormComp(KEY_NORMALISE_COMPOSITION, TRANS("Normalise Comp."),
+				normaliseComposition,TRANS("Convert cluster counts to composition"));
+			propertyList.addProperty(pNormComp,curGroup);
 		}
 	}
 
@@ -1144,14 +1069,10 @@ void ClusterAnalysisFilter::getProperties(FilterPropGroup &propertyList) const
 		if(enableBulkLink)
 		{
 			
-			p.name=TRANS("Enable/Disable All");
-			bool allEnabled=false;
-			allEnabled=(std::find(ionBulkEnabled.begin(),ionBulkEnabled.end(),false) == ionBulkEnabled.end());
-			p.data=boolStrEnc(allEnabled);
-			p.type=PROPERTY_TYPE_BOOL;
-			p.helpText=TRANS("Enable/disable all ions");
-			p.key=KEY_BULK_ALL;
-			propertyList.addProperty(p,curGroup);
+			bool allEnabled=(std::find(ionBulkEnabled.begin(),ionBulkEnabled.end(),false) == ionBulkEnabled.end());
+			FilterProperty pBulkAll(KEY_BULK_ALL,TRANS("Enable/Disable All"),
+				allEnabled,TRANS("Enable/disable all ions for bulk"));
+			propertyList.addProperty(pBulkAll,curGroup);
 
 			for(size_t ui=0;ui<ionNames.size();ui++)
 			{
@@ -1935,16 +1856,16 @@ unsigned int ClusterAnalysisFilter::refreshLinkClustering(const std::vector<cons
 
 		if(bulkLink > linkDist/2.0)
 		{
-			consoleOutput.push_back("");
-			consoleOutput.push_back(TRANS(" --------------------------- Parameter selection notice ------------- ")  );
-			consoleOutput.push_back(TRANS("You have specified a bulk distance larger than half your link distance.")  );
-			consoleOutput.push_back(TRANS("You can do this; thats OK, but the output is no longer independent of the computational process;")  );
-			consoleOutput.push_back(TRANS("This will be a problem in the case where two or more clusters can equally lay claim to a \"bulk\" ion. ")  );
-			consoleOutput.push_back(TRANS(" If your inter-cluster distance is sufficiently large (larger than your bulk linking distance), then you can get away with this.")  );
-			consoleOutput.push_back(TRANS(" In theory it is possible to \"join\" the clusters, but this has not been implemented for speed reasons."));
-			consoleOutput.push_back(TRANS("If you want this, please contact the author, or just use the source to add this in yourself.")  );
-			consoleOutput.push_back(TRANS("---------------------------------------------------------------------- ")  );
-			consoleOutput.push_back("");
+			appendConsoleMessage("");
+			appendConsoleMessage(TRANS(" --------------------------- Parameter selection notice ------------- ")  );
+			appendConsoleMessage(TRANS("You have specified a bulk distance larger than half your link distance.")  );
+			appendConsoleMessage(TRANS("You can do this; thats OK, but the output is no longer independent of the computational process;")  );
+			appendConsoleMessage(TRANS("This will be a problem in the case where two or more clusters can equally lay claim to a \"bulk\" ion. ")  );
+			appendConsoleMessage(TRANS(" If your inter-cluster distance is sufficiently large (larger than your bulk linking distance), then you can get away with this.")  );
+			appendConsoleMessage(TRANS(" In theory it is possible to \"join\" the clusters, but this has not been implemented for speed reasons."));
+			appendConsoleMessage(TRANS("If you want this, please contact the author, or just use the source to add this in yourself.")  );
+			appendConsoleMessage(TRANS("---------------------------------------------------------------------- ")  );
+			appendConsoleMessage("");
 		}	
 
 	}
@@ -2009,7 +1930,6 @@ unsigned int ClusterAnalysisFilter::refreshLinkClustering(const std::vector<cons
 	//When this queue is exhausted, move to the next cluster
 	for(size_t ui=0;ui<coreTree.size();ui++)
 	{
-		size_t curPt;
 		//Indicies of each cluster
 		vector<size_t> soluteCluster,dummy;
 		//Queue for atoms in this cluster waiting to be checked
@@ -2031,6 +1951,7 @@ unsigned int ClusterAnalysisFilter::refreshLinkClustering(const std::vector<cons
 		//Keep looping throughe queue until it is exhausted
 		do
 		{
+			size_t curPt;
 			//search atom's position
 			curPt=thisClusterQueue.front();
 
@@ -2678,6 +2599,17 @@ PlotStreamData* ClusterAnalysisFilter::clusterSizeDistribution(const vector<vect
 
 	dist->dataLabel=SIZE_DIST_DATALABEL;
 	dist->logarithmic=logClusterSize;
+	//Set desired Y bound (lower)
+	if(dist->logarithmic)
+		dist->setPlotBound(1,0,0.1);
+	else
+		dist->setPlotBound(1,0,0);
+
+	//Set X bound (lower)
+	if(nMin)
+		dist->setPlotBound(0,0,nMin-1);
+	else
+		dist->setPlotBound(0,0,0);
 
 	dist->plotStyle=PLOT_LINE_STEM;
 	dist->plotMode=PLOT_MODE_1D;
@@ -2904,11 +2836,24 @@ void ClusterAnalysisFilter::genCompositionVersusSize(const vector<vector<IonHit>
 				size_t sum=0;
 				for(size_t uk=0; uk<it->second.size();uk++)
 					sum+=it->second[uk];
-				p->xyData[offset].second /=(float)sum;
+				if(sum)
+					p->xyData[offset].second /=(float)sum;
+				else
+					p->xyData[offset].second=0;
 			}
 			offset++;
 		}
 
+		//Set the X-bounds we would like to use
+		if(nMin)
+			p->setPlotBound(0,0,nMin-1);
+		else
+			p->setPlotBound(0,0,0);
+
+		//Set Y bound we would like to use [0,1]
+		p->setPlotBound(1,0,0);
+		p->setPlotBound(1,1,1);
+
 		plots.push_back(p);
 	}
 
