@@ -1,6 +1,6 @@
 /*
  *	dataLoad.cpp - filter to  load datasets from various source
- *	Copyright (C) 2015, D Haley 
+ *	Copyright (C) 2018, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -47,6 +47,8 @@ enum
 	FILEDATA_TYPE_POS,
 	FILEDATA_TYPE_TEXT,
 	FILEDATA_TYPE_ATO,
+	FILEDATA_TYPE_TAPSIM,
+	FILEDATA_TYPE_3DAP_OPS,
 	FILEDATA_TYPE_ENUM_END, // Not a data type, just end of enum
 };
 
@@ -66,6 +68,8 @@ const char *ENDIAN_MODE_STR[] = { NTRANS("Auto"),
 const char *AVAILABLE_FILEDATA_TYPES[] = { 	NTRANS("POS Data"),
 					NTRANS("Text Data"),
 					NTRANS("ATO Data"),
+					NTRANS("Tapsim Data"),
+					NTRANS("3Dap/Posap OPS Data"),
 					};
 const char *DEFAULT_LABEL="Mass-to-Charge (Da/e)";
 
@@ -86,6 +90,13 @@ DataLoadFilter::DataLoadFilter() : fileType(FILEDATA_TYPE_POS), doSample(true),
 		index[i] = i;
 	}
 
+	//Use a 50nm radius cylinder by default
+	reconParams.radiusCylinder=50;
+
+	//Atomic volume of Al atom, in nm^3, including packing factor
+	reconParams.ionVolume= 0.0165418128;
+	reconParams.detectionEfficiency= 0.4;
+	 
 }
 
 Filter *DataLoadFilter::cloneUncached() const
@@ -113,6 +124,7 @@ Filter *DataLoadFilter::cloneUncached() const
 	p->cacheOK=false;
 	p->enabled=enabled;
 	p->userString=userString;
+	p->reconParams=reconParams;
 
 	p->wantMonitor=wantMonitor;
 	p->numColumns=numColumns;
@@ -135,6 +147,9 @@ void DataLoadFilter::setFileMode(unsigned int fileMode)
 		case DATALOAD_LAWATAP_ATO_FILE:
 			fileType=FILEDATA_TYPE_ATO;
 			break;
+		case DATALOAD_THREEDAP_OPS_FILE:
+			fileType=FILEDATA_TYPE_3DAP_OPS;
+			break;
 		default:
 			ASSERT(false);
 	}
@@ -287,7 +302,7 @@ unsigned int DataLoadFilter::refresh(const std::vector<const FilterStreamData *>
 				if((uiErr = LimitLoadPosFile(numColumns, INDEX_LENGTH, index, ionData->data, ionFilename.c_str(),
 									maxIons,progress.filterProgress,(*Filter::wantAbort),strongRandom)))
 				{
-					consoleOutput.push_back(string(TRANS("Error loading file: ")) + ionFilename);
+					appendConsoleMessage(string(TRANS("Error loading file: ")) + ionFilename);
 					delete ionData;
 					errStr=TRANS(POS_ERR_STRINGS[uiErr]);
 					return uiErr;
@@ -300,7 +315,7 @@ unsigned int DataLoadFilter::refresh(const std::vector<const FilterStreamData *>
 				if((uiErr = GenericLoadFloatFile(numColumns, INDEX_LENGTH, index, ionData->data, ionFilename.c_str(),
 									progress.filterProgress,(*Filter::wantAbort))))
 				{
-					consoleOutput.push_back(string(TRANS("Error loading file: ")) + ionFilename);
+					appendConsoleMessage(string(TRANS("Error loading file: ")) + ionFilename);
 					delete ionData;
 					errStr=TRANS(POS_ERR_STRINGS[uiErr]);
 					return uiErr;
@@ -317,7 +332,7 @@ unsigned int DataLoadFilter::refresh(const std::vector<const FilterStreamData *>
 				string strNumLoaded,strNumAvailable;
 				stream_cast(strNumLoaded,ionData->data.size());
 				stream_cast(strNumAvailable,numAvailable);
-				consoleOutput.push_back(string(TRANS("Sampling is active, loaded ")) + strNumLoaded + 
+				appendConsoleMessage(string(TRANS("Sampling is active, loaded ")) + strNumLoaded + 
 					string( TRANS(" of " ) ) + strNumAvailable + string(TRANS(" available.")));
 
 			}
@@ -326,7 +341,7 @@ unsigned int DataLoadFilter::refresh(const std::vector<const FilterStreamData *>
 				string strNumAvailable;
 				stream_cast(strNumAvailable,numAvailable);
 
-				consoleOutput.push_back(string(TRANS("Loaded entire dataset, " )) + strNumAvailable + string(TRANS(" points.")));
+				appendConsoleMessage(string(TRANS("Loaded entire dataset, " )) + strNumAvailable + string(TRANS(" points.")));
 			}
 			//--
 			break;
@@ -344,7 +359,7 @@ unsigned int DataLoadFilter::refresh(const std::vector<const FilterStreamData *>
 						TEXT_DELIMINATORS,maxIons,progress.filterProgress,(*Filter::wantAbort),strongRandom)))
 
 				{
-					consoleOutput.push_back(string(TRANS("Error loading file: ")) + ionFilename);
+					appendConsoleMessage(string(TRANS("Error loading file: ")) + ionFilename);
 					delete ionData;
 					errStr=TEXT_LOAD_ERR_STRINGS[uiErr];
 					return uiErr;
@@ -355,7 +370,7 @@ unsigned int DataLoadFilter::refresh(const std::vector<const FilterStreamData *>
 				//Load the entire text data using 
 				if((uiErr=loadTextData(ionFilename.c_str(),outDat,headerData,TEXT_DELIMINATORS)))
 				{
-					consoleOutput.push_back(string(TRANS("Error loading file: ")) + ionFilename);
+					appendConsoleMessage(string(TRANS("Error loading file: ")) + ionFilename);
 					delete ionData;
 					errStr=TEXT_LOAD_ERR_STRINGS[uiErr];
 					return uiErr;
@@ -369,7 +384,7 @@ unsigned int DataLoadFilter::refresh(const std::vector<const FilterStreamData *>
 				std::string sizeStr;
 				stream_cast(sizeStr,outDat.size());
 
-				consoleOutput.push_back(
+				appendConsoleMessage(
 					string(TRANS("Data file contained incorrect number of columns -- should be 3 or 4, was ")) + sizeStr );
 				delete ionData;
 
@@ -415,7 +430,7 @@ unsigned int DataLoadFilter::refresh(const std::vector<const FilterStreamData *>
 			if((uiErr = LoadATOFile(ionFilename.c_str(), ionData->data,
 						progress.filterProgress,(*Filter::wantAbort))))
 			{
-				consoleOutput.push_back(string(TRANS("Error loading file: ")) + ionFilename);
+				appendConsoleMessage(string(TRANS("Error loading file: ")) + ionFilename);
 				delete ionData;
 				errStr=TRANS(LAWATAP_ATO_ERR_STRINGS[uiErr]);
 				return uiErr;
@@ -424,11 +439,50 @@ unsigned int DataLoadFilter::refresh(const std::vector<const FilterStreamData *>
 		
 			std::string tmpSize;
 			stream_cast(tmpSize,ionData->data.size());
-			consoleOutput.push_back(string(TRANS("Loaded dataset, " )) + tmpSize
+			appendConsoleMessage(string(TRANS("Loaded dataset, " )) + tmpSize
 								 + string(TRANS(" points.")));
 
 			break;
 		}
+		case FILEDATA_TYPE_TAPSIM:
+		{
+			//TODO: Load tapsim file with sampling
+			//Load the file
+			if((uiErr = loadTapsimBinFile(ionFilename.c_str(), 
+						progress.filterProgress,(*Filter::wantAbort),ionData->data)))
+
+			{
+				appendConsoleMessage(string(TRANS("Error loading file: ")) + ionFilename);
+				delete ionData;
+				errStr=TRANS(TAPSIM_ERR_STRINGS[uiErr]);
+				return uiErr;
+			}
+				
+		
+			std::string tmpSize;
+			stream_cast(tmpSize,ionData->data.size());
+			appendConsoleMessage(string(TRANS("Loaded dataset, " )) + tmpSize
+								 + string(TRANS(" points.")));
+
+			break;
+		}
+		case FILEDATA_TYPE_3DAP_OPS:
+		{
+			if( (uiErr = reconstruct3DapData(ionFilename.c_str(), reconParams, progress.filterProgress,(*Filter::wantAbort), ionData->data)) )
+			{
+				appendConsoleMessage(string(TRANS("Error loading file: ")) + ionFilename);
+				delete ionData;
+				errStr=TRANS(OPS_ENUM_ERRSTRINGS[uiErr]);
+				return uiErr;
+
+			}
+			
+			std::string tmpSize;
+			stream_cast(tmpSize,ionData->data.size());
+			appendConsoleMessage(string(TRANS("Loaded dataset, " )) + tmpSize
+								 + string(TRANS(" points.")));
+			break;
+		}
 		default:
 			ASSERT(false);
 	}
@@ -456,7 +510,7 @@ unsigned int DataLoadFilter::refresh(const std::vector<const FilterStreamData *>
 
 	if(dataCube.isNumericallyBig())
 	{
-		consoleOutput.push_back(
+		appendConsoleMessage(
 			TRANS("Warning:One or more bounds of the loaded data approaches "
 			       "the limits of numerical stability for the internal data type"
 			       "(magnitude too large). Consider rescaling data before loading"));
@@ -479,28 +533,21 @@ void DataLoadFilter::getProperties(FilterPropGroup &propertyList) const
 
 	size_t curGroup=0;
 
-	p.type=PROPERTY_TYPE_FILE;
-	p.key=DATALOAD_KEY_FILE;
-	p.name=TRANS("File");
-	p.helpText=TRANS("File from which to load data");
-	p.data=ionFilename;
+	FilterProperty pFile(DATALOAD_KEY_FILE, PROPERTY_TYPE_FILE, TRANS("File"),
+				ionFilename,TRANS("File from which to load data"));
 	//Wx- acceptable string format
-	p.dataSecondary = TRANS("Readable files (*.xml, *.pos, *.txt,*.csv, *.ato)|*.xml;*.pos;*.txt;*.csv;*.ato|All Files|*") ;
+	pFile.dataSecondary = TRANS("Readable files (*.xml, *.pos, *.txt,*.csv, *.ato, *.ops)|*.xml;*.pos;*.txt;*.csv;*.ato;*.ops|All Files|*") ;
 
-	propertyList.addProperty(p,curGroup);
+	propertyList.addProperty(pFile,curGroup);
 
 	vector<pair<unsigned int,string> > choices;
 
 	for(unsigned int ui=0;ui<FILEDATA_TYPE_ENUM_END; ui++)
 		choices.push_back(make_pair(ui,AVAILABLE_FILEDATA_TYPES[ui]));	
-					
-	p.data=choiceString(choices,fileType);
-	p.name=TRANS("File type");
-	p.type=PROPERTY_TYPE_CHOICE;
-	p.helpText=TRANS("Type of file to be loaded");
-	p.key=DATALOAD_KEY_FILETYPE;
-	
-	propertyList.addProperty(p,curGroup);
+
+	FilterProperty pMode(DATALOAD_KEY_FILETYPE, TRANS("File type"),
+			choices,fileType,TRANS("Type of fil to be loaded"));
+	propertyList.addProperty(pMode,curGroup);
 
 	propertyList.setGroupTitle(curGroup,TRANS("File"));
 	//---------
@@ -511,13 +558,32 @@ void DataLoadFilter::getProperties(FilterPropGroup &propertyList) const
 	{
 		case FILEDATA_TYPE_POS:
 		{
-			stream_cast(colStr,numColumns);
-			p.name=TRANS("Entries per point");
-			p.helpText=TRANS("Number of decimal values in file per 3D point (normally 4)");
-			p.data=colStr;
-			p.key=DATALOAD_KEY_NUMBER_OF_COLUMNS;
-			p.type=PROPERTY_TYPE_INTEGER;
-			propertyList.addProperty(p,curGroup);
+			FilterProperty pEntries(DATALOAD_KEY_NUMBER_OF_COLUMNS, TRANS("Entries per point"),
+				numColumns,TRANS("Number of decimal values in file per 3D point (normally 4)"));
+			propertyList.addProperty(pEntries,curGroup);
+			
+			choices.clear();
+			for (unsigned int i = 0; i < numColumns; i++) {
+				string tmp;
+				stream_cast(tmp,i);
+				choices.push_back(make_pair(i,tmp));
+			}
+			
+			FilterProperty pX(DATALOAD_KEY_SELECTED_COLUMN0, TRANS("X"),
+				choices,index[0],TRANS("Relative offset of each entry in file for point's X position"));
+			propertyList.addProperty(pX,curGroup);
+			
+			FilterProperty pY(DATALOAD_KEY_SELECTED_COLUMN1, TRANS("Y"),
+				choices,index[1],TRANS("Relative offset of each entry in file for point's Y position"));
+			propertyList.addProperty(pY,curGroup);
+			
+			FilterProperty pZ(DATALOAD_KEY_SELECTED_COLUMN2, TRANS("Z"),
+				choices,index[2],TRANS("Relative offset of each entry in file for point's Z position"));;
+			propertyList.addProperty(pZ,curGroup);
+			
+			FilterProperty pValue(DATALOAD_KEY_SELECTED_COLUMN3, TRANS("Value"),
+				choices,index[3],TRANS("Relative offset of each entry in file to use for scalar value of 3D point"));
+			propertyList.addProperty(pValue,curGroup);
 			break;
 		}
 		case FILEDATA_TYPE_TEXT:
@@ -531,138 +597,92 @@ void DataLoadFilter::getProperties(FilterPropGroup &propertyList) const
 
 			for(unsigned int ui=0;ui<ENDIAN_MODE_ENUM_END;ui++)
 				endianChoices[ui]=make_pair(ui,ENDIAN_MODE_STR[ui]);
-
-			p.name=TRANS("File \"Endianness\"");
-			p.helpText=TRANS("On-disk data storage format. If file won\'t load, just try each");
-			p.data=choiceString(endianChoices,endianMode);
-			p.key=DATALOAD_KEY_ENDIANNESS;
-			p.type=PROPERTY_TYPE_CHOICE;
-			propertyList.addProperty(p,curGroup);
+			
+			FilterProperty pEndian(DATALOAD_KEY_ENDIANNESS, TRANS("File \"Endianness\""),
+				endianChoices,endianMode,TRANS("On-disk data storage format. If file won\'t load, just try each"));
+			propertyList.addProperty(pEndian,curGroup);
 			break;
 
 		}
+		case FILEDATA_TYPE_3DAP_OPS:
+		{
+
+			FilterProperty pScale(DATALOAD_KEY_SCALERADIUS,
+					TRANS("Recon. Width"),reconParams.radiusCylinder, 
+					TRANS("Width of box onto which to place reconstructed detector hits (similar to tip radius)"));
+			propertyList.addProperty(pScale,curGroup);
+			
+			FilterProperty pIonVolume(DATALOAD_KEY_IONVOLUME,
+					TRANS("Ion Volume"),reconParams.ionVolume, 
+					TRANS("Effective volume of an ion (incl. free packing volume)"));
+			propertyList.addProperty(pIonVolume,curGroup);
+			
+			FilterProperty pDetectionEff(DATALOAD_KEY_DETECTIONEFFICIENCY,
+					TRANS("Det. Efficiency"),reconParams.detectionEfficiency, 
+					TRANS("Effiency of system (ions evaporated towards detector/ions detected by detector)"));
+			propertyList.addProperty(pDetectionEff,curGroup);
+			break;
+		}
 		default:
 			ASSERT(false);
 			
 	}
 
-	choices.clear();
-	for (unsigned int i = 0; i < numColumns; i++) {
-		string tmp;
-		stream_cast(tmp,i);
-		choices.push_back(make_pair(i,tmp));
-	}
 	
-	colStr= choiceString(choices,index[0]);
-	p.name="X";
-	p.data=colStr;
-	p.key=DATALOAD_KEY_SELECTED_COLUMN0;
-	p.type=PROPERTY_TYPE_CHOICE;
-	p.helpText=TRANS("Relative offset of each entry in file for point's X position");
-	propertyList.addProperty(p,curGroup);
-	
-	colStr= choiceString(choices,index[1]);
-	p.name="Y";
-	p.data=colStr;
-	p.key=DATALOAD_KEY_SELECTED_COLUMN1;
-	p.type=PROPERTY_TYPE_CHOICE;
-	p.helpText=TRANS("Relative offset of each entry in file for point's Y position");
-	propertyList.addProperty(p,curGroup);
-	
-	colStr= choiceString(choices,index[2]);
-	p.name="Z";
-	p.data=colStr;
-	p.key=DATALOAD_KEY_SELECTED_COLUMN2;
-	p.type=PROPERTY_TYPE_CHOICE;
-	p.helpText=TRANS("Relative offset of each entry in file for point's Z position");
-	propertyList.addProperty(p,curGroup);
-	
-	colStr= choiceString(choices,index[3]);
-	p.name=TRANS("Value");
-	p.data=colStr;
-	p.key=DATALOAD_KEY_SELECTED_COLUMN3;
-	p.type=PROPERTY_TYPE_CHOICE;
-	p.helpText=TRANS("Relative offset of each entry in file to use for scalar value of 3D point");
-	propertyList.addProperty(p,curGroup);
-	
-	p.name=TRANS("Value Label");
-	p.data=valueLabel;
-	p.key=DATALOAD_KEY_VALUELABEL;
-	p.type=PROPERTY_TYPE_STRING;
-	p.helpText=TRANS("Name for the scalar value associated with each point");
-	propertyList.addProperty(p,curGroup);
+	FilterProperty pValueLabel(DATALOAD_KEY_VALUELABEL,
+			TRANS("Value Label"),valueLabel,
+			TRANS("Name for the scalar value associated with each point"));
+	propertyList.addProperty(pValueLabel,curGroup);
 	
 	propertyList.setGroupTitle(curGroup,TRANS("Format params."));
-
 	curGroup++;
 
-	string tmpStr;
-	stream_cast(tmpStr,enabled);
-	p.name=TRANS("Enabled");
-	p.data=tmpStr;
-	p.key=DATALOAD_KEY_ENABLED;
-	p.type=PROPERTY_TYPE_BOOL;
-	p.helpText=TRANS("Load this file?");
-	propertyList.addProperty(p,curGroup);
+	FilterProperty pEnabled(DATALOAD_KEY_ENABLED,
+			TRANS("Enabled"),enabled,TRANS("Load this file?"));
+	propertyList.addProperty(pEnabled,curGroup);
 
 	if(enabled)
 	{
 		std::string tmpStr;
 
 		//FIXME: ATO Files need an implementation of sampling read
-		if(fileType!=FILEDATA_TYPE_ATO)
+		if(fileType==FILEDATA_TYPE_POS)
 		{
-			stream_cast(tmpStr,doSample);
-			p.name=TRANS("Sample data");
-			p.data=tmpStr;
-			p.type=PROPERTY_TYPE_BOOL;
-			p.helpText=TRANS("Perform random selection on file contents, instead of loading entire file");
-			p.key=DATALOAD_KEY_SAMPLE;
-			propertyList.addProperty(p,curGroup);
+			FilterProperty pSample(DATALOAD_KEY_SAMPLE,
+					TRANS("Sample data"),doSample,TRANS("Perform random selection on file contents,instead of loading entire file?"));
+			propertyList.addProperty(pSample,curGroup);
 
 			if(doSample)
 			{
-				stream_cast(tmpStr,maxIons*sizeof(float)*4/(1024*1024));
-				p.name=TRANS("Load Limit (MB)");
-				p.data=tmpStr;
-				p.type=PROPERTY_TYPE_INTEGER;
-				p.helpText=TRANS("Limit for size of data to load");
-				p.key=DATALOAD_KEY_SIZE;
-				propertyList.addProperty(p,curGroup);
+				FilterProperty pMaxLoad(DATALOAD_KEY_SIZE,
+						TRANS("Load Limit (MB)"),(unsigned int)(maxIons*sizeof(float)*4/(1024*1024)),
+						TRANS("Limit for size of data to load"));
+				propertyList.addProperty(pMaxLoad,curGroup);
 			}
 		}
 
-		stream_cast(tmpStr,wantMonitor);
-		p.name=TRANS("Monitor");
-		p.data=tmpStr;
-		p.key=DATALOAD_KEY_MONITOR;
-		p.type=PROPERTY_TYPE_BOOL;
-		p.helpText=TRANS("Watch file timestamp to track changes to file contents from other programs");
-		propertyList.addProperty(p,curGroup);
+		FilterProperty pMonitor(DATALOAD_KEY_MONITOR,
+				TRANS("Monitor"),wantMonitor,
+				TRANS("Watch file timestamp to track changes to file contents from other programs"));
+		propertyList.addProperty(pMonitor,curGroup);
 	}
 	
 	propertyList.setGroupTitle(curGroup,TRANS("Load params."));
 
 	if(enabled)
 	{
-		
 		curGroup++;
-
-		p.name=TRANS("Default colour ");
-		p.data=rgbaf.toColourRGBA().rgbaString(); 
-		p.type=PROPERTY_TYPE_COLOUR;
-		p.helpText=TRANS("Default colour for points, if not overridden by other filters");
-		p.key=DATALOAD_KEY_COLOUR;
-		propertyList.addProperty(p,curGroup);
-
-		stream_cast(tmpStr,ionSize);
-		p.name=TRANS("Draw Size");
-		p.data=tmpStr; 
-		p.type=PROPERTY_TYPE_REAL;
-		p.helpText=TRANS("Default size for points, if not overridden by other filters");
-		p.key=DATALOAD_KEY_IONSIZE;
 		
-		propertyList.addProperty(p,curGroup);
+		FilterProperty pColour(DATALOAD_KEY_COLOUR,
+				TRANS("Default colour"),rgbaf,
+				TRANS("Default colour for points, if not overridden by other filters"));
+		propertyList.addProperty(pColour,curGroup);
+
+		FilterProperty pDrawSize(DATALOAD_KEY_IONSIZE,
+				TRANS("Draw Size"),ionSize,
+				TRANS("Default size for points, if not overridden by other filters"));
+		propertyList.addProperty(pDrawSize,curGroup);
+		
 		propertyList.setGroupTitle(curGroup,TRANS("Appearance"));
 	}
 
@@ -943,6 +963,64 @@ bool DataLoadFilter::setProperty(  unsigned int key,
 			needUpdate=true;
 			break;
 		}
+		
+		case DATALOAD_KEY_SCALERADIUS:
+		{
+			float ltmp;
+			if(stream_cast(ltmp,value))
+				return false;
+
+			if(ltmp < 0)
+				return false;
+
+			if(ltmp == reconParams.radiusCylinder)
+				break;
+
+			reconParams.radiusCylinder=ltmp;
+
+			needUpdate=true;
+			clearCache();
+
+			break;
+		}
+		case DATALOAD_KEY_IONVOLUME:
+		{
+			float ltmp;
+			if(stream_cast(ltmp,value))
+				return false;
+
+			if(ltmp < 0)
+				return false;
+
+			if(ltmp == reconParams.ionVolume)
+				break;
+
+			reconParams.ionVolume=ltmp;
+
+			needUpdate=true;
+			clearCache();
+
+			break;
+		}
+		case DATALOAD_KEY_DETECTIONEFFICIENCY:
+		{
+			float ltmp;
+			if(stream_cast(ltmp,value))
+				return false;
+
+			if(ltmp < 0)
+				return false;
+
+			if(ltmp == reconParams.detectionEfficiency)
+				break;
+
+			reconParams.detectionEfficiency=ltmp;
+
+			needUpdate=true;
+			clearCache();
+
+			break;
+		}
 		default:
 			ASSERT(false);
 			break;
@@ -991,12 +1069,7 @@ bool DataLoadFilter::readState(xmlNodePtr &nodePtr, const std::string &stateFile
 		fileType=FILEDATA_TYPE_POS;
 
 
-	//Override the string, as needed
-	if( (stateFileDir.size()) &&
-		(ionFilename.size() > 2 && ionFilename.substr(0,2) == "./") )
-	{
-		ionFilename=stateFileDir + ionFilename.substr(2);
-	}
+	convertFileStringToAbsolute(stateFileDir, ionFilename);
 
 	//Filenames need to be converted from unix format (which I make canonical on disk) into native format 
 	ionFilename=convertFileStringToNative(ionFilename);
@@ -1174,6 +1247,7 @@ bool DataLoadFilter::writeState(std::ostream &f,unsigned int format, unsigned in
 			f << tabs(depth+1) << "<colour r=\"" <<  rgbaf.r() << "\" g=\"" << rgbaf.g() 
 				<< "\" b=\"" << rgbaf.b() << "\" a=\"" << rgbaf.a() << "\"/>" <<endl;
 			f << tabs(depth+1) << "<ionsize value=\"" << ionSize << "\"/>" << endl;
+			f << tabs(depth+1) << "<threedaprecon scaleradius=\"" << reconParams.radiusCylinder << "\" detectionefficiency=\"" << reconParams.detectionEfficiency <<"\" ionvolume=\"" << reconParams.ionVolume << "\"/>" << endl;
 			f << tabs(depth) << "</" << trueName() << ">" << endl;
 			break;
 		}
