@@ -1,6 +1,6 @@
 /*
  *	filterCommon.h - Helper routines for filter classes
- *	Copyright (C) 2015, D Haley 
+ *	Copyright (C) 2018, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -27,31 +27,21 @@
 
 #include "backend/APT/APTRanges.h"
 
-//QHull library
-#ifdef __POWERPC__
-	#pragma push_macro("__POWERPC__")
-	#define __POWERPC__ 1
-#endif
-extern "C"
-{
-	#include <qhull/qhull_a.h>
-}
-#ifdef __POWERPC__
-	#pragma pop_macro("__POWERPC__")
-#endif
+#include "config.h"
 
 
-enum
-{
-	HULL_ERR_NO_MEM=1,
-	HULL_ERR_USER_ABORT,
-	HULL_ERR_ENUM_END
-};
-
 //TODO: Namespace this into the filter base class?
 
 const size_t PROGRESS_REDUCE=5000;
 
+
+//Create a "choice" string that is used ot populate drop-down combo boxes
+// this is created from a vector of int-string pairs
+std::string choiceString(std::vector<std::pair<unsigned int, std::string> > comboString, 
+									unsigned int curChoice);
+
+
+
 //serialise 3D std::vectors to specified output stream in XML format
 void writeVectorsXML(std::ostream &f, const char *containerName,
 		const std::vector<Point3D> &vectorParams, unsigned int depth);
@@ -119,91 +109,14 @@ bool parseXMLColour(xmlNodePtr &nodePtr, ColourRGBAf &rgbaf);
 // is ranged tht way. Otherwise returns -1.
 unsigned int getIonstreamIonID(const IonStreamData *d, const RangeFile *r);
 
-inline 
-void assignIonData(Point3D &p, const IonHit &h)
-{
-	p = h.getPosRef();
-}
-
-inline
-void assignIonData(IonHit &p, const IonHit &h)
-{
-	p = h;
-}
-
-//!Extend a point data vector using some ion data. 
-// Offset is the position to start inserting in the destination array.
-// Will fail if user abort is detected 
-template<class T>
-unsigned int extendDataVector(std::vector<T> &dest, const std::vector<IonHit> &vIonData,
-				unsigned int &progress, size_t offset)
-{
-	unsigned int curProg=NUM_CALLBACK;
-	unsigned int n =offset;
-#ifdef _OPENMP
-	//Parallel version
-	bool spin=false;
-	#pragma omp parallel for shared(spin)
-	for(size_t ui=0;ui<vIonData.size();ui++)
-	{
-		if(spin)
-			continue;
-		assignIonData(dest[offset+ ui],vIonData[ui]);
-		
-		//update progress every CALLBACK entries
-		if(!curProg--)
-		{
-			#pragma omp critical
-			{
-			n+=NUM_CALLBACK;
-			progress= (unsigned int)(((float)n/(float)dest.size())*100.0f);
-			if(!omp_get_thread_num())
-			{
-				if(*Filter::wantAbort)
-					spin=true;
-			}
-			}
-		}
-
-	}
-
-	if(spin)
-		return 1;
-#else
-
-	for(size_t ui=0;ui<vIonData.size();ui++)
-	{
-		assignIonData(dest[offset+ ui],vIonData[ui]);
-		
-		//update progress every CALLBACK ions
-		if(!curProg--)
-		{
-			n+=NUM_CALLBACK;
-			progress= (unsigned int)(((float)n/(float)dest.size())*100.0f);
-			if(*(Filter::wantAbort))
-				return 1;
-		}
-
-	}
-#endif
-
-
-	return 0;
-}
+//obtain the number of basic items in the input, given the stream type mask
+size_t getTotalSizeByType(const std::vector<const FilterStreamData*> &dataIn, unsigned int streamTypeMask);
 
 const RangeFile *getRangeFile(const std::vector<const FilterStreamData*> &dataIn);
 
-//Compute the convex hull of a set of input points from fiilterstream data
-unsigned int computeConvexHull(const std::vector<const FilterStreamData*> &data, 
-			unsigned int *progress, 
-			std::vector<Point3D> &hullPts, bool wantVolume, bool freeHull=true);
-//Compute the convex hull of a set of input points
-unsigned int computeConvexHull(const std::vector<Point3D> &data, 
-			unsigned int *progress, const bool &abortPtr,
-			std::vector<Point3D> &hullPts, bool wantVolume, bool freeHull=true);
 
-//Release the memory held by qhull, and notify the computeConvexHull routines that this has been done
-void freeConvexHull();
+
+
 //Draw a colour bar
 DrawColourBarOverlay *makeColourBar(float minV, float maxV,size_t nColours,size_t colourMap, bool reverseMap=false, float alpha=1.0f) ;
 
@@ -212,4 +125,17 @@ DrawColourBarOverlay *makeColourBar(float minV, float maxV,size_t nColours,size_
 // - note that any subdirs will be automatically created if needed.
 std::string createTmpFilename(const char *dir=NULL,const char *extension=NULL);
 
+//Input buffer should be bufXY[ui*2 + offset] (X-> offset=0, Y->offset=1)
+// n - number of input points
+// output is the vertices of the triangulation
+unsigned int do2DDelaunay(double *bufXY, unsigned int n, 
+		std::vector<size_t> &ta, std::vector<size_t> &tb, std::vector<size_t> &tc);
+
+#ifdef DEBUG 
+
+//Test functions
+bool testCommon();
+
+#endif
+
 #endif
