@@ -1,6 +1,6 @@
 /*
  *	geometryHelpers.cpp - Various spatial geometry operators for point clouds
- *	Copyright (C) 2015, D Haley 
+ *	Copyright (C) 2018, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -19,8 +19,6 @@
 
 #include "geometryHelpers.h"
 
-#include "backend/APT/ionhit.h"
-
 #include "backend/filter.h"
 
 #ifdef _OPENMP
@@ -30,15 +28,15 @@
 
 using std::vector;
 
-const size_t DEFAULT_NUM_CALLBACK=5000;
-
 //These numbers have not been optimised. On a 2CPU system, I cannot find
 // a case where the // option is outrun by the linear one, over 3 averages
 //---
 //Minimum number of input points before we will do reserve testing
 const size_t MIN_SAMPLE_TEST = 1000;
-//Minimim number of input points before we will engage a parallel algorithm
+#ifdef _OPENMP
+//Minimum number of input points before we will engage a parallel algorithm
 const size_t MIN_PARALLELISE = 20000;
+#endif
 //---
 
 CropHelper::CropHelper(	size_t totalData,size_t filterMode,
