@@ -1,6 +1,6 @@
 /*
  *	ionClip.cpp -3Depict filter to perform clipping of 3D point clouds
- *	Copyright (C) 2015, D Haley 
+ *	Copyright (C) 2018, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -392,71 +392,74 @@ unsigned int IonClipFilter::refresh(const std::vector<const FilterStreamData *>
 	try
 	{
 
-		std::map<pair<size_t,bool>,size_t> primitiveTypeMap;
-
-		using std::make_pair;
-		//map the primitive enum type, and the clip inversion state
-		// to the CropHelper clip mode
-		primitiveTypeMap[make_pair((size_t)PRIMITIVE_SPHERE,false)]=CROP_SPHERE_INSIDE;
-		primitiveTypeMap[make_pair((size_t)PRIMITIVE_SPHERE,true)]=CROP_SPHERE_OUTSIDE;
-		primitiveTypeMap[make_pair((size_t)PRIMITIVE_PLANE,false)]=CROP_PLANE_FRONT;
-		primitiveTypeMap[make_pair((size_t)PRIMITIVE_PLANE,true)]=CROP_PLANE_BACK;
-		primitiveTypeMap[make_pair((size_t)PRIMITIVE_CYLINDER,false)]=CROP_CYLINDER_INSIDE_AXIAL;
-		primitiveTypeMap[make_pair((size_t)PRIMITIVE_CYLINDER,true)]=CROP_CYLINDER_OUTSIDE;
-		primitiveTypeMap[make_pair((size_t)PRIMITIVE_AAB,false)]=CROP_AAB_INSIDE;
-		primitiveTypeMap[make_pair((size_t)PRIMITIVE_AAB,true)]=CROP_AAB_OUTSIDE;
-
-		size_t mode;
-		mode = primitiveTypeMap[make_pair(primitiveType,invertedClip)];
-		size_t totalSize=numElements(dataIn);
-		CropHelper cropper(totalSize,mode,vectorParams,scalarParams  );
-
-		float minProg,maxProg;
-		size_t cumulativeSize=0;
-		for(unsigned int ui=0;ui<dataIn.size() ;ui++)
+		size_t totalSize=numElements(dataIn,STREAM_TYPE_IONS);
+		if(totalSize)
 		{
-			//Loop through each data set
-			switch(dataIn[ui]->getStreamType())
+			std::map<pair<size_t,bool>,size_t> primitiveTypeMap;
+
+			using std::make_pair;
+			//map the primitive enum type, and the clip inversion state
+			// to the CropHelper clip mode
+			primitiveTypeMap[make_pair((size_t)PRIMITIVE_SPHERE,false)]=CROP_SPHERE_INSIDE;
+			primitiveTypeMap[make_pair((size_t)PRIMITIVE_SPHERE,true)]=CROP_SPHERE_OUTSIDE;
+			primitiveTypeMap[make_pair((size_t)PRIMITIVE_PLANE,false)]=CROP_PLANE_FRONT;
+			primitiveTypeMap[make_pair((size_t)PRIMITIVE_PLANE,true)]=CROP_PLANE_BACK;
+			primitiveTypeMap[make_pair((size_t)PRIMITIVE_CYLINDER,false)]=CROP_CYLINDER_INSIDE_AXIAL;
+			primitiveTypeMap[make_pair((size_t)PRIMITIVE_CYLINDER,true)]=CROP_CYLINDER_OUTSIDE;
+			primitiveTypeMap[make_pair((size_t)PRIMITIVE_AAB,false)]=CROP_AAB_INSIDE;
+			primitiveTypeMap[make_pair((size_t)PRIMITIVE_AAB,true)]=CROP_AAB_OUTSIDE;
+
+			size_t mode;
+			mode = primitiveTypeMap[make_pair(primitiveType,invertedClip)];
+			CropHelper cropper(totalSize,mode,vectorParams,scalarParams  );
+
+			float minProg,maxProg;
+			size_t cumulativeSize=0;
+			for(unsigned int ui=0;ui<dataIn.size() ;ui++)
 			{
-				case STREAM_TYPE_IONS:
+				//Loop through each data set
+				switch(dataIn[ui]->getStreamType())
 				{
-					d=new IonStreamData;
-					d->parent=this;
-					minProg=cumulativeSize/(float)totalSize;
-					cumulativeSize+=d->data.size();
-					maxProg=cumulativeSize/(float)totalSize;
-
-					//Filter input data to output data.
-					if(cropper.runFilter(((const IonStreamData *)dataIn[ui])->data,
-						d->data,minProg,maxProg,progress.filterProgress))
+					case STREAM_TYPE_IONS:
 					{
-						delete d;
-						return CALLBACK_FAIL; 
+						d=new IonStreamData;
+						d->parent=this;
+						minProg=cumulativeSize/(float)totalSize;
+						cumulativeSize+=d->data.size();
+						maxProg=cumulativeSize/(float)totalSize;
+
+						//Filter input data to output data.
+						if(cropper.runFilter(((const IonStreamData *)dataIn[ui])->data,
+							d->data,minProg,maxProg,progress.filterProgress))
+						{
+							delete d;
+							return CALLBACK_FAIL; 
+						}
+
+						if(d->data.size())
+						{
+							//Copy over other attributes
+							d->r = ((IonStreamData *)dataIn[ui])->r;
+							d->g = ((IonStreamData *)dataIn[ui])->g;
+							d->b =((IonStreamData *)dataIn[ui])->b;
+							d->a =((IonStreamData *)dataIn[ui])->a;
+							d->ionSize =((IonStreamData *)dataIn[ui])->ionSize;
+
+							//getOut is const, so shouldn't be modified
+							cacheAsNeeded(d);
+
+							getOut.push_back(d);
+							d=0;
+						}
+						else
+							delete d;
+						break;
 					}
-
-					if(d->data.size())
-					{
-						//Copy over other attributes
-						d->r = ((IonStreamData *)dataIn[ui])->r;
-						d->g = ((IonStreamData *)dataIn[ui])->g;
-						d->b =((IonStreamData *)dataIn[ui])->b;
-						d->a =((IonStreamData *)dataIn[ui])->a;
-						d->ionSize =((IonStreamData *)dataIn[ui])->ionSize;
-
-						//getOut is const, so shouldn't be modified
-						cacheAsNeeded(d);
-
-						getOut.push_back(d);
-						d=0;
-					}
-					else
-						delete d;
-					break;
+					default:
+						//Just copy across the ptr, if we are unfamiliar with this type
+						getOut.push_back(dataIn[ui]);	
+						break;
 				}
-				default:
-					//Just copy across the ptr, if we are unfamiliar with this type
-					getOut.push_back(dataIn[ui]);	
-					break;
 			}
 		}
 	}
@@ -494,29 +497,17 @@ void IonClipFilter::getProperties(FilterPropGroup &propertyList) const
 	choices.push_back(make_pair((unsigned int)PRIMITIVE_AAB,
 				primitiveStringFromID(PRIMITIVE_AAB)));
 
-	tmpStr= choiceString(choices,primitiveType);
-	p.name=TRANS("Primitive");
-	p.data=tmpStr;
-	p.type=PROPERTY_TYPE_CHOICE;
-	p.helpText=TRANS("Shape of clipping object");
-	p.key=KEY_PRIMITIVE_TYPE;
-	propertyList.addProperty(p,curGroup);
+	FilterProperty pPrimitive(KEY_PRIMITIVE_TYPE, TRANS("Primitive"),
+			choices,primitiveType,TRANS("Shape of clipping object"));
+	propertyList.addProperty(pPrimitive,curGroup);
 	
-	stream_cast(tmpStr,showPrimitive);
-	p.key=KEY_PRIMITIVE_SHOW;
-	p.name=TRANS("Show Primitive");
-	p.data= tmpStr;
-	p.type=PROPERTY_TYPE_BOOL;
-	p.helpText=TRANS("Display the 3D interaction object");
-	propertyList.addProperty(p,curGroup);
+	FilterProperty pShow(KEY_PRIMITIVE_SHOW, TRANS("Show Primitive"),
+		 showPrimitive,TRANS("Display the 3D interaction object"));
+	propertyList.addProperty(pShow,curGroup);
 	
-	stream_cast(tmpStr,invertedClip);
-	p.key=KEY_PRIMITIVE_INVERTCLIP;
-	p.name=TRANS("Invert Clip");
-	p.data= tmpStr;
-	p.type=PROPERTY_TYPE_BOOL;
-	p.helpText=TRANS("Switch between retaining points inside (false) and outside (true) of primitive");
-	propertyList.addProperty(p,curGroup);
+	FilterProperty pInvertClip(KEY_PRIMITIVE_INVERTCLIP,TRANS("Invert Clip"),
+		 invertedClip,TRANS("Switch between retaining points inside (disabled) and outside (enabled) of primitive"));
+	propertyList.addProperty(pInvertClip,curGroup);
 
 	switch(primitiveType)
 	{
@@ -524,43 +515,29 @@ void IonClipFilter::getProperties(FilterPropGroup &propertyList) const
 		{
 			ASSERT(vectorParams.size() == 1);
 			ASSERT(scalarParams.size() == 1);
-			stream_cast(tmpStr,vectorParams[0]);
-			p.key=KEY_ORIGIN;
-			p.name=TRANS("Origin");
-			p.data= tmpStr;
-			p.type=PROPERTY_TYPE_POINT3D;
-			p.helpText=TRANS("Position for centre of sphere");
-			propertyList.addProperty(p,curGroup);
 			
-			stream_cast(tmpStr,scalarParams[0]);
-			p.key=KEY_RADIUS;
-			p.name=TRANS("Radius");
-			p.data=tmpStr;
-			p.type=PROPERTY_TYPE_REAL;
-			p.helpText=TRANS("Radius of sphere");
-			propertyList.addProperty(p,curGroup);
-
+			FilterProperty pOrigin(KEY_ORIGIN,TRANS("Origin"),
+				 vectorParams[0],TRANS("Position for centre of sphere"));
+			propertyList.addProperty(pOrigin,curGroup);
+			
+			FilterProperty pRadius(KEY_RADIUS,TRANS("Radius"),
+				 scalarParams[0],TRANS("Radius of sphere"));
+			propertyList.addProperty(pRadius,curGroup);
+			
 			break;
 		}
 		case PRIMITIVE_PLANE:
 		{
 			ASSERT(vectorParams.size() == 2);
 			ASSERT(scalarParams.size() == 0);
-			stream_cast(tmpStr,vectorParams[0]);
-			p.key=KEY_ORIGIN;
-			p.name=TRANS("Origin");
-			p.data=tmpStr;
-			p.type=PROPERTY_TYPE_POINT3D;
-			p.helpText=TRANS("Position that plane passes through");
-			propertyList.addProperty(p,curGroup);
+			FilterProperty pOrigin(KEY_ORIGIN,TRANS("Origin"),
+				 vectorParams[0],TRANS("Position that plane passes through"));
+			propertyList.addProperty(pOrigin,curGroup);
+			
+			FilterProperty pNormal(KEY_NORMAL,TRANS("Plane Normal"),
+				 vectorParams[1],TRANS("Perpendicular direction for plane"));
+			propertyList.addProperty(pNormal,curGroup);
 			
-			stream_cast(tmpStr,vectorParams[1]);
-			p.key=KEY_NORMAL;
-			p.name=TRANS("Plane Normal");
-			p.data= tmpStr;
-			p.type=PROPERTY_TYPE_POINT3D;
-			p.helpText=TRANS("Perpendicular direction for plane");
-			propertyList.addProperty(p,curGroup);
 
 			break;
 		}
@@ -568,58 +545,36 @@ void IonClipFilter::getProperties(FilterPropGroup &propertyList) const
 		{
 			ASSERT(vectorParams.size() == 2);
 			ASSERT(scalarParams.size() == 1);
-			stream_cast(tmpStr,vectorParams[0]);
-			p.key=KEY_ORIGIN;
-			p.name=TRANS("Origin");
-			p.data= tmpStr;
-			p.type=PROPERTY_TYPE_POINT3D;
-			p.helpText=TRANS("Centre of cylinder");
-			propertyList.addProperty(p,curGroup);
-			
-			stream_cast(tmpStr,vectorParams[1]);
-			p.key=KEY_NORMAL;
-			p.name=TRANS("Axis");
-			p.data= tmpStr;
-			p.type=PROPERTY_TYPE_POINT3D;
-			p.helpText=TRANS("Positive vector for cylinder");
-			propertyList.addProperty(p,curGroup);
-			
-			tmpStr=boolStrEnc(lockAxisMag);
-			p.key=KEY_AXIS_LOCKMAG;
-			p.name=TRANS("Lock Axis Mag.");
-			p.data=tmpStr;
-			p.type=PROPERTY_TYPE_BOOL;
-			p.helpText=TRANS("Prevent changing length of cylinder during 3D interaction");
-			propertyList.addProperty(p,curGroup);
 
-			stream_cast(tmpStr,scalarParams[0]);
-			p.key=KEY_RADIUS;
-			p.name=TRANS("Radius");
-			p.data=tmpStr;
-			p.type=PROPERTY_TYPE_POINT3D;
-			p.helpText=TRANS("Radius of cylinder");
-			propertyList.addProperty(p,curGroup);
+			FilterProperty pOrigin(KEY_ORIGIN,TRANS("Origin"),
+				 vectorParams[0],TRANS("Centre of cylinder"));
+			propertyList.addProperty(pOrigin,curGroup);
+			
+			FilterProperty pAxis(KEY_NORMAL,TRANS("Axis"),
+				 vectorParams[1],TRANS("Positive vector for cylinder"));
+			propertyList.addProperty(pAxis,curGroup);
+			
+			FilterProperty pLockAxis(KEY_AXIS_LOCKMAG,TRANS("Lock Axis Mag."),
+				 lockAxisMag,TRANS("Prevent changing length of cylinder during 3D interaction"));
+			propertyList.addProperty(pLockAxis,curGroup);
+
+			FilterProperty pRadius(KEY_RADIUS,TRANS("Radius."),
+				 scalarParams[0],TRANS("Radius of cylinder"));
+			propertyList.addProperty(pRadius,curGroup);
 			break;
 		}
 		case PRIMITIVE_AAB:
 		{
 			ASSERT(vectorParams.size() == 2);
 			ASSERT(scalarParams.size() == 0);
-			stream_cast(tmpStr,vectorParams[0]);
-			p.key=KEY_ORIGIN;
-			p.name=TRANS("Origin");
-			p.data= tmpStr;
-			p.type=PROPERTY_TYPE_POINT3D;
-			p.helpText=TRANS("Centre of axis aligned box");
-			propertyList.addProperty(p,curGroup);
+			FilterProperty pOrigin(KEY_ORIGIN,TRANS("Origin"),
+				 vectorParams[0],TRANS("Centre of Aligned Box"));
+			propertyList.addProperty(pOrigin,curGroup);
+			
+			FilterProperty pCorner(KEY_CORNER,TRANS("Corner offset"),
+				 vectorParams[1],TRANS("Vector to corner of box"));
+			propertyList.addProperty(pCorner,curGroup);
 			
-			stream_cast(tmpStr,vectorParams[1]);
-			p.key=KEY_CORNER;
-			p.name=TRANS("Corner offset");
-			p.data=tmpStr;
-			p.type=PROPERTY_TYPE_POINT3D;
-			p.helpText=TRANS("Vector to corner of box");
-			propertyList.addProperty(p,curGroup);
 			break;
 		}
 		default:
