@@ -1,6 +1,6 @@
 /*
  *	ionColour.cpp - Filter to create coloured batches of ions based upon value
- *	Copyright (C) 2015, D Haley 
+ *	Copyright (C) 2018, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -18,6 +18,8 @@
 #include "ionColour.h"
 
 #include "filterCommon.h"
+#include "../state.h"
+#include "common/stringFuncs.h"
 
 #include "common/colourmap.h"
 
@@ -37,6 +39,7 @@ enum
 	KEY_IONCOLOURFILTER_REVERSE,
 	KEY_IONCOLOURFILTER_SHOWBAR,
 	KEY_IONCOLOURFILTER_ALPHA,
+	KEY_IONCOLOURFILTER_AUTOEXTREMA,
 };
 
 enum
@@ -44,11 +47,12 @@ enum
 	IONCOLOUR_ABORT_ERR
 };
 
-IonColourFilter::IonColourFilter() : colourMap(0),reverseMap(false), 
+IonColourFilter::IonColourFilter() : colourMap(COLOURMAP_VIRIDIS),reverseMap(false), 
 		nColours(MAX_NUM_COLOURS),showColourBar(true), alpha(1.0f)
 {
 	mapBounds[0] = 0.0f;
 	mapBounds[1] = 100.0f;
+	autoExtrema=true;	
 
 	cacheOK=false;
 	cache=true; //By default, we should cache, but decision is made higher up
@@ -61,6 +65,7 @@ Filter *IonColourFilter::cloneUncached() const
 	p->colourMap = colourMap;
 	p->mapBounds[0]=mapBounds[0];
 	p->mapBounds[1]=mapBounds[1];
+	p->autoExtrema=autoExtrema;	
 	p->nColours =nColours;	
 	p->alpha = alpha;
 	p->showColourBar =showColourBar;	
@@ -76,7 +81,7 @@ Filter *IonColourFilter::cloneUncached() const
 
 size_t IonColourFilter::numBytesForCache(size_t nObjects) const
 {
-		return (size_t)((float)(nObjects*IONDATA_SIZE));
+	return (size_t)((float)(nObjects*IONDATA_SIZE));
 }
 
 
@@ -95,6 +100,50 @@ unsigned int IonColourFilter::refresh(const std::vector<const FilterStreamData *
 		return 0;
 	}
 
+	size_t totalSize=numElements(dataIn,STREAM_TYPE_IONS);
+	if(autoExtrema)
+	{
+		float tmpBounds[2];
+		tmpBounds[0]= std::numeric_limits<float>::max();
+		tmpBounds[1]=-std::numeric_limits<float>::max();
+		
+
+		for(unsigned int ui=0;ui<dataIn.size() ;ui++)
+		{
+			//Only process stream_type_ions. Do not propagate anything,
+			//except for the spectrum
+			if(dataIn[ui]->getStreamType() == STREAM_TYPE_IONS)
+			{
+				const IonStreamData *ions;
+				ions = (IonStreamData *)dataIn[ui];
+				for(unsigned int uj=0;uj<ions->data.size(); uj++)
+				{
+					float mToc;
+					mToc =ions->data[uj].getMassToCharge();
+
+					if(isinf(mToc))
+						continue;
+					tmpBounds[0] = std::min(tmpBounds[0],mToc);
+					tmpBounds[1] = std::max(tmpBounds[1],mToc);
+
+				}
+
+			}
+			
+		}
+	
+		if(tmpBounds[0] != std::numeric_limits<float>::max() &&
+			tmpBounds[1] != -std::numeric_limits<float>::max())
+		{
+			mapBounds[0] = tmpBounds[0];
+			mapBounds[1] = tmpBounds[1];
+		}
+		else
+		{
+			//This will happen if, e.g. no data was given 
+			appendConsoleMessage(TRANS("Unable to determine colour limits"));
+		}
+	}
 
 	ASSERT(nColours >0 && nColours<=MAX_NUM_COLOURS);
 	IonStreamData *d[nColours];
@@ -125,7 +174,6 @@ unsigned int IonColourFilter::refresh(const std::vector<const FilterStreamData *
 
 	//Did we find any ions in this pass?
 	bool foundIons=false;	
-	unsigned int totalSize=numElements(dataIn);
 	unsigned int curProg=NUM_CALLBACK;
 	size_t n=0;
 	for(unsigned int ui=0;ui<dataIn.size() ;ui++)
@@ -249,64 +297,43 @@ void IonColourFilter::getProperties(FilterPropGroup &propertyList) const
 
 	size_t curGroup=0;
 
-	for(unsigned int ui=0;ui<NUM_COLOURMAPS; ui++)
+	for(unsigned int ui=0;ui<COLOURMAP_ENUM_END; ui++)
 		choices.push_back(make_pair(ui,getColourMapName(ui)));
+	FilterProperty pColourMap(KEY_IONCOLOURFILTER_COLOURMAP, TRANS("Colour Map"),choices,
+			colourMap,TRANS("Colour scheme used to assign points colours by value"));
+	propertyList.addProperty(pColourMap,curGroup);
 
-	tmpStr=choiceString(choices,colourMap);
+	FilterProperty pReverse(KEY_IONCOLOURFILTER_REVERSE, TRANS("Reverse map"), 
+			reverseMap,TRANS("Reverse the colour scale"));
+	propertyList.addProperty(pReverse,curGroup);
 	
-	p.name=TRANS("Colour Map"); 
-	p.data=tmpStr;
-	p.key=KEY_IONCOLOURFILTER_COLOURMAP;
-	p.type=PROPERTY_TYPE_CHOICE;
-	p.helpText=TRANS("Colour scheme used to assign points colours by value");
-	propertyList.addProperty(p,curGroup);
-
+	FilterProperty pShowBar(KEY_IONCOLOURFILTER_SHOWBAR, TRANS("Show Bar"), 
+			showColourBar,TRANS("Display the colour legend in the 3D view"));
+	propertyList.addProperty(pShowBar,curGroup);
 	
-	p.name=TRANS("Reverse map");
-	p.helpText=TRANS("Reverse the colour scale");
-	p.data= boolStrEnc(reverseMap);
-	p.key=KEY_IONCOLOURFILTER_REVERSE;
-	p.type=PROPERTY_TYPE_BOOL;
-	propertyList.addProperty(p,curGroup);
+	FilterProperty pOpacity(KEY_IONCOLOURFILTER_ALPHA, TRANS("Opacity"), 
+			alpha,TRANS("How see-through to make the legend (0- transparent, 1- solid)"));
+	propertyList.addProperty(pOpacity,curGroup);
 	
+	FilterProperty pNumColours(KEY_IONCOLOURFILTER_NCOLOURS, TRANS("Num Colours"), 
+			nColours,TRANS("Number of unique colours to use in colour map")); 
+	propertyList.addProperty(pNumColours,curGroup);
 
-	p.name=TRANS("Show Bar");
-	p.key=KEY_IONCOLOURFILTER_SHOWBAR;
-	p.data=boolStrEnc(showColourBar);
-	p.type=PROPERTY_TYPE_BOOL;
-	propertyList.addProperty(p,curGroup);
-	
-	p.name=TRANS("Opacity");
-	p.key=KEY_IONCOLOURFILTER_ALPHA;
-	stream_cast(p.data,alpha);
-	p.type=PROPERTY_TYPE_REAL;
-	propertyList.addProperty(p,curGroup);
+	FilterProperty pAutoBounds(KEY_IONCOLOURFILTER_AUTOEXTREMA, TRANS("Auto bounds"), 
+			autoExtrema,TRANS("Automatically scale colour to min/max of data range"));
+	propertyList.addProperty(pAutoBounds,curGroup);
+	if(!autoExtrema)
+	{
 
-	stream_cast(tmpStr,nColours);
-	p.name=TRANS("Num Colours");
-	p.data=tmpStr;
-	p.helpText=TRANS("Number of unique colours to use in colour map"); 
-	p.key=KEY_IONCOLOURFILTER_NCOLOURS;
-	p.type=PROPERTY_TYPE_INTEGER;
-	propertyList.addProperty(p,curGroup);
+		FilterProperty pMapStart(KEY_IONCOLOURFILTER_MAPSTART, TRANS("Map start"), 
+				mapBounds[0],TRANS("Assign points with this value to the first colour in map")) ;
+		propertyList.addProperty(pMapStart,curGroup);
 
-	stream_cast(tmpStr,mapBounds[0]);
-	p.name=TRANS("Map start");
-	p.helpText=TRANS("Assign points with this value to the first colour in map"); 
-	p.data= tmpStr;
-	p.key=KEY_IONCOLOURFILTER_MAPSTART;
-	p.type=PROPERTY_TYPE_REAL;
-	propertyList.addProperty(p,curGroup);
-
-	stream_cast(tmpStr,mapBounds[1]);
-	p.name=TRANS("Map end");
-	p.helpText=TRANS("Assign points with this value to the last colour in map"); 
-	p.data= tmpStr;
-	p.key=KEY_IONCOLOURFILTER_MAPEND;
-	p.type=PROPERTY_TYPE_REAL;
-	propertyList.addProperty(p,curGroup);
+		FilterProperty pMapEnd(KEY_IONCOLOURFILTER_MAPEND, TRANS("Map End"), 
+				mapBounds[1],TRANS("Assign points with this value to the last colour in map")); 
+		propertyList.addProperty(pMapEnd,curGroup);
+	}	
 	propertyList.setGroupTitle(curGroup,TRANS("Data"));
-	
 
 }
 
@@ -321,7 +348,7 @@ bool IonColourFilter::setProperty(  unsigned int key,
 		{
 			unsigned int tmpMap;
 			tmpMap=(unsigned int)-1;
-			for(unsigned int ui=0;ui<NUM_COLOURMAPS;ui++)
+			for(unsigned int ui=0;ui<COLOURMAP_ENUM_END;ui++)
 			{
 				if(value== getColourMapName(ui))
 				{
@@ -330,7 +357,7 @@ bool IonColourFilter::setProperty(  unsigned int key,
 				}
 			}
 
-			if(tmpMap >=NUM_COLOURMAPS || tmpMap ==colourMap)
+			if(tmpMap >=COLOURMAP_ENUM_END || tmpMap ==colourMap)
 				return false;
 
 			clearCache();
@@ -398,6 +425,12 @@ bool IonColourFilter::setProperty(  unsigned int key,
 				return false;
 			break;
 		}
+		case KEY_IONCOLOURFILTER_AUTOEXTREMA:
+		{
+			if(!applyPropertyNow(autoExtrema,value,needUpdate))
+				return false;
+			break;
+		}
 		default:
 			ASSERT(false);
 	}	
@@ -427,7 +460,7 @@ bool IonColourFilter::writeState(std::ostream &f,unsigned int format, unsigned i
 			f << tabs(depth+1) << "<userstring value=\""<< escapeXML(userString) << "\"/>"  << endl;
 
 			f << tabs(depth+1) << "<colourmap value=\"" << colourMap << "\"/>" << endl;
-			f << tabs(depth+1) << "<extrema min=\"" << mapBounds[0] << "\" max=\"" 
+			f << tabs(depth+1) << "<extrema auto=\""  << boolStrEnc(autoExtrema) << "\" min=\"" << mapBounds[0] << "\" max=\"" 
 				<< mapBounds[1] << "\"/>" << endl;
 			f << tabs(depth+1) << "<ncolours value=\"" << nColours << "\" opacity=\"" << alpha << "\"/>" << endl;
 
@@ -474,7 +507,13 @@ bool IonColourFilter::readState(xmlNodePtr &nodePtr, const std::string &stateFil
 	if(stream_cast(colourMap,tmpStr))
 		return false;
 
-	if(colourMap>= NUM_COLOURMAPS)
+	//We defaulted to Jet prior to version 0.0.20, and did not store
+	// the colour map data properly, so it loaded as jet 
+	if( AnalysisState::getStateWriterVersion() < getVersionNumber(0,0,20)  && colourMap == 0)
+		colourMap=COLOURMAP_JET;
+		
+			
+	if(colourMap>= COLOURMAP_ENUM_END)
 	       return false;	
 	xmlFree(xmlString);
 
@@ -495,6 +534,18 @@ bool IonColourFilter::readState(xmlNodePtr &nodePtr, const std::string &stateFil
 	if(XMLHelpFwdToElem(nodePtr,"extrema"))
 		return false;
 
+	if(XMLHelpGetProp(autoExtrema,nodePtr,"auto"))
+	{
+		if(AnalysisState::getStateWriterVersion() <=19)
+		{
+			//DEPRECATE ME
+			//This property did not exist prior to 0.0.20
+			autoExtrema=false;
+		}
+		else
+			autoExtrema=true;
+	}
+
 	xmlString=xmlGetProp(nodePtr,(const xmlChar *)"min");
 	if(!xmlString)
 		return false;
@@ -637,7 +688,7 @@ bool ionCountTest()
 	delete f;
 	delete d;
 	
-	TEST(streamOut.size() == 99,"stream count");
+	TEST(streamOut.size() == 100,"stream count");
 
 	for(unsigned int ui=0;ui<streamOut.size();ui++)
 	{
