@@ -1,6 +1,6 @@
 /*
  *	ionDownsample.cpp - Filter to perform sampling without replacement on input ion data
- *	Copyright (C) 2015, D Haley 
+ *	Copyright (C) 2018, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -46,7 +46,14 @@ IonDownsampleFilter::IonDownsampleFilter()
 
 	cacheOK=false;
 	cache=true; //By default, we should cache, but decision is made higher up
+}
 
+IonDownsampleFilter::~IonDownsampleFilter()
+{
+#ifndef HAVE_CPP_1X
+	if(rsdIncoming)
+		delete rsdIncoming;
+#endif
 }
 
 void IonDownsampleFilter::initFilter(const std::vector<const FilterStreamData *> &dataIn,
@@ -69,10 +76,14 @@ void IonDownsampleFilter::initFilter(const std::vector<const FilterStreamData *>
 	if(!c)
 	{
 		//delete the old incoming range pointer
+#ifdef HAVE_CPP_1X
+		rsdIncoming.reset(nullptr);
+#else
+
 		if(rsdIncoming)
 			delete rsdIncoming;
 		rsdIncoming=0;
-
+#endif
 		//Well, don't use per-species info anymore
 		perSpecies=false;
 	}
@@ -85,10 +96,14 @@ void IonDownsampleFilter::initFilter(const std::vector<const FilterStreamData *>
 		// so we have to copy.
 		if(!rsdIncoming)
 		{
+#ifdef HAVE_CPP_1X
+			rsdIncoming.reset(new RangeStreamData());
+#else
 			rsdIncoming = new RangeStreamData;
+#endif
 			*rsdIncoming=*c;
 
-			if(ionFractions.size() != c->rangeFile->getNumIons())
+			if(ionFractions.size() != c->rangeFile->getNumIons()+1)
 			{
 				//set up some defaults; seeded from normal
 				ionFractions.resize(c->rangeFile->getNumIons()+1,fraction);
@@ -104,10 +119,13 @@ void IonDownsampleFilter::initFilter(const std::vector<const FilterStreamData *>
 			//Do a pointer comparison (its a hack, yes, but it should work)
 			if(rsdIncoming->rangeFile != c->rangeFile)
 			{
+#ifdef HAVE_CPP_1X
+				rsdIncoming.reset(new RangeStreamData());
+#else
 				//hmm, it is different. well, trash the old incoming rng
 				delete rsdIncoming;
-
 				rsdIncoming = new RangeStreamData;
+#endif
 				*rsdIncoming=*c;
 
 				ionFractions.resize(c->rangeFile->getNumIons()+1,fraction);
@@ -142,7 +160,7 @@ Filter *IonDownsampleFilter::cloneUncached() const
 	p->maxAfterFilter=maxAfterFilter;
 	p->fraction=fraction;
 	p->perSpecies=perSpecies;
-	p->rsdIncoming=rsdIncoming;
+	p->rsdIncoming=0;
 
 	p->ionFractions.resize(ionFractions.size());
 	std::copy(ionFractions.begin(),ionFractions.end(),p->ionFractions.begin());
@@ -182,6 +200,7 @@ unsigned int IonDownsampleFilter::refresh(const std::vector<const FilterStreamDa
 	{
 		propagateStreams(dataIn,getOut,STREAM_TYPE_IONS,true);
 		propagateCache(getOut);
+		progress.filterProgress=100;
 
 		return 0;
 	}
@@ -191,6 +210,15 @@ unsigned int IonDownsampleFilter::refresh(const std::vector<const FilterStreamDa
 	progress.stepName=TRANS("Sampling");
 
 	size_t totalSize = numElements(dataIn,STREAM_TYPE_IONS);
+
+	//Check to see if we have anything to do
+	if(!totalSize)
+	{
+		progress.filterProgress=100;
+		return 0;
+	}
+
+	//Switch between per-species and normal modes
 	if(!perSpecies)	
 	{
 		for(size_t ui=0;ui<dataIn.size() ;ui++)
@@ -199,8 +227,6 @@ unsigned int IonDownsampleFilter::refresh(const std::vector<const FilterStreamDa
 			{
 				case STREAM_TYPE_IONS: 
 				{
-					if(!totalSize)
-						continue;
 
 					IonStreamData *d;
 					d=new IonStreamData;
@@ -441,30 +467,22 @@ void IonDownsampleFilter::getProperties(FilterPropGroup &propertyList) const
 	FilterProperty p;
 	size_t curGroup=0;
 
-	string tmpStr;
-	stream_cast(tmpStr,fixedNumOut);
-	p.data=tmpStr;
-	p.name=TRANS("By Count");
-	p.key=KEY_IONDOWNSAMPLE_FIXEDOUT;
-	p.type=PROPERTY_TYPE_BOOL;
-	p.helpText=TRANS("Sample up to a fixed number of ions");
-	propertyList.addProperty(p,curGroup);
+	FilterProperty pFixedOut(KEY_IONDOWNSAMPLE_FIXEDOUT, TRANS("By Count"),
+		 fixedNumOut,TRANS("Sample up to a fixed number of ions"));
+	propertyList.addProperty(pFixedOut,curGroup);
 
 	if(rsdIncoming)
 	{
-		stream_cast(tmpStr,perSpecies);
-		p.name=TRANS("Per Species");
-		p.data=tmpStr;
-		p.key=KEY_IONDOWNSAMPLE_PERSPECIES;
-		p.type=PROPERTY_TYPE_BOOL;
-		p.helpText=TRANS("Use species specific (from ranging) sampling values");
-		propertyList.addProperty(p,curGroup);
+		FilterProperty pPerSpecies(KEY_IONDOWNSAMPLE_PERSPECIES, TRANS("Per Species"),
+			 perSpecies,TRANS("Use species specific (from ranging) sampling values"));
+		propertyList.addProperty(pPerSpecies,curGroup);
 	}	
 
 
 	propertyList.setGroupTitle(curGroup,TRANS("Mode"));
 	curGroup++;
-	if(rsdIncoming && perSpecies)
+
+	if(rsdIncoming && perSpecies && rsdIncoming->enabledIons.size())
 	{
 		unsigned int typeVal;
 		if(fixedNumOut)
@@ -472,6 +490,7 @@ void IonDownsampleFilter::getProperties(FilterPropGroup &propertyList) const
 		else
 			typeVal=PROPERTY_TYPE_REAL;
 
+		std::string tmpStr;
 		unsigned int numIons=rsdIncoming->enabledIons.size();
 		//create a  single line for each
 		for(unsigned  int ui=0; ui<numIons; ui++)
@@ -508,24 +527,17 @@ void IonDownsampleFilter::getProperties(FilterPropGroup &propertyList) const
 	{
 		if(fixedNumOut)
 		{
-			stream_cast(tmpStr,maxAfterFilter);
-			p.key=KEY_IONDOWNSAMPLE_COUNT;
-			p.name=TRANS("Output Count");
-			p.data=tmpStr;
-			p.type=PROPERTY_TYPE_INTEGER;
-			p.helpText=TRANS("Sample up to this value of points");
+			FilterProperty pOutputCount(KEY_IONDOWNSAMPLE_COUNT, TRANS("Output Count"),
+				 (unsigned int)maxAfterFilter,TRANS("Sample up to this value of points"));
+			propertyList.addProperty(pOutputCount,curGroup);
 		}
 		else
 		{
-			stream_cast(tmpStr,fraction);
-			p.name=TRANS("Out Fraction");
-			p.data=tmpStr;
-			p.key=KEY_IONDOWNSAMPLE_FRACTION;
-			p.type=PROPERTY_TYPE_REAL;
-			p.helpText=TRANS("Sample this fraction of points");
+			FilterProperty pOutputFrac(KEY_IONDOWNSAMPLE_COUNT, TRANS("Out Fraction"),
+				 float(fraction),TRANS("Sample this fraction of points"));
+			propertyList.addProperty(pOutputFrac,curGroup);
 
 		}
-		propertyList.addProperty(p,curGroup);
 		propertyList.setGroupTitle(curGroup,TRANS("Sampling rates"));
 	}
 }
