@@ -1,6 +1,6 @@
 /*
  *	ionInfo.cpp -Filter to compute various properties of valued point cloud
- *	Copyright (C) 2015, D Haley 
+ *	Copyright (C) 2018, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -19,6 +19,9 @@
 
 #include "filterCommon.h"
 #include "algorithms/mass.h"
+#include "algorithms/convexHull.h"
+
+#include "config.h"
 
 using std::vector;
 using std::string;
@@ -197,7 +200,7 @@ unsigned int IonInfoFilter::refresh(const std::vector<const FilterStreamData *>
 
 	if(!numTotalPoints)
 	{
-		consoleOutput.push_back((TRANS("No ions")));
+		appendConsoleMessage((TRANS("No ions")));
 		return 0;
 	}
 
@@ -223,8 +226,8 @@ unsigned int IonInfoFilter::refresh(const std::vector<const FilterStreamData *>
 				if(doFitBackground(dataIn,backParams))
 				{
 					//display a warning that the background failed
-					consoleOutput.push_back(TRANS("Background fit failed - input data was considered ill formed (gauss-test)"));
-					consoleOutput.push_back(TRANS("Following data has not been corrected"));
+					appendConsoleMessage(TRANS("Background fit failed - input data was considered ill formed (gauss-test)"));
+					appendConsoleMessage(TRANS("Following data has not been corrected"));
 				}
 				else 
 				{
@@ -277,7 +280,7 @@ unsigned int IonInfoFilter::refresh(const std::vector<const FilterStreamData *>
 			
 			stream_cast(str,numTotalPoints);
 			str=std::string(TRANS("--Counts--") );
-			consoleOutput.push_back(str);
+			appendConsoleMessage(str);
 			
 			//sum all ions *except* the unranged.
 			for(size_t ui=0;ui<numIons.size()-1;ui++)
@@ -293,8 +296,8 @@ unsigned int IonInfoFilter::refresh(const std::vector<const FilterStreamData *>
 				stream_cast(str,numTotalPoints);
 				str=TRANS("Total (incl. unranged)\t")+str;
 			}
-			consoleOutput.push_back(str);
-			consoleOutput.push_back("");
+			appendConsoleMessage(str);
+			appendConsoleMessage("");
 
 			//Print out the ion count table
 			for(size_t ui=0;ui<numIons.size();ui++)
@@ -317,10 +320,10 @@ unsigned int IonInfoFilter::refresh(const std::vector<const FilterStreamData *>
 					str=std::string(TRANS("Unranged")) + std::string("\t") + str;
 				}
 
-				consoleOutput.push_back(str);
+				appendConsoleMessage(str);
 			}
 			str=std::string("----------");
-			consoleOutput.push_back(str);
+			appendConsoleMessage(str);
 	
 		}
 		else
@@ -328,7 +331,7 @@ unsigned int IonInfoFilter::refresh(const std::vector<const FilterStreamData *>
 			//ok, no ranges -- just give us the total
 			stream_cast(str,numTotalPoints);
 			str=std::string(TRANS("Number of points : ") )+ str;
-			consoleOutput.push_back(str);
+			appendConsoleMessage(str);
 		}
 	}
 
@@ -358,10 +361,10 @@ unsigned int IonInfoFilter::refresh(const std::vector<const FilterStreamData *>
 					
 					s=TRANS("Rectilinear Bounds : ");
 					s+= tmpLow + " / "  + tmpHi;
-					consoleOutput.push_back(s);
+					appendConsoleMessage(s);
 					
 					stream_cast(s,computedVol);
-					consoleOutput.push_back(string(TRANS("Volume (len^3): ")) + s);
+					appendConsoleMessage(string(TRANS("Volume (len^3): ")) + s);
 				}
 
 				break;
@@ -378,10 +381,10 @@ unsigned int IonInfoFilter::refresh(const std::vector<const FilterStreamData *>
 				stream_cast(s,computedVol);
 				if(computedVol>0)
 				{
-					consoleOutput.push_back(string(TRANS("Convex Volume (len^3): ")) + s);
+					appendConsoleMessage(string(TRANS("Convex Volume (len^3): ")) + s);
 				}
 				else
-					consoleOutput.push_back(string(TRANS("Unable to compute volume")));
+					appendConsoleMessage(string(TRANS("Unable to compute volume")));
 
 
 				break;
@@ -398,7 +401,7 @@ unsigned int IonInfoFilter::refresh(const std::vector<const FilterStreamData *>
 
 
 	//"Pairwise events" - where we perform an action if both 
-	//These
+	//these are set
 	if(wantIonCounts && wantVolume)
 	{
 		if(computedVol > sqrtf(std::numeric_limits<float>::epsilon()))
@@ -410,16 +413,19 @@ unsigned int IonInfoFilter::refresh(const std::vector<const FilterStreamData *>
 			{
 				density=(float)numRanged/computedVol;
 				stream_cast(s,density);
-				consoleOutput.push_back(string(TRANS("Ranged Density (pts/vol):")) + s );
+				appendConsoleMessage(string(TRANS("Ranged Density (pts/vol):")) + s );
 			}
 			
 			density=(float)numTotalPoints/computedVol;
 			stream_cast(s,density);
-			consoleOutput.push_back(string(TRANS("Total Density (pts/vol):")) + s );
+			appendConsoleMessage(string(TRANS("Total Density (pts/vol):")) + s );
 	
 		}
 	}
 
+
+	progress.filterProgress=100;
+	
 	return 0;
 }
 
@@ -438,34 +444,27 @@ void IonInfoFilter::getProperties(FilterPropGroup &propertyList) const
 	vector<pair<unsigned int,string> > choices;
 	string tmpStr;
 
-	stream_cast(str,wantIonCounts);
-	p.key=IONINFO_KEY_TOTALS;
 	if(range)
 	{
-		p.name=TRANS("Compositions");
-		p.helpText=TRANS("Display compositional data for points in console");
+		FilterProperty pCounts(IONINFO_KEY_TOTALS, TRANS("Compositions"),
+			 wantIonCounts,TRANS("Display compositional data for points in console"));
+						
+		propertyList.addProperty(pCounts,curGroup);
 	}
 	else
 	{
-		p.name=TRANS("Counts");
-		p.helpText=TRANS("Display count data for points in console");
+		FilterProperty pCounts(IONINFO_KEY_TOTALS, TRANS("Counts"),
+			 wantIonCounts,TRANS("Display count data for points in console"));
+		propertyList.addProperty(pCounts,curGroup);
 	}
-	p.data= str;
-	p.type=PROPERTY_TYPE_BOOL;
-	propertyList.addProperty(p,curGroup);
 
 
 	propertyList.setGroupTitle(curGroup,TRANS("Ion data"));
 	if(wantIonCounts && range)
 	{
-		stream_cast(str,wantNormalise);
-		p.name=TRANS("Normalise");
-		p.data=str;
-		p.key=IONINFO_KEY_NORMALISE;
-		p.type=PROPERTY_TYPE_BOOL;
-		p.helpText=TRANS("Normalise count data");
-
-		propertyList.addProperty(p,curGroup);
+		FilterProperty pNormalise(IONINFO_KEY_NORMALISE, TRANS("Normalise"),
+			 wantNormalise,TRANS("Normalise count data by totals"));
+		propertyList.addProperty(pNormalise,curGroup);
 
 		/*
 		p.name=TRANS("Back. Correct");
@@ -525,13 +524,9 @@ void IonInfoFilter::getProperties(FilterPropGroup &propertyList) const
 
 	curGroup++;
 
-	stream_cast(str,wantVolume);
-	p.key=IONINFO_KEY_VOLUME;
-	p.name=TRANS("Volume");
-	p.data= str;
-	p.type=PROPERTY_TYPE_BOOL;
-	p.helpText=TRANS("Compute volume for point data");
-	propertyList.addProperty(p,curGroup);
+	FilterProperty pVolume(IONINFO_KEY_VOLUME, TRANS("Volume"),
+			 wantVolume,TRANS("Compute volume for point data"));
+	propertyList.addProperty(pVolume,curGroup);
 
 	if(wantVolume)
 	{	
@@ -542,22 +537,9 @@ void IonInfoFilter::getProperties(FilterPropGroup &propertyList) const
 						TRANS(volumeModeString[ui])));
 		}
 		
-		tmpStr= choiceString(choices,volumeAlgorithm);
-		p.name=TRANS("Algorithm");
-		p.data=tmpStr;
-		p.type=PROPERTY_TYPE_CHOICE;
-		p.helpText=TRANS("Select volume counting technique");
-		p.key=IONINFO_KEY_VOLUME_ALGORITHM;
-		propertyList.addProperty(p,curGroup);
-
-
-		switch(volumeAlgorithm)
-		{
-			case VOLUME_MODE_RECTILINEAR:
-			case VOLUME_MODE_CONVEX:
-				break;
-		}
-	
+		FilterProperty pAlgorithm(IONINFO_KEY_VOLUME_ALGORITHM, TRANS("Algorithm"),
+				 choices,volumeAlgorithm,TRANS("Select volume counting technique"));
+		propertyList.addProperty(pAlgorithm,curGroup);
 	}
 	propertyList.setGroupTitle(curGroup,TRANS("Volume data"));
 }
@@ -703,27 +685,25 @@ unsigned int IonInfoFilter::convexHullEstimateVol(const vector<const FilterStrea
 	//Compute the convex hull, leaving the qhull data structure intact
 	const bool NO_FREE_QHULL=false;
 	const bool WANT_QHULL_VOL=true;
+
 	vector<Point3D> hullPts;
+#if HAVE_SWEEP_HULL
+	vector<HULLTRI> facets;
+	if(computeConvexHull(data,&dummyProgress,hullPts,facets))
+		return ERR_BAD_QHULL;
+
+	volume = getConvexVolume(hullPts,facets);
+#else
 	if(computeConvexHull(data,&dummyProgress,
 			hullPts,WANT_QHULL_VOL,NO_FREE_QHULL))
 		return ERR_BAD_QHULL;
 
-	Point3D midPt(0,0,0);
-
-	for(size_t ui=0;ui<hullPts.size();ui++)
-		midPt+=hullPts[ui];
-	
-	midPt*=1.0f/(float)hullPts.size();
-
-	//We don't need this result anymore
-	hullPts.clear();
-
 	volume=qh totvol;
 
-
 	//Free the convex hull mem
 	freeConvexHull();
-
+#endif
+	
 	return 0;
 }
 
@@ -947,7 +927,7 @@ bool volumeBoxTest()
 	//No ions come out of the info
 	TEST(streamOut.empty(),"stream size test");
 
-	vector<string> consoleStrings;
+	vector<ConsoleMessage > consoleStrings;
 	f->getConsoleStrings(consoleStrings); 
 	
 	//weak test for the console string size
@@ -1016,13 +996,14 @@ bool volumeSphereTest()
 	//No ions come out of the info
 	TEST(streamOut.empty(),"stream size test");
 
-	vector<string> consoleStrings;
+	vector<ConsoleMessage > consoleStrings;
 	f->getConsoleStrings(consoleStrings); 
 
 	//weak test for the console string size
 	TEST(consoleStrings.size(), "console strings existence test");
 
 
+#if !defined(HAVE_SWEEP_HULL)
 	float volMeasure,volReal;
 	volMeasure=f->getLastVolume();
 	//Bounding box for sphere is diameter^3.
@@ -1034,8 +1015,7 @@ bool volumeSphereTest()
 	TEST(f->setProperty(IONINFO_KEY_VOLUME_ALGORITHM,
 		volumeModeString[VOLUME_MODE_CONVEX],needUp),"Set prop");
 	
-	vector<string> dummy;
-	f->getConsoleStrings(dummy);
+	f->getConsoleStrings(consoleStrings);
 
 	TEST(!f->refresh(streamIn,streamOut,p),"refresh error code");
 
@@ -1046,7 +1026,9 @@ bool volumeSphereTest()
 	TEST(fabs(volMeasure -volReal) < 0.05*volReal, "volume test, convex est. of sphere");
 	
 	TEST(consoleStrings.size(), "console strings existence test");
-
+#else
+	MESSAGE("Skipping spherical volume test when using SHULL- upstream has been notified of the problem...");
+#endif
 	delete d;
 	delete f;
 	return true;
