@@ -1,6 +1,6 @@
 /*
  *	profile.cpp - Compute composition or density profiles from valued point clouds
- *	Copyright (C) 2015, D Haley 
+ *	Copyright (C) 2018, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -328,26 +328,30 @@ unsigned int ProfileFilter::refresh(const std::vector<const FilterStreamData *>
 	vector<vector<size_t> > ionFrequencies;
 	
 	RangeStreamData *rngData=0;
-	for(unsigned int ui=0;ui<dataIn.size() ;ui++)
+	if(!wantDensity)
 	{
-		if(dataIn[ui]->getStreamType() == STREAM_TYPE_RANGE)
+		for(unsigned int ui=0;ui<dataIn.size() ;ui++)
 		{
-			rngData =((RangeStreamData *)dataIn[ui]);
-			break;
+			if(dataIn[ui]->getStreamType() == STREAM_TYPE_RANGE)
+			{
+				rngData =((RangeStreamData *)dataIn[ui]);
+				break;
+			}
 		}
 	}
 
-	unsigned int numBins, errCode;
+	unsigned int numBins;
 	{
+	unsigned int errCode;
 	float length;
 	errCode=getBinData(numBins,length);
 
 	if(!numBins)
 		return 0;
-	}
 
 	if(errCode)
 		return errCode;
+	}
 
 	//Indirection vector to convert ionFrequencies position to ionID mapping.
 	//Should only be used in conjunction with rngData == true
@@ -408,7 +412,7 @@ unsigned int ProfileFilter::refresh(const std::vector<const FilterStreamData *>
 
 
 	size_t n=0;
-	size_t totalSize=numElements(dataIn);
+	size_t totalSize=max(numElements(dataIn),(size_t)1);
 
 	map<size_t,size_t> primitiveMap;
 	primitiveMap[PRIMITIVE_CYLINDER_AXIAL] = CROP_CYLINDER_INSIDE_AXIAL;
@@ -596,7 +600,12 @@ unsigned int ProfileFilter::refresh(const std::vector<const FilterStreamData *>
 			//sum composition = 1 otherwise use volume of bin
 			//as normalisation factor
 			if(rngData)
+			{
 				plotData[ui]->yLabel= TRANS("Fraction");
+				//Set y bounds to [0,1],
+				plotData[ui]->setPlotBound(1,0,0);
+				plotData[ui]->setPlotBound(1,1,1);
+			}
 			else
 				plotData[ui]->yLabel= TRANS("Density (\\frac{\\#}{len^3})");
 		}
@@ -677,7 +686,7 @@ unsigned int ProfileFilter::refresh(const std::vector<const FilterStreamData *>
 		}
 		else
 		{
-			consoleOutput.push_back(TRANS("No data remained in profile - cannot display result"));
+			appendConsoleMessage(TRANS("No data remained in profile - cannot display result"));
 			delete plotData[ui];
 		}
 	}
@@ -963,13 +972,9 @@ void ProfileFilter::getProperties(FilterPropGroup &propertyList) const
 
 	if(haveRangeParent)
 	{
-		stream_cast(tmpStr,wantDensity);
-		p.name=TRANS("Total Density");	
-		p.data=tmpStr;
-		p.key=PROFILE_KEY_DENSITY_ONLY;
-		p.type=PROPERTY_TYPE_BOOL;
-		p.helpText=TRANS("Do not do per-species analysis, perform density computation only");
-		propertyList.addProperty(p,curGroup);
+		FilterProperty pTotalDensity(PROFILE_KEY_DENSITY_ONLY, TRANS("Total Density"),
+				wantDensity,TRANS("Do not do per-species analysis, perform density computation only"));
+		propertyList.addProperty(pTotalDensity,curGroup);
 	}
 
 	//Allow primitive selection if we have more than one primitive
@@ -982,24 +987,17 @@ void ProfileFilter::getProperties(FilterPropGroup &propertyList) const
 			str =TRANS(PRIMITIVE_NAME[ui]);
 			choices.push_back(make_pair(ui,str));
 		}
-		p.name=TRANS("Primitive type");
-		p.data=choiceString(choices,primitiveType);
-		p.key=PROFILE_KEY_PRIMITIVETYPE;
-		p.type=PROPERTY_TYPE_CHOICE;
-		p.helpText=TRANS("Basic shape to use for profile");
-		propertyList.addProperty(p,curGroup);
+		FilterProperty pPrimitive(PROFILE_KEY_PRIMITIVETYPE, TRANS("Primitive type"),
+				choices,primitiveType,TRANS("Basic shape to use for profile"));
+		propertyList.addProperty(pPrimitive,curGroup);
 		propertyList.setGroupTitle(curGroup,TRANS("Primitive"));	
 		curGroup++;
 	}
 
 	
-	stream_cast(tmpStr,showPrimitive);
-	p.name=TRANS("Show Primitive");	
-	p.data=tmpStr;
-	p.key=PROFILE_KEY_SHOWPRIMITIVE;
-	p.type=PROPERTY_TYPE_BOOL;
-	p.helpText=TRANS("Display the 3D composition profile interaction object");
-	propertyList.addProperty(p,curGroup);
+	FilterProperty pShowPrimitive(PROFILE_KEY_SHOWPRIMITIVE, TRANS("Show Primitive"),
+			showPrimitive,TRANS("Display the 3D composition profile interaction object"));
+	propertyList.addProperty(pShowPrimitive,curGroup);
 
 	switch(primitiveType)
 	{
@@ -1008,37 +1006,20 @@ void ProfileFilter::getProperties(FilterPropGroup &propertyList) const
 		{
 			ASSERT(vectorParams.size() == 2);
 			ASSERT(scalarParams.size() == 1);
-			stream_cast(str,vectorParams[0]);
-			p.key=PROFILE_KEY_ORIGIN;
-			p.name=TRANS("Origin");
-			p.data=str;
-			p.type=PROPERTY_TYPE_POINT3D;
-			p.helpText=TRANS("Position for centre of cylinder");
-			propertyList.addProperty(p,curGroup);
+			FilterProperty pOrigin(PROFILE_KEY_ORIGIN, TRANS("Origin"),
+					vectorParams[0],TRANS("Position for centre of cylinder"));
+			propertyList.addProperty(pOrigin,curGroup);
+			FilterProperty pAxis(PROFILE_KEY_NORMAL, TRANS("Axis"),
+					vectorParams[1],TRANS("Vector between ends of cylinder"));
+			propertyList.addProperty(pAxis,curGroup);
 			
-			stream_cast(str,vectorParams[1]);
-			p.key=PROFILE_KEY_NORMAL;
-			p.name=TRANS("Axis");
-			p.data=str;
-			p.type=PROPERTY_TYPE_POINT3D;
-			p.helpText=TRANS("Vector between ends of cylinder");
-			propertyList.addProperty(p,curGroup);
-
-			str=boolStrEnc(lockAxisMag);
-			p.key=PROFILE_KEY_LOCKAXISMAG;
-			p.name=TRANS("Lock Axis Mag.");
-			p.data= str;
-			p.type=PROPERTY_TYPE_BOOL;
-			p.helpText=TRANS("Prevent length of cylinder changing during interaction");
-			propertyList.addProperty(p,curGroup);
+			FilterProperty pLockAxis(PROFILE_KEY_LOCKAXISMAG, TRANS("Lock Axis Mag"),
+					lockAxisMag,TRANS("Prevent length of cylinder changing during interaction"));
+			propertyList.addProperty(pLockAxis,curGroup);
+			FilterProperty pRadius(PROFILE_KEY_RADIUS, TRANS("Radius"),
+					scalarParams[0],TRANS("Radius of cylinder"));
+			propertyList.addProperty(pRadius,curGroup);
 			
-			stream_cast(str,scalarParams[0]);
-			p.key=PROFILE_KEY_RADIUS;
-			p.name=TRANS("Radius");
-			p.data= str;
-			p.type=PROPERTY_TYPE_POINT3D;
-			p.helpText=TRANS("Radius of cylinder");
-			propertyList.addProperty(p,curGroup);
 			break;
 		}
 		case PRIMITIVE_SPHERE:
@@ -1046,21 +1027,13 @@ void ProfileFilter::getProperties(FilterPropGroup &propertyList) const
 			
 			ASSERT(vectorParams.size() == 1);
 			ASSERT(scalarParams.size() == 1);
-			stream_cast(str,vectorParams[0]);
-			p.key=PROFILE_KEY_ORIGIN;
-			p.name=TRANS("Origin");
-			p.data=str;
-			p.type=PROPERTY_TYPE_POINT3D;
-			p.helpText=TRANS("Position for centre of sphere");
-			propertyList.addProperty(p,curGroup);
+			FilterProperty pAxis(PROFILE_KEY_ORIGIN, TRANS("Origin"),
+					vectorParams[0],TRANS("Position for centre of sphere"));
+			propertyList.addProperty(pAxis,curGroup);
 			
-			stream_cast(str,scalarParams[0]);
-			p.key=PROFILE_KEY_RADIUS;
-			p.name=TRANS("Radius");
-			p.data= str;
-			p.type=PROPERTY_TYPE_POINT3D;
-			p.helpText=TRANS("Radius of sphere");
-			propertyList.addProperty(p,curGroup);
+			FilterProperty pRadius(PROFILE_KEY_RADIUS, TRANS("Radius"),
+					scalarParams[0],TRANS("Radius of sphere"));
+			propertyList.addProperty(pRadius,curGroup);
 			break;
 		}
 		default:
@@ -1070,67 +1043,38 @@ void ProfileFilter::getProperties(FilterPropGroup &propertyList) const
 	//Must be fixed bin num in radial mode. Disallow turning this off
 	if(primitiveType!= PRIMITIVE_CYLINDER_RADIAL)
 	{
-		p.key=PROFILE_KEY_FIXEDBINS;
-		stream_cast(str,fixedBins);
-		p.name=TRANS("Fixed Bin Num");
-		p.data=str;
-		p.type=PROPERTY_TYPE_BOOL;
-		p.helpText=TRANS("If true, use a fixed number of bins for profile, otherwise use fixed step size");
-		propertyList.addProperty(p,curGroup);
+		FilterProperty pFixedBins(PROFILE_KEY_FIXEDBINS, TRANS("Fixed Bin Num"),
+				fixedBins,TRANS("If true, use a fixed number of bins for profile, otherwise use fixed step size"));
+		propertyList.addProperty(pFixedBins,curGroup);
 	}
 
 	if(fixedBins)
 	{
-		stream_cast(tmpStr,nBins);
-		str = TRANS("Num Bins");
-		p.name=str;
-		p.data=tmpStr;
-		p.key=PROFILE_KEY_NUMBINS;
-		p.type=PROPERTY_TYPE_INTEGER;
-		p.helpText=TRANS("Number of bins to use for profile");
-		propertyList.addProperty(p,curGroup);
+		FilterProperty pNumBins(PROFILE_KEY_NUMBINS, TRANS("Num Bins"),
+				nBins,TRANS("Number of bins to use for profile"));
+		propertyList.addProperty(pNumBins,curGroup);
 	}
 	else
 	{
 		ASSERT(primitiveType!=PRIMITIVE_CYLINDER_RADIAL);
-		str = TRANS("Bin width");
-		stream_cast(tmpStr,binWidth);
-		p.name=str;
-		p.data=tmpStr;
-		p.key=PROFILE_KEY_BINWIDTH;
-		p.type=PROPERTY_TYPE_REAL;
-		p.helpText=TRANS("Size of each bin in profile");
-		propertyList.addProperty(p,curGroup);
+		FilterProperty pBinWidth(PROFILE_KEY_BINWIDTH, TRANS("Bin Width"),
+						nBins,TRANS("Size of each bin in profile"));
+		propertyList.addProperty(pBinWidth,curGroup);
 	}
 
-	stream_cast(tmpStr,normalise);
-	p.name= TRANS("Normalise");	
-	p.data=tmpStr;
-	p.key=PROFILE_KEY_NORMALISE;
-	p.type=PROPERTY_TYPE_BOOL;
-	p.helpText=TRANS("Convert bin counts into relative frequencies in each bin");
-	propertyList.addProperty(p,curGroup);
-
-	stream_cast(tmpStr,minEvents);
-	p.name= TRANS("Min. events");	
-	p.data=tmpStr;
-	p.key=PROFILE_KEY_MINEVENTS;
-	p.type=PROPERTY_TYPE_INTEGER;
-	p.helpText=TRANS("Drop data that does not have this many events");
-	propertyList.addProperty(p,curGroup);
-
+	FilterProperty pNormalise(PROFILE_KEY_NORMALISE, TRANS("Normalise"),
+		nBins,TRANS("Convert bin counts into relative frequencies in each bin"));
+	propertyList.addProperty(pNormalise,curGroup);
+	
+	FilterProperty pMinEvents(PROFILE_KEY_MINEVENTS, TRANS("Min. Events"),
+		minEvents,TRANS("Drop data that does not have this many events"));
+	propertyList.addProperty(pMinEvents,curGroup);
+	
 	propertyList.setGroupTitle(curGroup,TRANS("Settings"));	
-
-
-
 	curGroup++;
 	
-	//use set 2 to store the plot properties
-	stream_cast(str,plotStyle);
 	//Let the user know what the valid values for plot type are
 	vector<pair<unsigned int,string> > choices;
-
-
 	tmpStr=plotString(PLOT_LINE_LINES);
 	choices.push_back(make_pair((unsigned int) PLOT_LINE_LINES,tmpStr));
 	tmpStr=plotString(PLOT_LINE_BARS);
@@ -1140,24 +1084,16 @@ void ProfileFilter::getProperties(FilterPropGroup &propertyList) const
 	tmpStr=plotString(PLOT_LINE_STEM);
 	choices.push_back(make_pair((unsigned int)PLOT_LINE_STEM,tmpStr));
 
-	tmpStr= choiceString(choices,plotStyle);
-	p.name=TRANS("Plot Type");
-	p.data=tmpStr;
-	p.type=PROPERTY_TYPE_CHOICE;
-	p.helpText=TRANS("Visual style for plot");
-	p.key=PROFILE_KEY_PLOTTYPE;
-	propertyList.addProperty(p,curGroup);
+	FilterProperty pPlotType(PROFILE_KEY_PLOTTYPE, TRANS("Plot Type"),
+			choices,plotStyle,TRANS("Visual style for plot"));
+	propertyList.addProperty(pPlotType,curGroup);
 
 	//If we are not doing per-species, then we need colour
 	if(doDensityPlot)
 	{
-		//Convert the colour to a hex string
-		p.name=TRANS("Colour");
-		p.data=rgba.toColourRGBA().rgbString();
-		p.type=PROPERTY_TYPE_COLOUR;
-		p.helpText=TRANS("Colour of plot");
-		p.key=PROFILE_KEY_COLOUR;
-		propertyList.addProperty(p,curGroup);
+		FilterProperty pColour(PROFILE_KEY_COLOUR, TRANS("Colour"),
+			rgba,TRANS("Colour of plot"));
+		propertyList.addProperty(pColour,curGroup);
 	}
 
 
@@ -1187,7 +1123,8 @@ void ProfileFilter::getProperties(FilterPropGroup &propertyList) const
 		p.helpText=TRANS("Number of bins to include in moving average filter");
 		p.key=PROFILE_KEY_AVGWINSIZE;
 		propertyList.addProperty(p,curGroup);
-	}	
+	}
+	
 	propertyList.setGroupTitle(curGroup,TRANS("Error analysis"));
 }
 
@@ -1298,7 +1235,7 @@ bool ProfileFilter::writeState(std::ostream &f,unsigned int format, unsigned int
 
 			f << tabs(depth+1) << "<scalarparams>" << endl;
 			for(unsigned int ui=0; ui<scalarParams.size(); ui++)
-				f << tabs(depth+2) << "<scalar value=\"" << scalarParams[0] << "\"/>" << endl; 
+				f << tabs(depth+2) << "<scalar value=\"" << scalarParams[ui] << "\"/>" << endl; 
 			
 			f << tabs(depth+1) << "</scalarparams>" << endl;
 			f << tabs(depth+1) << "<normalise value=\"" << normalise << "\" minevents=\"" << minEvents << "\" />" << endl;
