@@ -1,6 +1,6 @@
 /*
  *	rangeFile.cpp - bins ions into different value ranges given an input range file
- *	Copyright (C) 2015, D Haley 
+ *	Copyright (C) 2018, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -113,6 +113,7 @@ unsigned int RangeFileFilter::refresh(const std::vector<const FilterStreamData *
 		//these.
 		propagateStreams(dataIn,getOut,getRefreshBlockMask(),true);
 			
+		progress.filterProgress=100;
 		return 0;
 	}
 
@@ -1125,13 +1126,8 @@ bool RangeFileFilter::readState(xmlNodePtr &nodePtr, const std::string &stateFil
 	rngName=(char *)xmlString;
 	xmlFree(xmlString);
 
-	//Override the string to strip leading ./ notation, as needed
-	if( (stateFileDir.size()) &&
-		(rngName.size() > 2 && rngName.substr(0,2) == "./") )
-	{
-		rngName=stateFileDir + rngName.substr(2);
-	}
 
+	convertFileStringToAbsolute(stateFileDir,rngName);
 	rngName=convertFileStringToNative(rngName);
 
 	//try using the extension name of the file to guess format
