@@ -1,6 +1,6 @@
 /*
  *	spatialAnalysis.h - Perform various data analysis on 3D point clouds
- *	Copyright (C) 2015, D Haley 
+ *	Copyright (C) 2018, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -41,12 +41,15 @@ class SpatialAnalysisFilter : public Filter
 
 		//!Do we have range data to use (is nonzero)
 		bool haveRangeParent;
-		//!The names of the incoming ions
+		//!The names of the incoming ions, which may be smaller than the rangefile
+		// that provided this list
 		std::vector<std::string > ionNames;
 		//!Are the sources/targets enabled for a  particular incoming range?
+		// these are of the same size as ionNames
 		std::vector<bool> ionSourceEnabled,ionTargetEnabled;
 
 		//!work out which ions to count in the numerator vs denominator
+		// these are of the same size as ionNames
 		std::vector<bool> ionNumeratorEnabled,ionDenominatorEnabled;
 
 		//RDF specific params
@@ -66,6 +69,9 @@ class SpatialAnalysisFilter : public Filter
 		
 		//!Do we want to display theoretical random NN distances on top?
 		bool wantRandomNNHist;
+
+		//!Do we want to perform normalisation of the RDF?
+		bool normaliseRDF;
 		//--------
 		
 		//Density filtering specific params
@@ -129,6 +135,19 @@ class SpatialAnalysisFilter : public Filter
 		//should we replace the current mass by the other file's ?
 		bool replaceMass;
 		//---------
+		
+		//Axial DF sweep specific
+		//---------
+		//Step size in degrees
+		float dAngularStep;
+		//Start and stop of sweep, in degrees (theta, then phi)
+		float angleStartStop[2][2];
+
+		//do we want to use log-scaling for the DF
+		bool wantDFLogScaled;
+
+		//Which colourmap do we want to use for the DF sweep?
+		unsigned int dfColourMap;
 	
 		//Radial distribution function - creates a 1D histogram of spherical atom counts, centered around each atom
 		size_t algorithmRDF(ProgressData &progress, size_t totalDataSize, 
@@ -165,22 +184,29 @@ class SpatialAnalysisFilter : public Filter
 		size_t algorithmLocalConcentration(ProgressData &progress, size_t totalDataSize, 
 			const std::vector<const FilterStreamData *>  &dataIn, 
 			std::vector<const FilterStreamData * > &getOut, const RangeFile *rngF);
-		//Create a 3D manipulable cylinder as an output drawable
-		// using the parameters stored inside the vector/scalar params
-		// both parameters are outputs from this function
-		void createCylinder(DrawStreamData* &d, SelectionDevice * &s) const;
-
-
-		//Wrapper routeine to create the appropriate selection
-		// device for whatever algorithm is in use; device list will be appended to
-		// and if needed, output object will be generated 
-		void createDevice(std::vector<const FilterStreamData *> &getOut);
+		
+		//Local concentration filtering algorithm
+		//	Filter hits based on their local concentration
+		size_t algorithmLocalConcentrationFilter(ProgressData &progress, size_t totalDataSize, 
+			const std::vector<const FilterStreamData *>  &dataIn, 
+			std::vector<const FilterStreamData * > &getOut, const RangeFile *rngF);
 
+		//This is unpublished (!!!) work, in which we perform fitting
+		size_t algorithmAxialDfSweep(ProgressData &progress,
+			size_t totalDataSize, const vector<const FilterStreamData *>  &dataIn, 
+			vector<const FilterStreamData * > &getOut);
 
 		//From the given input ions, filter them down using the user
 		// selection for ranges. If sourceFilter is true, filter by user
 		// source selection, otherwise by user target selection
 		void filterSelectedRanges(const std::vector<IonHit> &ions, bool sourceFilter, const RangeFile *rngF, std::vector<IonHit> &output) const;
+
+
+		//!Expand a "sublist" of enabled (ions/numerator/denom) to the parent rangefile
+		// by using the ionNames. The provided vector must match ionNames in size
+		vector<bool> expandEnabledToParentRange(const vector<bool> &vb, 
+							const RangeFile *rngF) const;
+
 	public:
 		SpatialAnalysisFilter(); 
 		//!Duplicate filter contents, excluding cache.
