@@ -1,6 +1,6 @@
 /*
  *	spectrumPlot.cpp - Compute histograms of values for valued 3D point data
- *	Copyright (C) 2015, D Haley 
+ *	Copyright (C) 2018, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -151,6 +151,7 @@ unsigned int SpectrumPlotFilter::refresh(const std::vector<const FilterStreamDat
 		//Only report the spectrum plot
 		propagateCache(getOut);
 
+		progress.filterProgress=100;
 		return 0;
 	}
 
@@ -276,6 +277,12 @@ unsigned int SpectrumPlotFilter::refresh(const std::vector<const FilterStreamDat
 	d->a = rgba.a();
 
 	d->logarithmic=logarithmic;
+	//Show y=0/0.1 region on plot (0 if not log, or 0.1 if so)
+	if(d->logarithmic)
+		d->setPlotBound(1,0,0.1);
+	else
+		d->setPlotBound(1,0,0);
+
 	d->plotStyle = plotStyle;
 	d->plotMode=PLOT_MODE_1D;
 
@@ -363,9 +370,9 @@ unsigned int SpectrumPlotFilter::refresh(const std::vector<const FilterStreamDat
 		d->xyData[ui].second=0;
 	}	
 	//Compute the plot bounds
-	d->autoSetHardBounds();
+//	d->autoSetHardBounds();
 	//Limit them to 1.0 or greater (due to log)
-	d->hardMinY=std::min(1.0f,d->hardMaxY);
+//	d->hardMinY=std::min(1.0f,d->hardMaxY);
 
 
 	//Number of ions currently processed
@@ -426,10 +433,29 @@ unsigned int SpectrumPlotFilter::refresh(const std::vector<const FilterStreamDat
 
 	
 		//fit a constant tof (1/sqrt (mass)) type background
-		if(doFitBackground(dataIn,backParams))
+		unsigned int errCode=doFitBackground(dataIn,backParams);
+		if(errCode)
 		{
 			//display a warning that the background failed
-			consoleOutput.push_back(TRANS("Background fit failed - input data was considered ill formed (gauss-test)"));
+			appendConsoleMessage(TRANS("Background fit failed")); 
+
+			switch(errCode)
+			{
+				case BACKGROUND_PARAMS::FIT_FAIL_MIN_REQ_BINS:
+					appendConsoleMessage(TRANS("Insufficient bins (data points) to perform fit. Try decreasing bin size, or expanding range"));
+					break;
+				case BACKGROUND_PARAMS::FIT_FAIL_AVG_COUNTS:
+					appendConsoleMessage(TRANS("Insufficient counts  to perform fit. Try expanding range, or increasing number of ions"));
+					break;
+				case BACKGROUND_PARAMS::FIT_FAIL_INSUFF_DATA:
+					appendConsoleMessage(TRANS("Insufficient data to compute Anderson-Darling statistic"));
+					break;
+				case BACKGROUND_PARAMS::FIT_FAIL_DATA_NON_GAUSSIAN:
+					appendConsoleMessage(TRANS("Background noise does not appear to be gaussian-like, ensure range contains no peaks"));
+					break;
+				default:
+					ASSERT(false);
+			}
 		}
 		else 
 		{
@@ -517,6 +543,8 @@ unsigned int SpectrumPlotFilter::refresh(const std::vector<const FilterStreamDat
 	
 	getOut.push_back(d);
 
+	progress.filterProgress=100;
+
 	return 0;
 }
 
@@ -576,49 +604,31 @@ void SpectrumPlotFilter::getProperties(FilterPropGroup &propertyList) const
 	size_t curGroup=0;
 	string str;
 
-	stream_cast(str,binWidth);
-	p.name=TRANS("Bin width");
-	p.data=str;
-	p.key=KEY_SPECTRUM_BINWIDTH;
-	p.type=PROPERTY_TYPE_REAL;
-	p.helpText=TRANS("Step size for spectrum");
-	propertyList.addProperty(p,curGroup);
+	FilterProperty pBinWidth(KEY_SPECTRUM_BINWIDTH, TRANS("Bin Width"),
+		 binWidth,TRANS("Step size for spectrum"));
+	propertyList.addProperty(pBinWidth,curGroup);
 
-	str=boolStrEnc(autoExtrema);
+	FilterProperty pAutoExtrema(KEY_SPECTRUM_AUTOEXTREMA, TRANS("Auto Min/Max"),
+		 autoExtrema,TRANS("Automatically compute spectrum upper and lower bound"));
+	propertyList.addProperty(pAutoExtrema,curGroup);
 
-	p.name=TRANS("Auto Min/max");
-	p.data=str;
-	p.key=KEY_SPECTRUM_AUTOEXTREMA;
-	p.type=PROPERTY_TYPE_BOOL;
-	p.helpText=TRANS("Automatically compute spectrum upper and lower bound");
-	propertyList.addProperty(p,curGroup);
+	if(!autoExtrema)
+	{
+		FilterProperty pSpectrumMin(KEY_SPECTRUM_MIN, TRANS("Min"),
+			 minPlot,TRANS("Starting position for spectrum"));
+		propertyList.addProperty(pSpectrumMin,curGroup);
 
-	stream_cast(str,minPlot);
-	p.data=str;
-	p.name=TRANS("Min");
-	p.key=KEY_SPECTRUM_MIN;
-	p.type=PROPERTY_TYPE_REAL;
-	p.helpText=TRANS("Starting position for spectrum");
-	propertyList.addProperty(p,curGroup);
-
-	stream_cast(str,maxPlot);
-	p.key=KEY_SPECTRUM_MAX;
-	p.name=TRANS("Max");
-	p.data=str;
-	p.type=PROPERTY_TYPE_REAL;
-	p.helpText=TRANS("Ending position for spectrum");
-	propertyList.addProperty(p,curGroup);
+		FilterProperty pSpectrumMax(KEY_SPECTRUM_MAX, TRANS("Max"),
+			 maxPlot,TRANS("Ending position for spectrum"));
+		propertyList.addProperty(pSpectrumMax,curGroup);
+	}
 
 	propertyList.setGroupTitle(curGroup,TRANS("Data"));
 	curGroup++;
 
-	str=boolStrEnc(logarithmic);
-	p.key=KEY_SPECTRUM_LOGARITHMIC;
-	p.name=TRANS("Logarithmic");
-	p.data=str;
-	p.type=PROPERTY_TYPE_BOOL;
-	p.helpText=TRANS("Convert the plot to logarithmic mode");
-	propertyList.addProperty(p,curGroup);
+	FilterProperty pLogarithmic(KEY_SPECTRUM_LOGARITHMIC, TRANS("Logarithmic"),
+		 logarithmic,TRANS("Convert the plot to logarithmic mode"));
+	propertyList.addProperty(pLogarithmic,curGroup);
 
 	vector<pair<unsigned int,string> > choices;
 	string tmpStr;
@@ -627,34 +637,20 @@ void SpectrumPlotFilter::getProperties(FilterPropGroup &propertyList) const
 		tmpStr=TRANS(NORMALISE_STRING[ui]);
 		choices.push_back(make_pair( ui,tmpStr));
 	}
-	
-	tmpStr= choiceString(choices,normaliseMode);
-	p.name=TRANS("Normalisation");
-	p.data=tmpStr;
-	p.type=PROPERTY_TYPE_CHOICE;
-	p.helpText=TRANS("Rescale the plot height, to make inter-spectrum comparisons easier");
-	p.key=KEY_SPECTRUM_NORMALISE;
-	propertyList.addProperty(p,curGroup);
+	FilterProperty pNormalise(KEY_SPECTRUM_NORMALISE, TRANS("Normalisation"),
+		 choices,normaliseMode,TRANS("Rescale the plot height, to make inter-spectrum comparisons easier"));
+	propertyList.addProperty(pNormalise,curGroup);
 
 
 	if(normaliseMode == NORMALISE_MAX_IN_BOUND)
 	{
-		p.name=TRANS("Lower Bound");
-		stream_cast(tmpStr,normaliseBounds.first);
-		p.data=tmpStr;
-		p.type=PROPERTY_TYPE_REAL;
-		p.helpText=TRANS("Do not use data below this x-value for normalisation");
-		p.key=KEY_SPECTRUM_NORMALISE_LOWERBOUND;
-		propertyList.addProperty(p,curGroup);
-
-		p.name=TRANS("Upper Bound");
-		stream_cast(tmpStr,normaliseBounds.second);
-		p.data=tmpStr;
-		p.type=PROPERTY_TYPE_REAL;
-		p.helpText=TRANS("Do not use data above this x-value for normalisation");
-		p.key=KEY_SPECTRUM_NORMALISE_UPPERBOUND;
-		propertyList.addProperty(p,curGroup);
-
+		FilterProperty pLowerBound(KEY_SPECTRUM_NORMALISE_LOWERBOUND, TRANS("Lower Bound"),
+			 normaliseBounds.first,TRANS("Do not use data below this x-value for normalisation"));
+		propertyList.addProperty(pLowerBound,curGroup);
+		
+		FilterProperty pUpperBound(KEY_SPECTRUM_NORMALISE_UPPERBOUND, TRANS("Upper Bound"),
+			 normaliseBounds.second,TRANS("Do not use data above this x-value for normalisation"));
+		propertyList.addProperty(pUpperBound,curGroup);
 	}
 	
 	//Let the user know what the valid values for plot type are
@@ -664,21 +660,14 @@ void SpectrumPlotFilter::getProperties(FilterPropGroup &propertyList) const
 		tmpStr=plotString(ui);
 		choices.push_back(make_pair( ui,tmpStr));
 	}
+	
+	FilterProperty pPlotType(KEY_SPECTRUM_PLOTTYPE, TRANS("Plot Type"),
+		 choices,plotStyle,TRANS("Visual style of plot"));
+	propertyList.addProperty(pPlotType,curGroup);
 
-	tmpStr= choiceString(choices,plotStyle);
-	p.name=TRANS("Plot Type");
-	p.data=tmpStr;
-	p.type=PROPERTY_TYPE_CHOICE;
-	p.helpText=TRANS("Visual style of plot");
-	p.key=KEY_SPECTRUM_PLOTTYPE;
-	propertyList.addProperty(p,curGroup);
-
-	p.name=TRANS("Colour");
-	p.data=rgba.toColourRGBA().rgbaString(); 
-	p.type=PROPERTY_TYPE_COLOUR;
-	p.helpText=TRANS("Colour of plotted spectrum");
-	p.key=KEY_SPECTRUM_COLOUR;
-	propertyList.addProperty(p,curGroup);
+	FilterProperty pColour(KEY_SPECTRUM_COLOUR, TRANS("Colour"),
+		 rgba,TRANS("Colour of plotted spectrum"));
+	propertyList.addProperty(pColour,curGroup);
 
 	propertyList.setGroupTitle(curGroup,TRANS("Appearance"));
 
@@ -691,12 +680,10 @@ void SpectrumPlotFilter::getProperties(FilterPropGroup &propertyList) const
 		choices.push_back(make_pair(ui,tmpStr));
 	}
 	
-	p.name=TRANS("Model");
-	p.key=KEY_SPECTRUM_BACKMODE;
-	p.type=PROPERTY_TYPE_CHOICE;
-	p.helpText=TRANS("Fitting method to use");
-	p.data=choiceString(choices,fitMode);
-	propertyList.addProperty(p,curGroup);
+	
+	FilterProperty pFitModel(KEY_SPECTRUM_BACKMODE, TRANS("Model"),
+		 choices,fitMode,TRANS("Background fitting method to use"));
+	propertyList.addProperty(pFitModel,curGroup);
 
 
 	switch(fitMode)
@@ -704,21 +691,16 @@ void SpectrumPlotFilter::getProperties(FilterPropGroup &propertyList) const
 		case FIT_MODE_NONE:
 			break;
 		case FIT_MODE_CONST_TOF:
+		{
 			//Add start/end TOF 
-			p.name=TRANS("Fit Start");
-			p.helpText=TRANS("Start mass value for fitting background");
-			p.type=PROPERTY_TYPE_REAL;
-			p.key=KEY_SPECTRUM_BACKMODE_FLAT_START;
-			stream_cast(p.data,massBackStart);
-			propertyList.addProperty(p,curGroup);
+			FilterProperty pFitStart(KEY_SPECTRUM_BACKMODE_FLAT_START, TRANS("Fit Start"),
+				 massBackStart,TRANS("Start mass value for fitting background"));
+			propertyList.addProperty(pFitStart,curGroup);
 
-			p.name=TRANS("Fit End");
-			p.helpText=TRANS("End mass value for fitting background");
-			p.type=PROPERTY_TYPE_REAL;
-			p.key=KEY_SPECTRUM_BACKMODE_FLAT_END;
-			stream_cast(p.data,massBackEnd);
-
-			propertyList.addProperty(p,curGroup);
+			FilterProperty pFitEnd(KEY_SPECTRUM_BACKMODE_FLAT_END, TRANS("Fit End"),
+				 massBackEnd,TRANS("End mass value for fitting background"));
+			propertyList.addProperty(pFitEnd,curGroup);
+		}
 			break;
 		default:
 			ASSERT(false);
@@ -726,12 +708,9 @@ void SpectrumPlotFilter::getProperties(FilterPropGroup &propertyList) const
 
 	if(fitMode != FIT_MODE_NONE)
 	{
-		p.name=TRANS("Corr. Only");
-		p.helpText=TRANS("Only show corrected spectrum, not fit");
-		p.key=KEY_SPECTRUM_CORRECTED_ONLY;
-		p.type=PROPERTY_TYPE_BOOL;
-		p.data=boolStrEnc(showOnlyCorrected);
-		propertyList.addProperty(p,curGroup);
+		FilterProperty pShowCorrected(KEY_SPECTRUM_CORRECTED_ONLY, TRANS("Corr. Only"),
+			 showOnlyCorrected,TRANS("Only show corrected spectrum, not fit"));
+		propertyList.addProperty(pShowCorrected,curGroup);
 
 	}
 
@@ -1317,8 +1296,11 @@ IonStreamData *synDataPoints(const unsigned int span[], unsigned int numPts)
 
 bool countTest()
 {
-	using std::auto_ptr;
-	auto_ptr<IonStreamData> d;
+#ifdef __HAVE_CPP11__
+	std::unique_ptr<IonStreamData> d;
+#else
+	std::auto_ptr<IonStreamData> d;
+#endif
 	const unsigned int VOL[]={
 				10,10,10
 				};
