@@ -1,6 +1,6 @@
 /*
  *	transform.cpp - Perform geometrical transform operations on point clouds
- *	Copyright (C) 2015, D Haley 
+ *	Copyright (C) 2018, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -200,6 +200,21 @@ unsigned int TransformFilter::refresh(const std::vector<const FilterStreamData *
 		//Propagate non-ion-types into output
 		propagateStreams(dataIn,getOut, getRefreshBlockMask(),true);
 		propagateCache(getOut);
+		progress.filterProgress=100;
+
+		//If the user is using a transform mode that requires origin selection 
+		// we have to regenerate this
+		if(showOrigin && (transformMode == MODE_ROTATE ||
+				transformMode == MODE_SCALE_ANISOTROPIC || 
+				transformMode == MODE_SCALE_ISOTROPIC) )
+		{
+			SelectionDevice *s;
+			DrawStreamData *d=makeMarkerSphere(s);
+			if(s)
+				devices.push_back(s);
+
+			getOut.push_back(d);
+		}
 		return 0;
 	}
 
@@ -262,7 +277,11 @@ unsigned int TransformFilter::refresh(const std::vector<const FilterStreamData *
 					}
 				}
 			}
-			vectorParams[0]=massCentre*1.0/(float)numCentres;
+
+			if(numCentres)
+				vectorParams[0]=massCentre*1.0/(float)numCentres;
+			else
+				vectorParams[0]=massCentre;
 			break;
 
 		}
@@ -282,8 +301,6 @@ unsigned int TransformFilter::refresh(const std::vector<const FilterStreamData *
 		if(s)
 			devices.push_back(s);
 
-		cacheAsNeeded(d);
-		
 		getOut.push_back(d);
 	}
 			
@@ -777,10 +794,15 @@ unsigned int TransformFilter::refresh(const std::vector<const FilterStreamData *
 									d->data.push_back(src->data[uj]);
 							}
 
-							d->estimateIonParameters(src);
-							cacheAsNeeded(d);
-							
-							getOut.push_back(d);
+							if(d->data.size())
+							{
+								d->estimateIonParameters(src);
+								cacheAsNeeded(d);
+								
+								getOut.push_back(d);
+							}
+							else
+								delete d;
 						}
 					}
 				}
@@ -998,8 +1020,10 @@ unsigned int TransformFilter::refresh(const std::vector<const FilterStreamData *
 					break;
 				}
 			
+			}
 		}
-		}
+
+		progress.filterProgress=100;
 	}
 	else
 	{
@@ -1016,15 +1040,7 @@ unsigned int TransformFilter::refresh(const std::vector<const FilterStreamData *
 		//Set up output ion stream 
 		IonStreamData *d=new IonStreamData;
 		d->parent=this;
-		
-		//TODO: Better output colouring/size
-		//Set up ion metadata
-		d->r = 0.5;
-		d->g = 0.5;
-		d->b = 0.5;
-		d->a = 0.5;
-		d->ionSize = 2.0;
-		d->valueType=TRANS("Mass-to-Charge (Da/e)");
+		d->estimateIonParameters(dataIn); // get best meta data we can
 
 		size_t curPos=0;
 		
@@ -1139,21 +1155,15 @@ void TransformFilter::getProperties(FilterPropGroup &propertyList) const
 
 	FilterProperty p;
 	size_t curGroup=0;
-	string tmpStr;
+	{
 	vector<pair<unsigned int,string> > choices;
 	for(unsigned int ui=0;ui<MODE_ENUM_END; ui++)
 		choices.push_back(make_pair(ui,TRANS(TRANSFORM_MODE_STRING[ui])));
-	
-	tmpStr=choiceString(choices,transformMode);
-	choices.clear();
-	
-	p.name=TRANS("Mode");
-	p.data=tmpStr;
-	p.type=PROPERTY_TYPE_CHOICE;
-	p.helpText=TRANS("Algorithm to use to transform point data");
-	p.key=KEY_MODE;
-	propertyList.addProperty(p,curGroup);
-	
+	FilterProperty pMode(KEY_MODE, TRANS("Mode"),
+			choices,transformMode,TRANS("Algorithm to use to transform point data"));
+	propertyList.addProperty(pMode,curGroup);
+	}
+		
 	propertyList.setGroupTitle(curGroup,TRANS("Algorithm"));
 	curGroup++;	
 	
@@ -1165,25 +1175,19 @@ void TransformFilter::getProperties(FilterPropGroup &propertyList) const
 		for(unsigned int ui=0;ui<ORIGINMODE_END;ui++)
 			choices.push_back(make_pair(ui,getOriginTypeString(ui)));
 		
-		tmpStr= choiceString(choices,originMode);
-
-		p.name=TRANS("Origin mode");
-		p.data=tmpStr;
-		p.type=PROPERTY_TYPE_CHOICE;
-		p.helpText=TRANS("Select how transform origin is computed");
-		p.key=KEY_ORIGINMODE;
-		propertyList.addProperty(p,curGroup);
+		FilterProperty pOriginMode(KEY_ORIGINMODE, TRANS("Origin mode"),
+				choices,originMode,TRANS("Select how transform origin is computed"));
+		propertyList.addProperty(pOriginMode,curGroup);
 	
-		stream_cast(tmpStr,showOrigin);	
-		p.name=TRANS("Show marker");
-		p.data=tmpStr;
-		p.type=PROPERTY_TYPE_BOOL;
-		if( originMode == ORIGINMODE_SELECT)
-			p.helpText=TRANS("Display an interactive object to set transform origin");
+		string helpStr;
+		if(originMode == ORIGINMODE_SELECT)
+			helpStr=TRANS("Display an interactive object to set transform origin");
 		else
-			p.helpText=TRANS("Display a small marker to denote transform origin");
-		p.key=KEY_TRANSFORM_SHOWORIGIN;
-		propertyList.addProperty(p,curGroup);
+			helpStr=TRANS("Display a small marker to denote transform origin");
+		
+		FilterProperty pShowMarker(KEY_TRANSFORM_SHOWORIGIN, TRANS("Show marker"),
+						showOrigin,helpStr);
+		propertyList.addProperty(pShowMarker,curGroup);
 	}
 
 	
@@ -1196,13 +1200,9 @@ void TransformFilter::getProperties(FilterPropGroup &propertyList) const
 			ASSERT(vectorParams.size() == 1);
 			ASSERT(scalarParams.size() == 0);
 			
-			stream_cast(tmpStr,vectorParams[0]);
-			p.name=TRANS("Translation");
-			p.data=tmpStr;
-			p.type=PROPERTY_TYPE_POINT3D;
-			p.helpText=TRANS("Translation vector for transform");
-			p.key=KEY_ORIGIN;
-			propertyList.addProperty(p,curGroup);
+			FilterProperty pTranslateVector(KEY_ORIGIN, TRANS("Translation"),
+							vectorParams[0],TRANS("Translation vector for transform"));
+			propertyList.addProperty(pTranslateVector,curGroup);
 			break;
 		}
 		case MODE_TRANSLATE_VALUE:
@@ -1210,14 +1210,9 @@ void TransformFilter::getProperties(FilterPropGroup &propertyList) const
 			ASSERT(vectorParams.size() == 0);
 			ASSERT(scalarParams.size() == 1);
 			
-			
-			stream_cast(tmpStr,scalarParams[0]);
-			p.name=TRANS("Offset");
-			p.data=tmpStr;
-			p.key=KEY_ORIGIN_VALUE;
-			p.type=PROPERTY_TYPE_REAL;
-			p.helpText=TRANS("Scalar to use to offset each point's associated value");
-			propertyList.addProperty(p,curGroup);
+			FilterProperty pOffset(KEY_ORIGIN_VALUE, TRANS("Offset"),
+				scalarParams[0],TRANS("Scalar to use to offset each point's associated value"));
+			propertyList.addProperty(pOffset,curGroup);
 			break;
 		}
 		case MODE_SCALE_ISOTROPIC:
@@ -1228,23 +1223,14 @@ void TransformFilter::getProperties(FilterPropGroup &propertyList) const
 			
 			if(originMode == ORIGINMODE_SELECT)
 			{
-				stream_cast(tmpStr,vectorParams[0]);
-				p.key=KEY_ORIGIN;
-				p.name=TRANS("Origin");
-				p.data=tmpStr;
-				p.type=PROPERTY_TYPE_POINT3D;
-				p.helpText=TRANS("Origin of scale trasnform");
-				propertyList.addProperty(p,curGroup);
+				FilterProperty pOrigin(KEY_ORIGIN, TRANS("Origin"),
+						vectorParams[0],TRANS("Origin of scale transform"));
+				propertyList.addProperty(pOrigin,curGroup);
 			}
 			
-			stream_cast(tmpStr,scalarParams[0]);
-			
-			p.key=KEY_SCALEFACTOR;
-			p.name=TRANS("Scale Fact.");
-			p.data=tmpStr;
-			p.type=PROPERTY_TYPE_REAL;
-			p.helpText=TRANS("Enlargement factor for scaling around origin");
-			propertyList.addProperty(p,curGroup);
+			FilterProperty pScaleFactor(KEY_SCALEFACTOR, TRANS("Scale Fact."),
+				scalarParams[0],TRANS("Enlargement factor for scaling around origin"));
+			propertyList.addProperty(pScaleFactor,curGroup);
 
 			break;
 		}
@@ -1255,23 +1241,14 @@ void TransformFilter::getProperties(FilterPropGroup &propertyList) const
 			
 			if(originMode == ORIGINMODE_SELECT)
 			{
-				stream_cast(tmpStr,vectorParams[0]);
-				p.key=KEY_ORIGIN;
-				p.name=TRANS("Origin");
-				p.data=tmpStr;
-				p.type=PROPERTY_TYPE_POINT3D;
-				p.helpText=TRANS("Origin of scale trasnform");
-				propertyList.addProperty(p,curGroup);
+				FilterProperty pOffset(KEY_ORIGIN, TRANS("Origin"),
+					vectorParams[0],TRANS("Origin of scale transform"));
+				propertyList.addProperty(pOffset,curGroup);
 			}
 			
-			stream_cast(tmpStr,vectorParams[1]);
-			
-			p.key=KEY_SCALEFACTOR_ANISOTROPIC;
-			p.name=TRANS("Scale Fact.");
-			p.data=tmpStr;
-			p.type=PROPERTY_TYPE_REAL;
-			p.helpText=TRANS("Enlargement factor for scaling around origin");
-			propertyList.addProperty(p,curGroup);
+			FilterProperty pScaleVec(KEY_SCALEFACTOR_ANISOTROPIC, TRANS("Scale Fact."),
+				vectorParams[1],TRANS("Enlargement factor for scaling around origin"));
+			propertyList.addProperty(pScaleVec,curGroup);
 
 			break;
 		}
@@ -1281,29 +1258,17 @@ void TransformFilter::getProperties(FilterPropGroup &propertyList) const
 			ASSERT(scalarParams.size() == 1);
 			if(originMode == ORIGINMODE_SELECT)
 			{
-				stream_cast(tmpStr,vectorParams[0]);
-				p.key=KEY_ORIGIN;
-				p.name=TRANS("Origin");
-				p.data=tmpStr;
-				p.type=PROPERTY_TYPE_POINT3D;
-				p.helpText=TRANS("Origin of rotation");
-				propertyList.addProperty(p,curGroup);
+				FilterProperty pOrigin(KEY_ORIGIN, TRANS("Origin"),
+						vectorParams[0],TRANS("Origin of Rotation"));
+				propertyList.addProperty(pOrigin,curGroup);
 			}
-			stream_cast(tmpStr,vectorParams[1]);
-			p.key=KEY_ROTATE_AXIS;
-			p.name=TRANS("Axis");
-			p.data=tmpStr;
-			p.type=(PROPERTY_TYPE_POINT3D);
-			p.helpText=TRANS("Axis around which to revolve");
-			propertyList.addProperty(p,curGroup);
+			FilterProperty pAxis(KEY_ROTATE_AXIS, TRANS("Axis"),
+					vectorParams[1],TRANS("Axis around which to revolve"));
+			propertyList.addProperty(pAxis,curGroup);
 
-			stream_cast(tmpStr,scalarParams[0]);
-			p.key=KEY_ROTATE_ANGLE;
-			p.name=TRANS("Angle (deg)");
-			p.data=tmpStr;
-			p.type=PROPERTY_TYPE_REAL;
-			p.helpText=TRANS("Angle to perform rotation (ACW, as viewed from axis towards origin)");
-			propertyList.addProperty(p,curGroup);
+			FilterProperty pAngle(KEY_ROTATE_ANGLE, TRANS("Angle (Deg)"),
+					scalarParams[0],TRANS("Angle to perform rotation (ACW, as viewed from axis towards origin)"));
+			propertyList.addProperty(pAngle,curGroup);
 			break;
 		}
 		case MODE_VALUE_SHUFFLE:
@@ -1314,34 +1279,32 @@ void TransformFilter::getProperties(FilterPropGroup &propertyList) const
 		}
 		case MODE_SPATIAL_NOISE:
 		{
+			vector<pair<unsigned int,string> > choices;
 			for(unsigned int ui=0;ui<NOISETYPE_END;ui++)
 				choices.push_back(make_pair(ui,getNoiseTypeString(ui)));
-			tmpStr= choiceString(choices,noiseType);
-			choices.clear();
-			
-			p.name=TRANS("Noise Type");
-			p.data=tmpStr;
-			p.type=PROPERTY_TYPE_CHOICE;
-			p.helpText=TRANS("Method to use to degrade point data");
-			p.key=KEY_NOISETYPE;
-			propertyList.addProperty(p,curGroup);
+			FilterProperty pNoiseType(KEY_NOISETYPE, TRANS("Noise Type"),
+							choices,noiseType,TRANS("Method to use to degrade point data"));
+			propertyList.addProperty(pNoiseType,curGroup);
 
 
+			string tmpStr, propName;
 			stream_cast(tmpStr,scalarParams[0]);
 			if(noiseType == NOISETYPE_WHITE)
-				p.name=TRANS("Noise level");
+			{
+				FilterProperty pNoiseLevel(KEY_NOISELEVEL, TRANS("Noise Level"),
+						scalarParams[0],TRANS("Amplitude of applied noise"));
+				propertyList.addProperty(pNoiseLevel,curGroup);
+			}
 			else if(noiseType == NOISETYPE_GAUSSIAN)
-				p.name=TRANS("Standard dev.");
+			{
+				FilterProperty pNoiseLevel(KEY_NOISELEVEL, TRANS("Standard dev."),
+						scalarParams[0],TRANS("Gaussian 1-sigma spread distance"));
+				propertyList.addProperty(pNoiseLevel,curGroup);
+			}
 			else
 			{
 				ASSERT(false);
 			}
-			
-			p.data=tmpStr;
-			p.type=PROPERTY_TYPE_REAL;
-			p.helpText=TRANS("Amplitude of noise");
-			p.key=KEY_NOISELEVEL;
-			propertyList.addProperty(p,curGroup);
 
 
 			break;	
@@ -1351,22 +1314,12 @@ void TransformFilter::getProperties(FilterPropGroup &propertyList) const
 			ASSERT(vectorParams.size() == 0);
 			ASSERT(scalarParams.size() == 2);
 			
-			
-			stream_cast(tmpStr,scalarParams[0]);
-			p.name=TRANS("Min Value");
-			p.data=tmpStr;
-			p.key=KEY_CROP_MINIMUM;
-			p.type=PROPERTY_TYPE_REAL;
-			p.helpText=TRANS("Minimum value to use for crop");
-			propertyList.addProperty(p,curGroup);
-			
-			stream_cast(tmpStr,scalarParams[1]);
-			p.name=TRANS("Max Value");
-			p.data=tmpStr;
-			p.key=KEY_CROP_MAXIMUM;
-			p.type=PROPERTY_TYPE_REAL;
-			p.helpText=TRANS("Maximum value to use for crop");
-			propertyList.addProperty(p,curGroup);
+			FilterProperty pMinValue(KEY_CROP_MINIMUM, TRANS("Min Value"),
+					scalarParams[0],TRANS("Minimum value to use for crop"));
+			propertyList.addProperty(pMinValue,curGroup);
+			FilterProperty pMaxValue(KEY_CROP_MAXIMUM, TRANS("Max Value"),
+					scalarParams[1],TRANS("Maximum value to use for crop"));
+			propertyList.addProperty(pMaxValue,curGroup);
 			break;
 		}
 		default:
@@ -1526,6 +1479,11 @@ bool TransformFilter::setProperty(  unsigned int key,
 		case KEY_CROP_MINIMUM:
 		{
 			ASSERT(scalarParams.size() ==2);
+			float tmp;
+			if(stream_cast(tmp,value) || tmp >=scalarParams[1])
+				return false;
+
+
 			if(!applyPropertyNow(scalarParams[0],value,needUpdate))
 				return false;
 			break;
@@ -1533,6 +1491,9 @@ bool TransformFilter::setProperty(  unsigned int key,
 		case KEY_CROP_MAXIMUM:
 		{
 			ASSERT(scalarParams.size() ==2);
+			float tmp;
+			if(stream_cast(tmp,value) || tmp <=scalarParams[0])
+				return false;
 			if(!applyPropertyNow(scalarParams[1],value,needUpdate))
 				return false;
 			break;
