@@ -1,6 +1,6 @@
 /*
  * 	filtertree.cpp - Filter tree topology and data propagation handling
- *	Copyright (C) 2015, D Haley 
+ *	Copyright (C) 2018, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -207,7 +207,7 @@ void FilterRefreshCollector::collectToLevel(unsigned int level)
 #ifdef DEBUG
 	checkSanity();
 #endif
-	for(size_t ui=level+1;ui<nodes.size();ui++)
+	for(size_t ui=level;ui<nodes.size();ui++)
 	{
 		for(list<const FilterStreamData*>::iterator it=nodes[ui].begin(); it!=nodes[ui].end(); ++it)
 			delete *it;
@@ -577,53 +577,11 @@ void FilterTree::getFilterRefreshStarts(vector<tree<Filter *>::iterator > &propS
 
 
 
-}
-
-void FilterTree::getConsoleMessagesToNodes(std::vector<tree<Filter *>::iterator > &nodes,
-			std::vector<pair<const Filter*,string> > &messages) const
-{
-
-
-	//obtain a unique list of all filters who are parents of the nodes
-	if(!nodes.size())
-		return;
-
-	std::set<Filter*> filterSet;
-	for(unsigned int ui=0;ui<nodes.size();ui++)
-	{
-		tree<Filter*>::iterator it;
-		it=nodes[ui];
-
-		if(it==filters.begin())
-			filterSet.insert(*it);
-		
-		while(filters.is_valid(it) && 
-			filters.parent(it)!= filters.begin())
-		{
-			filterSet.insert(*it);	
-			it=filters.parent(it);
-		}
-	}
-
-	filterSet.insert(*(filters.begin()));
-
-
-	//now loop through the filters and obtain the console messages
-	for(set<Filter *>::iterator it=filterSet.begin();it!=filterSet.end();++it)
-	{
-		vector<string> tmpMsgs;
-		(*it)->getConsoleStrings(tmpMsgs);
-
-		for(size_t ui=0;ui<tmpMsgs.size();ui++)
-		{
-			messages.push_back(make_pair(*it,tmpMsgs[ui]));
-		}
-	}
 }
 
 unsigned int FilterTree::refreshFilterTree(list<FILTER_OUTPUT_DATA > &outData, 
 		std::vector<SelectionDevice *> &devices,
-		vector<pair<const Filter* , string> > &consoleMessages,
+		vector<ConsoleMessage > &consoleMessages,
 	       	ProgressData &curProg, ATOMIC_BOOL &abortRefresh) const
 {
 
@@ -777,8 +735,12 @@ unsigned int FilterTree::refreshFilterTree(list<FILTER_OUTPUT_DATA > &outData,
 			//---
 			//
 
+			//If we have no cache, clear the previous console messages for this filter
+			// otherwise, mark the messages from the filter as cached.
 			if(!currentFilter->haveCache())
 				currentFilter->clearConsole();
+			else
+				currentFilter->markConsoleCached();
 
 			currentFilter->clearDevices();
 
@@ -801,15 +763,21 @@ unsigned int FilterTree::refreshFilterTree(list<FILTER_OUTPUT_DATA > &outData,
 #ifdef DEBUG
 			//Perform sanity checks on filter output
 			checkRefreshValidity(curData,currentFilter);
-			ASSERT(curProg.step == curProg.maxStep || errCode);
+			string s;
+			s=string("Filter (") + currentFilter->getUserString() + ") exited cleanly, but did not reach its maximum step";
+			WARN( curProg.step == curProg.maxStep || errCode,s.c_str()) ;
 			//when completing, we should have full progress 
 			std::string progWarn = std::string("Progress did not reach 100\% for filter: ");
 			progWarn+=currentFilter->getUserString();
 			
 			WARN( (curProg.filterProgress == 100 || errCode),progWarn.c_str());
+
+			std::string cacheWarnStr="Filter cached when asked not to. Filter : ";
+			cacheWarnStr+=currentFilter->getUserString();
+			WARN( !(cacheStrategy == CACHE_NEVER &&currentFilter->haveCache()),cacheWarnStr.c_str());
+
 #endif
-			//Ensure that (1) yield is called, regardless of what filter does
-			//(2) yield is called after 100% update	
+			//Ensure that progress is set to 100% after update	
 			curProg.filterProgress=100;	
 
 
@@ -824,12 +792,13 @@ unsigned int FilterTree::refreshFilterTree(list<FILTER_OUTPUT_DATA > &outData,
 			curDevices.clear();
 
 			//Retrieve any console messages from the filter
-			vector<string> tmpMessages;
+			vector<ConsoleMessage > tmpMessages;
 			currentFilter->getConsoleStrings(tmpMessages);
+
 			//Accumulate the messages
 			consoleMessages.reserve(consoleMessages.size()+tmpMessages.size());
 			for(size_t ui=0;ui<tmpMessages.size();ui++)
-				consoleMessages.push_back(make_pair(currentFilter,tmpMessages[ui]));
+				consoleMessages.push_back(tmpMessages[ui]);
 
 			//check for any error in filter update (including user abort)
 			if(errCode || abortRefresh)
@@ -1317,22 +1286,22 @@ bool FilterTree::hasHazardousContents() const
 
 void FilterTree::stripHazardousContents()
 {
+	vector<Filter *> vecFilt;
+
 	for(tree<Filter * >::pre_order_iterator it=filters.begin();
-					it!=filters.end(); ++it)
+		it!=filters.end(); ++it)
 	{
 		if ((*it)->canBeHazardous())
 		{
-			//delete filters from this branch
-			for(tree<Filter *>::pre_order_iterator itj(it); itj!=filters.end(); ++itj)
-				delete *itj;
-	
 
-			//nuke this branch
-			it=filters.erase(it);
-			--it;
+			vecFilt.push_back(*it);
 		}
 	}
 
+
+	for(unsigned int ui=0;ui<vecFilt.size(); ui++)
+		removeSubtree(vecFilt[ui]);
+
 }
 
 bool FilterTree::isChild(const tree<Filter *> &treeInst,
@@ -1739,6 +1708,35 @@ bool FilterTree::copyFilter(Filter *toCopy,const Filter *newParent)
 }
 
 
+bool FilterTree::respliceFilter(Filter *toMove, const Filter *newParent)
+{
+	if(toMove == newParent)
+		return false;
+
+	//Find the iterator of the filter to move
+	tree<Filter*>::iterator moveFilterIt=std::find(filters.begin(),filters.end(),toMove);
+
+	Filter* oldParent;
+	if(depth(moveFilterIt))
+		oldParent=*(filters.parent(moveFilterIt));
+	else
+		oldParent= 0;
+
+	//reparent all the children of the move filter
+	typedef tree<Filter*>::sibling_iterator TREE_SIBIT;
+	vector<TREE_SIBIT> moveChildren;
+	for(TREE_SIBIT it=moveFilterIt.begin(); it!=moveFilterIt.end(); ++it)
+		moveChildren.push_back(it);
+
+	for(unsigned int ui=0;ui<moveChildren.size(); ui++)
+		reparentFilter(*(moveChildren[ui]),oldParent);
+
+	reparentFilter(toMove,newParent);
+
+	return true;	
+}
+
+
 void FilterTree::removeSubtree(Filter *removeFilt)
 {
 	ASSERT(removeFilt);	
