@@ -1,6 +1,6 @@
 /*
  * 	filtertree.h - Filter tree topology and data propagation handling
- *	Copyright (C) 2015, D Haley 
+ *	Copyright (C) 2018, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -62,10 +62,6 @@ class FilterTree
 		//and tree topology
 		void getFilterRefreshStarts(std::vector<tree<Filter *>::iterator > &propStarts) const;
 	
-		//!Obtain the tree nodes up until (but excluding) these nodes
-		void getConsoleMessagesToNodes(std::vector<tree<Filter *>::iterator> &nodes, 
-				std::vector<std::pair< const Filter*,std::string> > &messages) const;	
-		
 #ifdef DEBUG
 		//!Check that the output of filter refresh functions
 		void checkRefreshValidity(const std::vector< const FilterStreamData *> &curData, 
@@ -99,9 +95,10 @@ class FilterTree
 		tree<Filter *>::pre_order_iterator depthBegin() const { return filters.begin();};
 		//Return iterator to tree contents end
 		tree<Filter *>::pre_order_iterator depthEnd() const { return filters.end();}
-		//Return depth of a given iterator
+		//return the maximum depth of the tree
 		size_t maxDepth() const;
 
+		//Return depth of a given iterator
 		size_t depth(const tree<Filter*>::pre_order_iterator &it) const ;
 
 		//Get a reference to the underlying tree
@@ -128,7 +125,7 @@ class FilterTree
 		// deleting the filterstream data correctly. To do this, use the "safeDeleteFilterList" function.
 		unsigned int refreshFilterTree(	
 			std::list<FILTER_OUTPUT_DATA> &outData,
-			std::vector<SelectionDevice *> &devices,std::vector<std::pair<const Filter *,std::string> > &consoleMessages,
+			std::vector<SelectionDevice *> &devices,std::vector<ConsoleMessage > &consoleMessages,
 						ProgressData &curProg, ATOMIC_BOOL &abortRefresh) const;
 
 		static std::string getRefreshErrString(unsigned int errCode);
@@ -172,7 +169,7 @@ class FilterTree
 		void removeSubtree(Filter *f);
 		
 		//!Add a new filter to the tree. Note that pointer will be released
-		// by filter destructor
+		// by filter destructor. Set parent=0 if inserting into base of tree
 		void addFilter(Filter *f, const Filter *parent);
 		
 		//!Add a new tree as a subtree to a node 
@@ -186,6 +183,12 @@ class FilterTree
 		//!Obtain a copy of the filters from the specified subtree,
 		// including the target filter and its descendants
 		void cloneSubtree(FilterTree &f,const Filter *targetFilt) const;
+
+		//!Move a filter from one point in the tree to another, with the
+		//  parent of the filter being moved adopting all that filters children
+		//  - newParent can be a child of toMove.
+		bool respliceFilter(Filter *toMove, const Filter *newParent);
+
 		//---------
 	
 
@@ -236,6 +239,11 @@ class FilterTree
 
 
 		size_t cacheCount(unsigned int typeMask = STREAMTYPE_MASK_ALL) const;
+
+#ifdef DEBUG
+		void print() const ;
+#endif
+
 };
 
 #endif
