@@ -1,6 +1,6 @@
 /*
  *	plot.cpp - mathgl plot wrapper class
- *	Copyright (C) 2015, D Haley 
+ *	Copyright (C) 2018, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
  *	along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 #include "plot.h"
-
+#include "common/colourmap.h"
 #include "common/stringFuncs.h"
 
 #include "common/translation.h"
@@ -29,7 +29,7 @@ const char *errModeStrings[] = {
 				NTRANS("Moving avg.")
 				};
 
-const char *plotTypeStrings[]= {
+const char *traceStyleStrings[]= {
 	NTRANS("Lines"),
 	NTRANS("Bars"),
 	NTRANS("Steps"),
@@ -114,20 +114,39 @@ std::string mglColourCode(float r, float g, float b)
 	return string("{x") + uppercase(s) + string("}");
 }
 
+std::string mglColourCode(float r, float g, float b,float p)
+{
+	ASSERT(r >=0.0f && g >=0.0f && b >=0.0f)
+	ASSERT(r <=255.0f && g <=255.0f && b <=255.0f)
+
+	ColourRGBA rgba(r*255.0,g*255.0,b*255.0);
+
+	std::string s;
+	//Make a #rrggbb hex string
+	s=rgba.rgbString();
+	s=s.substr(1);
+
+	std::string sp;
+	stream_cast(sp,p);
+	ASSERT(p >=0 && p <=1);
+
+	return string("{x") + uppercase(s) + string(",") + sp + string("}");
+}
+
 //TODO: Refactor these functions to use a common string map
 //-----------
 string plotString(unsigned int plotMode)
 {
 	ASSERT(plotMode< PLOT_TYPE_ENUM_END);
-	return TRANS(plotTypeStrings[plotMode]); 
+	return TRANS(traceStyleStrings[plotMode]); 
 }
 
 unsigned int plotID(const std::string &plotString)
 {
-	COMPILE_ASSERT(THREEDEP_ARRAYSIZE(plotTypeStrings) == PLOT_TYPE_ENUM_END);
+	COMPILE_ASSERT(THREEDEP_ARRAYSIZE(traceStyleStrings) == PLOT_TYPE_ENUM_END);
 	for(unsigned int ui=0;ui<PLOT_TYPE_ENUM_END; ui++)
 	{
-		if(plotString==TRANS(plotTypeStrings[ui]))
+		if(plotString==TRANS(traceStyleStrings[ui]))
 			return ui;
 	}
 
@@ -298,7 +317,7 @@ std::string PlotRegion::getName() const
 
 PlotWrapper::PlotWrapper()
 {
-	//COMPILE_ASSERT(THREEDEP_ARRAYSIZE(plotTypeStrings) == PLOT_TYPE_ENUM_END);
+	//COMPILE_ASSERT(THREEDEP_ARRAYSIZE(traceStyleStrings) == PLOT_TYPE_ENUM_END);
 
 	applyUserBounds=false;
 	plotChanged=true;
@@ -494,7 +513,7 @@ void PlotWrapper::getBounds(float &xMin, float &xMax,
 	else
 		scanBounds(xMin,xMax,yMin,yMax);
 
-	ASSERT(xMin <xMax && yMin <=yMax);
+	ASSERT(xMin <= xMax && yMin <=yMax);
 }
 
 void PlotWrapper::scanBounds(float &xMin,float &xMax,float &yMin,float &yMax) const
@@ -523,7 +542,7 @@ void PlotWrapper::scanBounds(float &xMin,float &xMax,float &yMin,float &yMax) co
 
 	}
 
-	ASSERT(xMin < xMax && yMin <=yMax);
+	ASSERT(xMin <= xMax && yMin <=yMax);
 }
 
 void PlotWrapper::bestEffortRestoreVisibility()
@@ -671,6 +690,7 @@ unsigned int PlotWrapper::getVisibleMode() const
 			if(visibleMode == PLOT_MODE_ENUM_END)
 			{
 				visibleMode=plottingData[ui]->getMode();
+				ASSERT(visibleMode < PLOT_MODE_ENUM_END);
 				continue;
 			}
 			else
@@ -684,6 +704,12 @@ unsigned int PlotWrapper::getVisibleMode() const
 	return visibleMode;
 }
 
+unsigned int PlotWrapper::getPlotMode(unsigned int plotId) const
+{
+	ASSERT(plotId < plottingData.size());
+	return plottingData[plotId]->getPlotMode();
+}
+
 void PlotWrapper::getVisibleIDs(vector<unsigned int> &visiblePlotIDs ) const
 {
 
@@ -791,7 +817,7 @@ void PlotWrapper::drawPlot(mglGraph *gr, bool &haveUsedLog) const
 				if(!plottingData[ui]->visible)
 					continue;
 
-				if(plottingData[ui]->getType()!= PLOT_MODE_1D)
+				if(plottingData[ui]->getMode()!= PLOT_MODE_1D)
 					continue;
 			
 				if(((Plot1D*)plottingData[ui])->wantLogPlot()) 
@@ -802,6 +828,7 @@ void PlotWrapper::drawPlot(mglGraph *gr, bool &haveUsedLog) const
 		
 
 			
+			//Detect if the bounds requested are invalid
 			//Allow for logarithmic mode, as needed.
 			// mathgl does not like a zero coordinate for plotting
 			if(min.y == 0 && useLogPlot)
@@ -809,7 +836,7 @@ void PlotWrapper::drawPlot(mglGraph *gr, bool &haveUsedLog) const
 				float minYVal=0.1;
 				for(size_t ui=0;ui<plottingData.size();ui++)
 				{
-					if(!plottingData[ui]->visible || plottingData[ui]->getType() !=PLOT_MODE_1D)
+					if(!plottingData[ui]->visible || plottingData[ui]->getMode() !=PLOT_MODE_1D)
 						continue;
 
 					float tmp ;
@@ -929,7 +956,7 @@ void PlotWrapper::drawPlot(mglGraph *gr, bool &haveUsedLog) const
 				Plot2DFunc *curPlot;
 				curPlot=(Plot2DFunc*)plottingData[ui];
 
-				if(curPlot->getType() == PLOT_2D_DENS)
+				if(curPlot->getMode() == PLOT_2D_DENS)
 				{
 					wantColourbar=true;
 				}
@@ -1047,11 +1074,6 @@ void PlotWrapper::getRegion(unsigned int plotId, unsigned int regionId, PlotRegi
 	plottingData[plotIDHandler.getPos(plotId)]->regionGroup.getRegion(regionId,region);
 }
 
-unsigned int PlotWrapper::plotType(unsigned int plotId) const
-{
-	return plottingData[plotIDHandler.getPos(plotId)]->getPlotMode();
-}
-
 
 void PlotWrapper::moveRegion(unsigned int plotID, unsigned int regionId, bool regionSelfUpdate,
 		unsigned int movementType, float newX, float newY) const
@@ -1104,6 +1126,11 @@ PlotBase::PlotBase()
 	parentObject=0;
 	parentPlotIndex=(unsigned int)-1;
 	visible=true;
+
+	//Disable use of "special"  bounds	
+	for(unsigned int ui=0;ui<2;ui++)
+		for(unsigned int uj=0;uj<2;uj++)
+			specialBoundsEnabled[ui][uj] = false;
 }
 
 
@@ -1121,9 +1148,47 @@ void PlotBase::getBounds(float &xMin,float &xMax,float &yMin,float &yMax) const
 	yMin=minY;
 	yMax=maxY;
 
+#ifdef DEBUG
+	//Check each axis is sane
+	for(unsigned int ui=0;ui<2;ui++)
+	{
+		//Min special bound should undershoot upper bounds
+		if(specialBoundsEnabled[ui][0] && specialBoundsEnabled[ui][1])
+		{
+			ASSERT(specialBounds[ui][0] < specialBounds[ui][1])
+		}
+	}
+#endif
+
+	//X-
+	if(specialBoundsEnabled[0][0])
+		xMin=std::min(xMin,specialBounds[0][0]);
+	//X+
+	if(specialBoundsEnabled[0][1])
+		xMax=std::max(xMax,specialBounds[0][1]);
+	//Y-
+	if(specialBoundsEnabled[1][0])
+		yMin=std::min(yMin,specialBounds[1][0]);
+	//Y+
+	if(specialBoundsEnabled[1][1])
+		yMax=std::max(yMax,specialBounds[1][1]);
+
+
 	ASSERT(yMin <=yMax);
 }
 
+void PlotBase::setSpecialBounds(unsigned int idx, bool axisMax, float value)
+{
+	unsigned int minIdx;
+	if(axisMax) 
+		minIdx=1;
+	else 
+		minIdx=0;
+
+	specialBoundsEnabled[idx][minIdx] =true;
+	specialBounds[idx][minIdx] =value;
+}
+
 void PlotBase::setStrings(const std::string &x, const std::string &y, const std::string &t)
 {
 
@@ -1135,7 +1200,7 @@ void PlotBase::setStrings(const std::string &x, const std::string &y, const std:
 
 void PlotBase::copyBase(PlotBase *target) const
 {
-	target->plotType=plotType;
+	target->traceStyle=traceStyle;
 	target->minX=minX;
 	target->maxX=maxX;
 	target->minY=minY;
@@ -1157,36 +1222,27 @@ void PlotBase::copyBase(PlotBase *target) const
 
 unsigned int PlotBase::getType() const
 {
-	return plotType;
+	ASSERT(traceStyle <  PLOT_TYPE_ENUM_END);
+	return traceStyle;
 }
 
 unsigned int PlotBase::getMode() const
 {
-	switch(plotType)
-	{
-		case PLOT_LINE_LINES:
-		case PLOT_LINE_BARS:
-		case PLOT_LINE_STEPS:
-		case PLOT_LINE_STEM:
-		case PLOT_LINE_POINTS:
-			return PLOT_MODE_1D;
+	ASSERT(plotMode < PLOT_MODE_ENUM_END);
+	return plotMode;
 
-		case PLOT_2D_DENS:
-		case PLOT_2D_SCATTER:
-			return PLOT_MODE_2D;	
-	}
-	ASSERT(false);
 }
 
 Plot1D::Plot1D()
 {
 	//Set the default plot properties
-	plotType=PLOT_LINE_LINES;
+	traceStyle=PLOT_LINE_LINES;
 	plotMode=PLOT_MODE_1D;
 	xLabel="";
 	yLabel="";
 	title="";
 	r=(0);g=(0);b=(1);
+	logarithmic=true;
 }
 
 
@@ -1489,7 +1545,7 @@ void Plot1D::drawPlot(mglGraph *gr) const
 
 
 	//Plot the appropriate form	
-	switch(plotMode)
+	switch(traceStyle)
 	{
 		case PLOT_LINE_LINES:
 			//Unfortunately, when using line plots, mathgl moves the data points to the plot boundary,
@@ -1502,19 +1558,21 @@ void Plot1D::drawPlot(mglGraph *gr) const
 				gr->Error(xDat,yDat,eDat,colourCode.c_str());
 			gr->SetCut(false);
 			break;
-		case PLOT_LINE_BARS:
-			gr->Bars(xDat,yDat,colourCode.c_str());
-			break;
 		case PLOT_LINE_STEPS:
 			//Same problem as for line plot. 
 			gr->SetCut(true);
 			gr->Step(xDat,yDat,colourCode.c_str());
 			gr->SetCut(false);
 			break;
+		case PLOT_LINE_BARS:
+			//FIXME: Bars mode doesn't work for sparse data.
+			// This has been fixed in mathgl > 2.3.5.1
+			//string sMode = colourCode.c_str() + "F^"; // F^ - centred and fixed size
+			//gr->Bars(xDat,yDat,sMode.c_str());
+			//break;
 		case PLOT_LINE_STEM:
-			gr->SetCut(true);
+			//Do not use cut mode, as this removes points that straddle the vertical axes
 			gr->Stem(xDat,yDat,colourCode.c_str());
-			gr->SetCut(false);
 			break;
 
 		case PLOT_LINE_POINTS:
@@ -1632,7 +1690,9 @@ float Plot1D::getSmallestNonzero() const
 Plot2DFunc::Plot2DFunc()
 {
 	plotMode = PLOT_MODE_2D;
-	plotType=PLOT_2D_DENS;
+	traceStyle=PLOT_2D_DENS;
+	wantLog=false;
+	wantColourbar=false;
 }
 
 void Plot2DFunc::setData(const Array2D<float> &a,
@@ -1671,24 +1731,71 @@ void Plot2DFunc::drawPlot(mglGraph *graph) const
 	w=xyValues.width();
 	h=xyValues.height();
 	
+	if(xyValues.empty())
+		return;
+
 	mglData xyData(w,h);
 
+	//Find min/max so we can rescale to [0,1], for mgl
+	// as this seems to be how colouring is done
+	float minV,maxV;
+	maxV=minV=xyValues.get(0,0);
+	for(size_t ui=0;ui<w;ui++)
+	{
+		for(size_t uj=0;uj<h;uj++)
+		{
+			minV=std::min(xyValues.get(ui,uj),minV);
+			maxV=std::max(xyValues.get(ui,uj),maxV);
+		}
+	}
+	//Provide normalised data to mathgl
 	#pragma omp parallel for
 	for(size_t ui=0;ui<w;ui++)
 	{
 		for(size_t uj=0;uj<h;uj++)
 		{
-			xyData[uj*w+ui]=xyValues[ui][uj];
+			xyData.a[uj*w+ui]= xyValues.get(ui,uj);
 		}
 	}
-
 	mglData xAxis(w),yAxis(h);
 	xAxis.Fill(minX,maxX);
 	yAxis.Fill(minY,maxY);
 
+	if(wantLog)
+	{
+		const float LOG_MIN=1e-4;
+		if(minV <LOG_MIN)
+		{
+			//arbitrarily increase scale to minimum=LOG_MIN
+			for(size_t ui=0;ui<w;ui++)
+				for(size_t uj=0;uj<h;uj++)
+					xyData.a[uj*w+ui]= xyData.a[uj*w+ui] +minV + LOG_MIN;
+		}
+			
+	    graph->SetFunc("","","","lg(c)");
+	}
+
+	if(wantColourbar)
+	    graph->Colorbar();
+
+	const unsigned int NUM_STOPS=64;
+	string colourStr;
+	for(unsigned int ui=0;ui<NUM_STOPS;ui++)
+	{
+		float pos;
+		pos = ui/(float)NUM_STOPS;
+		unsigned char rgb[3];
+		colourMapWrap(colourMap,rgb,pos,0,1,false);
+
+		colourStr+=mglColourCode(rgb[0]/255.0f,rgb[1]/255.0f,rgb[2]/255.0f,pos);
+	}
+
 	graph->Axis("xy");
 	graph->SetCut(false);
-	graph->Dens(xAxis,yAxis,xyData);
+	graph->SetRange('c',minV,maxV); //set colour range
+	string mglStr;
+	mglStr = string("|") + colourStr;
+	graph->Dens(xAxis,yAxis,xyData,mglStr.c_str()); //no spatial interp (|)
 	graph->SetCut(true);
 }
 
@@ -1701,11 +1808,18 @@ void Plot2DFunc::getRawData(std::vector<std::vector<float> >  &rawData,
 
 }
 
+void Plot2DFunc::setColourMap(unsigned int newColMap)
+{	
+	ASSERT(newColMap < COLOURMAP_ENUM_END);
+	colourMap=newColMap;
+}
+
 //--
 
 Plot2DScatter::Plot2DScatter()
 {
-	plotType=PLOT_2D_SCATTER;
+	traceStyle=PLOT_2D_SCATTER;
+	plotMode=PLOT_MODE_2D;
 	scatterIntensityLog=false;
 }
 
