@@ -1,6 +1,6 @@
 /*
  *	plot.h - plotting wraper for mathgl
- *	Copyright (C) 2015, D Haley 
+ *	Copyright (C) 2018, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -43,7 +43,7 @@
 #undef isnan
 
 
-//Plot style/mode enum
+//Plot trace style enum
 enum 
 {
 	PLOT_LINE_LINES=0,
@@ -57,9 +57,8 @@ enum
 	PLOT_TYPE_ENUM_END
 };
 
-//This is the plot mode,
-// it can be determined as a function of the
-// plot-style/mode enum
+//Plot mode enum, which governs how the axes need to be set
+// for this plot type
 enum
 {
 	PLOT_MODE_1D,
@@ -217,7 +216,7 @@ class PlotOverlays
 class PlotBase
 {
 	protected:
-		//!Sub type of plot (eg lines, bars for 1D)
+		//!Type of plot 
 		unsigned int plotMode;
 		//!xaxis label
 		std::string xLabel;
@@ -228,9 +227,16 @@ class PlotBase
 		
 		//plot colour (for single coloured plots)
 		float r,g,b;
+
+		//Do we want a special set of bounds for our bounding box
+		// format is [X/Y][low/high] 
+		// these act like bounding value in the dataset, but are not part of the data.
+		bool specialBoundsEnabled[2][2];
+		float specialBounds[2][2];
 		
-		//The type of plot (ie what class is it?)	
-		unsigned int plotType;
+		//The sub-style of the plot trace (eg lines, points, bars, etc)
+		// FIXME: This is badly named, change to traceStyle, or dataStyle, or something
+		unsigned int traceStyle;
 		
 		void copyBase(PlotBase *target) const;
 
@@ -297,8 +303,14 @@ class PlotBase
 		void setStrings(const std::string &x, 
 			const std::string &y,const std::string &t);
 
+		//Set the colour of the plot trace
 		void setColour(float rNew, float gNew, float bNew);
 
+		//set the visual style for the trace (dots, lines, etc)
+		void setTraceStyle(unsigned int newStyle) { traceStyle=newStyle;}
+
+		void setSpecialBounds(unsigned int idx, bool axisMin, float value);
+
 		std::string getXLabel() const { return xLabel;}
 		std::string getTitle() const { return title;}
 		std::string getYLabel() const { return yLabel;}
@@ -309,6 +321,7 @@ class PlotBase
 		void setPlotMode(unsigned int newMode) { plotMode= newMode;}
 
 
+		//get the  colour of the trace
 		void getColour(float &r, float &g, float &b) const ;
 
 #ifdef DEBUG
@@ -396,6 +409,15 @@ class Plot2DFunc : public PlotBase
 	private: 	
 		//2D array, for f(x,y) plots 
 		Array2D<float> xyValues;
+
+		//Do we want to use logarithmic scaling
+		bool wantLog;
+
+		//Do we want to draw a colour bar?
+		bool wantColourbar;
+
+		//Which colourmap (COLOURMAP_*) do we want to use?
+		unsigned int colourMap;
 	public:
 		Plot2DFunc();
 		virtual bool isEmpty() const;
@@ -411,6 +433,9 @@ class Plot2DFunc : public PlotBase
 	
 		void setData(const Array2D<float> &a, float xLow, float xHigh, float yLow, float yHigh) ;
 
+		void setLog(bool doWantLog){wantLog=doWantLog;};
+		
+		void setColourMap(unsigned int newColMap); 
 };
 
 //!2D scatter plot, {x,y}_i
@@ -613,7 +638,7 @@ class PlotWrapper
 	
 
 		//!obtain the type of a plot, given the plot's uniqueID
-		unsigned int plotType(unsigned int plotId) const;
+		unsigned int getPlotMode(unsigned int plotId) const;
 
 		//Retrieve the types of visible plots
 		unsigned int getVisibleMode() const;
