@@ -1,6 +1,6 @@
 /*
  *	state.cpp - user session state handler
- *	Copyright (C) 2015, D Haley 
+ *	Copyright (C) 2018, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -34,30 +34,34 @@ using std::make_pair;
 using std::map;
 using std::endl; //TODO: Remove me?
 
-//FIXME: Global - need to make part of AnalysisState.
-//	 then provide references as needed
-//true if modification to state has occurred
-int stateModifyLevel=STATE_MODIFIED_NONE;
+unsigned int AnalysisState::currentStateWriterVersion=0;
 
-void setStateModifyLevel(int newLevel)
+
+void AnalysisState::setStateModifyLevel(unsigned int newLevel)
 {
 	stateModifyLevel=std::max(newLevel,stateModifyLevel);
 }
 
-int getStateModifyLevel()
+unsigned int AnalysisState::getStateModifyLevel() const
 {
-	return stateModifyLevel;
+	unsigned int levelMod;
+	levelMod=std::max(stateModifyLevel,treeState.getModifyLevel());
+	ASSERT(levelMod < STATE_MODIFIED_ENUM_END);
+	return levelMod;
 }
 
 
 AnalysisState::AnalysisState()
 {
+	MESSAGE("AnalysisState Constructor");
 	useRelativePathsForSave=false;
 	activeCamera=0;
 	
 	savedCameras.push_back(new CameraLookAt);	
 
 	plotLegendEnable=true;
+	worldAxisMode=0;
+	stateModifyLevel=STATE_MODIFIED_NONE;
 
 	rBack=gBack=bBack=0;
 }
@@ -91,6 +95,7 @@ void AnalysisState::operator=(const AnalysisState &oth)
 	gBack=oth.gBack;
 	bBack=oth.bBack;
 
+	stateModifyLevel=oth.stateModifyLevel;
 	worldAxisMode=oth.worldAxisMode;
 	activeCamera=oth.activeCamera;
 
@@ -108,6 +113,10 @@ AnalysisState::~AnalysisState()
 
 void AnalysisState::clear()
 {
+	//Has the state been modified?
+	if(treeState.size() || stashedTrees.size()) 
+		setStateModifyLevel(STATE_MODIFIED_DATA);
+
 	treeState.clear();
 	
 	stashedTrees.clear();
@@ -138,8 +147,14 @@ void AnalysisState::clearEffects()
 	effects.clear();
 
 }
+
+unsigned int AnalysisState::getStateWriterVersion()
+{
+	return currentStateWriterVersion;
+}
+
 bool AnalysisState::save(const char *cpFilename, std::map<string,string> &fileMapping,
-		bool writePackage, bool setStateModifyLevel) const
+		bool writePackage, bool setStateModifyLevel) 
 {
 	//Open file for output
 	std::ofstream f(cpFilename);
@@ -280,7 +295,11 @@ bool AnalysisState::save(const char *cpFilename, std::map<string,string> &fileMa
 	ASSERT(isValidXML(cpFilename));
 
 	if(setStateModifyLevel)
+	{
+		//reset the current state modification level
 		stateModifyLevel=STATE_MODIFIED_NONE;
+		treeState.resetModifyLevel();
+	}
 
 	return true;
 }
@@ -390,11 +409,18 @@ bool AnalysisState::loadInternal(const char *cpFilename, bool doMerge, std::ostr
 						errStream << TRANS("State was created by a newer version of this program.. ")
 							<< TRANS("file reading will continue, but may fail.") << endl ;
 					}
+
+					
+					if(!parseVersion(tmpVer,currentStateWriterVersion))
+						currentStateWriterVersion=0;
+
 				}
 				else
 				{
 					errStream<< TRANS("Warning, unparseable version number in state file. File reading will continue, but may fail") << endl;
 				}
+
+				
 				xmlFree(xmlString);
 			}
 		}
@@ -599,12 +625,10 @@ bool AnalysisState::loadInternal(const char *cpFilename, bool doMerge, std::ostr
 				haveCameraAlready=false;
 				for(unsigned int ui=0; ui<newCameraVec.size() ; ui++)
 				{
+					//Ignore duplicate camera
 					if(thisCam->getUserString() == newCameraVec[ui]->getUserString())
 					{
 						haveCameraAlready=true;
-#ifdef DEBUG
-						cerr << "Found duplicate camera, ignoring" << endl;
-	#endif
 						break;
 					} 
 				}
@@ -794,6 +818,8 @@ bool AnalysisState::loadInternal(const char *cpFilename, bool doMerge, std::ostr
 		nodeStack.pop();
 
 		nodeStack.push(nodePtr);
+
+		currentStateWriterVersion=0;
 	}
 	catch (int)
 	{
@@ -832,8 +858,9 @@ bool AnalysisState::loadInternal(const char *cpFilename, bool doMerge, std::ostr
 	Camera *c=new CameraLookAt();
 	savedCameras.push_back(c);
 	
-	bool defaultSet = false;
 	//spin through
+
+	bool defaultSet = false;
 	for(unsigned int ui=0;ui<newCameraVec.size();ui++)
 	{
 		//If there is no userstring, then its a  "default"
@@ -855,6 +882,11 @@ bool AnalysisState::loadInternal(const char *cpFilename, bool doMerge, std::ostr
 		}
 
 	}
+	//There should be at least the default camera
+	ASSERT(savedCameras.size());
+
+	if(activeCamera >=savedCameras.size())
+		activeCamera=savedCameras.size()-1;
 
 	fileName=cpFilename;
 
@@ -888,9 +920,16 @@ bool AnalysisState::loadInternal(const char *cpFilename, bool doMerge, std::ostr
 
 	// state is overwritten
 	setStateModifyLevel(STATE_MODIFIED_NONE);
+	treeState.resetModifyLevel();
 
 #ifdef DEBUG
 	checkSane();
+#else
+	//NASTY HACK. After a few attempts, this still doesnt't work 
+	// as it should. If we are not in debug mode, then we should simply 
+	// fix it as best we can
+	if ( activeCamera >= savedCameras.size())
+		activeCamera=0;
 #endif
 	//Perform sanitisation on results
 	return true;
@@ -940,7 +979,7 @@ void AnalysisState::merge(const AnalysisState &otherState)
 	treeState.clearUndoRedoStacks();	
 
 	if(f.size())
-		treeState.addFilterTree(f,true);
+		treeState.addFilterTree(f);
 	
 
 	const vector<Camera *> &newCameraVec = otherState.savedCameras;	
@@ -1157,7 +1196,9 @@ void AnalysisState::addStashedToFilters(const Filter *parentFilter, unsigned int
 	pair<string,FilterTree> f;
 	copyStashedTree(stashOffset,f);
 
-	treeState.addFilterTree(f.second,parentFilter);
+	size_t filterId;
+	filterId =treeState.getIdByFilter(parentFilter);
+	treeState.addFilterTree(f.second,filterId);
 }
 
 void AnalysisState::copyStashedTrees(std::vector<std::pair<string,FilterTree > > &s) const
@@ -1244,6 +1285,15 @@ bool AnalysisState::hasStateOverrides() const
 	return false;
 }
 
+
+TreeState::TreeState()
+{
+	pendingUpdates=false;
+	wantAbort=false;
+	stateModifyLevel=STATE_MODIFIED_NONE;
+}
+
+
 void TreeState::operator=(const TreeState &oth) 
 {
 #ifdef DEBUG
@@ -1263,8 +1313,9 @@ void TreeState::operator=(const TreeState &oth)
 	
 }
 
-void TreeState::addFilter(Filter *f, bool isBase,size_t parentId)
+void TreeState::addFilter(Filter *f, size_t parentId)
 { 
+	bool isBase = (parentId == (size_t)-1);
 	pushUndoStack();
 	if(!isBase)
 		filterTree.addFilter(f,filterMap[parentId]);
@@ -1294,9 +1345,9 @@ void TreeState::addFilter(Filter *f, bool isBase,size_t parentId)
 	filterMap[idToUse] = f;
 }
 
-void TreeState::addFilterTree(FilterTree &f, bool isBase,size_t parentId)
+void TreeState::addFilterTree(FilterTree &f, size_t parentId)
 { 
-	ASSERT(!(isBase && parentId==(size_t)-1));
+	bool isBase = (parentId == (size_t)-1);
 
 	if(isBase)
 		filterTree.addFilterTree(f,0);
@@ -1440,6 +1491,25 @@ bool TreeState::reparentFilter(size_t filter, size_t newParent)
 	return true;
 }
 
+bool TreeState::respliceFilter(size_t filter, size_t newParent)
+{
+	//Save current filter state to undo stack
+	pushUndoStack();
+
+	//Try to reparent this filter. It might not work, 
+	// eg if parent and filter are the same thing 
+	if(!filterTree.respliceFilter(filterMap[filter],filterMap[newParent]))
+	{
+		//Didn't work. Pop the undo stack, to reverse our 
+		//push, but don't restore it,
+		// as this would cost us our filter caches
+		popUndoStack(false);
+		return false;
+	}
+	
+	return true;
+}
+
 bool TreeState::setFilterProperty(size_t filterId, 
 				unsigned int key, const std::string &value, bool &needUpdate)
 {
@@ -1456,6 +1526,10 @@ bool TreeState::setFilterProperty(size_t filterId,
 		// restoring would destroy the cache
 		popUndoStack(false);
 	}
+	else
+	{
+		elevateModifyLevel(STATE_MODIFIED_DATA);
+	}
 
 	return setOK;
 }
@@ -1469,7 +1543,7 @@ void TreeState::setFilterString(size_t filterId, const std::string &s)
 }
 
 unsigned int TreeState::refresh(std::list<FILTER_OUTPUT_DATA> &refreshData,
-				std::vector<std::pair<const Filter*, std::string> > &consoleMessages, ProgressData &curProg)
+				std::vector<ConsoleMessage > &consoleMessages, ProgressData &curProg)
 {
 	//Attempt to acquire a lock. Return -1 if locking fails
 	wxMutexLocker lock(amRefreshing);
@@ -1527,13 +1601,14 @@ void TreeState::popUndoStack(bool restorePopped)
 	{
 		//Swap the current filter cache out with the undo stack result
 		filterTree.swap(undoFilterStack.back());
+
+		elevateModifyLevel(STATE_MODIFIED_DATA);
 		
 	}
 
 	//Pop the undo stack
 	undoFilterStack.pop_back();
 
-	setStateModifyLevel(STATE_MODIFIED_DATA);
 }
 
 void TreeState::popRedoStack()
@@ -1547,7 +1622,7 @@ void TreeState::popRedoStack()
 	//Pop the redo stack
 	redoFilterStack.pop_back();
 
-	setStateModifyLevel(STATE_MODIFIED_DATA);
+	elevateModifyLevel(STATE_MODIFIED_DATA);
 }
 
 void TreeState::applyBindings(const std::vector<std::pair<const Filter *,SelectionBinding> > &bindings)
@@ -1586,6 +1661,7 @@ void TreeState::applyBindings(const std::vector<std::pair<const Filter *,Selecti
 
 	}
 
+	elevateModifyLevel(STATE_MODIFIED_DATA);
 }
 
 void TreeState::applyBindingsToTree()
@@ -1603,7 +1679,8 @@ void TreeState::applyBindingsToTree()
 	//Clear the modifications to the selection devices
 	for(unsigned int ui=0;ui<selectionDevices.size();ui++)
 		selectionDevices[ui]->resetModifiedBindings();
-
+	
+	elevateModifyLevel(STATE_MODIFIED_DATA);
 
 }
 
@@ -1623,7 +1700,21 @@ bool TreeState::hasMonitorUpdates() const
 	return false;
 }
 
+unsigned int TreeState::getModifyLevel() const
+{
+	ASSERT(stateModifyLevel < STATE_MODIFIED_ENUM_END);
+	return stateModifyLevel;
+}
 
+void TreeState::elevateModifyLevel(unsigned int newLevel)
+{
+	stateModifyLevel=std::max(stateModifyLevel,newLevel);
+}
+
+void TreeState::resetModifyLevel()
+{
+	stateModifyLevel=STATE_MODIFIED_NONE;
+}
 
 #ifdef DEBUG
 
@@ -1632,6 +1723,7 @@ bool testStateReload();
 
 bool runStateTests()
 {
+	//FIXME: There could be a lot more testing here.
 	return testStateReload();
 }
 
