@@ -1,6 +1,6 @@
 /*
  *	state.h - user session state handler
- *	Copyright (C) 2015, D Haley 
+ *	Copyright (C) 2018, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -38,15 +38,14 @@
 bool runStateTests();
 #endif
 
-void setStateModifyLevel(int newLevel);
-int getStateModifyLevel();
 
 enum
 {
 	STATE_MODIFIED_NONE=0,
 	STATE_MODIFIED_VIEW, // the 3D view has chaged
 	STATE_MODIFIED_ANCILLARY, //Eg stashes, inactive cameras, and other things that might get saved
-	STATE_MODIFIED_DATA // actual data output is latered
+	STATE_MODIFIED_DATA,// actual data output is altered
+	STATE_MODIFIED_ENUM_END // not a level, just end of enum marker
 };
 
 
@@ -80,13 +79,16 @@ class TreeState
 		//Do we want to abort the refresh? This is passed to the tree
 		// to signal if the primary thread would like to abort
 		ATOMIC_BOOL wantAbort;
+
+		//How much has the tree been modified since last save?
+		unsigned int stateModifyLevel;
 	public:
-		TreeState() {pendingUpdates=false; wantAbort=false;}
+		TreeState() ;
 		
 		void operator=(const TreeState &otherState);  
 
 		unsigned int refresh(std::list<FILTER_OUTPUT_DATA> &outData,
-				std::vector<std::pair<const Filter*, std::string> > &consoleMessages, ProgressData &prog);
+				std::vector<ConsoleMessage> &consoleMessages, ProgressData &prog);
 
 		//set the abort flag 
 		void setAbort() { wantAbort=true;}
@@ -106,16 +108,17 @@ class TreeState
 
 		const FilterTree &getTreeRef() const { return filterTree ;};
 
-		//!Add a new filter to the tree. Set isbase=false and parentID for not
-		//setting a parent (ie making filter base)
-		void addFilter(Filter *f, bool isBase, size_t parentId);
+		//!Add a new filter to the tree. parentID == -1 for not
+		//setting a parent (ie making filter at base of tree)
+		void addFilter(Filter *f, size_t parentId=(size_t)-1);
 		
-		//!Add a new subtree to the tree. Note that the tree will be cleared
-		// as a result of this operation. Control of all pointers will be handled internally.
-		// Currently, If you wish to use ::getFilterById you *must* rebuild the tree control with
-		// ::updateWxTreeCtrl. This should be fixed.
-		void addFilterTree(FilterTree &f,bool isBase=true, 
-						size_t parentId=(unsigned int)-1); 
+		//!Add a new subtree to the tree, optionally specifying a parent filter.
+		// - Note that the input tree (the one to be added) will be cleared
+		// as a result of this operation. 
+		// Control of all pointers will be handled internally by this calss.
+		// - If you wish to use ::getFilterById you *must* rebuild the tree control with
+		// ::updateWxTreeCtrl. 
+		void addFilterTree(FilterTree &f, size_t parentId=(size_t)-1); 
 
 		//!Grab the filter tree from the internal one, and swap the 
 		// internal with a cloned copy of the internal.
@@ -144,18 +147,18 @@ class TreeState
 		//!Return all of a given type of filter from the filter tree. Type must be the exact type of filter - it is not a mask
 		void getFiltersByType(std::vector<const Filter *> &filters, unsigned int type)  const;
 
-		//!Return the number of filters currently in the main tree
-		size_t numFilters() const { return filterTree.size();};
-
 		//!Clear the cache for the filters
 		void purgeFilterCache() { filterTree.purgeCache();};
 
 		//!Delete a filter and all its children
 		void removeFilterSubtree(size_t filterId);
 
-		//Move a filter from one part of the tree to another
+		//Move a filter, and all its children, from one part of the tree to another
 		bool reparentFilter(size_t filterID, size_t newParentID);
 
+		//Move a filter from one part of the tree to another, leaving all the children in-place
+		bool respliceFilter(size_t filterID, size_t newParentID);
+
 		//!Set the properties using a key-value result 
 		/*
 		 * The return code tells whether to reject or accept the change. 
@@ -213,7 +216,14 @@ class TreeState
 	
 		//Return the selection devices obtained from the last refresh
 		std::vector<SelectionDevice *> &getSelectionDevices() { return selectionDevices;};
-	
+
+
+		//Increase the current state's modification level, if possible
+		void elevateModifyLevel(unsigned int newLevel);
+		//obtain the modification level since last uage
+		unsigned int getModifyLevel() const;
+		//Clear the current modification level
+		void resetModifyLevel();
 };
 
 //The underlying data for any given state in the analysis toolchain
@@ -267,10 +277,17 @@ class AnalysisState
 		//!User-set animation properties
 		PropertyAnimator animationState;
 
-		//TODO: Migrte into some state wrapper class with animationState
+		//TODO: Migrate into some state wrapper class with animationState
 		//Additional state information for animation
 		std::vector<std::pair<std::string,size_t>  > animationPaths;
 
+		//FIXME: this should not be static, but should be available to the filters at state read time.
+		// This variable is only set during a state read
+		static unsigned int currentStateWriterVersion;
+
+		//To what level has the state been modified since last save, or program start?
+		unsigned int stateModifyLevel;  
+
 		bool camNameExists(const std::string &s)  const ;
 
 		//Clear the effect vector
@@ -289,6 +306,11 @@ class AnalysisState
 #endif
 	public:
 
+	
+		//This is only valid during a state read, and should only be used by filters
+		// THis returns the version of the program that wrote the state	
+		static unsigned int getStateWriterVersion();
+
 		TreeState treeState;
 
 		AnalysisState();
@@ -301,7 +323,11 @@ class AnalysisState
 
 		void operator=(const AnalysisState &oth);
 
-		
+		//To what extent the state has been modified since last save
+		unsigned int getStateModifyLevel() const;
+		//Set the extent to which state has been modified since last save
+		// FIXME: Rename. This can only elevate the state modification level, not decrease it
+		void setStateModifyLevel(unsigned int newLevel);
 
 		//Load an XML representation of the analysis state
 		// - returns true on success, false on fail
@@ -318,7 +344,7 @@ class AnalysisState
 		// 	- write package says if state should attempt to ensure that output
 		// 		state is fully self-contained, and locally referenced
 		bool save(const char *cpFilename, std::map<std::string,std::string> &fileMapping,
-				bool writePackage,bool setModifyLevel=true) const ;
+				bool writePackage,bool setModifyLevel=true);
 
 		//Combine a separate state file into this one, avoiding clashes
 		void merge(const AnalysisState &srcState);
