@@ -1,7 +1,7 @@
 /*
 
  * 	viscontrol.h - Visualisation control header; "glue" between user interface and scene
- *	Copyright (C) 2015, D Haley 
+ *	Copyright (C) 2018, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -23,6 +23,7 @@
 #include "wx/propertyGridUpdater.h"
 #include "wx/wxcomponents.h"
 #include "common/voxels.h"
+#include "common/colourmap.h"
 
 using std::list;
 using std::vector;
@@ -53,9 +54,16 @@ unsigned int RefreshController::refresh()
 }
 
 
+void ConsoleMessage::operator=(const ConsoleMessage &oth)
+{
+	parent=oth.parent;
+	message=oth.message;
+	cached=oth.cached;
+}
+
 bool VisController::stateIsModified(unsigned int minLevel) const
 {
-	return state.hasStateData() && getStateModifyLevel();
+	return state.hasStateData() && state.getStateModifyLevel();
 }
 
 void VisController::transferSceneCameraToState()
@@ -114,7 +122,30 @@ void VisController::updateScene(RefreshController *r)
 		dataOnly.push_back(t);
 	}	
 	
-	updateScene(dataOnly,false);
+	updateScene(dataOnly,true);
+
+	vector<ConsoleMessage> cMsg;
+	cMsg = r->getConsoleMessages();
+	if(!cMsg.empty())
+	{
+		//Check that the messages have not been emitted before. 
+		// If that is the case, add a notice to the scene, to inform
+		// the user that there are console messages pending
+		bool msgIsNotCached=false;
+
+		for(unsigned int ui=0;ui<cMsg.size();ui++)
+		{
+			if(!cMsg[ui].cached)
+			{
+				msgIsNotCached=true;
+				break;
+			}
+		}
+
+		//There are uncached messages in the console, let the user know
+		if(msgIsNotCached)
+			scene.addNotice(">> See console...");
+	}
 }
 
 void VisController::updateScene(list<vector<const FilterStreamData *> > &sceneData, 
@@ -133,6 +164,9 @@ void VisController::updateScene(list<vector<const FilterStreamData *> > &sceneDa
 	
 	targetPlots.clear(true); //Clear, but preserve selection information.
 
+	float rBack,gBack,bBack;
+	state.getBackgroundColour(rBack,gBack,bBack);
+	scene.setBackgroundColour(rBack,gBack,bBack);
 
 	//Names for plots
 	vector<std::pair<size_t,string> > plotLabels;
@@ -149,16 +183,18 @@ void VisController::updateScene(list<vector<const FilterStreamData *> > &sceneDa
 		ASSERT(it->size());
 		for(unsigned int ui=0;ui<it->size(); ui++)
 		{
+			const FilterStreamData *filterData;
+			filterData=(*it)[ui];
 			//Filter must specify whether it is cached or not. Other values
 			//are inadmissible, but useful to catch uninitialised values
-			ASSERT((*it)[ui]->cached == 0 || (*it)[ui]->cached == 1);
+			ASSERT(filterData->cached == 0 || filterData->cached == 1);
 			
 			//set to true if we need to free up ram pointed to by 
 			// scene object iterator
 			bool deleteIt;
 			deleteIt=false;
 
-			switch((*it)[ui]->getStreamType())
+			switch(filterData->getStreamType())
 			{
 				case STREAM_TYPE_IONS:
 				{
@@ -174,7 +210,7 @@ void VisController::updateScene(list<vector<const FilterStreamData *> > &sceneDa
 					// (in throttlemap) if they are there, to prevent
 					// overloading the display system.
 					const IonStreamData *ionData;
-					ionData=((const IonStreamData *)((*it)[ui]));
+					ionData=((const IonStreamData *)(filterData));
 					if(throttleMap.find(ionData) != throttleMap.end())
 					{
 						ionData=throttleMap[ionData];
@@ -208,7 +244,7 @@ void VisController::updateScene(list<vector<const FilterStreamData *> > &sceneDa
 				case STREAM_TYPE_PLOT:
 				{
 					const PlotStreamData *plotData;
-					plotData=((PlotStreamData *)((*it)[ui]));
+					plotData=((PlotStreamData *)(filterData));
 					
 					//The plot should have some data in it.
 					ASSERT(plotData->getNumBasicObjects());
@@ -226,6 +262,19 @@ void VisController::updateScene(list<vector<const FilterStreamData *> > &sceneDa
 					Plot1D *plotNew= new Plot1D;
 
 					plotNew->setData(plotData->xyData);
+					for(unsigned int ui=0;ui<2;ui++)
+					{
+						for(unsigned int uj=0;uj<2;uj++)
+						{
+							if(plotData->getPlotBoundActive(ui,uj))
+							{
+								plotNew->setSpecialBounds(ui,uj,
+									plotData->getPlotBound(ui,uj));
+							}
+
+						}
+					}
+
 					plotNew->setLogarithmic(plotData->logarithmic);
 					plotNew->titleAsRawDataLabel=plotData->useDataLabelAsYDescriptor;
 					plotNew->setErrMode(plotData->errDat);
@@ -249,7 +298,7 @@ void VisController::updateScene(list<vector<const FilterStreamData *> > &sceneDa
 						plotData->yLabel,plotData->dataLabel);
 					
 					//set the appearance of the plot
-					//plotNew->setTraceStyle(plotStyle);
+					plotNew->setTraceStyle(plotData->plotStyle);
 					plotNew->setColour(plotData->r,plotData->g,plotData->b);
 					
 					
@@ -266,7 +315,7 @@ void VisController::updateScene(list<vector<const FilterStreamData *> > &sceneDa
 				case STREAM_TYPE_PLOT2D:
 				{
 					const Plot2DStreamData *plotData;
-					plotData=((Plot2DStreamData *)((*it)[ui]));
+					plotData=((Plot2DStreamData *)(filterData));
 					//The plot should have some data in it.
 					ASSERT(plotData->getNumBasicObjects());
 					//The plot should have an index, so we can keep
@@ -275,17 +324,22 @@ void VisController::updateScene(list<vector<const FilterStreamData *> > &sceneDa
 					unsigned int plotID;
 		
 					PlotBase *plotNew;
-					switch(plotData->plotType) 
+					switch(plotData->plotStyle) 
 					{
 						case PLOT_2D_DENS:
 						{
 							//Create a 2D plot
 							plotNew= new Plot2DFunc;
 
+							Plot2DFunc *p2D=(Plot2DFunc*)plotNew;
+
 							//set the plot info
-							((Plot2DFunc*)plotNew)->setData(plotData->xyData,
+							p2D->setData(plotData->xyData,
 										plotData->xMin, plotData->xMax, 
 										plotData->yMin,plotData->yMax);
+							p2D->setLog(plotData->wantImageLog);
+							ASSERT(plotData->colourMap<COLOURMAP_ENUM_END);
+							p2D->setColourMap(plotData->colourMap);
 						
 							break;
 						}
@@ -325,7 +379,7 @@ void VisController::updateScene(list<vector<const FilterStreamData *> > &sceneDa
 				case STREAM_TYPE_DRAW:
 				{
 					DrawStreamData *drawData;
-					drawData=((DrawStreamData *)((*it)[ui]));
+					drawData=((DrawStreamData *)(filterData));
 					
 					//Retrieve vector
 					const std::vector<DrawableObj *> *drawObjs;
@@ -352,7 +406,9 @@ void VisController::updateScene(list<vector<const FilterStreamData *> > &sceneDa
 						//prevent vector destructor from deleting pointers
 						//we have transferred ownership of to scene
 						drawData->drawables.clear();
-						deleteIt=true;
+						
+						if(releaseData)
+							deleteIt=true;
 					}
 					break;
 				}
@@ -362,7 +418,7 @@ void VisController::updateScene(list<vector<const FilterStreamData *> > &sceneDa
 				case STREAM_TYPE_VOXEL:
 				{
 					//Technically, we are violating const-ness
-					VoxelStreamData *vSrc = (VoxelStreamData *)((*it)[ui]);
+					VoxelStreamData *vSrc = (VoxelStreamData *)(filterData);
 					//Create a new Field3D
 					Voxels<float> *v = new Voxels<float>;
 
@@ -374,16 +430,46 @@ void VisController::updateScene(list<vector<const FilterStreamData *> > &sceneDa
 
 					switch(vSrc->representationType)
 					{
+#ifdef ENABLE_LIBVD
+						case VOXEL_REPRESENT_VOLUME_RENDER:
+#endif
 						case VOXEL_REPRESENT_POINTCLOUD:
 						{
+							//Create 3D scalar field drawable
 							DrawField3D  *d = new DrawField3D;
 							d->setField(v);
-							d->setColourMapID(0);
-							d->setColourMinMax();
+#ifdef ENABLE_LIBVD
+							if(vSrc->representationType == VOXEL_REPRESENT_VOLUME_RENDER)
+								d->setRenderMode(VOLUME_RENDER_ACCUMULATE);
+							else
+								d->setRenderMode(VOLUME_POINTS);
+#endif
+
+
+							//Choose the colour map mode
+							d->setColourMapMode(vSrc->colourMapMode);
+
+							switch(vSrc->colourMapMode)
+							{
+								case COLOURMAP_MODE_BUILTIN:
+									d->setColourMapID(vSrc->colourMap);
+									break;
+								case COLOURMAP_MODE_CUSTOM:
+									//Provide colourmap data
+									d->setCustomColourMap(vSrc->customColourMap);
+									break;
+								default:
+									ASSERT(false);
+							}
+						
+							//Set the min/max colour values
+							d->setColourMinMax(vSrc->colourMin,vSrc->colourMax);
+							
 							d->setBoxColours(vSrc->r,vSrc->g,vSrc->b,vSrc->a);
 							d->setPointSize(vSrc->splatSize);
 							d->setAlpha(vSrc->a);
 							d->wantsLight=false;
+							d->setDrawBox(false);
 
 							sceneDrawables.push_back(d);
 							break;
@@ -402,6 +488,7 @@ void VisController::updateScene(list<vector<const FilterStreamData *> > &sceneDa
 							sceneDrawables.push_back(d);
 							break;
 						}
+	
 						default:
 							ASSERT(false);
 							delete v;
@@ -413,12 +500,10 @@ void VisController::updateScene(list<vector<const FilterStreamData *> > &sceneDa
 			}
 			
 			//delete drawables as needed
-			if( (!(*it)[ui]->cached && releaseData) || deleteIt)
+			if( (!filterData->cached && releaseData) || deleteIt)
 			{
-				//Ensure that we didnt force deletion of a cached obejct
-				ASSERT(deleteIt != (*it)[ui]->cached);
-				delete (*it)[ui];
-				(*it)[ui]=0;	
+				delete filterData;
+				filterData=0;	
 			}
 			
 
@@ -495,6 +580,7 @@ void VisController::updateScene(list<vector<const FilterStreamData *> > &sceneDa
 	
 	scene.clearObjs();
 	scene.clearRefObjs();
+	scene.clearNotifications();
 
 
 
@@ -672,7 +758,6 @@ void VisController::updateWxTreeCtrl(wxTreeCtrl *t, const Filter *visibleFilt)
 	upWxTreeCtrl(state.treeState.getTreeRef(),t,	
 			filterMap,persistentFilters,visibleFilt);
 
-	cerr << "Rebuilt filter map" <<endl;
 	state.treeState.swapFilterMap(filterMap);
 }
 
