@@ -1,7 +1,7 @@
 /*
 
  * 	viscontrol.h - Visualisation control header; "glue" between user interface and scene
- *	Copyright (C) 2015, D Haley 
+ *	Copyright (C) 2018, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -20,6 +20,12 @@
 #ifndef VISCONTROL_H
 #define VISCONTROL_H
 
+#if defined(_WIN32) || defined(_WIN64)
+//Required to solve bug similar to wx bug 674, due
+// to redefinition of CreateDialog as macro
+#include <wx/wx.h>
+#endif
+
 #include <wx/listbox.h>
 #include <wx/textctrl.h>
 #include <wx/grid.h>
@@ -37,14 +43,13 @@ class wxTreeCtrl;
 #include <list>
 
 //!Tree controller
-
 class RefreshController
 {
 	private:
 		TreeState *treeState;
 		//!Results of last refresh
 		std::list<FILTER_OUTPUT_DATA> refreshData;
-		std::vector<std::pair<const Filter*, std::string> > consoleMessages;
+		std::vector<ConsoleMessage> consoleMessages;
 	public:
 
 		//Initialisation requires treecontroller
@@ -59,7 +64,7 @@ class RefreshController
 
 
 		std::list<FILTER_OUTPUT_DATA> &getRefreshData() { return refreshData;};
-		std::vector<std::pair<const Filter*, std::string> > &getConsoleMessages() { return consoleMessages;};
+		std::vector<ConsoleMessage> &getConsoleMessages() { return consoleMessages;};
 	
 
 };
@@ -100,10 +105,6 @@ class VisController
 		std::map<size_t, size_t> plotMap;
 
 
-
-
-		//!Update the console strings
-		void updateConsole(const std::vector<std::string> &v, const Filter *f) const;
 		//!Limit the number of objects we are sending to the scene
 		void throttleSceneInput(std::list<std::vector<const FilterStreamData *> > &outputData, 
 			std::map<const IonStreamData *, const IonStreamData *> &throttleMap) const;
@@ -111,7 +112,7 @@ class VisController
 		AnalysisState state;
 		Scene scene;
 
-		VisController() {ASSERT(!isInstantiated); isInstantiated=true; scene.setVisControl(this);}; 
+		VisController() {MESSAGE("VisController constructor"); ASSERT(!isInstantiated); isInstantiated=true; scene.setVisControl(this);}; 
 	
 		void setActiveCam(unsigned int cam);
 
