@@ -1,6 +1,6 @@
 /*
  *	basics.cpp  - basic functions header
- *	Copyright (C) 2015, D Haley 
+ *	Copyright (C) 2018, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -219,7 +219,7 @@ string veryFuzzyTimeSince( time_t origTime, time_t nowTime)
 					true,	//hour
 					false,	//45m	
 					false,	//30m	
-					false,	//20m	
+					true,	//20m	
 					false,	//15m	
 					false,	//10m	
 					false,	//5m
@@ -259,7 +259,7 @@ string veryFuzzyTimeSince( time_t origTime, time_t nowTime)
 			 NTRANS("a few hours ago"),
 			 "", //45m
 			 "", //30m
-			 "", //20m
+			 NTRANS("tens of minutes ago"), //20m
 			 "", //15m
 			 "", //10m
 			 "", //5m
@@ -564,7 +564,7 @@ float ColourRGBAf::at(unsigned int idx) const
 }
 
 		
-ColourRGBAf ColourRGBAf::interpolate(float delta, const ColourRGBAf &other)
+ColourRGBAf ColourRGBAf::interpolate(float delta, const ColourRGBAf &other) const
 {
 	ColourRGBAf result;
 
@@ -573,6 +573,12 @@ ColourRGBAf ColourRGBAf::interpolate(float delta, const ColourRGBAf &other)
 	return result;
 }
 
+void ColourRGBAf::copyToRGBArray(unsigned char *array) const
+{
+	for(unsigned int ui=0;ui<3;ui++)
+		array[ui] = data[ui]*255;
+}
+
 ColourRGBA ColourRGBAf::toColourRGBA() const
 {
 	ColourRGBA tmp(data[0]*255.0f,data[1]*255.0f,
@@ -624,6 +630,71 @@ bool ColourRGBAf::operator!=(const ColourRGBAf  &oth) const
 {
 	return !(*this == oth);
 }
+
+void getContrastingColour(float r, float g, float b, 
+		float &rCont, float &gCont, float &bCont)
+{
+
+	//luma colour space - value is perceived brightness
+	//https://en.wikipedia.org/wiki/HSV_color_space#Lightness
+	float value= sqrt(0.299*r*r + 0.587*g*g + 0.114*b*b);
+
+	//Clamp to 0-1
+	value=std::min(value,1.0f);
+	value=std::max(value,0.0f);
+
+	//Quick and dirty : white if background dark, black if not.
+	if(value < 0.5)
+		rCont=gCont=bCont=1.0f;
+	else
+		rCont=gCont=bCont=0.0f;
+
+	/* TODO: We can pick a desired hue, and flip the value
+	float hue,sat;
+	float cMax = std::max(r,std::max(g,b));
+	float cMin = std::min(r,std::min(g,b));
+
+	float delta;
+	delta = cMax-cMin;
+
+	if(delta == 0)
+		hue=0;
+	else if ( cMax == r)
+	{
+		hue = ( mod((g -b)/delta,6.0f));
+	}
+	else if (cMax == g)
+	{
+		hue = (b-r)/delta + 2;
+	}
+	else if (cMax == b)
+	{
+		hue = (r-g)/delta + 4;
+	}
+
+	hue*=60;
+
+	if(value== 1.0f)
+		sat = 0;
+	else
+		sat = delta/value;
+
+	//OKAY, so we now have  an HSV triplet
+	*/
+
+
+}
+
+BoundCube::BoundCube(const Point3D &p1, const Point3D &p2)
+{
+	for(unsigned int ui=0;ui<3;ui++)
+	{
+		valid[ui][0] = valid[ui][1] = true;
+		bounds[ui][0] =std::min(p1[ui],p2[ui]);
+		bounds[ui][1] =std::max(p1[ui],p2[ui]);
+	}
+}
+
 void BoundCube::getBound(Point3D &retBound, unsigned int minMax) const
 {
 	retBound=Point3D(bounds[0][minMax],
@@ -728,8 +799,10 @@ void BoundCube::getPlaneIntersectVertices(const Point3D &planeOrigin, const Poin
 	//		and  (idx,idx+1) (idx in 0,2,4,6)	: 4 edges (vertical)
 
 	//Adjacency graph for cube edges
-	const unsigned int eStartIdx[12] = {0,1,2,3, 0,1,4,5, 0,2,4,6};
-	const unsigned int eEndIdx[12] = {4,5,6,7, 2,3,6,7, 1,3,5,7};
+	const unsigned int eStartIdx[12] = { 0,1,2,3, 0,1,4,5, 0,2,4,6
+						};
+	const unsigned int eEndIdx[12] = { 4,5,6,7, 2,3,6,7, 1,3,5,7
+						};
 	
 
 	for(unsigned int ui=0;ui<12;ui++)
@@ -748,9 +821,75 @@ void BoundCube::getPlaneIntersectVertices(const Point3D &planeOrigin, const Poin
 
 		float numerator = (planeOrigin - eStart).dotProd(normal);
 		float v;
-		v= numerator/denom;	
-		intersectPts.push_back((eEnd-eStart)*v+ eStart);
+		v= numerator/denom;
+		if(v <=1.0f && v>=0.0f)	
+			intersectPts.push_back((eEnd-eStart)*v+ eStart);
 	}
+
+	vector<bool> killV;
+	killV.resize(intersectPts.size(),false);
+	//Eliminate duplicates
+	//---
+	for(unsigned int ui=0;ui<intersectPts.size() ; ui++)
+	{
+		//we already marked this one, move on
+		if(killV[ui])
+			continue;
+
+		//find the dups for this entry
+		for(unsigned int uj=ui+1;uj<intersectPts.size();uj++)
+		{
+			if(intersectPts[ui] == intersectPts[uj])
+				killV[ui]=true;
+		}
+
+	}
+
+	//kill the duplicated entries
+	vectorMultiErase(intersectPts,killV);
+	//---
+
+
+	//The generated points may not be simplical,
+	// we may need to "untwist" the polygon
+	if(intersectPts.size() > 3)
+	{
+		//Find the clockwise angle from the first vector (vertex-midpt)
+		// to all the other vectors
+		Point3D midPt = Point3D::centroid(intersectPts);
+		vector<std::pair<size_t, float> > angles;
+		angles.resize(intersectPts.size());
+		Point3D v1= intersectPts[0] - midPt;
+		v1.normalise(); 
+		angles[0]=std::make_pair(0,0);
+		for(unsigned int ui=0;ui<intersectPts.size()-1;ui++)
+		{
+			Point3D v2;
+			v2 = intersectPts[(ui+1)%intersectPts.size()] - midPt;
+			v2.normalise();
+
+			angles[ui+1] = std::make_pair(ui+1,clockwiseAngle(v1,v2,normal));
+		}
+
+		//sort by increasing angle
+		ComparePairSecond cmp;
+		std::sort(angles.begin(),angles.end(),cmp);
+	
+
+		//reshuffle the points to be in-order
+		vector<Point3D> newPts;
+		newPts.resize(intersectPts.size());
+		for(unsigned int ui=0;ui<intersectPts.size();ui++)
+			newPts[ui] = intersectPts[angles[ui].first];
+
+		//swap re-ordered points out
+		intersectPts.swap(newPts);
+
+	}
+
+	//Can be at most a hexagon
+	ASSERT(intersectPts.size() <=6);	
+
 }
 
 Point3D BoundCube::getVertex(unsigned int idx) const
@@ -956,6 +1095,20 @@ float BoundCube::getSize(unsigned int dim) const
 	return fabs(bounds[dim][1] - bounds[dim][0]);
 }
 
+Point3D BoundCube::getSize() const
+{
+#ifdef DEBUG
+	for(unsigned int ui=0;ui<3; ui++)
+	{
+		ASSERT(valid[0][1] && valid [0][0]);
+	}
+#endif
+	Point3D p;
+	for(unsigned int ui=0; ui<3; ui++)
+		p[ui] = bounds[ui][1] -bounds[ui][0];
+
+	return p;
+}
 //checks intersection with sphere [centre,centre+radius)
 bool BoundCube::intersects(const Point3D &pt, float sqrRad) const
 {
@@ -1347,7 +1500,8 @@ unsigned int loadTextData(const char *cpFilename, vector<vector<float> > &dataVe
 	while(CFile.good() && !CFile.eof() && atHeader)
 	{
 		//Grab a line from the file
-		CFile.getline(inBuffer,BUFFER_SIZE);
+		if(!CFile.getline(inBuffer,BUFFER_SIZE))
+			break;
 
 		if(!CFile.good())
 			return ERR_FILE_FORMAT;
@@ -1457,10 +1611,8 @@ unsigned int loadTextData(const char *cpFilename, vector<vector<float> > &dataVe
 			
 		}
 		//Grab a line from the file
-		CFile.getline(inBuffer,BUFFER_SIZE);
-		
-		if(!CFile.good() && !CFile.eof())
-			return ERR_FILE_FORMAT;
+		if(!CFile.getline(inBuffer,BUFFER_SIZE))
+			break;
 	}
 
 	return 0;
@@ -1516,6 +1668,8 @@ unsigned int loadTextStringData(const char *cpFilename, vector<vector<string> >
 }
 
 
+
+
 #if !defined(__WIN32__) && !defined(__WIN64)
 	
 bool isNotDirectory(const char *filename)
@@ -1566,8 +1720,52 @@ bool isValidXML(const char *filename)
 		return result ==0;
 	}
 
-	//Debug check ineffective
+#if !(defined(WIN32) || defined(WIN64))
+	//Debug check ineffective - don't bother warning under windows, as the binary
+	// is unlikely to be present.
 	WARN(!result,"xmllint not installed in system PATH, cannot perform debug check")
+#endif
 	return true;
 }
+
+
+bool BoundCube::test()
+{
+	//Try to obtain the plane intersections
+	BoundCube b(Point3D(0,0,0),Point3D(1,1,1));
+
+	vector<Point3D> pts;
+	b.getPlaneIntersectVertices(
+		Point3D(0.5,0.5,0.5),Point3D(0,0,1),pts);
+
+	TEST(pts.size() == 4,"plane intersect test");
+	for(unsigned int ui=0;ui<pts.size();ui++)
+	{
+		TEST(EQ_TOL(pts[ui][2],0.5),"intersect test");
+	}
+
+	pts.clear();
+	b.getPlaneIntersectVertices(
+		Point3D(0.5,0.5,0.5),Point3D(1,1,0),pts);
+	TEST(pts.size() == 4,"plane intersect test");
+	for(unsigned int ui=0;ui<pts.size();ui++)
+	{
+		TEST( (EQ_TOL(pts[ui][2],0) || EQ_TOL(pts[ui][2],1)) ,"intersect test");
+	}
+	pts.clear();
+	b.getPlaneIntersectVertices(
+		Point3D(0.95,0.95,0.95),Point3D(1,1,1),pts);
+	TEST(pts.size() == 3,"plane intersect test");
+
+	return true;
+}
+
+bool runBasicsTests()
+{
+	if(!BoundCube::test())
+		return false;
+
+	return true;
+}
+
 #endif
