@@ -1,6 +1,6 @@
 /*
  *	common/basics.h - Basic functionality header 
- *	Copyright (C) 2015, D Haley 
+ *	Copyright (C) 2018, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -331,6 +331,8 @@ public:
         setInvalid();
     }
 
+    BoundCube(const Point3D &p1, const Point3D &p2);
+
     void setBounds(float xMin,float yMin,float zMin,
                    float xMax,float yMax,float zMax) {
         bounds[0][0]=xMin; bounds[1][0]=yMin; bounds[2][0]=zMin;
@@ -370,9 +372,12 @@ public:
     //!Get the bounds
     void getBounds(Point3D &low, Point3D &high) const ;
 
-    //!Return the size of the cube along the specified dimension
+    //!Return the side length of the cube along the specified dimension
     float getSize(unsigned int dim) const;
 
+    //!Return sizes of bounds for each dimension (max-min)
+    Point3D getSize() const;
+
     //! Returns true if all bounds are valid
     bool isValid() const;
 
@@ -439,6 +444,9 @@ public:
 
     friend  std::ostream &operator<<(std::ostream &stream, const BoundCube& b);
 
+#ifdef DEBUG
+	static bool test();
+#endif
     //FIXME: Hack!
     friend class K3DTree;
     friend class K3DTreeMk2;
@@ -452,6 +460,9 @@ typedef struct RGBf
 	float blue;
 } RGBf;
 
+void getContrastingColour(float r, float g, float b, 
+		float &rCont, float &gCont, float &bCont);
+
 class ColourRGBAf;
 
 //Colour storage class. Uses uchar internally
@@ -496,33 +507,43 @@ class ColourRGBA
 		
 		unsigned char at(unsigned int idx) const;
 
+		unsigned char &operator[](unsigned int idx) { ASSERT(idx <4); return data[idx];}
+
 };
 
 //Colour storage class. Uses float internally
 class ColourRGBAf
 {
 	private:
+		//colour intensity data in RGBA format.
+		// scaled to [0,1]
 		float data[4];
 	public:
 		ColourRGBAf();
 		ColourRGBAf(float, float, float);
 		ColourRGBAf(float, float, float,float);
+		//Return colours
 		float r() const;
 		float g() const;
 		float b() const;
 		float a() const;
 		
+		//Assign colours
 		void r(float);
 		void g(float);
 		void b(float);
 		void a(float);
 
 
-		ColourRGBAf interpolate(float delta, const ColourRGBAf &other);
+		//Interpolate between two RGBAfs to yield an RGBAf
+		ColourRGBAf interpolate(float delta, const ColourRGBAf &other) const;
 		//convert to a ColourRGBA (uchar representation)
 		//TODO : Rename me!
 		ColourRGBA toColourRGBA() const; 
 		RGBf toRGBf() const; 
+
+		//!Copy the data to an unsigned char array.
+		void copyToRGBArray(unsigned char *a) const;
 		bool operator==(const ColourRGBA &oth) const;
 		bool operator!=(const ColourRGBA &oth) const;
 		bool operator==(const ColourRGBAf &oth) const;
