@@ -1,6 +1,6 @@
 /*
  *	endianttest.h - Platform endian testing
- *	Copyright (C) 2015, D Haley 
+ *	Copyright (C) 2018, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -25,10 +25,12 @@
 #include <endian.h>
 #endif
 #endif
+#include <stdint.h>
 
 #ifdef __BYTE_ORDER
 //if both are not defined it is TRUE!
-#if __BYTE_ORDER == __BIG_ENDIAN
+// on F28 ppc64, is defined as __ORDER_BIG_ENDIAN
+#if (__BYTE_ORDER == __BIG_ENDIAN) || (__BYTE_ORDER == __ORDER_BIG_ENDIAN)
 #ifndef __BIG_ENDIAN__
 #define __BIG_ENDIAN__
 #endif
@@ -51,7 +53,7 @@ inline int is_bigendian() { return (*(char*)&ENDIAN_TEST) == 0 ;}
 
 inline int is_littleendian() { return (*(char*)&ENDIAN_TEST) == 1 ;}
 
-
+//this may fail on some compilers??
 inline void floatSwapBytes(float *inFloat)
 {
 	//Use a union to avoid strict-aliasing error
@@ -69,5 +71,19 @@ inline void floatSwapBytes(float *inFloat)
 
 	*inFloat=fb.f;
 }
+inline void uint16_tSwapBytes(uint16_t *inShort)
+{
+	//Use a union to avoid strict-aliasing error
+	union Uint16SwapUnion{
+	   uint16_t s;
+	   char c[2];
+	} ;
+	Uint16SwapUnion sa,sb;
+	sa.s = *inShort;
 
+	sb.c[0] = sa.c[1];
+	sb.c[1] = sa.c[0];
+
+	*inShort=sb.s;
+}
 #endif
