@@ -1,6 +1,6 @@
 /*
  *	mathfuncs.cpp - Miscellaneous mathematical functions implementation. 
- *	Copyright (C) 2015, D Haley 
+ *	Copyright (C) 2018, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -235,6 +235,11 @@ float Point3D::sqrMag() const
 	return value[0]*value[0] + value[1]*value[1] + value[2]*value[2];
 }
 
+float Point3D::mag() const
+{
+	return sqrtf(sqrMag());
+}
+
 Point3D Point3D::normalise()
 {
 	float mag = sqrtf(sqrMag());
@@ -301,6 +306,13 @@ void Point3D::sphericalAngles(float &theta, float &phi) const
 #endif
 }
 
+void Point3D::setISOSpherical(float theta, float phi, float r)
+{
+	value[0] = r*sin(theta)*cos(phi);
+	value[1] = r*sin(theta)*sin(phi);
+	value[2] = r*cos(theta);
+}
+
 bool Point3D::orthogonalise(const Point3D &pt)
 {
 	Point3D crossp;
@@ -368,16 +380,20 @@ bool Point3D::parse(const std::string &str)
 	string tmpStr;
 	tmpStr=stripWhite(str);
 
+	//are we using polar notation?
+	bool polarNotation = (tmpStr[0] == '<' && tmpStr[tmpStr.size()-1] == '>');
 
 	//Two strings must be in sync
 	std::string allowableStartChars, allowableEndChars;
 	allowableStartChars="([{<'";
 	allowableEndChars=")]}>'";
 
+	//Find the start/end chars
 	size_t startPos,endPos;
 	startPos=allowableStartChars.find(tmpStr[0]);
 	endPos=allowableEndChars.find(tmpStr[tmpStr.size()-1]);
 
+
 	//Strip the start/end chars 
 	if(startPos !=std::string::npos && endPos != std::string::npos)
 		tmpStr=tmpStr.substr(1,tmpStr.size()-1);
@@ -420,13 +436,25 @@ bool Point3D::parse(const std::string &str)
 			return false;
 	}
 
+	if(!polarNotation)
+		setValueArr(p);
+	else
+	{
+		//r,theta,phi, as users should input
+		// <r,theta,phi>
+		//Convert from degrees to radiians
+		p[1]*=M_PI/180;	
+		p[2]*=M_PI/180;	
+
+		//set spherical co-ordinates
+		setISOSpherical(p[1],p[2],p[0]);
+	}
+
 
-	setValueArr(p);
 	
 	return true;
 }
 
-#ifdef __LITTLE_ENDIAN__
 
 void Point3D::switchEndian()
 {
@@ -434,7 +462,6 @@ void Point3D::switchEndian()
 	floatSwapBytes(&value[1]);
 	floatSwapBytes(&value[2]);
 }
-#endif
 
 std::ostream& operator<<(std::ostream &stream, const Point3D &pt)
 {
@@ -769,6 +796,14 @@ void quat_rot_apply_quat(Point3f *point, const Quaternion *rotQuat)
 	quat_mult_no_second_a(&temp,rotQuat,&pointQuat);
 	quat_pointmult(point, &temp,rotQuat);
 }
+
+//Invert the given quaternion (this for example, can generate the inverse rotation) 
+void quat_invert(Quaternion *quat)
+{
+	quat->b=-quat->b;
+	quat->c=-quat->c;
+	quat->d=-quat->d;
+}
 	
 //For the table to work, we need the sizeof(size_T) at preprocess time
 #ifndef SIZEOF_SIZE_T
@@ -1024,5 +1059,182 @@ void rotateByMatrix(const vector<Point3D> &vpts, const gsl_matrix *m, vector<Poi
 	gsl_vector_free(rv);
 }
 
-	
+float clockwiseAngle(const Point3D &v1, const Point3D &v2,const Point3D &normal)
+{
 
+	//sin(theta) = determinant
+	// cos(tehta) = |u||v|
+	float det = normal.dotProd(v1.crossProd(v2));
+	float angle= 	atan2(det,v1.dotProd(v2));
+	//shift range to [0,2*PI]
+	if(angle < 0)
+		angle +=2.0*M_PI;
+	return angle;
+}
+
+
+Point3D generatePointInTri(const Point3D &origin,
+		const Point3D &sideV1, const Point3D &sideV2, RandNumGen &rng)
+{
+	float r, s;
+	r = rng.genUniformDev();
+	s= rng.genUniformDev();
+
+	if(r+s >=1)
+	{
+		//Remap to be inside triangle, as the 
+		// vector sum lies in a parallelogram, but
+		// not in the triangle 
+		r = 1-r;
+		s= 1-s;
+	}
+
+
+	return origin + sideV1*r + sideV2*s;	
+
+}
+
+void scatterPointsInPolygon(const vector<Point3D> &polygon, const Point3D &normal,
+		float scatterDensity, RandNumGen &rng, vector<Point3D> &result)
+{
+	ASSERT(polygon.size() > 2);
+
+	//Find the area of the polygon
+	Point3D centroid;
+	centroid = Point3D::centroid(polygon);
+
+	//Keep track of the relative area of each triangle,
+	// and its vectors
+	vector<float> weights;
+	vector<std::pair<Point3D, Point3D> > triVecs;
+	weights.resize(polygon.size());
+	triVecs.resize(polygon.size());
+
+	float area=0;
+	for(unsigned int ui=0;ui<polygon.size();ui++)
+	{
+		//Compute side vectors
+		Point3D v1,v2;
+		v1 = polygon[ui] -centroid;
+		v2 = polygon[(ui+1)%polygon.size()] - centroid;
+		
+		//remember for later
+		triVecs[ui].first = v1;
+		triVecs[ui].second=v2;
+
+		//add this triangle's area
+		float thisArea;
+		thisArea= v1.mag()*v2.mag()*0.5*sin(v1.angle(v2));
+		weights[ui] = thisArea;
+		area+=thisArea;
+	}
+
+	if(area < sqrt(std::numeric_limits<float>::epsilon()))
+		return;
+
+	//normalise the weight to total area,
+	// and accumulate
+	float cumSum=0.0f;;
+	for(unsigned int ui=0;ui<weights.size();ui++)
+	{
+		weights[ui]/=area;
+		cumSum+=weights[ui];
+		weights[ui]=cumSum;
+	}
+	
+	unsigned int nScatter = ceil(scatterDensity*area);
+
+	result.resize(nScatter);
+	for(unsigned int ui=0;ui<nScatter; ui++)
+	{
+		//Randomly pick a triangle
+		unsigned int triangle;
+		triangle =std::lower_bound(weights.begin(),
+			weights.end(), rng.genUniformDev()) - weights.begin();
+
+		//generate a random point in it
+		Point3D p;
+		result[ui] = generatePointInTri(centroid,triVecs[triangle].first,
+					triVecs[triangle].second,rng);
+	}
+
+}
+
+
+void rotatePointsToXYPlane(vector<Point3D> &pts, const Point3D &normal, const Point3D &origin, Quaternion &q)
+{
+	//Move all points to origin;
+	for(size_t ui=0;ui<pts.size();ui++)
+		pts[ui]-=origin;
+
+	//FIXME : Quaternion not inited
+	float rotateAngle = normal.angle(Point3D(0,0,1)); 
+	if(fabs(rotateAngle) < sqrt(std::numeric_limits<float>::epsilon()))
+		return;
+
+	//Compute rotation axis
+	Point3D rotateAxis = normal.crossProd(Point3D(0,0,1));
+	rotateAxis.normalise();
+
+	Point3f ra;
+	ra.fx=rotateAxis[0]; 
+	ra.fy=rotateAxis[1];
+	ra.fz=rotateAxis[2];
+	quat_get_rot_quat(&ra,rotateAngle,&q);
+
+	//perform in-place rotation
+	for(unsigned int ui=0;ui<pts.size();ui++)
+	{
+		Point3f pf;
+		pf.fx=pts[ui][0];
+		pf.fy=pts[ui][1];
+		pf.fz=pts[ui][2];
+		
+		quat_rot_apply_quat(&pf,&q);
+
+		pts[ui] = Point3D(pf.fx,pf.fy,pf.fz);
+	}
+
+
+}
+
+void applyQuaternion(vector<Point3D> &pts,Quaternion &q)
+{
+	//perform in-place rotation
+	for(unsigned int ui=0;ui<pts.size();ui++)
+	{
+		Point3f pf;
+		pf.fx=pts[ui][0];
+		pf.fy=pts[ui][1];
+		pf.fz=pts[ui][2];
+		
+		quat_rot_apply_quat(&pf,&q);
+
+		pts[ui] = Point3D(pf.fx,pf.fy,pf.fz);
+	}
+
+}
+
+#ifdef DEBUG
+
+bool testMathfuncs()
+{
+	Point3D p1,p2,n;
+	p1=Point3D(1,0,0);
+	p2=Point3D(0,1,0);
+	n=Point3D(0,0,1);
+
+	TEST( EQ_TOL(clockwiseAngle(p1,p2,n), M_PI/2.0f),"clockwise angle test");
+
+	p2=Point3D(1,1,0);
+	TEST( EQ_TOL(clockwiseAngle(p1,p2,n), M_PI/4.0f),"clockwise angle test");
+	
+	p2=Point3D(-1,1,0);
+	TEST( EQ_TOL(clockwiseAngle(p1,p2,n), M_PI/2.0f  + M_PI/4.0f),"clockwise angle test");
+	p2=Point3D(-1,-1,0);
+	TEST( EQ_TOL(clockwiseAngle(p1,p2,n), M_PI  + M_PI/4.0f),"clockwise angle test");
+
+	return true;
+}
+
+#endif
