@@ -1,6 +1,6 @@
 /*
  *	stringFuncs.cpp - string manipulation routines
- *	Copyright (C) 2015, D Haley 
+ *	Copyright (C) 2018, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -24,89 +24,43 @@
 
 using std::string;
 using std::vector;
+using std::pair;
 
 std::string getMaxVerStr(const std::vector<std::string> &verStrings)
 {
-	std::vector<std::pair<size_t,std::vector<unsigned int> > > verNum;
-	std::vector<unsigned int> thisVer;
-
+	vector<unsigned int > verNumList;
 
 	//break string up into numeric components
 	for(unsigned int ui=0;ui<verStrings.size();ui++)
 	{
-		std::vector<std::string> strVerNum;
-		strVerNum.clear();
+		unsigned int verNumber;
+		if(!parseVersion(verStrings[ui],verNumber))
+			continue;
 
-		// period or hyphen are valid version number separators
-		splitStrsRef(verStrings[ui].c_str(),".-",strVerNum);
+		verNumList.push_back(verNumber);
+	}
 
-		//Check to see if we can interpret the values
-		for(unsigned int uj=0;uj<strVerNum.size();uj++)
+	unsigned int maxV=0;
+	unsigned int maxP=(unsigned int)-1;
+	for(unsigned int ui=0;ui<verNumList.size();ui++)
+	{
+		if(verNumList[ui] > maxV)
 		{
-			int i;
-			
-			//Try to cast the string (returns true on failure)
-			if(!stream_cast(i,strVerNum[uj]))
-				thisVer.push_back(i);
-
-		}
-
-		if(thisVer.size())
-		{
-			verNum.push_back(make_pair(ui,thisVer));
-			thisVer.clear();
+			maxP=ui;
+			maxV=verNumList[ui];
 		}
 	}
-		
-	
-	if(verNum.empty())
+
+	if(maxP == (unsigned int)-1)
 		return std::string("");
 
-	//OK, so now we have an integral list.
-	//Find the minimal element in each set until we 
-	//knock out all elements but one
-	size_t maxVerLen=0;
-	for(unsigned int ui=0;ui<verNum.size();ui++)
-		maxVerLen=std::max(maxVerLen,verNum[ui].second.size());
+	return verStrings[maxP];
+}
 
-		
-	unsigned int pos=0;
-	while(pos<maxVerLen && verNum.size() > 1)
-	{
-		unsigned int thisMax;
-		thisMax=0;
-
-		for(unsigned int ui=0;ui<verNum.size();ui++)
-		{
-			//If the version string has enough digits, check to see if it is the maximum for this digit
-			if(pos < verNum[ui].second.size() )
-				thisMax=std::max(thisMax,verNum[ui].second[pos]);
-		}
-
-		//Kill off any version numbers that were not 
-		//the max value, or had insufficient numbers
-		for(unsigned int ui=verNum.size();ui;)
-		{
-			ui--;
-
-			if(verNum[ui].second.size() <=pos || 
-				verNum[ui].second[pos] < thisMax )
-			{
-				std::swap(verNum[ui],verNum.back());
-				verNum.pop_back();
-			}
-		}
-
-		//move to next number
-		pos++;
-	}
-
-
-	//Should contain at least one version (ie the maximum, or multiple copies thereof)
-	ASSERT(verNum.size());
-
-
-	return verStrings[verNum[0].first];
+unsigned int getVersionNumber(unsigned int major, unsigned int minor, unsigned int revision)
+{
+	ASSERT(minor < 100 && revision < 100);
+	return major*100*100 + minor*100 + revision;
 }
 
 bool isVersionNumberString(const std::string &s)
@@ -124,6 +78,43 @@ bool isVersionNumberString(const std::string &s)
 	return true;
 }
 
+bool parseVersion(const std::string &s, unsigned int &version)
+{
+	if(!isVersionNumberString(s))
+		return false;
+		
+	std::vector<std::string> strVerNum;
+
+	// period or hyphen are valid version number separators
+	splitStrsRef(s.c_str(),".-",strVerNum);
+
+	
+	//reverse, so we are parsing from the right-most version number
+	std::reverse(strVerNum.begin(),strVerNum.end());
+
+	unsigned int number=0;
+
+	//Check to see if we can interpret the values
+	unsigned int factor=1;
+	for(unsigned int uj=0;uj<strVerNum.size();uj++)
+	{
+		unsigned int i;
+		factor*=10;
+
+		//Try to cast the string (returns true on failure)
+		if(stream_cast(i,strVerNum[uj]))
+			return false;
+
+		//partial version numbers should be at most 99 
+		if(i >=100)
+			return false;
+		number +=(i*factor);
+	}
+
+	version=number;
+
+	return true;		
+}
 
 bool boolStrDec(const std::string &s, bool &b)
 {
@@ -363,6 +354,81 @@ std::string choiceString(std::vector<std::pair<unsigned int, std::string> > comb
 	return s;
 }
 
+std::string transferFunctionString(const std::vector<pair<float,ColourRGBAf> >  &colWithOffset)
+{
+	vector<pair<float,ColourRGBAf> > tmp;
+	tmp=colWithOffset;
+	//Ensure the transfer function x-value is monotonically increasing
+	ComparePairFirst cmpF;
+	std::sort(tmp.begin(),tmp.end(),cmpF);
+
+	std::string res,tmpS;
+	for(unsigned int ui=0; ui<tmp.size();ui++)
+	{
+		//Record X
+		stream_cast(tmpS,tmp[ui].first);
+		res+=tmpS+ ",";
+
+		//Record RGBA components
+		for(unsigned int uj=0;uj<4;uj++)
+		{
+			stream_cast(tmpS,tmp[ui].second[uj]);
+			res+=tmpS;
+
+			if(uj !=3)
+				res+=",";
+		}
+
+		//Separate records
+		if(ui != colWithOffset.size()-1)
+			res+="|";
+	}
+
+	return res;
+}
+
+bool fromTransferFunctionString(const std::string &s , std::vector<pair<float,ColourRGBAf> > &colWithOffset)
+{
+	vector<string> components;
+	splitStrsRef(s.c_str(),'|',components);
+
+	if(!components.size())
+		return false;
+
+	colWithOffset.clear();
+
+	pair<float,ColourRGBAf> colPair;
+	for(unsigned int ui=0;ui<components.size();ui++)
+	{
+		vector<string> thisCompStrVec;
+		splitStrsRef(components[ui].c_str(),',',thisCompStrVec);
+
+		//Should be 5 vector
+		if(thisCompStrVec.size() %5)
+			return false;
+
+		for(unsigned int uj=0;uj<thisCompStrVec.size(); uj++)
+		{
+			float u;
+			if(uj == 0)
+			{
+				if(stream_cast(colPair.first,thisCompStrVec[uj]))
+					return false;
+			}
+			else
+			{
+				if(stream_cast(u,thisCompStrVec[uj]))
+					return false;
+				colPair.second[uj-1] = u;
+			}
+
+		}
+		colWithOffset.push_back(colPair);
+
+	}
+
+	return true;
+}
 
 //Strip "whitespace"
 std::string stripWhite(const std::string &str)
