@@ -1,6 +1,6 @@
  /*
  * common/voxels.h - Voxelised data manipulation class
- * Copyright (C) 2015  D. Haley
+ * Copyright (C) 2018, D Haley
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -129,10 +129,6 @@ template<class T> class Voxels
 		//to maxBound
 		Point3D minBound, maxBound;
 	
-			
-
-		void localPaddedConvolve(long long ui,long long uj, long long uk, 
-			const Voxels<T> &kernel,Voxels<T> &result, unsigned int mode) const;
 	public:
 
 		//!Constructor.
@@ -269,7 +265,7 @@ template<class T> class Voxels
 		//!Get the bounding box vertex (min/max) 
 		Point3D getMinBounds() const;
 		Point3D getMaxBounds() const;
-		//Obtain the ounds for a specified axis
+		//Obtain the bounds for a specified axis
 		void getAxisBounds(size_t axis, float &minV, float &maxV) const;
 		///! Get the spacing for a unit cell
 		Point3D getPitch() const;
@@ -277,6 +273,7 @@ template<class T> class Voxels
 		void setBounds(const Point3D &pMin, const Point3D &pMax);
 		//!Get the bounding size
 		void getBounds(Point3D &pMin, Point3D &pMax) const { pMin=minBound;pMax=maxBound;}
+		void getBounds(BoundCube &bc) const { bc.setBounds(minBound,maxBound);}
 
 		//!Initialise the voxel storage
 		size_t init(size_t nX,size_t nY,size_t nZ, const BoundCube &bound);
@@ -1143,7 +1140,6 @@ template<class T>
 T Voxels<T>::getSum(const T &initialValue) const
 {
 	ASSERT(voxels.size());
-
 	T tmp(initialValue);
 	size_t n=voxels.size();
 #pragma omp parallel for reduction(+:tmp)
@@ -1398,6 +1394,13 @@ void Voxels<T>::getIndex(size_t &x, size_t &y,
 	x=(size_t)((p[0]-minBound[0])/(maxBound[0]-minBound[0])*(float)binCount[0]);
 	y=(size_t)((p[1]-minBound[1])/(maxBound[1]-minBound[1])*(float)binCount[1]);
 	z=(size_t)((p[2]-minBound[2])/(maxBound[2]-minBound[2])*(float)binCount[2]);
+
+	if(x == binCount[0])
+		x--;
+	if(y == binCount[1])
+		y--;
+	if(z == binCount[2])
+		z--;
 }
 
 template<class T>
@@ -1599,18 +1602,23 @@ void Voxels<T>::getInterpolatedData(const Point3D &p, T &v) const
 
 	//interpolate data values at cube vertices that surround point. We are coming from below the point
 	// so we are simply extending the field on the upper edge by duplicating values as needed
-	for(unsigned int ui=0;ui<4;ui++)
-	{
-		c[(ui&1)][(ui&2)>>1] = getData( index[0],index[1]+ iPlus[1]*(ui&1),index[2] + iPlus[2]*(ui&2) )*(1-fraction[0]) 
-				+  getData(index[0]+iPlus[0],index[1]+ iPlus[1]*(ui&1),index[2] + iPlus[2]*(ui&2));
-	}
+
+	float xf = 1-fraction[0];
+	
+	size_t xLow,xHigh;
+	xLow = index[0];
+	xHigh = index[0] + iPlus[0];
+	c[0][0] = getData(xLow,index[1],index[2])*xf			+ getData(xHigh,index[1],index[2])*fraction[0] ;
+	c[0][1] = getData(xLow,index[1],index[2]+iPlus[2])*xf		+ getData(xHigh,index[1],index[2]+iPlus[2])*fraction[0] ;
+	c[1][0] = getData(xLow,index[1]+iPlus[1],index[2])*xf		+ getData(xHigh,index[1]+iPlus[1],index[2])*fraction[0] ;
+	c[1][1] = getData(xLow,index[1]+iPlus[1],index[2]+iPlus[2])*xf  + getData(xHigh,index[1]+iPlus[1],index[2]+iPlus[2])*fraction[0] ;
 
 
 	float c0,c1;
 	c0 = c[0][0]*(1-fraction[1]) + c[1][0]*fraction[1];
 	c1 = c[0][1]*(1-fraction[1]) + c[1][1]*fraction[1];
 
-	v= c0*(1-fraction[2])*c1;	
+	v= c0*(1-fraction[2])+c1*fraction[2];	
 	
 }	
 
