@@ -1,6 +1,6 @@
 /* 
  * XMLHelper.h - libXML2 wrapper functions
- * Copyright (C) 2015  D Haley
+ * Copyright (C) 2018, D Haley
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -134,6 +134,10 @@ bool XMLGetAttrib(xmlNodePtr &nodePtr, T&v, const char *attrib)
 	return true;
 }
 
+template<>
+bool XMLGetAttrib(xmlNodePtr &nodePtr, std::string &v, const char *attrib);
+
+
 /* Defined in the bowels of the xmlLib2 library
  * Enum xmlElementType {
  *	XML_ELEMENT_NODE = 1
