@@ -1,6 +1,6 @@
 /*
  *	drawables.cpp - opengl drawable objects cpp file
- *	Copyright (C) 2015, D Haley 
+ *	Copyright (C) 2018, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -29,8 +29,6 @@
 #include "glDebug.h"
 
 
-const float DEPTH_SORT_REORDER_EPSILON = 1e-2;
-
 //Static class variables
 //====
 const Camera *DrawableObj::curCamera = 0;
@@ -223,6 +221,62 @@ void drawBox(Point3D pMin, Point3D pMax, float r,float g, float b, float a)
 	glEnd();
 }
 
+//Test the volume renderer libvd
+/*
+void testVD()
+{
+	libDisplay::Volume disp;
+
+	unsigned short *data = new unsigned short[128*128*128];
+
+	float dx,dy,dz;
+	for(unsigned int ui=0;ui<128;ui++)
+	{
+		dx = (float)ui/128.0f;
+		for(unsigned int uj=0;uj<128;uj++)
+		{
+			dy = (float)uj/128.0f;
+			for(unsigned int uk=0;uk<128;uk++)
+			{
+				dz = (float)uk/128.0f;
+
+				float r;
+				r=dx*dx+dy*dy+dz*dz;
+				r=sqrt(r);
+
+				if(r < 1.0f)
+					data[128*128*ui + 128*uj + uk] =255*(1-r);
+				else
+					data[128*128*ui + 128*uj + uk] =0;
+
+			}
+		}
+	}
+
+
+
+	glDisable(GL_CULL_FACE);
+	//The object will duplicate the pointer, and not
+	// destroy its contents.
+	disp.update(data,128,128,128);
+
+	disp.updateRamp(0,255);
+
+
+	disp.updateLut(lut,STEPS);
+
+	//Draw the volume
+	glPushMatrix();
+	glScaled(20.0,20.0,20.0);
+	disp.display();
+	glEnable(GL_CULL_FACE);
+	glPopMatrix();
+	
+	delete[] data;
+	delete[] lut;
+
+}
+*/
 
 DrawableObj::DrawableObj() : active(true), haveChanged(true), canSelect(false), wantsLight(false)
 {
@@ -286,15 +340,22 @@ void DrawableObj::clearTexPool()
 Point3D DrawableObj::getCentroid() const
 {
 	ASSERT(!isExplodable());
+
+	//Deliberately do not return here, to force a detectable memory error
+	// The function does not need to be called by all derived objects
 }
 
 //=====
 
-DrawPoint::DrawPoint() : origin(0.0f,0.0f,0.0f), r(1.0f), g(1.0f), b(1.0f), a(1.0f)
+DrawPoint::DrawPoint() : origin(0.0f,0.0f,0.0f), r(1.0f), g(1.0f), b(1.0f), a(1.0f),size(1.0f)
 {
 }
 
-DrawPoint::DrawPoint(float x, float y, float z) : origin(x,y,z), r(1.0f), g(1.0f), b(1.0f)
+DrawPoint::DrawPoint(float x, float y, float z) : origin(x,y,z), r(1.0f), g(1.0f), b(1.0f),a(1.0f), size(1.0f)
+{
+}
+
+DrawPoint::DrawPoint(const Point3D &p) : origin(p), r(1.0f), g(1.0f), b(1.0f),a(1.0f),size(1.0f)
 {
 }
 
@@ -328,8 +389,9 @@ void DrawPoint::setOrigin(const Point3D &pt)
 
 void DrawPoint::draw() const
 {
+	glPointSize(size);
+	glBegin(GL_POINTS);
 	glColor4f(r,g,b,a);
-	glBegin(GL_POINT);
 	glVertex3fv(origin.getValueArr());
 	glEnd();
 }
@@ -350,9 +412,9 @@ DrawableObj* DrawVector::clone() const
 	return d;
 }
 
-void DrawVector::getBoundingBox(BoundCube &b) const 
+void DrawVector::getBoundingBox(BoundCube &box) const 
 {
-	b.setBounds(origin,vector+origin);
+	box.setBounds(origin,vector+origin);
 }
 
 void DrawVector::setColour(float rnew, float gnew, float bnew, float anew)
@@ -524,9 +586,9 @@ DrawableObj* DrawQuad::clone() const
 }
 
 
-void DrawQuad::getBoundingBox(BoundCube &b) const
+void DrawQuad::getBoundingBox(BoundCube &box) const
 {
-	b.setBounds(vertices,4);
+	box.setBounds(vertices,4);
 }
 
 void DrawQuad::draw() const
@@ -603,7 +665,7 @@ DrawTexturedQuad::~DrawTexturedQuad()
 {
 	//hack to work around static construct/destruct.
 	// normally we use the texture pool do to everything
-	if(texPool && textureId != -1)
+	if(texPool && textureId != (unsigned int)-1)
 	{
 		texPool->closeTexture(textureId);
 		textureId=-1;
@@ -626,6 +688,7 @@ void DrawTexturedQuad::draw() const
 	glPushAttrib(GL_CULL_FACE);
 	glDisable(GL_CULL_FACE);
 	
+	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 	
 	glBindTexture(GL_TEXTURE_2D,textureId);
 	const float COORD_SEQ_X[]={ 0,0,1,1};
@@ -715,6 +778,171 @@ void DrawTexturedQuad::setData(size_t x, size_t y, unsigned char *entry)
 }
 
 
+void DrawPolygon::setVertices(const vector<Point3D> &p)
+{
+	vertices = p;
+}
+
+
+DrawableObj *DrawPolygon::clone() const
+{
+	DrawPolygon *d= new DrawPolygon;
+	d->r=r;
+	d->g=g;
+	d->b=b;
+	d->a=a;
+	d->vertices=vertices;
+	d->normal=normal;
+
+	return d;
+}
+
+
+void DrawPolygon::setColour(float rN, float gN, float bN, float aN)
+{
+	r=rN; 
+	g=gN;
+	b=bN;
+	a=aN;
+}
+
+
+void DrawPolygon::draw() const
+{
+	switch(drawMode)
+	{
+		case POLYGON_DRAW_OUTLINE:
+		{
+			glColor4f(r,g,b,a);
+			glBegin(GL_LINE_LOOP);
+
+			for(unsigned int ui=0;ui<vertices.size();ui++)
+				glVertex3fv(vertices[ui].getValueArr());
+			glEnd();
+			break;
+		};
+		default:
+			ASSERT(false);
+
+	}
+}
+
+
+void DrawPolygon::getBoundingBox(BoundCube &box) const 
+{
+	box.setBounds(vertices);
+}
+DrawTriangleMesh::DrawTriangleMesh()
+{
+	drawDoubleSided=true;
+	drawMeshEdges=false;
+	haveCachedBounds=false;
+	drawMode=TRIMESH_DRAW_SURF;
+	deltaOrigin=Point3D(0,0,0);
+	alpha=1.0;
+}
+
+DrawTriangleMesh::~DrawTriangleMesh()
+{
+}
+
+void DrawTriangleMesh::draw() const
+{
+
+	if(drawDoubleSided)
+		glDisable(GL_CULL_FACE);
+	else
+		glEnable(GL_CULL_FACE);
+
+	switch(drawMode)
+	{
+		case TRIMESH_DRAW_SURF:
+		{
+			glPushMatrix();
+			if(deltaOrigin.sqrMag() > 0)
+				glTranslatef(deltaOrigin[0],deltaOrigin[1],deltaOrigin[2]);
+			glBegin(GL_TRIANGLES);
+			for(unsigned int ui=0;ui<triIndices.size();ui++)
+			{
+				for(unsigned int uj=0;uj<3;uj++)
+				{
+					float rgba[4];
+					for( unsigned int uk=0;uk<3; uk++)
+						rgba[uk]=triIndices[ui].col[uj].v[uk];
+					rgba[3]=alpha;
+					glColor4fv(rgba);
+					glVertex3fv(pts[triIndices[ui].idx[uj]].getValueArr());
+				}
+			}
+			glEnd();
+
+			
+			//Draw the outline of the mesh as needed
+			if(drawMeshEdges)
+			{
+				glBegin(GL_LINES);
+				glColor4f(0,0,1.0f,1.0f);
+				for(unsigned int ui=0;ui<triIndices.size();ui++)
+				{
+					for(unsigned int uj=0;uj<4;uj++)
+					{
+						glVertex3fv(pts[triIndices[ui].idx[(uj%3)]].getValueArr());
+					}
+				}
+				glEnd();
+			}
+			glPopMatrix();
+			break;
+		}
+		default:
+			ASSERT(false);
+	}
+
+	glDisable(GL_CULL_FACE);
+}
+
+void DrawTriangleMesh::setData(std::vector<Point3D> &p,
+			std::vector<TRI_IDX> &idx)
+{
+	ASSERT(p.size() >=3);
+	pts.swap(p);
+	idx.swap(triIndices);
+
+	BoundCube bc;
+	getBoundingBox(bc);	
+	origCentroid=bc.getCentroid();
+}
+
+void DrawTriangleMesh::getBoundingBox(BoundCube &box) const
+{
+	if(haveCachedBounds)
+	{
+		box=cachedBounds;
+		return;
+	}
+
+	cachedBounds.setBounds(pts);
+	box=cachedBounds;
+	haveCachedBounds=true;
+	return;
+}
+
+void DrawTriangleMesh::recomputeParams(const vector<Point3D> &vecs, 
+			const vector<float> &scalars, unsigned int mode)
+{
+	switch(mode)
+	{
+		case DRAW_TRIMESH_BIND_ORIGIN:
+		{
+			//FIXME: This is a hack
+			deltaOrigin=vecs[0]-origCentroid;
+			break;
+		}
+		default:
+			ASSERT(false);
+	}
+}
+
 DrawSphere::DrawSphere() : radius(1.0f), latSegments(8),longSegments(8)
 {
 	q=gluNewQuadric();
@@ -742,12 +970,12 @@ DrawableObj *DrawSphere::clone() const
 	return d;
 }
 
-void DrawSphere::getBoundingBox(BoundCube &b) const
+void DrawSphere::getBoundingBox(BoundCube &box) const
 {
 	for(unsigned int ui=0;ui<3;ui++)
 	{
-		b.setBound(ui,0,origin[ui] - radius);
-		b.setBound(ui,1,origin[ui] + radius);
+		box.setBound(ui,0,origin[ui] - radius);
+		box.setBound(ui,1,origin[ui] + radius);
 	}
 }
 
@@ -953,7 +1181,7 @@ void DrawCylinder::setColour(float rnew, float gnew, float bnew, float anew)
 	a=anew;
 }
 
-void DrawCylinder::getBoundingBox(BoundCube &b) const
+void DrawCylinder::getBoundingBox(BoundCube &box) const
 {
 
 	float tmp;
@@ -987,7 +1215,7 @@ void DrawCylinder::getBoundingBox(BoundCube &b) const
 	p[2]= offset+(-direction*0.5+origin);
 	p[3]= -offset+(-direction*0.5+origin);
 
-	b.setBounds(p);
+	box.setBounds(p);
 }
 
 
@@ -1009,7 +1237,7 @@ DrawableObj* DrawManyPoints::clone() const
 	return d;
 }
 
-void DrawManyPoints::getBoundingBox(BoundCube &b) const
+void DrawManyPoints::getBoundingBox(BoundCube &box) const
 {
 
 	//Update the cache as needed
@@ -1019,7 +1247,7 @@ void DrawManyPoints::getBoundingBox(BoundCube &b) const
 		cachedBounds.setBounds(pts);
 	}
 
-	b=cachedBounds;
+	box=cachedBounds;
 	return;
 }
 
@@ -1127,9 +1355,9 @@ bool DrawDispList::startList(bool execute)
 void DrawDispList::addDrawable(const DrawableObj *d)
 {
 	ASSERT(listActive);
-	BoundCube b;
-	d->getBoundingBox(b);
-	boundBox.expand(b);
+	BoundCube box;
+	d->getBoundingBox(box);
+	boundBox.expand(box);
 	d->draw();
 }
 
@@ -1208,7 +1436,8 @@ DrawGLText::DrawGLText(std::string fontFile, unsigned int mode) :font(0),fontStr
 	//Use unicode
 	font->CharMap(ft_encoding_unicode);
 
-	alignMode = DRAWTEXT_ALIGN_LEFT;
+	alignHorizMode = DRAWTEXT_ALIGN_LEFT;
+	alignVertMode = DRAWTEXT_ALIGN_LOWER;
 }
 
 DrawGLText::DrawGLText(const DrawGLText &oth) : font(0), fontString(oth.fontString),
@@ -1264,7 +1493,7 @@ void DrawGLText::draw() const
 		return;
 
 	//Translate the drawing position to the origin
-	Point3D offsetVec=textDir;
+	Point3D hOffsetVec=textDir;
 	float advance, halfHeight;
 
 	{
@@ -1276,20 +1505,36 @@ void DrawGLText::draw() const
 	halfHeight/=2.0f;
 	}
 
-	switch(alignMode)
+	switch(alignHorizMode)
 	{
 		case DRAWTEXT_ALIGN_LEFT:
 			break;
-		case DRAWTEXT_ALIGN_CENTRE:
-			offsetVec=offsetVec*advance/2.0f;
+		case DRAWTEXT_ALIGN_CENTREX:
+			hOffsetVec=hOffsetVec*advance/2.0f;
 			break;
 		case DRAWTEXT_ALIGN_RIGHT:
-			offsetVec=offsetVec*advance;
+			hOffsetVec=hOffsetVec*advance;
 			break;
 		default:
 			ASSERT(false);
 	}
-	
+
+	Point3D vOffsetVec=Point3D(0,0,0);	
+	switch(alignVertMode)
+	{
+		case DRAWTEXT_ALIGN_LOWER:
+			break;
+		case DRAWTEXT_ALIGN_CENTREY:
+			vOffsetVec=up*halfHeight;
+			break;
+		case DRAWTEXT_ALIGN_UPPER:
+			hOffsetVec=up*(2*halfHeight);
+			break;
+		default:
+			ASSERT(false);
+	}
+
+	Point3D offsetVec = hOffsetVec + vOffsetVec;
 
 	glPushMatrix();
 
@@ -1451,7 +1696,10 @@ void DrawGLText::draw() const
 DrawGLText::~DrawGLText()
 {
 	if(font)
+	{
 		delete font;
+		font=0;
+	}
 }
 
 void DrawGLText::setColour(float rnew, float gnew, float bnew, float anew)
@@ -1462,7 +1710,7 @@ void DrawGLText::setColour(float rnew, float gnew, float bnew, float anew)
 	a=anew;
 }
 
-void DrawGLText::getBoundingBox(BoundCube &b) const
+void DrawGLText::getBoundingBox(BoundCube &box) const
 {
 	//Box forwards transformations
 	// * Translation by [origin-textDir*  (maxx - minx)]
@@ -1478,10 +1726,10 @@ void DrawGLText::getBoundingBox(BoundCube &b) const
 		float dy=maxY-minY;
 
 
-		b.setBounds(minX,minY,minZ,
+		box.setBounds(minX,minY,minZ,
 				maxX,maxY,maxZ);
 		vector<Point3D> p;
-		b.getVertices(p,true);
+		box.getVertices(p,true);
 
 		for(size_t ui=0;ui<p.size();ui++)
 			p[ui]-=Point3D(0,-dy*0.5,0);
@@ -1550,17 +1798,23 @@ void DrawGLText::getBoundingBox(BoundCube &b) const
 		for(size_t ui=0;ui<p.size();ui++)
 			p[ui]+=origin ; 
 
-		b.setBounds(p);
+		box.setBounds(p);
 	}
 	else
-		b.setInverseLimits();	
+		box.setInverseLimits();	
 	
 }
 
-void DrawGLText::setAlignment(unsigned int newMode)
+void DrawGLText::setVertAlignment(unsigned int newMode)
 {
-	ASSERT(newMode < DRAWTEXT_ALIGN_ENUM_END);
-	alignMode=newMode;
+	ASSERT(newMode < DRAWTEXT_ALIGN_VERT_ENUM_END);
+	alignVertMode=newMode;
+}
+
+void DrawGLText::setHorizAlignment(unsigned int newMode)
+{
+	ASSERT(newMode < DRAWTEXT_ALIGN_HORIZ_ENUM_END);
+	alignHorizMode=newMode;
 }
 
 void DrawGLText::recomputeParams(const vector<Point3D> &vecs, 
@@ -1577,6 +1831,159 @@ void DrawGLText::recomputeParams(const vector<Point3D> &vecs,
 	}
 }
 
+
+
+DrawGL2DText::DrawGL2DText(const std::string &fontFile)
+{
+
+	r=g=b=a=1.0f;
+	fontString=fontFile;
+
+	const unsigned int mode = FTGL_POLYGON;
+	font=0;
+	switch(mode)
+	{
+		case FTGL_BITMAP:
+			font = new FTGLBitmapFont(fontFile.c_str());
+			break;
+		case FTGL_PIXMAP:
+			font = new FTGLPixmapFont(fontFile.c_str());
+			break;
+		case FTGL_OUTLINE:
+			font = new FTGLOutlineFont(fontFile.c_str());
+			break;
+		case FTGL_POLYGON:
+			font = new FTGLPolygonFont(fontFile.c_str());
+			break;
+		case FTGL_EXTRUDE:
+			font = new FTGLExtrdFont(fontFile.c_str());
+			break;
+		case FTGL_TEXTURE:
+			font = new FTGLTextureFont(fontFile.c_str());
+			break;
+		default:
+			//Don't do this. Use valid font numbers
+			ASSERT(false);
+			font=0; 
+	}
+
+	if(!font || font->Error())
+	{
+		isOK=false;
+		return;
+	}
+
+	font->FaceSize(1);
+	font->Depth(1);
+
+	font->CharMap(ft_encoding_unicode);
+
+	alignHorizMode=DRAWTEXT_ALIGN_LEFT;
+	alignVertMode=DRAWTEXT_ALIGN_LOWER;
+
+	isOK=true;
+}
+
+
+DrawGL2DText::~DrawGL2DText()
+{
+	if(font)
+	{
+		delete font;
+		font=0;
+	}
+}
+
+void DrawGL2DText::setColour(float rNew, float gNew,float bNew)
+{
+	r=rNew;
+	g=gNew;
+	b=bNew;
+}
+
+void DrawGL2DText::draw() const
+{
+	ASSERT(font);
+	//There is only one scaling variable for this object
+	ASSERT(height == width);
+	ASSERT(height > 0);
+
+
+	//skip if there is nothing to do, or if we are not good to go
+	if(!isOK || strText.empty())
+		return;
+
+	//FIXME: Ensure scene switches to overlay mode, then draw
+	// to save continously flipping out the matrices
+	glMatrixMode(GL_PROJECTION);	
+	glPushMatrix();
+	glLoadIdentity();
+	gluOrtho2D(0, 1, 1, 0);
+
+	glMatrixMode(GL_MODELVIEW);
+
+
+	float ASPECT_RATIO;
+	if(winX)
+		ASPECT_RATIO=(float)winY/(float)winX;
+	else
+		ASPECT_RATIO=0.0f;
+
+	if(ASPECT_RATIO)
+	{
+		glPushMatrix();
+		glLoadIdentity();
+			
+			glColor4f(r,g,b,a);
+			glDisable(GL_CULL_FACE);
+
+			font->FaceSize(1.0);
+			glPushMatrix();
+				glTranslatef(position[0],position[1],0);
+				glScaled(height,-height/ASPECT_RATIO,height);
+				font->Render(strText.c_str());
+			glPopMatrix();
+
+		glPopMatrix(); //Pop modelview matrix
+	}
+
+	glMatrixMode(GL_PROJECTION);
+	glPopMatrix();
+
+	glMatrixMode(GL_MODELVIEW);
+}
+
+
+void DrawGL2DText::getBoundingBox(BoundCube &b) const
+{
+	WARN(false,"Requested 3D bounding box for 2D object");
+	//This function is problematic
+	b.setInverseLimits();
+}
+
+
+DrawableObj *DrawGL2DText::clone() const
+{
+	DrawGL2DText *d = new DrawGL2DText(fontString);
+
+	d->strText=strText;
+	d->position[0]=position[0];
+	d->position[1]=position[1];
+	d->r=r;
+	d->g=g;
+	d->b=b;
+	d->a=a;
+	d->height=height;
+	d->width=width;
+
+	d->alignHorizMode = alignHorizMode;
+	d->alignVertMode = alignVertMode;
+	d->isOK=isOK;
+
+	return d;
+}
+
+
 DrawRectPrism::DrawRectPrism() : drawMode(DRAW_WIREFRAME), r(1.0f), g(1.0f), b(1.0f), a(1.0f), lineWidth(1.0f)
 {
 }
@@ -1591,9 +1998,9 @@ DrawableObj *DrawRectPrism::clone() const
 	return dR;
 }
 
-void DrawRectPrism::getBoundingBox(BoundCube &b) const
+void DrawRectPrism::getBoundingBox(BoundCube &box) const
 {
-	b.setBounds(pMin[0],pMin[1],pMin[2],
+	box.setBounds(pMin[0],pMin[1],pMin[2],
 			pMax[0],pMax[1],pMax[2]);
 }
 
@@ -1679,9 +2086,9 @@ void DrawRectPrism::setAxisAligned( const Point3D &p1, const Point3D &p2)
 
 }
 
-void DrawRectPrism::setAxisAligned( const BoundCube &b)
+void DrawRectPrism::setAxisAligned( const BoundCube &box)
 {
-	b.getBounds(pMin,pMax);
+	box.getBounds(pMin,pMax);
 }
 
 void DrawRectPrism::setColour(float rnew, float gnew, float bnew, float anew)
@@ -1844,7 +2251,9 @@ void DrawProgressCircleOverlay::draw( )const
 	glLoadIdentity();
 
 	glDisable(GL_CULL_FACE);
-	glEnable(GL_DEPTH_TEST);
+	glPushAttrib(GL_DEPTH_BUFFER_BIT);
+	glDisable(GL_DEPTH_TEST);
+				                       
 	//size of gap to fit, in degrees
 	const float FILTER_SPACING_ANGLE= 20.0f/(float)totalFilters;
 	//Angular step when drawing wheel
@@ -1956,7 +2365,7 @@ void DrawProgressCircleOverlay::draw( )const
 
 	glMatrixMode(GL_MODELVIEW);
 
-	glDisable(GL_DEPTH_TEST);
+	glPopAttrib();
 	glEnable(GL_CULL_FACE);
 }
 
@@ -2092,7 +2501,7 @@ void DrawAnimatedOverlay::draw() const
 
 	//TODO: Find correct blending mode. Default is good, but may change...	
 //	glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_BLEND); 
-//	glBlendFunc(GL_ZERO, GL_ONE_MINUS_SRC_ALPHA);
+	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 
 	// Draw overlay quad 
 	ASSERT(width == height); // width/height should be the same
@@ -2367,8 +2776,12 @@ void DrawPointLegendOverlay::addItem(const std::string &s, float r, float g, flo
 
 DrawField3D::DrawField3D() : ptsCacheOK(false), alphaVal(0.2f), pointSize(1.0f), drawBoundBox(true),
 	boxColourR(1.0f), boxColourG(1.0f), boxColourB(1.0f), boxColourA(1.0f),
-	volumeGrid(false), volumeRenderMode(0), field(0) 
+	volumeGrid(false), colourMapID(COLOURMAP_VIRIDIS), colourMapMode(COLOURMAP_MODE_BUILTIN),
+	 volumeRenderMode(0), field(0) 
 {
+#ifdef ENABLE_LIBVD
+	volumeRenderCacheOK=false;
+#endif
 }
 
 DrawField3D::~DrawField3D()
@@ -2378,10 +2791,10 @@ DrawField3D::~DrawField3D()
 }
 
 
-void DrawField3D::getBoundingBox(BoundCube &b) const
+void DrawField3D::getBoundingBox(BoundCube &box) const
 {
 	ASSERT(field)
-	b.setBounds(field->getMinBounds(),field->getMaxBounds());
+	box.setBounds(field->getMinBounds(),field->getMaxBounds());
 }
 
 
@@ -2390,19 +2803,30 @@ void DrawField3D::setField(const Voxels<float> *newField)
 	field=newField;
 }
 
+#ifdef ENABLE_LIBVD
 void DrawField3D::setRenderMode(unsigned int mode)
 {
 	volumeRenderMode=mode;
 }
+#endif
 
-void DrawField3D::setColourMinMax()
+void DrawField3D::setColourMinMax(float minV, float maxv)
 {
-	colourMapBound[0]=field->min();
-	colourMapBound[1]=field->max();
+	colourMapBound[0]=minV;
+	colourMapBound[1]=maxv;
 
-	ASSERT(colourMapBound[0] <=colourMapBound[1]);
 }
 
+void DrawField3D::setCustomColourMap(const std::vector<std::pair<float,ColourRGBAf> > &cMap) 
+{
+	colourMap=cMap;
+}
+
+void DrawField3D::setColourMapMode(unsigned int mode)
+{
+	ASSERT(mode < COLOURMAP_MODE_ENUM_END);
+	colourMapMode=mode;
+}
 			
 void DrawField3D::draw() const
 {
@@ -2437,13 +2861,30 @@ void DrawField3D::draw() const
 							v=field->getData(uiX,uiY,uiZ);
 							if(v > std::numeric_limits<float>::epsilon())
 							{
-								RGBThis rgb;
-								//Set colour and point loc
-								colourMapWrap(colourMapID,rgb.v,
-										field->getData(uiX,uiY,uiZ), 
-										colourMapBound[0],colourMapBound[1],false);
-								
-								ptsCache.push_back(make_pair(field->getPoint(uiX,uiY,uiZ)+delta,rgb));
+								unsigned char rgb[3];
+								if(colourMapMode == COLOURMAP_MODE_BUILTIN)
+								{
+									//Set colour and point loc
+									colourMapWrap(colourMapID,rgb,v,
+											colourMapBound[0],colourMapBound[1],false);
+								}
+								else if (colourMapMode ==COLOURMAP_MODE_CUSTOM)
+								{
+									//FIXME: At this point we are discarding the alpha channel information
+									// we should preserve this. However the RGB storage needs to be adjusted
+									unsigned char alphaThis;
+									applyCustomColourMap(colourMap,rgb,alphaThis,v,
+											colourMapBound[0],colourMapBound[1]);
+
+								}
+								else
+								{
+									ASSERT(false);
+								}
+							
+								RGBThis rThis;
+								rThis.v[0]=rgb[0]; rThis.v[1]=rgb[1];rThis.v[2]=rgb[2];
+								ptsCache.push_back(make_pair(field->getPoint(uiX,uiY,uiZ)+delta,rThis));
 							}
 						}
 					}
@@ -2510,7 +2951,102 @@ void DrawField3D::draw() const
 			}
 			break;
 		}
+#ifdef ENABLE_LIBVD
+		case VOLUME_RENDER_ACCUMULATE:
+		{
+			if(!volumeRenderCacheOK)
+			{
 
+				size_t sx,sy,sz;
+				field->getSize(sx,sy,sz);
+				
+				unsigned short *data = new unsigned short[128*128*128];
+
+				size_t fieldSize=field->size();
+
+				//Copy the contents of the field into an arry that the volume render
+				// library can understand.
+				for(unsigned int ui=0;ui<fieldSize;ui++)
+				{
+					//obtain data and normalise to 0-255 scale
+					float f;
+					f= field->getData(ui);
+					f=(f - colourMapBound[0])/(colourMapBound[1]-colourMapBound[0])*255;
+
+					data[ui]=std::min(std::max(0.0f,f),255.0f); 
+				}
+				//Send the data to the volume renderer
+				volumeRenderer.update(data,sx,sy,sz);
+				delete[] data;
+				
+				
+				//FIXME: We need to use a proper transfer function editor
+				//Create the colour scale (look-up table "LUT")
+				const unsigned int STEPS=256;
+				float *lut = new float[STEPS*4]; //RGBA
+
+				for(unsigned int ui=1;ui<STEPS;ui++)
+				{
+					//This returns an RGB, we have to map A ourselves
+					unsigned char rgb[3];
+					unsigned char alphaThis;
+
+					if(colourMapMode == COLOURMAP_MODE_BUILTIN)
+					{
+						colourMapWrap(colourMapID, rgb,(float)ui,0,STEPS,false);
+						alphaThis=(float)ui/(float)STEPS*255*alphaVal;
+					}
+					else if(colourMapMode == COLOURMAP_MODE_CUSTOM)
+						applyCustomColourMap(colourMap,rgb,alphaThis,(float)ui,0,STEPS);
+
+					//Set the colour section
+					float *p;
+					p=lut+ui*4;
+					p[0]=(float)rgb[0]/255.0f;
+					p[1]=(float)rgb[1]/255.0f;
+					p[2]=(float)rgb[2]/255.0f;
+
+					//Set alpha
+					p[3] = (float)alphaThis/255.0f; 
+				}
+
+				volumeRenderer.updateLut(lut,256);
+
+				delete[] lut;
+
+				volumeRenderer.setQuality(0.2);
+				volumeRenderCacheOK=true;
+					
+			
+			}
+			BoundCube bc;
+			field->getBounds(bc);
+		
+			glPushAttrib(GL_CULL_FACE);
+			glDisable(GL_CULL_FACE);
+			glPushAttrib(GL_BLEND);
+
+			//Resize and scale to correctly size the
+			// volume to be rendered
+			glPushMatrix();
+			glTranslatef(bc.getSize(0)/2.0f+ bc.getBound(0,0),bc.getSize(1)/2.0f+bc.getBound(1,0),bc.getSize(2)/2.0f+bc.getBound(2,0));
+			glScaled(bc.getSize(0),bc.getSize(1),bc.getSize(2));
+
+			//Draw the volume
+			volumeRenderer.display();
+
+
+			glPopAttrib();
+			glPopAttrib();
+			glPopMatrix();
+
+			glEnable(GL_BLEND);
+			glBlendFunc(GL_SRC_ALPHA,GL_DST_ALPHA);
+			glBlendEquation(GL_FUNC_ADD);
+
+			break;
+		}
+#endif
 		default:
 			//Not implemented
 			ASSERT(false); 
@@ -2543,7 +3079,7 @@ void DrawField3D::setPointSize(float size)
 
 void DrawField3D::setMapColours(unsigned int mapID)
 {
-	ASSERT(mapID < NUM_COLOURMAPS);
+	ASSERT(mapID < COLOURMAP_ENUM_END);
 	colourMapID= mapID;
 }
 
@@ -2556,8 +3092,23 @@ void DrawField3D::setBoxColours(float rNew, float gNew, float bNew, float aNew)
 
 }
 
+bool DrawField3D::needsDepthSorting() const
+{
+	switch(volumeRenderMode)
+	{
+		case VOLUME_POINTS:
+			return alphaVal < 1.0f;
+#ifdef ENABLE_LIBVD
+		case VOLUME_RENDER_ACCUMULATE:
+			return true;
+#endif
 
-DrawIsoSurface::DrawIsoSurface() : cacheOK(false),  drawMode(DRAW_SMOOTH),
+	}
+
+	ASSERT(false);
+}
+
+DrawIsoSurface::DrawIsoSurface() : cacheOK(false),  
 	threshold(0.5f), r(0.5f), g(0.5f), b(0.5f), a(0.5f) 
 {
 #ifdef DEBUG
@@ -2595,15 +3146,15 @@ void DrawIsoSurface::updateMesh() const
 
 }
 
-void DrawIsoSurface::getBoundingBox(BoundCube &b) const
+void DrawIsoSurface::getBoundingBox(BoundCube &box) const
 {
 	if(voxels)
 	{
-		b.setBounds(voxels->getMinBounds(),
+		box.setBounds(voxels->getMinBounds(),
 				voxels->getMaxBounds());
 	}
 	else
-		b.setInverseLimits();
+		box.setInverseLimits();
 }
 
 
@@ -2841,9 +3392,9 @@ void DrawAxis::draw() const
 	glPopMatrix();
 }
 
-void DrawAxis::getBoundingBox(BoundCube &b) const
+void DrawAxis::getBoundingBox(BoundCube &cube) const
 {
-	b.setInvalid();
+	cube.setInvalid();
 }
 
 Draw2DCircle::Draw2DCircle()
@@ -2897,10 +3448,10 @@ void Draw2DCircle::draw() const
 	}
 }
 
-void Draw2DCircle::getBoundingBox(BoundCube &b) const
+void Draw2DCircle::getBoundingBox(BoundCube &cube) const
 {
 
-	b.setBounds(centre[0]-radius, centre[1]-radius,
+	cube.setBounds(centre[0]-radius, centre[1]-radius,
 			centre[0]+radius, centre[1]+radius,
 			0,0);
 }
