@@ -1,6 +1,6 @@
 /*
  *	drawables.h - Opengl drawable objects header
- *	Copyright (C) 2015, D Haley 
+ *	Copyright (C) 2018, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -20,14 +20,22 @@
 #define DRAWABLES_H
 
 //MacOS is "special" and puts it elsewhere
+
+#ifdef ENABLE_LIBVD 
+#include <GL/glew.h>
+#include <libvd/display3d.h>
+#endif
+
+
 #ifdef __APPLE__ 
 	#include <OpenGL/glu.h>
 #else
 	#include <GL/glu.h>
 #endif
-
 #include <sys/time.h>
 
+
+
 #include "textures.h"
 #include "cameras.h"
 #include "isoSurface.h"
@@ -46,6 +54,7 @@ const size_t MAX_NUM_DRAWABLE_POINTS=10;
 #include <ft2build.h>
 #include <FTGL/ftgl.h>
 
+//!FTGL Font modes
 enum
 {
 	FTGL_BITMAP=0,
@@ -60,9 +69,17 @@ enum
 enum
 {
 	DRAWTEXT_ALIGN_LEFT,
-	DRAWTEXT_ALIGN_CENTRE,
+	DRAWTEXT_ALIGN_CENTREX,
 	DRAWTEXT_ALIGN_RIGHT,
-	DRAWTEXT_ALIGN_ENUM_END
+	DRAWTEXT_ALIGN_HORIZ_ENUM_END
+};
+
+enum
+{
+	DRAWTEXT_ALIGN_LOWER,
+	DRAWTEXT_ALIGN_CENTREY,
+	DRAWTEXT_ALIGN_UPPER,
+	DRAWTEXT_ALIGN_VERT_ENUM_END
 };
 
 
@@ -81,6 +98,21 @@ enum
 	AXIS_IN_SPACE
 };
 
+
+//!Polygon drawing styles
+enum
+{
+	POLYGON_DRAW_OUTLINE,
+	POLYGON_DRAW_ENUM_END
+};
+
+//!Triangle mesh drawing styles
+enum
+{
+	TRIMESH_DRAW_SURF,
+	TRIMESH_DRAW_ENUM_END
+};
+
 //!Drawable types
 enum
 {
@@ -89,10 +121,12 @@ enum
 	DRAW_TYPE_VECTOR,
 	DRAW_TYPE_TRIANGLE,
 	DRAW_TYPE_QUAD,
+	DRAW_TYPE_POLYGON,
 	DRAW_TYPE_SPHERE,
 	DRAW_TYPE_CYLINDER,
 	DRAW_TYPE_DISPLAYLIST,
 	DRAW_TYPE_GLTEXT,
+	DRAW_TYPE_GL2DTEXT,
 	DRAW_TYPE_2D_CIRCLE,
 	DRAW_TYPE_RECTPRISM,
 	DRAW_TYPE_COLOURBAR,
@@ -103,6 +137,7 @@ enum
 	DRAW_TYPE_AXIS,
 	DRAW_TYPE_LEGENDOVERLAY,
 	DRAW_TYPE_PROGRESSCIRCLE_OVERLAY,
+	DRAW_TYPE_TRIANGLE_MESH,
 };
 
 //TODO: It seems unnecessary to have multiple types for the bind
@@ -127,10 +162,20 @@ enum
 	DRAW_QUAD_BIND_ORIGIN,
 	//DRAW_TEXT_BIND_TEXTDIR, //FIXME: Implement me for annotation todo.
 	//DRAW_TEXT_BIND_UPDIR,
+	DRAW_TRIMESH_BIND_ORIGIN,
 	DRAW_BIND_ENUM_END
 };
 
+struct RGBFloat
+{
+	float v[3];
+};
 
+struct TRI_IDX
+{
+	size_t idx[3];
+	RGBFloat col[3];
+};
 
 
 //!An abstract bas class for drawing primitives
@@ -225,6 +270,30 @@ class DrawableObj
 
 };
 
+//Abstract class as base for overlays
+class DrawableOverlay : public DrawableObj
+{
+	protected:
+		//alpha (transparancy) value
+		float a;
+		//!Height and width of overlay (total)
+		float height,width;
+		//Fractional coordinates for the  top left of the overlay
+		float position[2];
+	public:
+		DrawableOverlay() {} ;
+		//Declared as pure virtual to force ABC
+		virtual ~DrawableOverlay() =0;
+		void setAlpha(float alpha) { a=alpha;};
+		void setSize(float widthN, float heightN) {height=heightN, width=widthN;} 
+		void setSize(float size) {width=height=size;};
+		void setPosition(float newTLX,float newTLY) { position[0]=newTLX; position[1]=newTLY;}
+
+		void getBoundingBox(BoundCube &b) const {b.setInvalid();};
+		//!This is an overlay
+		bool isOverlay() const {return true;};
+};
+
 //A single point drawing class 
 class DrawPoint : public DrawableObj
 {
@@ -233,11 +302,15 @@ class DrawPoint : public DrawableObj
 		Point3D origin;
 		//!Point colour (r,g,b,a) range: [0.0f,1.0f]
 		float r,g,b,a;
+		//!Size of point
+		float size;
 	public:
 		//!Constructor
 		DrawPoint();
-		//!Constructor that takes in positional argments
+		//!Constructor that takes in positional augments
 		DrawPoint(float,float,float);
+		//!Constructor that takes in positional augments
+		DrawPoint(const Point3D &p);
 		//!Destructor
 		virtual ~DrawPoint();
 
@@ -249,9 +322,13 @@ class DrawPoint : public DrawableObj
 		void setColour(float r, float g, float b, float alpha);
 		//!Draws the points
 		void draw() const;
-		//!Sets the location of the poitns
+		//!Sets the location of the point
 		void setOrigin(const Point3D &);
 
+		//!Set the size of the point
+		void setSize(float s) { size=s;}
+	
+
 		void getBoundingBox(BoundCube &b) const { b.setInvalid();};
 
 		Point3D getCentroid() const{ return origin;}
@@ -474,7 +551,6 @@ class DrawTexturedQuad : public DrawQuad
 		unsigned char *textureData;
 		size_t nX,nY;
 		size_t channels;
-		size_t displayMode;
 
 		//FIXME: This should be non-mutable.  We need
 		// to move texture rebinding to a pre-processing step, not at draw time
@@ -482,9 +558,6 @@ class DrawTexturedQuad : public DrawQuad
 		// to opengl
 		mutable unsigned int textureId;
 		
-		//!FTGL font instance
-		FTFont *font;
-		
 		//disallow resetting base colour to white 
 		bool noColour;
 
@@ -509,6 +582,96 @@ class DrawTexturedQuad : public DrawQuad
 		void setUseColouring(bool useColouring) {noColour= !useColouring;};
 };
 
+//Draw an irregular 2D n-gon in 3D.
+class DrawPolygon : public DrawableObj
+{
+	protected:
+		//!Vertices of the quad
+		vector<Point3D> vertices;
+
+		//!Colour data for the quad
+		//!The lighting normal of the triangle 
+		/*! Lighting for this class is per triangle only no
+		 * per vertex lighting */
+		Point3D normal;
+		//!Colours of polygon (rgba colour model)
+		float r,g,b,a;
+		//!The drawings style to use (eg filled, outline etc)
+		unsigned int drawMode;
+	public:
+		//!Constructor
+		DrawPolygon() {drawMode=POLYGON_DRAW_OUTLINE;};
+		//!Destructor
+		virtual ~DrawPolygon() {};
+		
+		virtual DrawableObj *clone() const;
+		
+		virtual unsigned int getType() const {return DRAW_TYPE_POLYGON;};	
+
+		//!Get bounding cube
+		virtual void getBoundingBox(BoundCube &b) const ;
+
+		void setVertices(const vector<Point3D> &p);
+		//!Set the colour of all vertices
+		void setColour(float r, float g, float b, float a);
+
+		//!Draw the triangle
+		void draw() const;
+		
+		//!Gets the arrow axis direction
+		Point3D getOrigin() const;
+		
+};
+
+
+//Coloured triangle mesh
+class DrawTriangleMesh : public DrawableObj
+{
+	protected:
+		//!Vector of points to draw
+		std::vector<Point3D> pts;
+		//!Indices of triangle vertices
+		std::vector<TRI_IDX> triIndices;
+
+		//FIXME: Redo some of the selection code, so we don't need
+		// this
+		Point3D origCentroid,deltaOrigin;
+
+		float alpha;
+
+		bool drawDoubleSided;
+		//If true, draw the mesh edges in addition to the
+		// coloured triangles
+		bool drawMeshEdges;
+		unsigned int drawMode;
+
+		mutable bool haveCachedBounds;
+		mutable BoundCube cachedBounds;
+	public:
+		DrawTriangleMesh();
+		~DrawTriangleMesh();
+
+		//Assign the triangle data - note this destroys the input data
+		void setData(std::vector<Point3D> &pts,
+			std::vector<TRI_IDX> &idx);
+
+		//Set whether the edges of the triangle should be drawn
+		void setDrawMeshEdges(bool doDraw=true) { drawMeshEdges=true;}
+
+		unsigned int getType() const {return DRAW_TYPE_TRIANGLE_MESH;}	
+		
+		//!Draws the triangle mesh
+		void draw() const;
+
+		//!Get the bounding box that encapulates this object
+		void getBoundingBox(BoundCube &b) const ;
+
+		//!Recompute the internal parameters using the input vector information
+		void recomputeParams(const std::vector<Point3D> &vecs, const std::vector<float> &scalars, unsigned int mode);
+
+		//!Set the opacity value (alpha)
+		void setAlpha(float f) { alpha=f;}
+};
 
 
 //!A sphere drawing 
@@ -559,6 +722,7 @@ class DrawSphere : public DrawableObj
 		void recomputeParams(const std::vector<Point3D> &vecs, 
 				const std::vector<float> &scalars, unsigned int mode);
 
+
 };
 
 //!A tapered cylinder drawing class
@@ -631,7 +795,10 @@ class DrawCylinder : public DrawableObj
 //!Drawing mode enumeration for scalar field
 enum
 {
-	VOLUME_POINTS=0
+	VOLUME_POINTS=0, // Display as a 3D field of points
+#ifdef ENABLE_LIBVD
+	VOLUME_RENDER_ACCUMULATE, // Display using volume rendering
+#endif
 };
 
 //!A display list generating class
@@ -689,9 +856,7 @@ class DrawDispList : public DrawableObj
  */
 class DrawGLText : public DrawableObj
 {
-
 	private:
-
 		//!FTGL font instance
 		FTFont *font;
 
@@ -705,7 +870,8 @@ class DrawGLText : public DrawableObj
 		//!Origin of text
 		Point3D origin;
 		//!Alignment mode
-		unsigned int alignMode;
+		unsigned int alignHorizMode;
+		unsigned int alignVertMode;
 
 		//!Colours for text 
 		float r,g,b,a;
@@ -745,7 +911,8 @@ class DrawGLText : public DrawableObj
 		 */
 		DrawGLText(std::string fontFile,
 					unsigned int ftglTextMode);
-		DrawGLText(const DrawGLText &other);
+		
+		DrawGLText(const DrawGLText &oth);
 
 		//!Destructor
 		virtual ~DrawGLText();
@@ -806,8 +973,11 @@ class DrawGLText : public DrawableObj
 		//!Get the bounding box for the text
 		void getBoundingBox(BoundCube &b) const; 
 
-		//!Set the text alignment (default is left)
-		void setAlignment(unsigned int mode);
+		//!Set the text horiz alignment (default is left)
+		void setHorizAlignment(unsigned int mode);
+		
+		//!Set the text y alignment (default is bottom)
+		void setVertAlignment(unsigned int mode);
 		
 		//Binding parameter recomputation
 		void recomputeParams(const std::vector<Point3D> &vecs, 
@@ -815,6 +985,56 @@ class DrawGLText : public DrawableObj
 };
 
 
+class DrawGL2DText : public DrawableOverlay
+{
+	private:
+
+		//!FTGL font instance
+		FTFont *font;
+
+		//!Font string
+		std::string fontString;
+		//!Text string
+		std::string strText;
+
+		//!Alignment mode
+		unsigned int alignHorizMode;
+		unsigned int alignVertMode;
+
+		//!Colours for text - alpha handled separately
+		float r,g,b;
+
+		//!True if no erro
+		bool isOK;
+
+	public:
+		DrawGL2DText(const std::string &fontFile);
+		DrawGL2DText(const DrawGL2DText &other);
+
+		virtual ~DrawGL2DText();
+
+		
+		virtual unsigned int getType() const {return DRAW_TYPE_GL2DTEXT;}
+		
+		//!Set the object's text string
+		inline void setString(std::string &s) { strText=s;}
+
+		//!Returs true if the class data is good
+		inline bool ok() const
+			{return isOK;};
+
+		inline void setText(const std::string &s)  { strText =s;};
+
+		//!Set the colour (rgba) of the object
+		void setColour(float r,float g,float b);
+
+		void draw() const;
+
+		void getBoundingBox(BoundCube &b) const;
+
+		DrawableObj *clone() const;
+};
+
 
 
 //!A class to draw rectangluar prisms
@@ -857,34 +1077,7 @@ class DrawRectPrism  : public DrawableObj
 		void recomputeParams(const std::vector<Point3D> &vecs, const std::vector<float> &scalars, unsigned int mode);
 };
 
-struct RGBFloat
-{
-	float v[3];
-};
 
-//Abstract class as base for overlays
-class DrawableOverlay : public DrawableObj
-{
-	protected:
-		//alpha (transparancy) value
-		float a;
-		//!Height and width of overlay (total)
-		float height,width;
-		//Fractional coordinates for the  top left of the overlay
-		float position[2];
-	public:
-		DrawableOverlay() {} ;
-		//Declared as pure virtual to force ABC
-		virtual ~DrawableOverlay() =0;
-		void setAlpha(float alpha) { a=alpha;};
-		void setSize(float widthN, float heightN) {height=heightN, width=widthN;} 
-		void setSize(float size) {width=height=size;};
-		void setPosition(float newTLX,float newTLY) { position[0]=newTLX; position[1]=newTLY;}
-
-		void getBoundingBox(BoundCube &b) const {b.setInvalid();};
-		//!This is an overlay
-		bool isOverlay() const {return true;};
-};
 
 class DrawColourBarOverlay : public DrawableOverlay
 {
@@ -1054,7 +1247,7 @@ class DrawPointLegendOverlay : public DrawableOverlay
 		void addItem(const std::string &s, float r, float g, float b);
 
 };
-
+//FIXME: Remove this.
 struct RGBThis
 {
 	unsigned char v[3];
@@ -1065,6 +1258,12 @@ class DrawField3D : public DrawableObj
 {
 	private:
 		mutable std::vector<std::pair<Point3D,RGBThis> > ptsCache;
+#ifdef ENABLE_LIBVD
+		//Volume renderer interface
+		mutable libvdDisplay::Volume volumeRenderer;
+		mutable bool volumeRenderCacheOK; 
+#endif
+
 		mutable bool ptsCacheOK;
 	protected:
 		//!Alpha transparancy of objects in field
@@ -1088,12 +1287,20 @@ class DrawField3D : public DrawableObj
 
 		//!Which colourmap to use
 		unsigned int colourMapID;
+		
+		//!Whether we should use eg a built-in colour map, custom
+		// or something else
+		unsigned int colourMapMode;
+
+		//!Custom colour map to use (if custom map specified)
+		std::vector<std::pair<float,ColourRGBAf> > colourMap;
 
 		//!Sets the render mode for the 3D volume 
 		/* Possible modes
 		 * 0: Alpha blended points
 		 */
 		unsigned int volumeRenderMode;
+
 		//!The scalar field - used to store data values
 		const Voxels<float> *field;
 	public:
@@ -1106,7 +1313,7 @@ class DrawField3D : public DrawableObj
 
 		//!Get the bounding box for this object
 		void getBoundingBox(BoundCube &b) const;
-		
+
 		//!Set the render mode (see volumeRenderMode variable for details)
 		void setRenderMode(unsigned int);
 		
@@ -1116,12 +1323,21 @@ class DrawField3D : public DrawableObj
 		//!Set the alpha value for elemnts
 		void setAlpha(float alpha);
 
-		//!Set the colour bar minima and maxima from current field values
-		void setColourMinMax();
+		//!SEt which type of colour map method we would like to use (none, built-in ro 
+		void setColourMapMode(unsigned int mode);
+		
+		//!SEt the transfer function for a custom colour map 
+		void setCustomColourMap(const std::vector<std::pair<float,ColourRGBAf> > &colourMap);
 
-		//!Set the colourMap ID
+		//!Set the colour bar minima and maxima from current field values
+		void setColourMinMax(float minV,float maxV);
+
+		//!Set the colourMap ID - only valid if using built-in colour map mode
 		void setColourMapID(unsigned int i){ colourMapID=i;};
 
+		//!Set if we want to draw the box surrounding the volume
+		void setDrawBox(bool doDraw) { drawBoundBox =doDraw;};
+
 		//!Render the field
 		void draw() const;
 
@@ -1134,6 +1350,8 @@ class DrawField3D : public DrawableObj
 		//!Set the coour of the bounding box
 		void setBoxColours(float r, float g, float b, float a);
 
+		//!Do we need depth sorting
+		bool needsDepthSorting() const;
 };
 
 class DrawIsoSurface: public DrawableObj
@@ -1142,13 +1360,6 @@ private:
 
 	mutable bool cacheOK;
 
-	//!should we draw the thing 
-	//	- in wireframe
-	//	-Flat
-	//	-Smooth
-	//
-	unsigned int drawMode;
-
 	//!Isosurface scalar threshold
 	float threshold;
 
@@ -1244,7 +1455,7 @@ class Draw2DCircle : public DrawableObj
 		virtual void draw() const;
 
 		void setCentre(float fx,float fy) { centre[0] = fx; centre[1]= fy;};
-		void setRadius(float r) { radius=r;}
+		void setRadius(float rad) { radius=rad;}
 		//Angular step in radiians
 		void setAngularStep(float da) { angularStep = da;};
 	
