@@ -1,6 +1,6 @@
 /*
  *	scene.cpp  - OpenGL 3D static scene implementation
- *	Copyright (C) 2015, D Haley 
+ *	Copyright (C) 2018, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -29,15 +29,16 @@ using std::vector;
 using std::string;
 
 
-//
+//Animation progress texture information
+const unsigned int ANIMATE_PROGRESS_NUMFRAMES=3;
 const char ANIMATE_PROGRESS_BASENAME[] = {
 	"textures/animProgress"};
-unsigned int ANIMATE_PROGRESS_NUMFRAMES=3;
 
 
 
-Scene::Scene() : tempCam(0), cameraSet(true), outWinAspect(1.0f)
+Scene::Scene() : tempCam(0), cameraSet(true), outWinAspect(1.0f), noticeText(getDefaultFontFile())
 {
+	MESSAGE("Scene Constructor");
 	glewInited=false;
 	visControl=0;
 
@@ -61,6 +62,10 @@ Scene::Scene() : tempCam(0), cameraSet(true), outWinAspect(1.0f)
 	lightPosition[2]= 1.0;
 	lightPosition[3]=0.0;
 
+	lastMessageTime.tv_sec=lastMessageTime.tv_usec=0;
+	noticeText.setPosition(0.02,0.95);
+	noticeText.setSize(0.015);
+
 	DrawableObj::setTexPool(new TexturePool);
 
 }
@@ -257,6 +262,29 @@ void Scene::draw(bool noUpdateCam)
 		updateCam(camToUse);
 
 
+	//FIXME: Don't always sort - use a "dirty" flag on camera change
+	{
+	//FIXME: Unify refObjects and objects into a single 
+	// vector, so we can sort them together
+	unsigned int depthSortCount=0;
+	//Perform depth sorting of object vectors if needed
+	for(unsigned int ui=0;ui<refObjects.size();ui++)
+	{
+		if(refObjects[ui]->needsDepthSorting())
+			depthSortCount++;
+	}
+	if(depthSortCount>1 )
+		depthSort(refObjects);
+
+	depthSortCount=0;
+	for(unsigned int ui=0;ui<objects.size();ui++)
+	{
+		if(objects[ui]->needsDepthSorting())
+			depthSortCount++;
+	}
+	if(depthSortCount>1 )
+		depthSort(objects);
+	}
 
 	for(unsigned int passNumber=0; passNumber<numberTotalPasses; passNumber++)
 	{
@@ -296,6 +324,7 @@ void Scene::draw(bool noUpdateCam)
 		drawObjectVector(refObjects,lightsOn,true);
 		//Draw normal objects
 		drawObjectVector(objects,lightsOn,true);
+
 		//-----------	
 		
 		//Second sub-pass with transparent objects
@@ -304,6 +333,7 @@ void Scene::draw(bool noUpdateCam)
 		drawObjectVector(refObjects,lightsOn,false);
 		//Draw normal objects
 		drawObjectVector(objects,lightsOn,false);
+
 		//-----------	
 		
 		
@@ -333,6 +363,10 @@ void Scene::draw(bool noUpdateCam)
 		if(!lockInteract&& lastHovered != (unsigned int)(-1) )
 			drawHoverOverlay();
 
+		//Update text notifications
+		updateNotifications();
+
+		//Draw all overlays
 		drawOverlays(noUpdateCam);
 
 		//Draw progress, if needed
@@ -451,12 +485,15 @@ void Scene::drawOverlays(bool noUpdateCam) const
 	}
 	
 
+	//Draw any scene notifications
+	drawNotifications();
+
 	if(!noUpdateCam)
 	{
-		//op our modelview matrix
+		//pop our modelview matrix
 		glPopMatrix();
 		
-		//ppop projection atrix
+		//pop projection atrix
 		glMatrixMode(GL_PROJECTION);
 		glPopMatrix();
 		//return to modelview mode
@@ -628,6 +665,92 @@ void Scene::drawProgressAnim() const
 		glEnable(GL_LIGHTING);
 }
 
+void Scene::addNotice(const std::string &s)
+{
+	noticeMessages.push_back(s);
+}
+
+void Scene::updateNotifications()
+{
+	//Don't do anything if we've no messages
+	if(noticeMessages.empty())
+		return;
+
+	
+	//Time until message completely fades out
+	const float MESSAGE_FADE_TIME=2.5; //Seconds
+	//Time from which we start fading the message
+	const float MESSAGE_FADE_START=1.0; //Seconds
+	ASSERT(MESSAGE_FADE_TIME > MESSAGE_FADE_START);
+
+
+	timeval tNow;
+	gettimeofday(&tNow,NULL);
+	float deltaMessageTime;
+
+	if(lastMessageTime.tv_sec ==0 && lastMessageTime.tv_usec == 0)
+	{
+		lastMessageTime=tNow;
+	}
+
+	deltaMessageTime = (tNow.tv_sec- lastMessageTime.tv_sec) + (tNow.tv_usec - lastMessageTime.tv_usec)/1e6;
+
+
+	//Check the last message time
+	if(deltaMessageTime > MESSAGE_FADE_TIME)
+	{
+		noticeMessages.pop_front();
+		lastMessageTime=tNow;
+	}
+
+	//Set up the text
+	if(noticeMessages.empty())
+	{
+		noticeText.setText("");
+		lastMessageTime.tv_usec=lastMessageTime.tv_sec=0;
+		return;
+	}
+	else
+	{
+		//Set up the notification text
+		noticeText.setText((noticeMessages.front()));
+	}
+
+	//cap the fade time 
+	deltaMessageTime=std::min(deltaMessageTime,MESSAGE_FADE_TIME);
+	//Linear fade out
+	float alphaV;
+	alphaV = 1.0 - (deltaMessageTime - MESSAGE_FADE_START)/(MESSAGE_FADE_TIME-MESSAGE_FADE_START);
+
+	noticeText.setAlpha(alphaV);
+
+	float rC,gC,bC;
+	getContrastingColour(rBack,gBack,bBack,
+				rC,gC,bC);
+
+	//HACK - make less stark
+	if(rC > 0.5)
+	{
+		rC*=0.75;
+		gC*=0.75;
+		bC*=0.75;
+	}
+	else
+	{
+		rC+=0.25;
+		gC+=0.25;
+		bC+=0.25;
+	}
+	
+	noticeText.setColour(rC,gC,bC);
+}
+
+void Scene::drawNotifications() const 
+{
+	noticeText.draw();
+
+}
+
 void Scene::commitTempCam()
 {
 	ASSERT(tempCam);
@@ -1071,3 +1194,50 @@ void Scene::clearEffects()
 	effects.clear();
 	effectIDs.clear();
 }
+
+void Scene::depthSort(vector<const DrawableObj *> &objVec)
+{
+	Point3D camOrigin = activeCam->getOrigin();
+	//We need to sort them by distance
+	//from eye (back to front), otherwise they will not blend properly
+	std::vector<std::pair<float,unsigned int >  > eyeDists;
+
+	eyeDists.resize(objVec.size());
+
+	//Set up an original index for the eye distances
+	#pragma omp parallel for shared(camOrigin)
+	for(unsigned int ui=0;ui<objVec.size();ui++)
+	{
+		Point3D centroid;
+
+		if(!objVec[ui]->isOverlay())
+		{
+			BoundCube bc;
+			objVec[ui]->getBoundingBox(bc);
+			centroid=bc.getCentroid();
+		}
+		else
+			centroid=camOrigin;
+
+		//Use the distance (actually distance^2) to the boundbox centre
+		// as the sort key
+		eyeDists[ui].first=centroid.sqrDist(camOrigin);
+		eyeDists[ui].second=ui;
+	}
+
+
+	//Perform distance sort (furthest object at back)
+	ComparePairFirstReverse cmp;
+	std::sort(eyeDists.begin(),eyeDists.end(),cmp);	
+
+	//Re-order the vector contents
+	//--
+	vector<const DrawableObj *> tmpV;
+	tmpV.resize(objVec.size());
+	for(unsigned int ui=0;ui<eyeDists.size(); ui++)
+		tmpV[ui]  = objVec[eyeDists[ui].second];
+
+	tmpV.swap(objVec);
+	//--
+}
+
