@@ -305,7 +305,7 @@ void trPerspective(TRcontext *tr,
                    GLdouble zNear, GLdouble zFar )
 {
    GLdouble xmin, xmax, ymin, ymax;
-   ymax = zNear * tan(fovy * 3.14159265 / 360.0);
+   ymax = zNear * tan(fovy * M_PI / 360.0);
    ymin = -ymax;
    xmin = ymin * aspect;
    xmax = ymax * aspect;
