@@ -1,6 +1,6 @@
 /*
  *	wxCropPanel.cpp - cropping window  for user interaction
- *	Copyright (C) 2015, D Haley 
+ *	Copyright (C) 2018, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -81,6 +81,8 @@ void CropPanel::mouseMove(wxMouseEvent &event)
 	//Do our calculations in reduced coordinates (0->1);
 	float xMouse,yMouse;
 	wxPoint mousePos =event.GetPosition();
+	//FIXME: This linearly distributes the distance across the control
+	// perhaps we want to have a less linear model?
 	//Add a 1px border around control
 	xMouse=(float)(mousePos.x+1)/(float)(w-2);
 	yMouse=(float)(mousePos.y+1)/(float)(h-2);
@@ -156,6 +158,16 @@ void CropPanel::mouseMove(wxMouseEvent &event)
 				float delta[2];
 				delta[0]=xMouse-mouseAtDragStart[0];
 				delta[1]=yMouse-mouseAtDragStart[1];
+				
+				//If shift down, only drag in one direction
+				if(event.ShiftDown())
+				{
+					if(abs(delta[1]) < abs(delta[0]))
+						delta[1]=0;
+					else
+						delta[0]=0;
+				}
+
 				for(unsigned int ui=0;ui<4;ui++)
 				{
 					float flip;
