@@ -1,6 +1,6 @@
 /*
  *	ExportRngDialog.cpp  - "Range" data export dialog
- *	Copyright (C) 2015, D Haley 
+ *	Copyright (C) 2018, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -237,7 +237,7 @@ void ExportRngDialog::updateRangeList()
 	       	itemIndex=listRanges->InsertItem(0, (rangeData->getUserString())); 
 		unsigned int nIons,nRngs; 
 		nIons = rangeData->getRange().getNumIons();
-		nRngs = rangeData->getRange().getNumIons();
+		nRngs = rangeData->getRange().getNumRanges();
 
 		stream_cast(tmpStr,nIons);
 		listRanges->SetItem(itemIndex, 1, (tmpStr)); 
