@@ -1,6 +1,6 @@
 /*
  *	animateFilterDialog - GUI for animate filter
- *	Copyright (C) 2015, D. Haley, A Ceguerra 
+ *	Copyright (C) 2018, D Haley, A Ceguerra 
  
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -152,9 +152,6 @@ private:
     size_t rangeExportMode;
 
 
-    //viewport aspect ratio for image output
-    float imageAspectRatio;
-
     //Used to jump out of wx events that are generated by 
     // the code, rather than the user, eg text events
     bool programmaticEvent;
@@ -162,13 +159,13 @@ private:
     void update(); 
 
 
-    //Enable/disable the OK button depending upon dialog state
+    //!Enable/disable the OK button depending upon dialog state
     void updateOKButton();
-    //update function for frame view page, grid control
+    //!update function for frame view page, grid control
     void updateFilterViewGrid();
-    //update function for frame view page, grid contorl
+    //!update function for frame view page, grid contorl
     void updateFrameViewGrid();
-    //Updates the slider on the frame view page
+    //!Updates the slider on the frame view page
     void updateFrameViewSlider();
     // begin wxGlade: ExportAnimationDialog::methods
     void set_properties();
