@@ -1,6 +1,6 @@
 /*
  *	rangeEditDialog.h - Point data export dialog
- *	Copyright (C) 2015, D Haley 
+ *	Copyright (C) 2018, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -299,8 +299,8 @@ void RangeEditorDialog::onPlotUpdate()
 	size_t lastEditedRegion,lastEditedPlot;
 	plotPanel->getLastEdited(lastEditedPlot,lastEditedRegion);
 
-	ASSERT(lastEditedRegion!= -1);
-	ASSERT(lastEditedPlot != -1);
+	ASSERT(lastEditedRegion!=(unsigned int) -1);
+	ASSERT(lastEditedPlot != (unsigned int)-1);
 #endif
 
 	generateRangeEntries();
