@@ -1,6 +1,6 @@
 /*
  *	glPane.cpp - OpenGL panel implementation
- *	Copyright (C) 2015, D Haley 
+ *	Copyright (C) 2018, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -594,29 +594,29 @@ void BasicGLPane::keyPressed(wxKeyEvent& event)
 			{
 				//Space only
 				case 0: 
-					visibleDir=3; 
+					visibleDir=CAMERA_DIR_XPLUS; 
 					break;
 				//Command down +space 
 				case 1:
-					visibleDir=0;
+					visibleDir=CAMERA_DIR_YPLUS;
 					break;
 				//Shift +space 
 				case 2:
-					visibleDir=2;
+					visibleDir=CAMERA_DIR_ZPLUS;
 					break;
 				//NO CASE 3	
 				//Double+space
 				case 4:
-					visibleDir=5; 
+					visibleDir=CAMERA_DIR_XMINUS; 
 					break;
 				//Doublespace+Cmd
 				case 5:
-					visibleDir=4;
+					visibleDir=CAMERA_DIR_YMINUS;
 					break;
 
 				//Space+Double+shift
 				case 6:
-					visibleDir=1;
+					visibleDir=CAMERA_DIR_ZMINUS;
 					break;
 				default:
 					;
