@@ -1,6 +1,6 @@
 /*
  * 	3Depict.h - main program header
- * 	Copyright (C) 2015 D Haley
+ * 	Copyright (C) 2018, D Haley
  *
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -112,7 +112,9 @@ private:
     void do_filtergrid_prop_layout();
     //Force a re-layout of the camera property grid
     void do_cameragrid_prop_layout();
-  
+    //Delete the tree filters, updating the remaining code as required
+    void deleteTreeFilter(wxTreeItemId &id);
+     
 	bool refreshThreadActive() { return refreshThread && refreshThread->IsRunning();};
  
    	//!Queue up a status message for display
@@ -124,7 +126,7 @@ private:
 	//!Update the progress information in the status bar
 	void updateProgressStatus();
 	//!Perform an update to the 3D Scene. Returns false if refresh failed
-	bool doSceneUpdate(bool ensureResultVisible=false);
+	void doSceneUpdate(bool ensureResultVisible=false);
 	
 	//!Complete the scene update. Returns false if failed
 	void finishSceneUpdate(unsigned int errCode);
@@ -261,6 +263,10 @@ protected:
     wxButton* buttonAlignCamYMinus;
     wxButton* buttonAlignCamZMinus;
     wxCheckBox* checkAlignCamResize;
+    wxStaticText* labelCameraAlignRotate;
+    wxButton* buttonCamRotatePlus;
+    wxButton* buttonCamRotateMinus;
+    wxSpinCtrl* spinCtrlRotateDegrees;
     wxScrolledWindow* noteCamera;
     wxCheckBox* checkPostProcessing;
     wxCheckBox* checkFxCrop;
@@ -337,12 +343,13 @@ public:
     void OnFileExportImage(wxCommandEvent &event); // wxGlade: <event_handler>
     void OnFileExportIons(wxCommandEvent &event); // wxGlade: <event_handler>
     void OnFileExportRange(wxCommandEvent &event); // wxGlade: <event_handler>
-    void OnViewBackground(wxCommandEvent &event); // wxGlade: <event_handler>
+    void OnFileExportVoxel(wxCommandEvent &event);
     void OnFileExit(wxCommandEvent &event); // wxGlade: <event_handler>
     void OnEditUndo(wxCommandEvent &event); // wxGlade: <event_handler>
     void OnEditRedo(wxCommandEvent &event); // wxGlade: <event_handler>
     void OnEditRange(wxCommandEvent &event); // wxGlade: <event_handler>
     void OnEditPreferences(wxCommandEvent &event); // wxGlade: <event_handler>
+    void OnViewBackground(wxCommandEvent &event); // wxGlade: <event_handler>
     void OnViewControlPane(wxCommandEvent &event); // wxGlade: <event_handler>
     void OnViewRawDataPane(wxCommandEvent &event); // wxGlade: <event_handler>
     void OnHelpHelp(wxCommandEvent &event); // wxGlade: <event_handler>
@@ -377,6 +384,8 @@ public:
     void OnButtonAlignCameraXMinus(wxCommandEvent &event); // wxGlade: <event_handler>
     void OnButtonAlignCameraYMinus(wxCommandEvent &event); // wxGlade: <event_handler>
     void OnButtonAlignCameraZMinus(wxCommandEvent &event); // wxGlade: <event_handler>
+    void OnButtonAlignCameraRotateMinus(wxCommandEvent &event); // wxGlade: <event_handler>
+    void OnButtonAlignCameraRotatePlus(wxCommandEvent &event); // wxGlade: <event_handler>
     void OnCheckPostProcess(wxCommandEvent &event); // wxGlade: <event_handler>
     void OnFxCropCheck(wxCommandEvent &event); // wxGlade: <event_handler>
     void OnFxCropCamFrameCheck(wxCommandEvent &event); // wxGlade: <event_handler>
@@ -391,8 +400,8 @@ public:
     void OnCheckWeakRandom(wxCommandEvent &event); // wxGlade: <event_handler>
     void OnSpectraListbox(wxCommandEvent &event); // wxGlade: <event_handler>
     void OnCheckLimitOutput(wxCommandEvent &event); // wxGlade: <event_handler>
-    void OnTextLimitOutput(wxCommandEvent &event); // wxGlade: <event_handler>
     void OnTextLimitOutputEnter(wxCommandEvent &event); // wxGlade: <event_handler>
+    void OnTextLimitOutput(wxCommandEvent &event); // wxGlade: <event_handler>
     void OnCheckCacheEnable(wxCommandEvent &event); // wxGlade: <event_handler>
     void OnCacheRamUsageSpin(wxSpinEvent &event); // wxGlade: <event_handler>
 
@@ -430,6 +439,8 @@ public:
 
     void OnTreeBeginLabelEdit(wxTreeEvent &evt);
     void OnTreeEndLabelEdit(wxTreeEvent &evt);
+    void OnTreeRightClick(wxContextMenuEvent &evt);
+    void OnTreePopupMenu(wxCommandEvent &evt);
     
     void OnUpdateTimer(wxTimerEvent &evt);
     void OnAutosaveTimer(wxTimerEvent &evt);
