@@ -1,6 +1,6 @@
 /*
  *	mathglPane.cpp - mathgl-wx interface panel control
- *	Copyright (C) 2015, D Haley 
+ *	Copyright (C) 2018, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -24,7 +24,6 @@
 
 #include "wx/wxcommon.h"
 #include "common/translation.h"
-#include "backend/plot.h"
 
 #ifndef pow10
 #define pow10(x) pow(10,x)
@@ -40,6 +39,11 @@ const float MGL_PAN_SPEED=2.0f;
 //Mathgl uses floating point loop computation, and can get stuck. Limit zoom precision
 const float MGL_ZOOM_LIMIT=10.0f*sqrt(std::numeric_limits<float>::epsilon());
 
+//We overdraw on the horizontal axis, because we don't like
+// the margin handling that mathgl provides.
+// This is the horizontal pre-scaling factor for the mgl plot from
+// the original window size
+const float HORIZ_MARGIN_FACTOR = 0.25;
 
 //Mouse action types
 enum
@@ -171,7 +175,7 @@ void MathGLPane::setPanCoords() const
 bool MathGLPane::readyForInput() const
 {
 	return (thePlot && gr && 
-		!thePlot->isInteractionLocked()  && thePlot->getNumTotal());
+		!thePlot->isInteractionLocked()  && thePlot->getNumVisible());
 }
 
 unsigned int MathGLPane::getAxisMask(int x, int y) const
@@ -274,6 +278,9 @@ void MathGLPane::render(wxPaintEvent &event)
 	if(doTrap)
 		trapfpe(false);
 #endif
+
+	const unsigned int WIDTH_AXIS_MARGIN=HORIZ_MARGIN_FACTOR*w;
+
 	//If the plot has changed, been resized or is performing
 	// a mouse action that requires updating, we need to update it
 	//likewise if we don't have a plot, we need one.
@@ -283,7 +290,7 @@ void MathGLPane::render(wxPaintEvent &event)
 		//clear the plot drawing entity
 		if(!gr)
 		{
-			gr = new mglGraph(0,w,h);
+			gr = new mglGraph(0,w+WIDTH_AXIS_MARGIN,h);
 #ifdef __APPLE__
 			//apparenty bug in mgl under osx - font wont load,
 			// use random string to force fallback
@@ -292,7 +299,7 @@ void MathGLPane::render(wxPaintEvent &event)
 		}
 		else
 		{
-			gr->SetSize(w,h);
+			gr->SetSize(w+WIDTH_AXIS_MARGIN,h);
 		}
 		
 		//change the plot by panningOneD it before we draw.
@@ -312,10 +319,28 @@ void MathGLPane::render(wxPaintEvent &event)
 		hasResized=false;
 
 		//Copy the plot's memory buffer into a wxImage object, then draw it	
-		char *rgbdata = (char*)malloc(w*h*3);
-		gr->GetRGB((char*)rgbdata,w*h*3);
-		
-		imageCacheBmp=wxBitmap(wxImage(w,h,(unsigned char*)rgbdata,true));
+		char *rgbdata = (char*)malloc((w+WIDTH_AXIS_MARGIN)*h*3);
+		gr->GetRGB((char*)rgbdata,(w+WIDTH_AXIS_MARGIN)*h*3);
+		wxImage img(w,h);
+
+	
+#pragma omp parallel for
+		for(unsigned int uj=0;uj<(unsigned int)h; uj++)
+		{
+			unsigned int offset;
+			offset = (w+WIDTH_AXIS_MARGIN)*uj*3;
+
+			for(unsigned int ui=WIDTH_AXIS_MARGIN/2;ui<w+WIDTH_AXIS_MARGIN/2; ui++)
+			{
+				unsigned char pixR,pixG,pixB;
+				pixR = rgbdata[offset+ui*3];
+				pixG = rgbdata[offset+ui*3+1];
+				pixB = rgbdata[offset+ui*3+2];
+				img.SetRGB(ui-WIDTH_AXIS_MARGIN/2,uj,pixR,pixG,pixB);
+			}
+		}
+
+		imageCacheBmp=wxBitmap(img);
 		free(rgbdata);
 	}
 
@@ -323,7 +348,7 @@ void MathGLPane::render(wxPaintEvent &event)
 	if(doTrap)
 		trapfpe(true);
 #endif
-	dc->DrawBitmap(wxBitmap(imageCacheBmp),0,0);
+	dc->DrawBitmap(imageCacheBmp,0,0);
 	//If we are engaged in a dragging operation
 	//draw the nice little bits we need
 	switch(mouseDragMode)
@@ -606,8 +631,9 @@ void MathGLPane::mouseDoubleMiddleClick(wxMouseEvent &event)
 void MathGLPane::oneDMouseDownAction(bool leftDown,bool middleDown,
 		 bool alternateDown, int dragX,int dragY)
 {
-	ASSERT(thePlot->getNumVisible());
-	
+	if(!readyForInput())
+		return;
+
 	float xMin,xMax,yMin,yMax;
 	thePlot->getBounds(xMin,xMax,yMin,yMax);
 
@@ -627,7 +653,7 @@ void MathGLPane::oneDMouseDownAction(bool leftDown,bool middleDown,
 			thePlot->getRegion(plotId,regionId,r);
 
 			//TODO: Implement a more generic region handler?
-			ASSERT(thePlot->plotType(plotId) == PLOT_MODE_1D);
+			ASSERT(thePlot->getPlotMode(plotId) == PLOT_MODE_1D);
 
 			float mglStartX,mglStartY;
 			toPlotCoords(draggingStart.x, draggingStart.y,mglStartX,mglStartY);
@@ -660,10 +686,9 @@ void MathGLPane::oneDMouseDownAction(bool leftDown,bool middleDown,
 void MathGLPane::twoDMouseDownAction(bool leftDown,bool middleDown,
 		 bool alternateDown, int dragX,int dragY)
 {
-	ASSERT(thePlot->getNumVisible());
-	
-	float xMin,xMax,yMin,yMax;
-	thePlot->getBounds(xMin,xMax,yMin,yMax);
+	//Ensure that we actually have an active plot
+	if(!readyForInput())
+		return;
 
 	//Set the interaction mode
 	if(leftDown && !alternateDown )
@@ -678,6 +703,10 @@ void MathGLPane::twoDMouseDownAction(bool leftDown,bool middleDown,
 		mouseDragMode=MOUSE_MODE_DRAG_PAN;
 		draggingStart = wxPoint(dragX,dragY);
 		
+		//Obtain the plot's bounding box
+		float xMin,xMax,yMin,yMax;
+		thePlot->getBounds(xMin,xMax,yMin,yMax);
+
 		origPanMinX=xMin;
 		origPanMaxX=xMax;
 		origPanMinY=yMin;
@@ -772,6 +801,7 @@ void MathGLPane::mouseWheelMoved(wxMouseEvent& event)
 	//Bigger numbers mean faster. 
 	const float SCROLL_WHEEL_ZOOM_RATE=0.20;
 
+	//negative for a wheel forwards/up. Positive for wheel down
 	float zoomRate=(float)event.GetWheelRotation()/(float)event.GetWheelDelta();
 	zoomRate=zoomRate*SCROLL_WHEEL_ZOOM_RATE;
 
@@ -780,12 +810,12 @@ void MathGLPane::mouseWheelMoved(wxMouseEvent& event)
 	if(zoomRate > 0.0f)
 	{
 		zoomFactor=1.0/(1.0+zoomRate);
-		ASSERT(zoomFactor> 1.0f);
+		ASSERT(zoomFactor< 1.0f);
 	}
 	else
 	{
 		zoomFactor=(1.0-zoomRate);
-		ASSERT(zoomFactor < 1.0f);
+		ASSERT(zoomFactor > 1.0f);
 	}
 
 
@@ -965,7 +995,7 @@ void MathGLPane::updateDragPos(const wxPoint &draggingEnd) const
 		endX=draggingStart.x;
 	}
 
-	if(h-draggingEnd.y > h-draggingStart.y)
+	if(draggingEnd.y > draggingStart.y)
 	{
 		startY=draggingStart.y;
 		endY=draggingEnd.y;
@@ -1417,7 +1447,13 @@ bool MathGLPane::toPlotCoords(int winX, int winY,float &resX, float &resY) const
 		WARN(false,"DEBUG ONLY - was outside window coord");
 		return false;
 	}
-		
+
+	//Account for the horizontal margin.
+	//Inverse transform = (Q_x-winx/2)/F + winX/2;
+	// Q_x : point in stretched space, F : stretch factor, winX: 
+	winX = winX + (width*HORIZ_MARGIN_FACTOR/2.0f);
+	
+
 	ASSERT(gr);
 	mglPoint pt = gr->CalcXYZ(winX,winY);
 	
@@ -1439,14 +1475,17 @@ bool MathGLPane::toPlotCoords(int winX, int winY,float &resX, float &resY) const
 }
 bool MathGLPane::toWinCoords(float plotX, float plotY, float &winX, float &winY) const
 {
+	int width, height;
+	GetClientSize(&width,&height);
+
 	mglPoint tmp;
 	tmp=gr->CalcScr(mglPoint(plotX,plotY));
-	winX=tmp.x; winY=tmp.y;
+	winX=tmp.x -width*HORIZ_MARGIN_FACTOR/2.0f; winY=tmp.y;
 
 	if(plotIsLogarithmic)
 	{
 		//FIXME: IMPLEMENT ME
-		WARN(false,"NOT IMPLEMENTED FOR LOG MODE");
+		WARN(false,"NOT IMPLEMENTED FOR LOG MODE (Y value)");
 		return true;
 	}
 	else
@@ -1471,7 +1510,7 @@ void MathGLPane::drawRegionDraggingOverlay(wxDC *dc) const
 		return;
 
 
-	ASSERT(thePlot->plotType(startMousePlot) == PLOT_MODE_1D);
+	ASSERT(thePlot->getPlotMode(startMousePlot) == PLOT_MODE_1D);
 
 	//See where extending the region is allowed up to.
 	thePlot->findRegionLimit(startMousePlot,startMouseRegion,
@@ -1550,7 +1589,7 @@ void MathGLPane::drawRegionDraggingOverlay(wxDC *dc) const
 		{
 			//This needs to be extended to support more
 			//plot types.
-			ASSERT(thePlot->plotType(startMousePlot) == PLOT_MODE_1D);
+			ASSERT(thePlot->getPlotMode(startMousePlot) == PLOT_MODE_1D);
 			
 			//Draw "ghost" limits markers for move,
 			//these appear as moving vertical bars to outline
