@@ -1,6 +1,6 @@
 /*
  *	filtertesting.cpp - unit testing implementation for filter code
- *	Copyright (C) 2015, D Haley 
+ *	Copyright (C) 2018, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -62,7 +62,7 @@ bool testFilterTree(const FilterTree &f,
 	std::list<std::pair<Filter *, std::vector<const FilterStreamData * > > > &outData )
 {
 	std::vector<SelectionDevice *> devices;
-	std::vector<std::pair<const Filter *, string > > consoleMessages;
+	std::vector<ConsoleMessage > consoleMessages;
 
 	ProgressData prog;
 #ifdef  HAVE_CPP_1X
@@ -152,6 +152,7 @@ bool filterCloneTests()
 	ConfigFile configFile;
     	configFile.read();
 	
+#ifndef WIN32
 	bool fileWarn=false;
 	for(unsigned int ui=0; ui<FILTER_TYPE_ENUM_END; ui++)
 	{
@@ -270,6 +271,7 @@ bool filterCloneTests()
 		delete f;
 		delete g;
 	}
+#endif
 	return true;
 }
 
