@@ -1,6 +1,6 @@
 /*
  *	testing.cpp - unit testing implementation
- *	Copyright (C) 2015, D Haley 
+ *	Copyright (C) 2018, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -29,6 +29,7 @@
 
 
 #include "backend/filters/allFilter.h"
+#include "backend/filters/filterCommon.h"
 #include "backend/APT/vtk.h"
 #include "backend/state.h"
 #include "backend/configFile.h"
@@ -36,13 +37,16 @@
 #include "backend/filters/algorithms/K3DTree-mk2.h"
 #include "backend/filters/algorithms/K3DTree.h"
 #include "backend/filters/algorithms/mass.h"
+#include "backend/filters/algorithms/convexHull.h"
 
 #include "backend/APT/ionhit.h"
 #include "backend/APT/APTFileIO.h"
 #include "backend/APT/abundanceParser.h"
 
+#include "common/mathfuncs.h"
 #include "common/stringFuncs.h"
 #include "common/xmlHelper.h"
+#include "common/colourmap.h"
 
 #include "gl/isoSurface.h"
 
@@ -121,12 +125,19 @@ bool basicFunctionTests()
 	LinearFeedbackShiftReg reg;
 	TEST(reg.verifyTable(16),"Check LFSR table integrity");
 
+	TEST(BoundCube::test(),"boundcube test");
 	
 	return true;
 }
 
 bool runUnitTests()
 {
+
+	//There is a sort-of-ordering required here, though it is soft and for convenience when debugging
+	// the basic unit tests should go first - things that test primitives and other low-dependency 
+	// units. Thus if there is a failure, it fails in the smallest location possible
+
+
 	//Set the abort pointer for the filter
 #ifdef HAVE_CPP_1X
 	ATOMIC_BOOL abortFlag(false);
@@ -142,6 +153,11 @@ bool runUnitTests()
 	K3DTreeMk2::setProgressPtr(&progressVar);
 
 	cerr << "Running unit tests..." ;
+	if(!testMathfuncs())
+		return false;
+
+	if(!testHull())
+		return false;
 
 	if(!algorithmTests())
 		return false;
@@ -151,6 +167,10 @@ bool runUnitTests()
 
 	if(!filterTests())
 		return false;
+
+	if(!testCommon())
+		return false;
+
 	if(!rangeFileLoadTests())
 		return false;
 
@@ -187,6 +207,10 @@ bool runUnitTests()
 
 	if(!fileFormatTests())
 		return false;
+
+	if(!testColourMap())
+		return false;
+
 	cerr << " OK" << endl << endl;
 
 	return true;
@@ -456,7 +480,8 @@ bool locateDataTests()
 	{
 		lineNum++;
 		std::string line;
-		getline(manifest,line);
+		if(!getline(manifest,line))
+			break;
 		line=stripWhite(line);
 
 		if(line.empty() || line[0] == '#')
