@@ -1,6 +1,6 @@
 /*
  *	wxcomponents.h - Custom wxWidgets components header
- *	Copyright (C) 2015, D Haley 
+ *	Copyright (C) 2018, D Haley 
 
  *	This program is free software: you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -33,8 +33,6 @@ using std::ofstream;
 using std::vector;
 using std::stack;
 
-const float FONT_HEADING_SCALEFACTOR=1.25f;
-
 void upWxTreeCtrl(const FilterTree &filterTree, wxTreeCtrl *t,
 		std::map<size_t,Filter *> &filterMap,vector<const Filter *> &persistentFilters,
 		const Filter *visibleFilt)
@@ -188,14 +186,14 @@ void CopyGrid::selectData()
 
 void CopyGrid::saveData()
 {
-	wxFileDialog *wxF = new wxFileDialog(this,TRANS("Save Data..."), wxT(""),
+	wxFileDialog wxF(this,TRANS("Save Data..."), wxT(""),
 		wxT(""),TRANS("Text File (*.txt)|*.txt|All Files (*)|*"),wxFD_SAVE);
 
-	if( (wxF->ShowModal() == wxID_CANCEL))
+	if( (wxF.ShowModal() == wxID_CANCEL))
 		return;
 	
 
-	std::string dataFile = stlStr(wxF->GetPath());
+	std::string dataFile = stlStr(wxF.GetPath());
 	ofstream f(dataFile.c_str());
 
 	if(!f)
@@ -483,13 +481,8 @@ void TextTreeCtrl::OnTreePaint(wxPaintEvent &event)
 		int startX;
 		startX=w/2 - textSize.GetWidth()/2; 
 
-#if !(defined(_WIN32) || defined(_WIN64) ) 
 		dc->DrawText((messageStrs[ui]),
 					startX,startY);	
-#else
-		dc->DrawTextW((messageStrs[ui]),
-					startX,startY);
-#endif
 		startY+=HEIGHT_SPACING*textSize.GetHeight();
 	}
 
